import 'lexer/Lexer.dart';
import 'lexer/enumns.dart';

import 'top_level/LibraryDefinition.dart';
import 'ParserExceptions.dart';

/// This object is used to parse a given string or file either sync or async.
class Parser {
  List<LibraryDefinition> libraryDefinitions;

  Parser() {
    libraryDefinitions = [];
  }

  Future<void> parse(String path) async {
    Lexer lexer = Lexer();
    await lexer.parse(path);
    _parseTokens(lexer);
  }

  void parseSync(String path) {
    Lexer lexer = Lexer()..parseSync(path);
    _parseTokens(lexer);
  }

  void parseStr(String str) {
    Lexer lexer = Lexer()..parseStr(str);
    _parseTokens(lexer);
  }

  void _parseTokens(Lexer lexer) {
    while (lexer.currentToken.type != TType.EOF) {
      int snapShot = lexer.currentIndex;
      try {
        while (lexer.currentToken.type == TType.COMMENT) {
          lexer.advance();
        }
        libraryDefinitions.add(LibraryDefinition(lexer));
      } on WrongTokenType {
        lexer.setPos(snapShot);
        lexer.advance();
      }
    }
  }

  String toString() {
    String rtn = '';
    libraryDefinitions.forEach((d) => rtn += '$d\n');
    return rtn.trim() + '\n';
  }
}
