import 'ParserExceptions.dart';
import 'lexer/Lexer.dart';

class ParserType {
  void checkVal(Lexer lexer, String val) {
    _skipComments(lexer);
    if (lexer.currentToken.val != val) {
      throw WrongTokenType(
          'Expected "$val", got ${lexer.currentToken.toString()}');
    }
  }

  void checkType(Lexer lexer, TType type) {
    _skipComments(lexer);
    if (lexer.currentToken.type != type) {
      throw WrongTokenType(
          'Expected <${lexer.currentToken.typeToString(type)}>, got ${lexer.currentToken.toString()}');
    }
  }

  void checkNotVal(Lexer lexer, String val) {
    _skipComments(lexer);
    if (lexer.currentToken.val == val) {
      throw WrongTokenType(
          'Expected not "$val", got ${lexer.currentToken.toString()}');
    }
  }

  void checkNotType(Lexer lexer, TType type) {
    _skipComments(lexer);
    if (lexer.currentToken.type == type) {
      throw WrongTokenType(
          'Expected not <${lexer.currentToken.typeToString(type)}>, got ${lexer.currentToken.toString()}');
    }
  }

  bool isVal(Lexer lexer, String val) {
    _skipComments(lexer);
    return lexer.currentToken.val == val;
  }

  bool isType(Lexer lexer, TType type) {
    _skipComments(lexer);
    return lexer.currentToken.type == type;
  }

  void skipIfType(Lexer lexer, TType type) {
    _skipComments(lexer);
    if (lexer.currentToken.type == type) {
      lexer.advance();
    }
  }

  void skipIfVal(Lexer lexer, String val) {
    _skipComments(lexer);
    if (lexer.currentToken.val == val) {
      lexer.advance();
    }
  }

  void _skipComments(Lexer lexer) {
    while (lexer.currentToken.type == TType.COMMENT) {
      lexer.advance();
    }
  }
}
