import '../ParserExceptions.dart';
import '../ParserType.dart';
import '../lexer/Lexer.dart';
import '../sub_types/Literals.dart';

/// <literal> ::= <nullLiteral>
/// | <booleanLiteral>
/// | <numericLiteral>
/// | <stringLiteral>
/// | <symbolLiteral>
/// | <setOrMapLiteral>
/// | <listLiteral>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 81
class Literal extends ParserType {
  int caseNum;
  NullLiteral nullLiteral;
  BooleanLiteral booleanLiteral;
  NumericLiteral numericLiteral;
  StringLiteral stringLiteral;
  SymbolLiteral symbolLiteral;
  SetOrMapLiteral setOrMapLiteral;
  ListLiteral listLiteral;

  Literal(Lexer lexer) {
    int attempt, maxAttempts = 7;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0:
            _case0(lexer);
            break;
          case 1:
            _case1(lexer);
            break;
          case 2:
            _case2(lexer);
            break;
          case 3:
            _case3(lexer);
            break;
          case 4:
            _case4(lexer);
            break;
          case 5:
            _case5(lexer);
            break;
          case 6:
            _case6(lexer);
            break;
        }

        break;
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts) {
      throw WrongTokenType('Tried all options in Literal: ALL FAILED');
    }
  }

  /// <nullLiteral>
  void _case0(Lexer lexer) {
    nullLiteral = NullLiteral(lexer);
    caseNum = 0;
  }

  /// <booleanLiteral>
  void _case1(Lexer lexer) {
    booleanLiteral = BooleanLiteral(lexer);
    caseNum = 1;
  }

  /// <numericLiteral>
  void _case2(Lexer lexer) {
    numericLiteral = NumericLiteral(lexer);
    caseNum = 2;
  }

  /// <stringLiteral>
  void _case3(Lexer lexer) {
    stringLiteral = StringLiteral(lexer);
    caseNum = 3;
  }

  /// <symbolLiteral>
  void _case4(Lexer lexer) {
    symbolLiteral = SymbolLiteral(lexer);
    caseNum = 4;
  }

  /// <setOrMapLiteral>
  void _case5(Lexer lexer) {
    setOrMapLiteral = SetOrMapLiteral(lexer);
    caseNum = 5;
  }

  /// <listLiteral>
  void _case6(Lexer lexer) {
    listLiteral = ListLiteral(lexer);
    caseNum = 6;
  }

  String toString() {
    switch (caseNum) {
      case 0:
        return '$nullLiteral';
        break;
      case 1:
        return '$booleanLiteral';
        break;
      case 2:
        return '$numericLiteral';
        break;
      case 3:
        return '$stringLiteral';
        break;
      case 4:
        return '$symbolLiteral';
        break;
      case 5:
        return '$setOrMapLiteral';
        break;
      case 6:
        return '$listLiteral';
        break;
    }
    return '';
  }
}
