import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import 'Identifier.dart';

/// <qualified> ::= <identifier> ('.' <identifier>)?
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 141
class Qualified extends ParserType {
  Identifier identifier1;
  Identifier identifier2;

  Qualified(Lexer lexer) {
    identifier1 = Identifier(lexer);

    int snapShot = lexer.currentIndex;
    try {
      checkType(lexer, TType.PERIOD);
      lexer.advance();

      identifier2 = Identifier(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    return '$identifier1' + (identifier2 != null ? '.$identifier2' : '');
  }
}
