import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/Types.dart';

/// <type> ::= <functionTypeTails>
/// | <typeNotFunction> <functionTypeTails>
/// | <typeNotFunction>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 176
class Type extends ParserType {
  int caseNum;
  FunctionTypeTails functionTypeTails;
  TypeNotFunction typeNotFunction;

  Type(Lexer lexer) {
    // print ('Starting Type');
    int attempt, maxAttempts = 3;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0:
            _case0(lexer);
            break;
          case 1:
            _case1(lexer);
            break;
          case 2:
            _case2(lexer);
            break;
        }

        // print ('Completed Type $attempt');
        break;
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts) {
      throw WrongTokenType('Tried all options in Type: ALL FAILED');
    }
  }

  /// <functionTypeTails>
  void _case0(Lexer lexer) {
    functionTypeTails = FunctionTypeTails(lexer);
    caseNum = 0;
  }

  /// <typeNotFunction> <functionTypeTails>
  void _case1(Lexer lexer) {
    typeNotFunction = TypeNotFunction(lexer);
    functionTypeTails = FunctionTypeTails(lexer);
    caseNum = 1;
  }

  /// <typeNotFunction>
  void _case2(Lexer lexer) {
    typeNotFunction = TypeNotFunction(lexer);
    caseNum = 2;
  }

  String toString() {
    switch (caseNum) {
      case 0:
        return '$functionTypeTails';
        break;
      case 1:
        return '$typeNotFunction $functionTypeTails';
        break;
      case 2:
        return '$typeNotFunction';
        break;
    }
    return '';
  }
}
