import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../main/Expression.dart';

/// <expressionList> ::= <expression> (',' <expression>)*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 74
class ExpressionList extends ParserType {
  List<Expression> expressions;

  ExpressionList(Lexer lexer) {
    // print ('started ExpressionList');
    expressions = [];
    expressions.add(Expression(lexer));
    // print ('valid');

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;
        lexer.advance();
        expressions.add(Expression(lexer));

        // This is because named arguments look the same but had a colon next
        // eg: ArgumentList('sometype, namedtype:0);
        if (isType(lexer, TType.COLON)) {
          expressions.removeLast();
          checkNotType(lexer, TType.COLON);
        }
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    // print ('finished ExpressionList');
  }

  String toString() {
    String rtn = '';
    for (int i = 0; i < expressions.length; ++i) {
      rtn += ', ${expressions[i]}';
    }
    return rtn.substring(2);
  }
}
