import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../core/Identifier.dart';

/// <identifierList> ::= <identifier> (',' <identifier>)*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 167
class IdentifierList extends ParserType {
  List<Identifier> identifiers;

  IdentifierList(Lexer lexer) {
    identifiers = [];
    identifiers.add(Identifier(lexer));

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;
        lexer.advance();
        identifiers.add(Identifier(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    String rtn = '';
    for (int i = 0; i < identifiers.length; ++i) {
      rtn += ', ${identifiers[i]}';
    }
    return rtn.substring(2);
  }
}
