import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/Identifiers.dart';

/// <initializedIdentifierList> ::= <initializedIdentifier> (',' <initializedIdentifier>)*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 14
class InitializedIdentifierList extends ParserType {
  List<InitializedIdentifier> initalizedIdentifiers;

  InitializedIdentifierList(Lexer lexer) {
    initalizedIdentifiers = [];
    initalizedIdentifiers.add(InitializedIdentifier(lexer));

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;
        lexer.advance();
        initalizedIdentifiers.add(InitializedIdentifier(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    String rtn = '';
    for (int i = 0; i < initalizedIdentifiers.length; ++i) {
      rtn += ', ${initalizedIdentifiers[i]}';
    }
    return rtn.substring(2);
  }
}
