import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/InitializerListEntry.dart';

/// <initializers> ::=
/// ':' <initializerListEntry> (',' <initializerListEntry>)*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 40
class Initializers extends ParserType {
  List<InitializerListEntry> initializerListEntries;

  Initializers(Lexer lexer) {
    checkType(lexer, TType.COLON);
    lexer.advance();

    initializerListEntries = [];
    initializerListEntries.add(InitializerListEntry(lexer));

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;
        lexer.advance();
        initializerListEntries.add(InitializerListEntry(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    String rtn = '';
    for (int i = 0; i < initializerListEntries.length; ++i) {
      rtn += ', ${initializerListEntries[i]}';
    }
    return ': ' + rtn.substring(2);
  }
}
