import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/StaticFinalDeclaration.dart';

/// <staticFinalDeclarationList> ::=
/// <staticFinalDeclaration> (',' <staticFinalDeclaration>)*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 27
class StaticFinalDeclarationList extends ParserType {
  List<StaticFinalDeclaration> staticFinalDeclarations;

  StaticFinalDeclarationList(Lexer lexer) {
    staticFinalDeclarations = [];
    staticFinalDeclarations.add(StaticFinalDeclaration(lexer));

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;
        lexer.advance();
        staticFinalDeclarations.add(StaticFinalDeclaration(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    String rtn = '';
    for (int i = 0; i < staticFinalDeclarations.length; ++i) {
      rtn += ', ${staticFinalDeclarations[i]}';
    }
    return rtn.substring(2);
  }
}
