import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../core/Type.dart';

/// <typeList> ::= <type> (',' <type>)*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 176
class TypeList extends ParserType {
  List<Type> types;

  TypeList(Lexer lexer) {
    types = [];
    types.add(Type(lexer));

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;

        lexer.advance();
        types.add(Type(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    String rtn = '';
    for (int i = 0; i < types.length; ++i) {
      rtn += ', ${types[i]}';
    }
    return rtn.substring(2);
  }
}
