import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/Types.dart';

/// <typeNotVoidList> ::= <typeNotVoid> (',' <typeNotVoid>)*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 176
class TypeNotVoidList extends ParserType {
  List<TypeNotVoid> typeNotVoids;

  TypeNotVoidList(Lexer lexer) {
    typeNotVoids = [];
    typeNotVoids.add(TypeNotVoid(lexer));

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;

        lexer.advance();
        typeNotVoids.add(TypeNotVoid(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    String rtn = '';
    for (int i = 0; i < typeNotVoids.length; ++i) {
      rtn += ', ${typeNotVoids[i]}';
    }
    return rtn.substring(2);
  }
}
