import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/Types.dart';

/// <typeNotVoidNotFunctionList> ::= <typeNotVoidNotFunction> (',' <typeNotVoidNotFunction>)*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 176
class TypeNotVoidNotFunctionList extends ParserType {
  List<TypeNotVoidNotFunction> typeNotVoidNotFunctions;

  TypeNotVoidNotFunctionList(Lexer lexer) {
    typeNotVoidNotFunctions = [];
    typeNotVoidNotFunctions.add(TypeNotVoidNotFunction(lexer));

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;

        lexer.advance();
        typeNotVoidNotFunctions.add(TypeNotVoidNotFunction(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    String rtn = '';
    for (int i = 0; i < typeNotVoidNotFunctions.length; ++i) {
      rtn += ', ${typeNotVoidNotFunctions[i]}';
    }
    return rtn.substring(2);
  }
}
