import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../main/TypeParameter.dart';

/// <typeParameters> ::= '<' <typeParameter> (',' <typeParameter>)* '>'
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 63

class TypeParameters extends ParserType {
  List<TypeParameter> typeParameters;

  TypeParameters(Lexer lexer) {
    typeParameters = [];

    checkType(lexer, TType.LEFT_ANGLE);
    lexer.advance();

    typeParameters.add(TypeParameter(lexer));

    int snapShot;
    try {
      snapShot = lexer.currentIndex;
      while (isType(lexer, TType.COMMA)) {
        lexer.advance();
        typeParameters.add(TypeParameter(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.RIGHT_ANGLE);
    lexer.advance();
  }

  String toString() {
    String rtn = '<';
    typeParameters.forEach((t) => rtn += '$t, ');
    rtn = rtn.substring(0, rtn.length - 2);
    return rtn + '>';
  }
}
