import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/FunctionBody.dart';
import '../sub_types/Declaration.dart';
import '../sub_types/Signatures.dart';

/// <classMemberDefinition> ::= <declaration> ';'
/// | <methodSignature> <functionBody>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 26

class ClassMemberDefinition extends ParserType {
  int caseNum;
  Declaration declaration;
  MethodSignature methodSignature;
  FunctionBody functionBody;

  ClassMemberDefinition(Lexer lexer) {
    int attempt, maxAttempts = 2;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0:
            _case0(lexer);
            break;
          case 1:
            _case1(lexer);
            break;
        }

        break;
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts) {
      throw WrongTokenType(
          'Tried all options in ClassMemberDefinition: ALL FAILED');
    }
  }

  /// <declaration> ';'
  void _case0(Lexer lexer) {
    declaration = Declaration(lexer);

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();

    caseNum = 0;
  }

  /// <methodSignature> <functionBody>
  void _case1(Lexer lexer) {
    methodSignature = MethodSignature(lexer);
    functionBody = FunctionBody(lexer);

    caseNum = 1;
  }

  String toString() {
    switch (caseNum) {
      case 0:
        return '$declaration;';
        break;
      case 1:
        return '$methodSignature $functionBody';
        break;
    }
    return '';
  }
}
