import '../ParserType.dart';
import '../lexer/Lexer.dart';

import '../main/MetaData.dart';
import '../core/Identifier.dart';

/// <enumType> ::= <metadata> enum <identifier> '{' <enumEntry> (',' <enumEntry>)* (',')? '}'
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 60
class EnumType extends ParserType {
  MetaData metaData;
  Identifier identifier;
  List<EnumEntry> enumEntries;

  EnumType(Lexer lexer) {
    metaData = MetaData(lexer);

    checkVal(lexer, 'enum');
    lexer.advance();

    identifier = Identifier(lexer);

    checkType(lexer, TType.LEFT_CURLY);
    lexer.advance();

    checkType(lexer, TType.IDENTIFIER);
    enumEntries = [];
    enumEntries.add(EnumEntry(lexer));

    while (isType(lexer, TType.COMMA)) {
      lexer.advance();
      if (isType(lexer, TType.IDENTIFIER)) {
        enumEntries.add(EnumEntry(lexer));
      } else {
        checkType(lexer, TType.RIGHT_CURLY);
      }
    }

    checkType(lexer, TType.RIGHT_CURLY);

    lexer.advance();
  }

  String toString() {
    return '$metaData' +
        'enum $identifier \{ ${enumEntries.join(', ').trim()} \}';
  }
}

/// <enumEntry> ::= <metadata> <identifier>
class EnumEntry extends ParserType {
  MetaData metaData;
  Identifier identifier;

  EnumEntry(Lexer lexer) {
    metaData = MetaData(lexer);
    identifier = Identifier(lexer);
  }

  String toString() {
    return '$metaData' + '$identifier';
  }
}
