import '../ParserType.dart';
import '../lexer/Lexer.dart';
import '../sub_types/ForLoopParts.dart';
import 'Statement.dart';

/// <forStatement> ::= await? for '(' <forLoopParts> ')' <statement>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 150

class ForStatement extends ParserType {
  ForLoopParts forLoopParts;
  Statement statement;
  bool isAwait;

  ForStatement(Lexer lexer) {
    isAwait = false;
    if (isType(lexer, TType.KEYWORD) && isVal(lexer, 'await')) {
      isAwait = true;
      lexer.advance();
    }

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'for');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    forLoopParts = ForLoopParts(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();

    statement = Statement(lexer);
  }

  String toString() {
    return (isAwait ? 'await ' : '') + 'for ($forLoopParts) $statement';
  }
}
