import '../ParserExceptions.dart';
import '../ParserType.dart';
import '../lexer/Lexer.dart';
import '../core/Qualified.dart';
import '../core/Identifier.dart';
import '../lists/Arguments.dart';

/// <metadata> ::= ('@' <qualified> ('.' <identifier>)? <arguments>?)*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 73
class MetaData extends ParserType {
  int length;

  // position 1 is qualified, 2 is identifier (maybe null), 3 is arguments (maybe null)
  Map<int, List<ParserType>> elements;
  // List<Qualified> qualified;
  // List<Identifier> identifier;
  // List<Arguments> arguments;

  MetaData(Lexer lexer) {
    length = 0;
    elements = Map();

    while (lexer.currentToken.type == TType.KUKAC) {
      elements[length] = [];

      lexer.advance();

      elements[length].add(Qualified(lexer));

      if (isType(lexer, TType.PERIOD)) {
        lexer.advance();
        elements[length].add(Identifier(lexer));
      } else {
        elements[length].add(null);
      }

      int snapShot = lexer.currentIndex;
      Arguments args;
      try {
        args = Arguments(lexer);
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
      elements[length].add(args);

      ++length;
    }
  }

  String toString() {
    return length > 0 ? _getStringFormatted().trimLeft() : '';
  }

  String _getStringFormatted() {
    String rtn = '';
    for (int i = 0; i < length; ++i) {
      rtn += '@' + elements[i][0].toString();
      rtn += elements[i][1] != null ? elements[i][1].toString() : '';
      rtn += elements[i][2] != null ? elements[i][2].toString() : '';
      rtn += ' ';
    }
    return rtn;
  }
}
