import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/AssignableSelector.dart';
import '../sub_types/ArgumentPart.dart';

/// <selector> ::= <assignableSelector>
/// | <argumentPart>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 137
class Selector extends ParserType {
  AssignableSelector assignableSelector;
  ArgumentPart argumentPart;

  Selector(Lexer lexer) {
    int snapShot = lexer.currentIndex;
    try {
      assignableSelector = AssignableSelector(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);

      argumentPart = ArgumentPart(lexer);
    }
  }

  String toString() {
    return (assignableSelector != null
        ? '$assignableSelector'
        : '$argumentPart');
  }
}
