import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../core/Label.dart';
import '../sub_types/NonLabelledStatement.dart';

/// <statement> ::= <label>* <nonLabelledStatement>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 145

class Statement extends ParserType {
  List<Label> labels;
  NonLabelledStatement nonLabelledStatement;

  Statement(Lexer lexer) {
    // print ('Started Statement');

    labels = [];

    int snapShot;
    try {
      while (true) {
        snapShot = lexer.currentIndex;
        labels.add(Label(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    nonLabelledStatement = NonLabelledStatement(lexer);

    // print ('Finished Statement');
  }

  String toString() {
    String rtn = '';
    labels.forEach((l) => rtn += '$l ');
    return rtn + '$nonLabelledStatement';
  }
}
