import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/Types.dart';
import '../main/Mixins.dart';

/// <superclass> ::= extends <typeNotVoid> <mixins>?
/// | <mixins>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 47
class SuperClass extends ParserType {
  int caseNum;
  TypeNotVoid typeNotVoid;
  Mixins mixins;

  SuperClass(Lexer lexer) {
    int attempt, maxAttempts = 2;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0:
            _case0(lexer);
            break;
          case 1:
            _case1(lexer);
            break;
        }

        break;
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts) {
      throw WrongTokenType('Tried all options in SuperClass: ALL FAILED');
    }
  }

  /// extends <typeNotVoid> <mixins>?
  void _case0(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'extends');
    lexer.advance();

    typeNotVoid = TypeNotVoid(lexer);

    int snapShotInside = lexer.currentIndex;
    try {
      mixins = Mixins(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShotInside);
    }

    caseNum = 0;
  }

  /// <mixins>
  void _case1(Lexer lexer) {
    mixins = Mixins(lexer);
    caseNum = 1;
  }

  String toString() {
    switch (caseNum) {
      case 0:
        return 'extends $typeNotVoid' + (mixins != null ? ' $mixins' : '');
        break;
      case 1:
        return '$mixins';
        break;
    }
    return '';
  }
}
