import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../sub_types/Cases.dart';
import 'Expression.dart';

/// <switchStatement> ::= switch '(' <expression> ')' '{' <switchCase>* <defaultCase>? '}'
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 153

class SwitchStatement extends ParserType {
  Expression expression;
  List<SwitchCase> switchCases;
  DefaultCase defaultCase;

  SwitchStatement(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'switch');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    expression = Expression(lexer);

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();

    checkType(lexer, TType.LEFT_CURLY);
    lexer.advance();

    switchCases = [];
    int snapShot;
    try {
      while (true) {
        snapShot = lexer.currentIndex;
        switchCases.add(SwitchCase(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    snapShot = lexer.currentIndex;
    try {
      defaultCase = DefaultCase(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.RIGHT_CURLY);
    lexer.advance();
  }

  String toString() {
    String rtn = 'switch ($expression) {';
    switchCases.forEach((c) => rtn += ' $c');
    if (defaultCase != null) {
      rtn += ' $defaultCase';
    }
    return rtn + ' }';
  }
}
