import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../main/MetaData.dart';
import '../core/Identifier.dart';
import '../sub_types/Types.dart';

/// <typeParameter> ::= <metadata> <identifier> (extends <typeNotVoid>)?
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 63

class TypeParameter extends ParserType {
  MetaData metaData;
  Identifier identifier;
  TypeNotVoid typeNotVoid;

  TypeParameter(Lexer lexer) {
    metaData = MetaData(lexer);
    identifier = Identifier(lexer);

    int snapShot = lexer.currentIndex;
    try {
      checkType(lexer, TType.KEYWORD);
      checkVal(lexer, 'extends');
      lexer.advance();

      typeNotVoid = TypeNotVoid(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    return '$metaData$identifier' +
        (typeNotVoid != null ? ' extends $typeNotVoid' : '');
  }
}
