import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../core/Identifier.dart';
import '../sub_types/Identifiers.dart';

/// <variableDeclaration> ::= <declaredIdentifier> (',' <identifier>)*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 14

class VariableDeclaration extends ParserType {
  DeclaredIdentifier declaredIdentifier;
  List<Identifier> identifiers;

  VariableDeclaration(Lexer lexer) {
    declaredIdentifier = DeclaredIdentifier(lexer);

    identifiers = [];
    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;
        lexer.advance();
        identifiers.add(Identifier(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    String rtn = '';
    identifiers.forEach((i) => rtn += ', $i');
    return '$declaredIdentifier' + rtn;
  }
}
