import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../main/Expression.dart';

/// <assertion> ::= assert '(' <expression> (',' <expression> )? ','? ')'
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 165

class Assertion extends ParserType {
  Expression expression1;
  Expression expression2;
  bool hasComma;

  Assertion(Lexer lexer) {
    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'assert');
    lexer.advance();

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    expression1 = Expression(lexer);

    int snapShot = lexer.currentIndex;
    try {
      checkType(lexer, TType.COMMA);
      lexer.advance();

      expression2 = Expression(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    if (isType(lexer, TType.COMMA)) {
      hasComma = true;
      lexer.advance();
    } else {
      hasComma = false;
    }

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();
  }

  String toString() {
    return 'assert ($expression1' +
        (expression2 != null ? ', $expression2' : '') +
        (hasComma ? ',' : '') +
        ')';
  }
}
