import '../ParserType.dart';
import '../ParserExceptions.dart';
import 'UnconditionalAssignableSelector.dart';
import '../lexer/Lexer.dart';
import '../core/Identifier.dart';

/// <assignableSelector> ::= <unconditionalAssignableSelector> | '?.' <identifier>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 140
class AssignableSelector extends ParserType {
  UnconditionalAssignableSelector unconditionalAssignableSelector;
  Identifier identifier;

  AssignableSelector(Lexer lexer) {
    int snapShot = lexer.currentIndex;
    try {
      unconditionalAssignableSelector = UnconditionalAssignableSelector(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);

      checkType(lexer, TType.Q_MARK);
      lexer.advance();
      checkType(lexer, TType.PERIOD);
      lexer.advance();

      identifier = Identifier(lexer);
    }
  }

  String toString() {
    return (identifier != null
        ? '?.$identifier'
        : '$unconditionalAssignableSelector');
  }
}
