import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import 'AssignableSelector.dart';
import 'ArgumentPart.dart';

/// <assignableSelectorPart> ::= <argumentPart>* <assignableSelector>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 140
class AssignableSelectorPart extends ParserType {
  List<ArgumentPart> argumentParts;
  AssignableSelector assignableSelector;

  AssignableSelectorPart(Lexer lexer) {
    argumentParts = [];
    int snapShot;
    try {
      while (true) {
        snapShot = lexer.currentIndex;
        argumentParts.add(ArgumentPart(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    assignableSelector = AssignableSelector(lexer);
  }

  String toString() {
    String rtn = '';
    argumentParts.forEach((p) => rtn += '$p ');
    return rtn.trimRight() + '$assignableSelector';
  }
}
