import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../lists/Statements.dart';
import '../main/Expression.dart';
import '../core/Label.dart';

/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 154

/// <switchCase> ::= <label>* case <expression> ':' <statements>
class SwitchCase extends ParserType {
  List<Label> labels;
  Expression expression;
  Statements statements;

  SwitchCase(Lexer lexer) {
    labels = [];

    int snapShot;
    try {
      while (true) {
        snapShot = lexer.currentIndex;
        labels.add(Label(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'case');
    lexer.advance();

    expression = Expression(lexer);

    checkType(lexer, TType.COLON);
    lexer.advance();

    statements = Statements(lexer);
  }

  String toString() {
    String rtn = '';
    labels.forEach((l) => rtn += '$l ');
    rtn += 'case $expression: $statements';
    return rtn;
  }
}

/// <defaultCase> ::= <label>* default ':' <statements>
class DefaultCase extends ParserType {
  List<Label> labels;
  Statements statements;

  DefaultCase(Lexer lexer) {
    labels = [];

    int snapShot;
    try {
      while (true) {
        snapShot = lexer.currentIndex;
        labels.add(Label(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'default');
    lexer.advance();

    checkType(lexer, TType.COLON);
    lexer.advance();

    statements = Statements(lexer);
  }

  String toString() {
    String rtn = '';
    labels.forEach((l) => rtn += '$l ');
    rtn += 'default: $statements';
    return rtn;
  }
}
