import '../ParserType.dart';
import '../lexer/Lexer.dart';
import '../core/Identifier.dart';
import '../lists/Arguments.dart';
import 'Types.dart';

/// <constructorInvocation> ::=
/// <typeName> <typeArguments> '.' <identifier> <arguments>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 137
class ConstructorInvocation extends ParserType {
  TypeName typeName;
  TypeArguments typeArguments;
  Identifier identifier;
  Arguments arguments;

  ConstructorInvocation(Lexer lexer) {
    typeName = TypeName(lexer);
    typeArguments = TypeArguments(lexer);

    checkType(lexer, TType.PERIOD);
    lexer.advance();

    identifier = Identifier(lexer);
    arguments = Arguments(lexer);
  }

  String toString() {
    return '$typeName$typeArguments.$identifier$arguments';
  }
}
