import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../main/Expression.dart';
import '../core/Identifier.dart';

/// <unconditionalAssignableSelector> ::= '[' <expression> ']' | '.' <identifier>
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 140
class UnconditionalAssignableSelector extends ParserType {
  Expression expression;
  Identifier identifier;

  UnconditionalAssignableSelector(Lexer lexer) {
    int snapShot = lexer.currentIndex;
    try {
      checkType(lexer, TType.LEFT_BRACKET);
      lexer.advance();

      expression = Expression(lexer);

      checkType(lexer, TType.RIGHT_BRACKET);
      lexer.advance();
    } on WrongTokenType {
      lexer.setPos(snapShot);

      checkType(lexer, TType.PERIOD);
      lexer.advance();

      identifier = Identifier(lexer);
    }
  }

  String toString() {
    return (expression != null ? '[$expression]' : '.$identifier');
  }
}
