import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../main/MetaData.dart';
import '../lists/DottedIdentifierList.dart';
import 'TopLevelDefinition.dart';
import 'LibraryImport.dart';
import 'LibraryExport.dart';
import 'Parts.dart';

/// TODO: Add scriptTags (need to modify lexer)
///
/// <libraryDefinition> ::=
/// <scriptTag>? <libraryName>? <importOrExport>* <partDirective>*
/// <topLevelDefinition>*
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 166
class LibraryDefinition extends ParserType {
  LibraryName libraryName;
  List<ImportOrExport> importOrExports;
  List<PartDirective> partDirectives;
  List<TopLevelDefinition> topLevelDefinitions;

  LibraryDefinition(Lexer lexer) {
    importOrExports = [];
    partDirectives = [];
    topLevelDefinitions = [];

    //TODO: add scriptTag

    int snapShot = lexer.currentIndex;
    try {
      libraryName = LibraryName(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    try {
      while (true) {
        snapShot = lexer.currentIndex;
        importOrExports.add(ImportOrExport(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    try {
      while (true) {
        snapShot = lexer.currentIndex;
        partDirectives.add(PartDirective(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    try {
      while (true) {
        snapShot = lexer.currentIndex;
        topLevelDefinitions.add(TopLevelDefinition(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }
  }

  String toString() {
    String rtn = (libraryName != null ? '$libraryName\n' : '');
    importOrExports.forEach((ie) => rtn += '$ie\n');
    rtn.trimRight();
    partDirectives.forEach((d) => rtn += '$d\n');
    rtn.trimRight();
    topLevelDefinitions.forEach((d) => rtn += '$d\n');
    return rtn;
  }
}

/// <libraryName> ::= <metadata> library <dottedIdentifierList> ';'
class LibraryName extends ParserType {
  MetaData metaData;
  DottedIdentifierList dottedIdentifierList;

  LibraryName(Lexer lexer) {
    metaData = MetaData(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'library');
    lexer.advance();

    dottedIdentifierList = DottedIdentifierList(lexer);

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    return '$metaData' + 'library $dottedIdentifierList;';
  }
}

/// <importOrExport> ::= <libraryImport> | <libraryExport>
class ImportOrExport extends ParserType {
  LibraryImport libraryImport;
  LibraryExport libraryExport;

  ImportOrExport(Lexer lexer) {
    int snapShot = lexer.currentIndex;
    try {
      libraryImport = LibraryImport(lexer);
    } on WrongTokenType {
      lexer.setPos(snapShot);

      libraryExport = LibraryExport(lexer);
    }
  }

  String toString() {
    return (libraryImport != null ? '$libraryImport' : '$libraryExport');
  }
}
