import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../main/MetaData.dart';
import 'Combinator.dart';
import 'Uries.dart';

/// <libraryExport> ::= <metadata> export <configurableUri> <combinator>* ';'
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 171
class LibraryExport extends ParserType {
  MetaData metaData;
  ConfigurableUri configurableUri;
  List<Combinator> combinators;

  LibraryExport(Lexer lexer) {
    combinators = [];

    metaData = MetaData(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'export');
    lexer.advance();

    configurableUri = ConfigurableUri(lexer);

    int snapShot;
    try {
      while (true) {
        snapShot = lexer.currentIndex;
        combinators.add(Combinator(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }

  String toString() {
    String rtn = '$metaData' + 'export $configurableUri';
    combinators.forEach((c) => rtn += ' $c');
    return rtn + ';';
  }
}
