import '../ParserType.dart';
import '../ParserExceptions.dart';
import '../lexer/Lexer.dart';
import '../lists/DottedIdentifierList.dart';
import '../main/MetaData.dart';
import 'TopLevelDefinition.dart';
import 'Uries.dart';

/// <partDirective> ::= <metadata> part <uri> ';'
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 173
class PartDirective extends ParserType {
  MetaData metaData;
  Uri uri;

  PartDirective(Lexer lexer) {
    metaData = MetaData(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'part');
    lexer.advance();

    uri = Uri(lexer);

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }
}

/// <partHeader> ::= <metadata> part of <identifier> ('.' <identifier>)* ';'
///
/// NOTE - MODIFIED TO BE:
/// <partHeader> ::= <metadata> part of <dottedIdentifierList> ';'
class PartHeader extends ParserType {
  MetaData metaData;
  DottedIdentifierList dottedIdentifierList;

  PartHeader(Lexer lexer) {
    metaData = MetaData(lexer);

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'part');
    lexer.advance();

    checkType(lexer, TType.KEYWORD);
    checkVal(lexer, 'of');
    lexer.advance();

    dottedIdentifierList = DottedIdentifierList(lexer);

    checkType(lexer, TType.SEMICOLON);
    lexer.advance();
  }
}

/// <partDeclaration> ::= <partHeader> <topLevelDefinition>* <EOF>
class PartDeclaration extends ParserType {
  PartHeader partHeader;
  List<TopLevelDefinition> topLevelDefinitions;

  PartDeclaration(Lexer lexer) {
    partHeader = PartHeader(lexer);

    int snapShot;
    try {
      while (true) {
        snapShot = lexer.currentIndex;
        topLevelDefinitions.add(TopLevelDefinition(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.EOF);
    lexer.advance();
  }
}
