import 'package:test/test.dart';
import 'test_lib.dart';

import 'package:source_parser/src/lexer/Lexer.dart';

import 'package:source_parser/src/sub_types/ArgumentPart.dart';
import 'package:source_parser/src/lists/Arguments.dart';
import 'package:source_parser/src/core/Label.dart';

void main() {

  Lexer lexer = Lexer();

  test('Special', () {

    runTest(NamedArgument(lexer.parseStr('withExpression: 3+3/2%10')), 'withExpression: 3 + 3 / 2 % 10');

  });

  test('Label', () {

    runTest(Label(lexer.parseStr('someLabel:')), 'someLabel:');

  });

  test('Named Argument', () {

    runTest(NamedArgument(lexer.parseStr('someId:4')), 'someId: 4');
    runTest(NamedArgument(lexer.parseStr('someId:true')), 'someId: true');
    runTest(NamedArgument(lexer.parseStr('someId:"HI!"')), 'someId: "HI!"');
    runTest(NamedArgument(lexer.parseStr('someId:someObject()')), 'someId: someObject()');
    runTest(NamedArgument(lexer.parseStr('someId:someObject.param')), 'someId: someObject.param');
    runTest(NamedArgument(lexer.parseStr('someId:someObject.param ? another.param : meh.someVal')), 'someId: someObject.param ? another.param : meh.someVal');

  });

  test('Argument List', () {

    runTest(ArgumentList(lexer.parseStr('234')), '234');
    runTest(ArgumentList(lexer.parseStr('234.432')), '234.432');
    runTest(ArgumentList(lexer.parseStr('1,2,3,4')), '1, 2, 3, 4');
    runTest(ArgumentList(lexer.parseStr('"someString"')), '"someString"');
    runTest(ArgumentList(lexer.parseStr('"1","2","3","4"')), '"1", "2", "3", "4"');
    runTest(ArgumentList(lexer.parseStr('someObject')), 'someObject');
    runTest(ArgumentList(lexer.parseStr('someObject,someObject,someObject')), 'someObject, someObject, someObject');

    runTest(ArgumentList(lexer.parseStr('someId:4')), 'someId: 4');
    runTest(ArgumentList(lexer.parseStr('someId:true')), 'someId: true');
    runTest(ArgumentList(lexer.parseStr('someId:"HI!"')), 'someId: "HI!"');
    runTest(ArgumentList(lexer.parseStr('someId:someObject()')), 'someId: someObject()');
    runTest(ArgumentList(lexer.parseStr('someId:someObject.param')), 'someId: someObject.param');
    runTest(ArgumentList(lexer.parseStr('someId:someObject.param ? another.param : meh.someVal')), 'someId: someObject.param ? another.param : meh.someVal');
    runTest(ArgumentList(lexer.parseStr('someId:4,id2:.23')), 'someId: 4, id2: .23');
    runTest(ArgumentList(lexer.parseStr('someId:someObject(),id2:object2(),id3:object3()')), 'someId: someObject(), id2: object2(), id3: object3()');
    runTest(ArgumentList(lexer.parseStr('someId:someObject.param, _id2:otherParam.param.subparam')), 'someId: someObject.param, _id2: otherParam.param.subparam');
    runTest(ArgumentList(lexer.parseStr('someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10')), 'someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10');

    runTest(ArgumentList(lexer.parseStr('234.432,someId:4')), '234.432, someId: 4');
    runTest(ArgumentList(lexer.parseStr('1,2,3,4,someId:someObject.param ? another.param : meh.someVal')), '1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal');
    runTest(ArgumentList(lexer.parseStr('"someString", someId:someObject(),id2:object2(),id3:object3()')), '"someString", someId: someObject(), id2: object2(), id3: object3()');
    runTest(ArgumentList(lexer.parseStr('someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10')), 'someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10');

  });

  test('Arguments', () {

    runTest(Arguments(lexer.parseStr('()')), '()');
    runTest(Arguments(lexer.parseStr('(234)')), '(234)');
    runTest(Arguments(lexer.parseStr('(234.432)')), '(234.432)');
    runTest(Arguments(lexer.parseStr('(1,2,3,4)')), '(1, 2, 3, 4)');
    runTest(Arguments(lexer.parseStr('("someString")')), '("someString")');
    runTest(Arguments(lexer.parseStr('("1","2","3","4")')), '("1", "2", "3", "4")');
    runTest(Arguments(lexer.parseStr('(someObject)')), '(someObject)');
    runTest(Arguments(lexer.parseStr('(someObject,someObject,someObject)')), '(someObject, someObject, someObject)');
    runTest(Arguments(lexer.parseStr('(someId:4)')), '(someId: 4)');
    runTest(Arguments(lexer.parseStr('(someId:true)')), '(someId: true)');
    runTest(Arguments(lexer.parseStr('(someId:"HI!")')), '(someId: "HI!")');
    runTest(Arguments(lexer.parseStr('(someId:someObject())')), '(someId: someObject())');
    runTest(Arguments(lexer.parseStr('(someId:someObject.param)')), '(someId: someObject.param)');
    runTest(Arguments(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal)')), '(someId: someObject.param ? another.param : meh.someVal)');
    runTest(Arguments(lexer.parseStr('(someId:4,id2:.23)')), '(someId: 4, id2: .23)');
    runTest(Arguments(lexer.parseStr('(someId:someObject(),id2:object2(),id3:object3())')), '(someId: someObject(), id2: object2(), id3: object3())');
    runTest(Arguments(lexer.parseStr('(someId:someObject.param, _id2:otherParam.param.subparam)')), '(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(Arguments(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), '(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Arguments(lexer.parseStr('(234.432,someId:4)')), '(234.432, someId: 4)');
    runTest(Arguments(lexer.parseStr('(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), '(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(Arguments(lexer.parseStr('("someString", someId:someObject(),id2:object2(),id3:object3())')), '("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(Arguments(lexer.parseStr('(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), '(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');

    runTest(Arguments(lexer.parseStr('(234,)')), '(234,)');
    runTest(Arguments(lexer.parseStr('(234.432,)')), '(234.432,)');
    runTest(Arguments(lexer.parseStr('(1,2,3,4,)')), '(1, 2, 3, 4,)');
    runTest(Arguments(lexer.parseStr('("someString",)')), '("someString",)');
    runTest(Arguments(lexer.parseStr('("1","2","3","4",)')), '("1", "2", "3", "4",)');
    runTest(Arguments(lexer.parseStr('(someObject,)')), '(someObject,)');
    runTest(Arguments(lexer.parseStr('(someObject,someObject,someObject,)')), '(someObject, someObject, someObject,)');
    runTest(Arguments(lexer.parseStr('(someId:4,)')), '(someId: 4,)');
    runTest(Arguments(lexer.parseStr('(someId:true,)')), '(someId: true,)');
    runTest(Arguments(lexer.parseStr('(someId:"HI!",)')), '(someId: "HI!",)');
    runTest(Arguments(lexer.parseStr('(someId:someObject(),)')), '(someId: someObject(),)');
    runTest(Arguments(lexer.parseStr('(someId:someObject.param,)')), '(someId: someObject.param,)');
    runTest(Arguments(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal,)')), '(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Arguments(lexer.parseStr('(someId:4,id2:.23,)')), '(someId: 4, id2: .23,)');
    runTest(Arguments(lexer.parseStr('(someId:someObject(),id2:object2(),id3:object3(),)')), '(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Arguments(lexer.parseStr('(someId:someObject.param, _id2:otherParam.param.subparam,)')), '(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(Arguments(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(Arguments(lexer.parseStr('(234.432,someId:4,)')), '(234.432, someId: 4,)');
    runTest(Arguments(lexer.parseStr('(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Arguments(lexer.parseStr('("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Arguments(lexer.parseStr('(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

  });

  test('Argument Part', () {

    runTest(ArgumentPart(lexer.parseStr('()')), '()');
    runTest(ArgumentPart(lexer.parseStr('(234)')), '(234)');
    runTest(ArgumentPart(lexer.parseStr('(234.432)')), '(234.432)');
    runTest(ArgumentPart(lexer.parseStr('(1,2,3,4)')), '(1, 2, 3, 4)');
    runTest(ArgumentPart(lexer.parseStr('("someString")')), '("someString")');
    runTest(ArgumentPart(lexer.parseStr('("1","2","3","4")')), '("1", "2", "3", "4")');
    runTest(ArgumentPart(lexer.parseStr('(someObject)')), '(someObject)');
    runTest(ArgumentPart(lexer.parseStr('(someObject,someObject,someObject)')), '(someObject, someObject, someObject)');
    runTest(ArgumentPart(lexer.parseStr('(someId:4)')), '(someId: 4)');
    runTest(ArgumentPart(lexer.parseStr('(someId:true)')), '(someId: true)');
    runTest(ArgumentPart(lexer.parseStr('(someId:"HI!")')), '(someId: "HI!")');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject())')), '(someId: someObject())');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject.param)')), '(someId: someObject.param)');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal)')), '(someId: someObject.param ? another.param : meh.someVal)');
    runTest(ArgumentPart(lexer.parseStr('(someId:4,id2:.23)')), '(someId: 4, id2: .23)');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject(),id2:object2(),id3:object3())')), '(someId: someObject(), id2: object2(), id3: object3())');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject.param, _id2:otherParam.param.subparam)')), '(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), '(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ArgumentPart(lexer.parseStr('(234.432,someId:4)')), '(234.432, someId: 4)');
    runTest(ArgumentPart(lexer.parseStr('(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), '(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(ArgumentPart(lexer.parseStr('("someString", someId:someObject(),id2:object2(),id3:object3())')), '("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(ArgumentPart(lexer.parseStr('(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), '(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ArgumentPart(lexer.parseStr('(234,)')), '(234,)');
    runTest(ArgumentPart(lexer.parseStr('(234.432,)')), '(234.432,)');
    runTest(ArgumentPart(lexer.parseStr('(1,2,3,4,)')), '(1, 2, 3, 4,)');
    runTest(ArgumentPart(lexer.parseStr('("someString",)')), '("someString",)');
    runTest(ArgumentPart(lexer.parseStr('("1","2","3","4",)')), '("1", "2", "3", "4",)');
    runTest(ArgumentPart(lexer.parseStr('(someObject,)')), '(someObject,)');
    runTest(ArgumentPart(lexer.parseStr('(someObject,someObject,someObject,)')), '(someObject, someObject, someObject,)');
    runTest(ArgumentPart(lexer.parseStr('(someId:4,)')), '(someId: 4,)');
    runTest(ArgumentPart(lexer.parseStr('(someId:true,)')), '(someId: true,)');
    runTest(ArgumentPart(lexer.parseStr('(someId:"HI!",)')), '(someId: "HI!",)');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject(),)')), '(someId: someObject(),)');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject.param,)')), '(someId: someObject.param,)');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal,)')), '(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ArgumentPart(lexer.parseStr('(someId:4,id2:.23,)')), '(someId: 4, id2: .23,)');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject(),id2:object2(),id3:object3(),)')), '(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject.param, _id2:otherParam.param.subparam,)')), '(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(ArgumentPart(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(ArgumentPart(lexer.parseStr('(234.432,someId:4,)')), '(234.432, someId: 4,)');
    runTest(ArgumentPart(lexer.parseStr('(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ArgumentPart(lexer.parseStr('("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ArgumentPart(lexer.parseStr('(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

    runTest(ArgumentPart(lexer.parseStr('<int>()')), '<int>()');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(234)')), '<int, String>(234)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(234.432)')), '<int, String>(234.432)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(1,2,3,4)')), '<int, String>(1, 2, 3, 4)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>("someString")')), '<int, String>("someString")');
    runTest(ArgumentPart(lexer.parseStr('<int,String>("1","2","3","4")')), '<int, String>("1", "2", "3", "4")');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someObject)')), '<int, String>(someObject)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someObject,someObject,someObject)')), '<int, String>(someObject, someObject, someObject)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:4)')), '<int, String>(someId: 4)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:true)')), '<int, String>(someId: true)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:"HI!")')), '<int, String>(someId: "HI!")');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject())')), '<int, String>(someId: someObject())');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject.param)')), '<int, String>(someId: someObject.param)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject.param ? another.param : meh.someVal)')), '<int, String>(someId: someObject.param ? another.param : meh.someVal)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:4,id2:.23)')), '<int, String>(someId: 4, id2: .23)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject(),id2:object2(),id3:object3())')), '<int, String>(someId: someObject(), id2: object2(), id3: object3())');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject.param, _id2:otherParam.param.subparam)')), '<int, String>(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), '<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(234.432,someId:4)')), '<int, String>(234.432, someId: 4)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), '<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>("someString", someId:someObject(),id2:object2(),id3:object3())')), '<int, String>("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), '<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(234,)')), '<int, String>(234,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(234.432,)')), '<int, String>(234.432,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(1,2,3,4,)')), '<int, String>(1, 2, 3, 4,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>("someString",)')), '<int, String>("someString",)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>("1","2","3","4",)')), '<int, String>("1", "2", "3", "4",)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someObject,)')), '<int, String>(someObject,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someObject,someObject,someObject,)')), '<int, String>(someObject, someObject, someObject,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:4,)')), '<int, String>(someId: 4,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:true,)')), '<int, String>(someId: true,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:"HI!",)')), '<int, String>(someId: "HI!",)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject(),)')), '<int, String>(someId: someObject(),)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject.param,)')), '<int, String>(someId: someObject.param,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject.param ? another.param : meh.someVal,)')), '<int, String>(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:4,id2:.23,)')), '<int, String>(someId: 4, id2: .23,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject(),id2:object2(),id3:object3(),)')), '<int, String>(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(234.432,someId:4,)')), '<int, String>(234.432, someId: 4,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ArgumentPart(lexer.parseStr('<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

  });

}
