import 'package:test/test.dart';
import 'test_lib.dart';

import 'package:source_parser/src/lexer/Lexer.dart';

import 'package:source_parser/src/sub_types/FormalParameterPart.dart';
import 'package:source_parser/src/sub_types/FormalParameters.dart';
import 'package:source_parser/src/lists/FormalParameterList.dart';
import 'package:source_parser/src/sub_types/FunctionBody.dart';
import 'package:source_parser/src/sub_types/Signatures.dart';
import 'package:source_parser/src/main/Block.dart';

void main() {

  Lexer lexer = Lexer();

  test('Block', () {

    runTest(Block(lexer.parseStr('{}')), '{  }');
    runTest(Block(lexer.parseStr('{{{{}}}}')), '{ { { {  } } } }');
    runTest(Block(lexer.parseStr('{dynamic x=0;}')), '{ dynamic x = 0; }');
    runTest(Block(lexer.parseStr('{var x;{{{}}}}')), '{ var x; { { {  } } } }');

  });

  test('Function Body', () {

    runTest(FunctionBody(lexer.parseStr('=> test;')), '=> test;');
    runTest(FunctionBody(lexer.parseStr('async => test;')), 'async => test;');
    runTest(FunctionBody(lexer.parseStr('=> test.someParam;')), '=> test.someParam;');
    runTest(FunctionBody(lexer.parseStr('=> test.someParam'), false), '=> test.someParam');
    runTest(FunctionBody(lexer.parseStr('{}')), '{  }');
    runTest(FunctionBody(lexer.parseStr('async {}')), 'async {  }');
    runTest(FunctionBody(lexer.parseStr('async* {}')), 'async* {  }');
    runTest(FunctionBody(lexer.parseStr('sync* {}')), 'sync* {  }');
    runTest(FunctionBody(lexer.parseStr('{int x = 0;}')), '{ int x = 0; }');

  });

  test('Function Formal Parameter', () {

    runTest(FunctionFormalParameter(lexer.parseStr('int x()')), 'int x ()');
    runTest(FunctionFormalParameter(lexer.parseStr('var x()')), 'var x ()');
    runTest(FunctionFormalParameter(lexer.parseStr('covariant someType x()')), 'covariant someType x ()');
    runTest(FunctionFormalParameter(lexer.parseStr('dynamic x ()')), 'dynamic x ()');
    runTest(FunctionFormalParameter(lexer.parseStr('dynamic x (covariant int y(x))')), 'dynamic x (covariant int y (x))');

  });

  test('Simple Formal Parameter', () {

    runTest(SimpleFormalParameter(lexer.parseStr('int x')), 'int x');
    runTest(SimpleFormalParameter(lexer.parseStr('var x')), 'var x');
    runTest(SimpleFormalParameter(lexer.parseStr('someType x')), 'someType x');
    runTest(SimpleFormalParameter(lexer.parseStr('const bool x')), 'const bool x');
    runTest(SimpleFormalParameter(lexer.parseStr('final var x')), 'final var x');
    runTest(SimpleFormalParameter(lexer.parseStr('dynamic x')), 'dynamic x');
    runTest(SimpleFormalParameter(lexer.parseStr('covariant x')), 'covariant x');
    runTest(SimpleFormalParameter(lexer.parseStr('covariant int x')), 'covariant int x');
    runTest(SimpleFormalParameter(lexer.parseStr('covariant const bool x')), 'covariant const bool x');
    runTest(SimpleFormalParameter(lexer.parseStr('covariant final var x')), 'covariant final var x');
    runTest(SimpleFormalParameter(lexer.parseStr('covariant dynamic x')), 'covariant dynamic x');

  });

  test('Field Formal Parameter', () {

    runTest(FieldFormalParameter(lexer.parseStr('this.x')), 'this.x');
    runTest(FieldFormalParameter(lexer.parseStr('int this.x')), 'int this.x');
    runTest(FieldFormalParameter(lexer.parseStr('var this.x')), 'var this.x');
    runTest(FieldFormalParameter(lexer.parseStr('someType this.x')), 'someType this.x');
    runTest(FieldFormalParameter(lexer.parseStr('const bool this.x')), 'const bool this.x');
    runTest(FieldFormalParameter(lexer.parseStr('final var this.x')), 'final var this.x');
    runTest(FieldFormalParameter(lexer.parseStr('dynamic this.x')), 'dynamic this.x');

    runTest(FieldFormalParameter(lexer.parseStr('int this.x ()')), 'int this.x ()');
    runTest(FieldFormalParameter(lexer.parseStr('var this.x (var y)')), 'var this.x (var y)');
    runTest(FieldFormalParameter(lexer.parseStr('someType this.x (const var t)')), 'someType this.x (const var t)');
    runTest(FieldFormalParameter(lexer.parseStr('const bool this.x (covariant int y)')), 'const bool this.x (covariant int y)');
    runTest(FieldFormalParameter(lexer.parseStr('final var this.x (const var this.y ( final int z))')), 'final var this.x (const var this.y (final int z))');

  });

  test('Normal Formal Parameter', () {

    runTest(NormalFormalParameter(lexer.parseStr('int x()')), 'int x ()');
    runTest(NormalFormalParameter(lexer.parseStr('var x()')), 'var x ()');
    runTest(NormalFormalParameter(lexer.parseStr('covariant someType x()')), 'covariant someType x ()');
    runTest(NormalFormalParameter(lexer.parseStr('dynamic x ()')), 'dynamic x ()');
    runTest(NormalFormalParameter(lexer.parseStr('dynamic x (covariant int y(x))')), 'dynamic x (covariant int y (x))');

    runTest(NormalFormalParameter(lexer.parseStr('int x')), 'int x');
    runTest(NormalFormalParameter(lexer.parseStr('var x')), 'var x');
    runTest(NormalFormalParameter(lexer.parseStr('someType x')), 'someType x');
    runTest(NormalFormalParameter(lexer.parseStr('const bool x')), 'const bool x');
    runTest(NormalFormalParameter(lexer.parseStr('final var x')), 'final var x');
    runTest(NormalFormalParameter(lexer.parseStr('dynamic x')), 'dynamic x');
    runTest(NormalFormalParameter(lexer.parseStr('covariant x')), 'covariant x');
    runTest(NormalFormalParameter(lexer.parseStr('covariant int x')), 'covariant int x');
    runTest(NormalFormalParameter(lexer.parseStr('covariant const bool x')), 'covariant const bool x');
    runTest(NormalFormalParameter(lexer.parseStr('covariant final var x')), 'covariant final var x');
    runTest(NormalFormalParameter(lexer.parseStr('covariant dynamic x')), 'covariant dynamic x');
    
    runTest(NormalFormalParameter(lexer.parseStr('int this.x')), 'int this.x');
    runTest(NormalFormalParameter(lexer.parseStr('var this.x')), 'var this.x');
    runTest(NormalFormalParameter(lexer.parseStr('someType this.x')), 'someType this.x');
    runTest(NormalFormalParameter(lexer.parseStr('const bool this.x')), 'const bool this.x');
    runTest(NormalFormalParameter(lexer.parseStr('final var this.x')), 'final var this.x');
    runTest(NormalFormalParameter(lexer.parseStr('dynamic this.x')), 'dynamic this.x');
    runTest(NormalFormalParameter(lexer.parseStr('int this.x ()')), 'int this.x ()');
    runTest(NormalFormalParameter(lexer.parseStr('var this.x (var y)')), 'var this.x (var y)');
    runTest(NormalFormalParameter(lexer.parseStr('someType this.x (const var t)')), 'someType this.x (const var t)');
    runTest(NormalFormalParameter(lexer.parseStr('const bool this.x (covariant int y)')), 'const bool this.x (covariant int y)');
    runTest(NormalFormalParameter(lexer.parseStr('final var this.x (const var this.y ( final int z))')), 'final var this.x (const var this.y (final int z))');

  });

  test('Normal Formal Parameters', () {

    runTest(NormalFormalParameters(lexer.parseStr('int x, var y')), 'int x, var y');
    runTest(NormalFormalParameters(lexer.parseStr('int x, final var y')), 'int x, final var y');
    runTest(NormalFormalParameters(lexer.parseStr('covariant int x, final var y')), 'covariant int x, final var y');
    runTest(NormalFormalParameters(lexer.parseStr('int x(), var y()')), 'int x (), var y ()');
    runTest(NormalFormalParameters(lexer.parseStr('covariant someType x(), bool y')), 'covariant someType x (), bool y');
    runTest(NormalFormalParameters(lexer.parseStr('dynamic x (covariant int y(x), type z)')), 'dynamic x (covariant int y (x), type z)');

    runTest(NormalFormalParameters(lexer.parseStr('var this.x, bool this.y')), 'var this.x, bool this.y');
    runTest(NormalFormalParameters(lexer.parseStr('var this.x, const bool this.y')), 'var this.x, const bool this.y');
    runTest(NormalFormalParameters(lexer.parseStr('var this.x (), const bool this.y')), 'var this.x (), const bool this.y');
    runTest(NormalFormalParameters(lexer.parseStr('var this.x (type this.y), const bool this.y')), 'var this.x (type this.y), const bool this.y');
    runTest(NormalFormalParameters(lexer.parseStr('final var this.x (const var this.y ( final int z))')), 'final var this.x (const var this.y (final int z))');

  });

  test('Default Formal Parameter', () {

    runTest(DefaultFormalParameter(lexer.parseStr('int x()')), 'int x ()');
    runTest(DefaultFormalParameter(lexer.parseStr('var x()')), 'var x ()');
    runTest(DefaultFormalParameter(lexer.parseStr('covariant someType x()')), 'covariant someType x ()');
    runTest(DefaultFormalParameter(lexer.parseStr('dynamic x ()')), 'dynamic x ()');
    runTest(DefaultFormalParameter(lexer.parseStr('dynamic x (covariant int y(x))')), 'dynamic x (covariant int y (x))');
    runTest(DefaultFormalParameter(lexer.parseStr('int x')), 'int x');
    runTest(DefaultFormalParameter(lexer.parseStr('var x')), 'var x');
    runTest(DefaultFormalParameter(lexer.parseStr('someType x')), 'someType x');
    runTest(DefaultFormalParameter(lexer.parseStr('const bool x')), 'const bool x');
    runTest(DefaultFormalParameter(lexer.parseStr('final var x')), 'final var x');
    runTest(DefaultFormalParameter(lexer.parseStr('dynamic x')), 'dynamic x');
    runTest(DefaultFormalParameter(lexer.parseStr('covariant x')), 'covariant x');
    runTest(DefaultFormalParameter(lexer.parseStr('covariant int x')), 'covariant int x');
    runTest(DefaultFormalParameter(lexer.parseStr('covariant const bool x')), 'covariant const bool x');
    runTest(DefaultFormalParameter(lexer.parseStr('covariant final var x')), 'covariant final var x');
    runTest(DefaultFormalParameter(lexer.parseStr('covariant dynamic x')), 'covariant dynamic x');
    runTest(DefaultFormalParameter(lexer.parseStr('int this.x')), 'int this.x');
    runTest(DefaultFormalParameter(lexer.parseStr('var this.x')), 'var this.x');
    runTest(DefaultFormalParameter(lexer.parseStr('someType this.x')), 'someType this.x');
    runTest(DefaultFormalParameter(lexer.parseStr('const bool this.x')), 'const bool this.x');
    runTest(DefaultFormalParameter(lexer.parseStr('final var this.x')), 'final var this.x');
    runTest(DefaultFormalParameter(lexer.parseStr('dynamic this.x')), 'dynamic this.x');
    runTest(DefaultFormalParameter(lexer.parseStr('int this.x ()')), 'int this.x ()');
    runTest(DefaultFormalParameter(lexer.parseStr('var this.x (var y)')), 'var this.x (var y)');
    runTest(DefaultFormalParameter(lexer.parseStr('someType this.x (const var t)')), 'someType this.x (const var t)');
    runTest(DefaultFormalParameter(lexer.parseStr('const bool this.x (covariant int y)')), 'const bool this.x (covariant int y)');
    runTest(DefaultFormalParameter(lexer.parseStr('final var this.x (const var this.y ( final int z))')), 'final var this.x (const var this.y (final int z))');

    runTest(DefaultFormalParameter(lexer.parseStr('var x = "testStr"')), 'var x="testStr"');
    runTest(DefaultFormalParameter(lexer.parseStr('int x() = 4')), 'int x ()=4');
    runTest(DefaultFormalParameter(lexer.parseStr('final var x = someVal')), 'final var x=someVal');
    runTest(DefaultFormalParameter(lexer.parseStr('dynamic x (covariant int y(x)) = 23 * 3')), 'dynamic x (covariant int y (x))=23 * 3');
    runTest(DefaultFormalParameter(lexer.parseStr('covariant int x = 0xfff')), 'covariant int x=0xfff');
    runTest(DefaultFormalParameter(lexer.parseStr('String this.x=msg')), 'String this.x=msg');
    runTest(DefaultFormalParameter(lexer.parseStr('const String this.x=msg')), 'const String this.x=msg');
    runTest(DefaultFormalParameter(lexer.parseStr('int this.x () = 0')), 'int this.x ()=0');
    runTest(DefaultFormalParameter(lexer.parseStr('var this.x (var y)=anotherVar')), 'var this.x (var y)=anotherVar');

  });

  test('Default Named Parameter', () {

    runTest(DefaultNamedParameter(lexer.parseStr('int x()')), 'int x ()');
    runTest(DefaultNamedParameter(lexer.parseStr('var x()')), 'var x ()');
    runTest(DefaultNamedParameter(lexer.parseStr('covariant someType x()')), 'covariant someType x ()');
    runTest(DefaultNamedParameter(lexer.parseStr('dynamic x ()')), 'dynamic x ()');
    runTest(DefaultNamedParameter(lexer.parseStr('dynamic x (covariant int y(x))')), 'dynamic x (covariant int y (x))');
    runTest(DefaultNamedParameter(lexer.parseStr('int x')), 'int x');
    runTest(DefaultNamedParameter(lexer.parseStr('var x')), 'var x');
    runTest(DefaultNamedParameter(lexer.parseStr('someType x')), 'someType x');
    runTest(DefaultNamedParameter(lexer.parseStr('const bool x')), 'const bool x');
    runTest(DefaultNamedParameter(lexer.parseStr('final var x')), 'final var x');
    runTest(DefaultNamedParameter(lexer.parseStr('dynamic x')), 'dynamic x');
    runTest(DefaultNamedParameter(lexer.parseStr('covariant x')), 'covariant x');
    runTest(DefaultNamedParameter(lexer.parseStr('covariant int x')), 'covariant int x');
    runTest(DefaultNamedParameter(lexer.parseStr('covariant const bool x')), 'covariant const bool x');
    runTest(DefaultNamedParameter(lexer.parseStr('covariant final var x')), 'covariant final var x');
    runTest(DefaultNamedParameter(lexer.parseStr('covariant dynamic x')), 'covariant dynamic x');
    runTest(DefaultNamedParameter(lexer.parseStr('int this.x')), 'int this.x');
    runTest(DefaultNamedParameter(lexer.parseStr('var this.x')), 'var this.x');
    runTest(DefaultNamedParameter(lexer.parseStr('someType this.x')), 'someType this.x');
    runTest(DefaultNamedParameter(lexer.parseStr('const bool this.x')), 'const bool this.x');
    runTest(DefaultNamedParameter(lexer.parseStr('final var this.x')), 'final var this.x');
    runTest(DefaultNamedParameter(lexer.parseStr('dynamic this.x')), 'dynamic this.x');
    runTest(DefaultNamedParameter(lexer.parseStr('int this.x ()')), 'int this.x ()');
    runTest(DefaultNamedParameter(lexer.parseStr('var this.x (var y)')), 'var this.x (var y)');
    runTest(DefaultNamedParameter(lexer.parseStr('someType this.x (const var t)')), 'someType this.x (const var t)');
    runTest(DefaultNamedParameter(lexer.parseStr('const bool this.x (covariant int y)')), 'const bool this.x (covariant int y)');
    runTest(DefaultNamedParameter(lexer.parseStr('final var this.x (const var this.y ( final int z))')), 'final var this.x (const var this.y (final int z))');

    runTest(DefaultNamedParameter(lexer.parseStr('var x = "testStr"')), 'var x="testStr"');
    runTest(DefaultNamedParameter(lexer.parseStr('int x() = 4')), 'int x ()=4');
    runTest(DefaultNamedParameter(lexer.parseStr('final var x = someVal')), 'final var x=someVal');
    runTest(DefaultNamedParameter(lexer.parseStr('dynamic x (covariant int y(x)) = 23 * 3')), 'dynamic x (covariant int y (x))=23 * 3');
    runTest(DefaultNamedParameter(lexer.parseStr('covariant int x = 0xfff')), 'covariant int x=0xfff');
    runTest(DefaultNamedParameter(lexer.parseStr('String this.x=msg')), 'String this.x=msg');
    runTest(DefaultNamedParameter(lexer.parseStr('const String this.x=msg')), 'const String this.x=msg');
    runTest(DefaultNamedParameter(lexer.parseStr('int this.x () = 0')), 'int this.x ()=0');
    runTest(DefaultNamedParameter(lexer.parseStr('var this.x (var y)=anotherVar')), 'var this.x (var y)=anotherVar');

    runTest(DefaultNamedParameter(lexer.parseStr('var x : "testStr"')), 'var x:"testStr"');
    runTest(DefaultNamedParameter(lexer.parseStr('int x() : 4')), 'int x ():4');
    runTest(DefaultNamedParameter(lexer.parseStr('final var x : someVal')), 'final var x:someVal');
    runTest(DefaultNamedParameter(lexer.parseStr('dynamic x (covariant int y(x)) : 23 * 3')), 'dynamic x (covariant int y (x)):23 * 3');
    runTest(DefaultNamedParameter(lexer.parseStr('covariant int x : 0xfff')), 'covariant int x:0xfff');
    runTest(DefaultNamedParameter(lexer.parseStr('String this.x:msg')), 'String this.x:msg');
    runTest(DefaultNamedParameter(lexer.parseStr('const String this.x:msg')), 'const String this.x:msg');
    runTest(DefaultNamedParameter(lexer.parseStr('int this.x () : 0')), 'int this.x ():0');
    runTest(DefaultNamedParameter(lexer.parseStr('var this.x (var y):anotherVar')), 'var this.x (var y):anotherVar');

  });

  test('Optional Positional Formal Parameters', () {

    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[int x()]')), '[int x ()]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[var x()]')), '[var x ()]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[covariant someType x()]')), '[covariant someType x ()]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[dynamic x ()]')), '[dynamic x ()]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[dynamic x (covariant int y(x))]')), '[dynamic x (covariant int y (x))]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[int x]')), '[int x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[var x]')), '[var x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[someType x]')), '[someType x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[const bool x]')), '[const bool x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[final var x]')), '[final var x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[dynamic x]')), '[dynamic x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[covariant x]')), '[covariant x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[covariant int x]')), '[covariant int x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[covariant const bool x]')), '[covariant const bool x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[covariant final var x]')), '[covariant final var x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[covariant dynamic x]')), '[covariant dynamic x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[int this.x]')), '[int this.x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[var this.x]')), '[var this.x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[someType this.x]')), '[someType this.x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[const bool this.x]')), '[const bool this.x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[final var this.x]')), '[final var this.x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[dynamic this.x]')), '[dynamic this.x]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[int this.x ()]')), '[int this.x ()]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[var this.x (var y)]')), '[var this.x (var y)]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[someType this.x (const var t)]')), '[someType this.x (const var t)]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[const bool this.x (covariant int y)]')), '[const bool this.x (covariant int y)]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[final var this.x (const var this.y ( final int z))]')), '[final var this.x (const var this.y (final int z))]');

    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[var x = "testStr"]')), '[var x="testStr"]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[int x() = 4]')), '[int x ()=4]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[final var x = someVal]')), '[final var x=someVal]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[dynamic x (covariant int y(x)) = 23 * 3]')), '[dynamic x (covariant int y (x))=23 * 3]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[covariant int x = 0xfff]')), '[covariant int x=0xfff]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[String this.x=msg]')), '[String this.x=msg]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[const String this.x=msg]')), '[const String this.x=msg]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[int this.x () = 0]')), '[int this.x ()=0]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[var this.x (var y)=anotherVar]')), '[var this.x (var y)=anotherVar]');

    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[var x = "testStr",]')), '[var x="testStr",]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[int x() = 4,]')), '[int x ()=4,]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[final var x = someVal,]')), '[final var x=someVal,]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[dynamic x (covariant int y(x)) = 23 * 3,]')), '[dynamic x (covariant int y (x))=23 * 3,]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[covariant int x = 0xfff,]')), '[covariant int x=0xfff,]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[String this.x=msg,]')), '[String this.x=msg,]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[const String this.x=msg,]')), '[const String this.x=msg,]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[int this.x () = 0,]')), '[int this.x ()=0,]');
    runTest(OptionalPositionalFormalParameters(lexer.parseStr('[var this.x (var y)=anotherVar,]')), '[var this.x (var y)=anotherVar,]');

  });

  test('Named Formal Parameters', () {

    runTest(NamedFormalParameters(lexer.parseStr('{int x()}')), '{int x ()}');
    runTest(NamedFormalParameters(lexer.parseStr('{var x()}')), '{var x ()}');
    runTest(NamedFormalParameters(lexer.parseStr('{covariant someType x()}')), '{covariant someType x ()}');
    runTest(NamedFormalParameters(lexer.parseStr('{dynamic x ()}')), '{dynamic x ()}');
    runTest(NamedFormalParameters(lexer.parseStr('{dynamic x (covariant int y(x))}')), '{dynamic x (covariant int y (x))}');
    runTest(NamedFormalParameters(lexer.parseStr('{int x}')), '{int x}');
    runTest(NamedFormalParameters(lexer.parseStr('{var x}')), '{var x}');
    runTest(NamedFormalParameters(lexer.parseStr('{someType x}')), '{someType x}');
    runTest(NamedFormalParameters(lexer.parseStr('{const bool x}')), '{const bool x}');
    runTest(NamedFormalParameters(lexer.parseStr('{final var x}')), '{final var x}');
    runTest(NamedFormalParameters(lexer.parseStr('{dynamic x}')), '{dynamic x}');
    runTest(NamedFormalParameters(lexer.parseStr('{covariant x}')), '{covariant x}');
    runTest(NamedFormalParameters(lexer.parseStr('{covariant int x}')), '{covariant int x}');
    runTest(NamedFormalParameters(lexer.parseStr('{covariant const bool x}')), '{covariant const bool x}');
    runTest(NamedFormalParameters(lexer.parseStr('{covariant final var x}')), '{covariant final var x}');
    runTest(NamedFormalParameters(lexer.parseStr('{covariant dynamic x}')), '{covariant dynamic x}');
    runTest(NamedFormalParameters(lexer.parseStr('{int this.x}')), '{int this.x}');
    runTest(NamedFormalParameters(lexer.parseStr('{var this.x}')), '{var this.x}');
    runTest(NamedFormalParameters(lexer.parseStr('{someType this.x}')), '{someType this.x}');
    runTest(NamedFormalParameters(lexer.parseStr('{const bool this.x}')), '{const bool this.x}');
    runTest(NamedFormalParameters(lexer.parseStr('{final var this.x}')), '{final var this.x}');
    runTest(NamedFormalParameters(lexer.parseStr('{dynamic this.x}')), '{dynamic this.x}');
    runTest(NamedFormalParameters(lexer.parseStr('{int this.x ()}')), '{int this.x ()}');
    runTest(NamedFormalParameters(lexer.parseStr('{var this.x (var y)}')), '{var this.x (var y)}');
    runTest(NamedFormalParameters(lexer.parseStr('{someType this.x (const var t)}')), '{someType this.x (const var t)}');
    runTest(NamedFormalParameters(lexer.parseStr('{const bool this.x (covariant int y)}')), '{const bool this.x (covariant int y)}');
    runTest(NamedFormalParameters(lexer.parseStr('{final var this.x (const var this.y ( final int z))}')), '{final var this.x (const var this.y (final int z))}');

    runTest(NamedFormalParameters(lexer.parseStr('{var x = "testStr"}')), '{var x="testStr"}');
    runTest(NamedFormalParameters(lexer.parseStr('{int x() = 4}')), '{int x ()=4}');
    runTest(NamedFormalParameters(lexer.parseStr('{final var x = someVal}')), '{final var x=someVal}');
    runTest(NamedFormalParameters(lexer.parseStr('{dynamic x (covariant int y(x)) = 23 * 3}')), '{dynamic x (covariant int y (x))=23 * 3}');
    runTest(NamedFormalParameters(lexer.parseStr('{covariant int x = 0xfff}')), '{covariant int x=0xfff}');
    runTest(NamedFormalParameters(lexer.parseStr('{String this.x=msg}')), '{String this.x=msg}');
    runTest(NamedFormalParameters(lexer.parseStr('{const String this.x=msg}')), '{const String this.x=msg}');
    runTest(NamedFormalParameters(lexer.parseStr('{int this.x () = 0}')), '{int this.x ()=0}');
    runTest(NamedFormalParameters(lexer.parseStr('{var this.x (var y)=anotherVar}')), '{var this.x (var y)=anotherVar}');

    runTest(NamedFormalParameters(lexer.parseStr('{var x = "testStr",}')), '{var x="testStr",}');
    runTest(NamedFormalParameters(lexer.parseStr('{int x() = 4,}')), '{int x ()=4,}');
    runTest(NamedFormalParameters(lexer.parseStr('{final var x = someVal,}')), '{final var x=someVal,}');
    runTest(NamedFormalParameters(lexer.parseStr('{dynamic x (covariant int y(x)) = 23 * 3,}')), '{dynamic x (covariant int y (x))=23 * 3,}');
    runTest(NamedFormalParameters(lexer.parseStr('{covariant int x = 0xfff,}')), '{covariant int x=0xfff,}');
    runTest(NamedFormalParameters(lexer.parseStr('{String this.x=msg,}')), '{String this.x=msg,}');
    runTest(NamedFormalParameters(lexer.parseStr('{const String this.x=msg,}')), '{const String this.x=msg,}');
    runTest(NamedFormalParameters(lexer.parseStr('{int this.x () = 0,}')), '{int this.x ()=0,}');
    runTest(NamedFormalParameters(lexer.parseStr('{var this.x (var y)=anotherVar,}')), '{var this.x (var y)=anotherVar,}');

  });

  test('Optional Formal Parameters', () {

    runTest(OptionalFormalParameters(lexer.parseStr('[int x()]')), '[int x ()]');
    runTest(OptionalFormalParameters(lexer.parseStr('[var x()]')), '[var x ()]');
    runTest(OptionalFormalParameters(lexer.parseStr('[covariant someType x()]')), '[covariant someType x ()]');
    runTest(OptionalFormalParameters(lexer.parseStr('[dynamic x ()]')), '[dynamic x ()]');
    runTest(OptionalFormalParameters(lexer.parseStr('[dynamic x (covariant int y(x))]')), '[dynamic x (covariant int y (x))]');
    runTest(OptionalFormalParameters(lexer.parseStr('[int x]')), '[int x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[var x]')), '[var x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[someType x]')), '[someType x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[const bool x]')), '[const bool x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[final var x]')), '[final var x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[dynamic x]')), '[dynamic x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[covariant x]')), '[covariant x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[covariant int x]')), '[covariant int x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[covariant const bool x]')), '[covariant const bool x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[covariant final var x]')), '[covariant final var x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[covariant dynamic x]')), '[covariant dynamic x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[int this.x]')), '[int this.x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[var this.x]')), '[var this.x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[someType this.x]')), '[someType this.x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[const bool this.x]')), '[const bool this.x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[final var this.x]')), '[final var this.x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[dynamic this.x]')), '[dynamic this.x]');
    runTest(OptionalFormalParameters(lexer.parseStr('[int this.x ()]')), '[int this.x ()]');
    runTest(OptionalFormalParameters(lexer.parseStr('[var this.x (var y)]')), '[var this.x (var y)]');
    runTest(OptionalFormalParameters(lexer.parseStr('[someType this.x (const var t)]')), '[someType this.x (const var t)]');
    runTest(OptionalFormalParameters(lexer.parseStr('[const bool this.x (covariant int y)]')), '[const bool this.x (covariant int y)]');
    runTest(OptionalFormalParameters(lexer.parseStr('[final var this.x (const var this.y ( final int z))]')), '[final var this.x (const var this.y (final int z))]');

    runTest(OptionalFormalParameters(lexer.parseStr('[var x = "testStr"]')), '[var x="testStr"]');
    runTest(OptionalFormalParameters(lexer.parseStr('[int x() = 4]')), '[int x ()=4]');
    runTest(OptionalFormalParameters(lexer.parseStr('[final var x = someVal]')), '[final var x=someVal]');
    runTest(OptionalFormalParameters(lexer.parseStr('[dynamic x (covariant int y(x)) = 23 * 3]')), '[dynamic x (covariant int y (x))=23 * 3]');
    runTest(OptionalFormalParameters(lexer.parseStr('[covariant int x = 0xfff]')), '[covariant int x=0xfff]');
    runTest(OptionalFormalParameters(lexer.parseStr('[String this.x=msg]')), '[String this.x=msg]');
    runTest(OptionalFormalParameters(lexer.parseStr('[const String this.x=msg]')), '[const String this.x=msg]');
    runTest(OptionalFormalParameters(lexer.parseStr('[int this.x () = 0]')), '[int this.x ()=0]');
    runTest(OptionalFormalParameters(lexer.parseStr('[var this.x (var y)=anotherVar]')), '[var this.x (var y)=anotherVar]');

    runTest(OptionalFormalParameters(lexer.parseStr('[var x = "testStr",]')), '[var x="testStr",]');
    runTest(OptionalFormalParameters(lexer.parseStr('[int x() = 4,]')), '[int x ()=4,]');
    runTest(OptionalFormalParameters(lexer.parseStr('[final var x = someVal,]')), '[final var x=someVal,]');
    runTest(OptionalFormalParameters(lexer.parseStr('[dynamic x (covariant int y(x)) = 23 * 3,]')), '[dynamic x (covariant int y (x))=23 * 3,]');
    runTest(OptionalFormalParameters(lexer.parseStr('[covariant int x = 0xfff,]')), '[covariant int x=0xfff,]');
    runTest(OptionalFormalParameters(lexer.parseStr('[String this.x=msg,]')), '[String this.x=msg,]');
    runTest(OptionalFormalParameters(lexer.parseStr('[const String this.x=msg,]')), '[const String this.x=msg,]');
    runTest(OptionalFormalParameters(lexer.parseStr('[int this.x () = 0,]')), '[int this.x ()=0,]');
    runTest(OptionalFormalParameters(lexer.parseStr('[var this.x (var y)=anotherVar,]')), '[var this.x (var y)=anotherVar,]');

    // ---------------------------------

    runTest(OptionalFormalParameters(lexer.parseStr('{int x()}')), '{int x ()}');
    runTest(OptionalFormalParameters(lexer.parseStr('{var x()}')), '{var x ()}');
    runTest(OptionalFormalParameters(lexer.parseStr('{covariant someType x()}')), '{covariant someType x ()}');
    runTest(OptionalFormalParameters(lexer.parseStr('{dynamic x ()}')), '{dynamic x ()}');
    runTest(OptionalFormalParameters(lexer.parseStr('{dynamic x (covariant int y(x))}')), '{dynamic x (covariant int y (x))}');
    runTest(OptionalFormalParameters(lexer.parseStr('{int x}')), '{int x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{var x}')), '{var x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{someType x}')), '{someType x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{const bool x}')), '{const bool x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{final var x}')), '{final var x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{dynamic x}')), '{dynamic x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{covariant x}')), '{covariant x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{covariant int x}')), '{covariant int x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{covariant const bool x}')), '{covariant const bool x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{covariant final var x}')), '{covariant final var x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{covariant dynamic x}')), '{covariant dynamic x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{int this.x}')), '{int this.x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{var this.x}')), '{var this.x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{someType this.x}')), '{someType this.x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{const bool this.x}')), '{const bool this.x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{final var this.x}')), '{final var this.x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{dynamic this.x}')), '{dynamic this.x}');
    runTest(OptionalFormalParameters(lexer.parseStr('{int this.x ()}')), '{int this.x ()}');
    runTest(OptionalFormalParameters(lexer.parseStr('{var this.x (var y)}')), '{var this.x (var y)}');
    runTest(OptionalFormalParameters(lexer.parseStr('{someType this.x (const var t)}')), '{someType this.x (const var t)}');
    runTest(OptionalFormalParameters(lexer.parseStr('{const bool this.x (covariant int y)}')), '{const bool this.x (covariant int y)}');
    runTest(OptionalFormalParameters(lexer.parseStr('{final var this.x (const var this.y ( final int z))}')), '{final var this.x (const var this.y (final int z))}');

    runTest(OptionalFormalParameters(lexer.parseStr('{var x = "testStr"}')), '{var x="testStr"}');
    runTest(OptionalFormalParameters(lexer.parseStr('{int x() = 4}')), '{int x ()=4}');
    runTest(OptionalFormalParameters(lexer.parseStr('{final var x = someVal}')), '{final var x=someVal}');
    runTest(OptionalFormalParameters(lexer.parseStr('{dynamic x (covariant int y(x)) = 23 * 3}')), '{dynamic x (covariant int y (x))=23 * 3}');
    runTest(OptionalFormalParameters(lexer.parseStr('{covariant int x = 0xfff}')), '{covariant int x=0xfff}');
    runTest(OptionalFormalParameters(lexer.parseStr('{String this.x=msg}')), '{String this.x=msg}');
    runTest(OptionalFormalParameters(lexer.parseStr('{const String this.x=msg}')), '{const String this.x=msg}');
    runTest(OptionalFormalParameters(lexer.parseStr('{int this.x () = 0}')), '{int this.x ()=0}');
    runTest(OptionalFormalParameters(lexer.parseStr('{var this.x (var y)=anotherVar}')), '{var this.x (var y)=anotherVar}');

    runTest(OptionalFormalParameters(lexer.parseStr('{var x = "testStr",}')), '{var x="testStr",}');
    runTest(OptionalFormalParameters(lexer.parseStr('{int x() = 4,}')), '{int x ()=4,}');
    runTest(OptionalFormalParameters(lexer.parseStr('{final var x = someVal,}')), '{final var x=someVal,}');
    runTest(OptionalFormalParameters(lexer.parseStr('{dynamic x (covariant int y(x)) = 23 * 3,}')), '{dynamic x (covariant int y (x))=23 * 3,}');
    runTest(OptionalFormalParameters(lexer.parseStr('{covariant int x = 0xfff,}')), '{covariant int x=0xfff,}');
    runTest(OptionalFormalParameters(lexer.parseStr('{String this.x=msg,}')), '{String this.x=msg,}');
    runTest(OptionalFormalParameters(lexer.parseStr('{const String this.x=msg,}')), '{const String this.x=msg,}');
    runTest(OptionalFormalParameters(lexer.parseStr('{int this.x () = 0,}')), '{int this.x ()=0,}');
    runTest(OptionalFormalParameters(lexer.parseStr('{var this.x (var y)=anotherVar,}')), '{var this.x (var y)=anotherVar,}');

  });

  test('Formal Parameter List', () {

    runTest(FormalParameterList(lexer.parseStr('()')), '()');

    runTest(FormalParameterList(lexer.parseStr('(int x, var y)')), '(int x, var y)');
    runTest(FormalParameterList(lexer.parseStr('(int x, final var y)')), '(int x, final var y)');
    runTest(FormalParameterList(lexer.parseStr('(covariant int x, final var y)')), '(covariant int x, final var y)');
    runTest(FormalParameterList(lexer.parseStr('(int x(), var y())')), '(int x (), var y ())');
    runTest(FormalParameterList(lexer.parseStr('(covariant someType x(), bool y)')), '(covariant someType x (), bool y)');
    runTest(FormalParameterList(lexer.parseStr('(dynamic x (covariant int y(x), type z))')), '(dynamic x (covariant int y (x), type z))');
    runTest(FormalParameterList(lexer.parseStr('(var this.x, bool this.y)')), '(var this.x, bool this.y)');
    runTest(FormalParameterList(lexer.parseStr('(var this.x, const bool this.y)')), '(var this.x, const bool this.y)');
    runTest(FormalParameterList(lexer.parseStr('(var this.x (), const bool this.y)')), '(var this.x (), const bool this.y)');
    runTest(FormalParameterList(lexer.parseStr('(var this.x (type this.y), const bool this.y)')), '(var this.x (type this.y), const bool this.y)');
    runTest(FormalParameterList(lexer.parseStr('(final var this.x (const var this.y ( final int z)))')), '(final var this.x (const var this.y (final int z)))');

    runTest(FormalParameterList(lexer.parseStr('(int x, var y,)')), '(int x, var y,)');
    runTest(FormalParameterList(lexer.parseStr('(int x, final var y,)')), '(int x, final var y,)');
    runTest(FormalParameterList(lexer.parseStr('(covariant int x, final var y,)')), '(covariant int x, final var y,)');
    runTest(FormalParameterList(lexer.parseStr('(int x(), var y(),)')), '(int x (), var y (),)');
    runTest(FormalParameterList(lexer.parseStr('(covariant someType x(), bool y,)')), '(covariant someType x (), bool y,)');
    runTest(FormalParameterList(lexer.parseStr('(dynamic x (covariant int y(x), type z),)')), '(dynamic x (covariant int y (x), type z),)');
    runTest(FormalParameterList(lexer.parseStr('(var this.x, bool this.y,)')), '(var this.x, bool this.y,)');
    runTest(FormalParameterList(lexer.parseStr('(var this.x, const bool this.y,)')), '(var this.x, const bool this.y,)');
    runTest(FormalParameterList(lexer.parseStr('(var this.x (), const bool this.y,)')), '(var this.x (), const bool this.y,)');
    runTest(FormalParameterList(lexer.parseStr('(var this.x (type this.y), const bool this.y,)')), '(var this.x (type this.y), const bool this.y,)');
    runTest(FormalParameterList(lexer.parseStr('(final var this.x (const var this.y ( final int z)),)')), '(final var this.x (const var this.y (final int z)),)');

    // ---------------------------------

    runTest(FormalParameterList(lexer.parseStr('([int x()])')), '([int x ()])');
    runTest(FormalParameterList(lexer.parseStr('([covariant someType x()])')), '([covariant someType x ()])');
    runTest(FormalParameterList(lexer.parseStr('([dynamic x (covariant int y(x))])')), '([dynamic x (covariant int y (x))])');
    runTest(FormalParameterList(lexer.parseStr('([final var x])')), '([final var x])');
    runTest(FormalParameterList(lexer.parseStr('([covariant x])')), '([covariant x])');
    runTest(FormalParameterList(lexer.parseStr('([covariant const bool x])')), '([covariant const bool x])');
    runTest(FormalParameterList(lexer.parseStr('([someType this.x])')), '([someType this.x])');
    runTest(FormalParameterList(lexer.parseStr('([var this.x (var y)])')), '([var this.x (var y)])');
    runTest(FormalParameterList(lexer.parseStr('([someType this.x (const var t)])')), '([someType this.x (const var t)])');
    runTest(FormalParameterList(lexer.parseStr('([const bool this.x (covariant int y)])')), '([const bool this.x (covariant int y)])');
    runTest(FormalParameterList(lexer.parseStr('([final var this.x (const var this.y ( final int z))])')), '([final var this.x (const var this.y (final int z))])');
    runTest(FormalParameterList(lexer.parseStr('([var x = "testStr"])')), '([var x="testStr"])');
    runTest(FormalParameterList(lexer.parseStr('([final var x = someVal])')), '([final var x=someVal])');
    runTest(FormalParameterList(lexer.parseStr('([covariant int x = 0xfff])')), '([covariant int x=0xfff])');
    runTest(FormalParameterList(lexer.parseStr('([const String this.x=msg])')), '([const String this.x=msg])');
    runTest(FormalParameterList(lexer.parseStr('([var this.x (var y)=anotherVar])')), '([var this.x (var y)=anotherVar])');
    runTest(FormalParameterList(lexer.parseStr('([int x() = 4,])')), '([int x ()=4,])');
    runTest(FormalParameterList(lexer.parseStr('([dynamic x (covariant int y(x)) = 23 * 3,])')), '([dynamic x (covariant int y (x))=23 * 3,])');
    runTest(FormalParameterList(lexer.parseStr('([String this.x=msg,])')), '([String this.x=msg,])');
    runTest(FormalParameterList(lexer.parseStr('([int this.x () = 0,])')), '([int this.x ()=0,])');

    runTest(FormalParameterList(lexer.parseStr('(int x, var y, [int x()])')), '(int x, var y, [int x ()])');
    runTest(FormalParameterList(lexer.parseStr('(dynamic x (covariant int y(x), type z), [covariant someType x()])')), '(dynamic x (covariant int y (x), type z), [covariant someType x ()])');
    runTest(FormalParameterList(lexer.parseStr('([dynamic x (covariant int y(x))])')), '([dynamic x (covariant int y (x))])');
    runTest(FormalParameterList(lexer.parseStr('(int x, var y, [final var x])')), '(int x, var y, [final var x])');
    runTest(FormalParameterList(lexer.parseStr('(var this.x (), const bool this.y, [someType this.x])')), '(var this.x (), const bool this.y, [someType this.x])');
    runTest(FormalParameterList(lexer.parseStr('([final var this.x (const var this.y ( final int z))])')), '([final var this.x (const var this.y (final int z))])');
    runTest(FormalParameterList(lexer.parseStr('(final var this.x (const var this.y ( final int z)), [dynamic x (covariant int y(x)) = 23 * 3,])')), '(final var this.x (const var this.y (final int z)), [dynamic x (covariant int y (x))=23 * 3,])');

    // ---------------------------------

    runTest(FormalParameterList(lexer.parseStr('({int x()})')), '({int x ()})');
    runTest(FormalParameterList(lexer.parseStr('({covariant someType x()})')), '({covariant someType x ()})');
    runTest(FormalParameterList(lexer.parseStr('({dynamic x (covariant int y(x))})')), '({dynamic x (covariant int y (x))})');
    runTest(FormalParameterList(lexer.parseStr('({final var x})')), '({final var x})');
    runTest(FormalParameterList(lexer.parseStr('({covariant x})')), '({covariant x})');
    runTest(FormalParameterList(lexer.parseStr('({covariant const bool x})')), '({covariant const bool x})');
    runTest(FormalParameterList(lexer.parseStr('({someType this.x})')), '({someType this.x})');
    runTest(FormalParameterList(lexer.parseStr('({var this.x (var y)})')), '({var this.x (var y)})');
    runTest(FormalParameterList(lexer.parseStr('({someType this.x (const var t)})')), '({someType this.x (const var t)})');
    runTest(FormalParameterList(lexer.parseStr('({const bool this.x (covariant int y)})')), '({const bool this.x (covariant int y)})');
    runTest(FormalParameterList(lexer.parseStr('({final var this.x (const var this.y ( final int z))})')), '({final var this.x (const var this.y (final int z))})');
    runTest(FormalParameterList(lexer.parseStr('({var x = "testStr"})')), '({var x="testStr"})');
    runTest(FormalParameterList(lexer.parseStr('({final var x = someVal})')), '({final var x=someVal})');
    runTest(FormalParameterList(lexer.parseStr('({covariant int x = 0xfff})')), '({covariant int x=0xfff})');
    runTest(FormalParameterList(lexer.parseStr('({const String this.x=msg})')), '({const String this.x=msg})');
    runTest(FormalParameterList(lexer.parseStr('({var this.x (var y)=anotherVar})')), '({var this.x (var y)=anotherVar})');
    runTest(FormalParameterList(lexer.parseStr('({int x() = 4,})')), '({int x ()=4,})');
    runTest(FormalParameterList(lexer.parseStr('({dynamic x (covariant int y(x)) = 23 * 3,})')), '({dynamic x (covariant int y (x))=23 * 3,})');
    runTest(FormalParameterList(lexer.parseStr('({String this.x=msg,})')), '({String this.x=msg,})');
    runTest(FormalParameterList(lexer.parseStr('({int this.x () = 0,})')), '({int this.x ()=0,})');

    runTest(FormalParameterList(lexer.parseStr('(int x, var y, {int x()})')), '(int x, var y, {int x ()})');
    runTest(FormalParameterList(lexer.parseStr('(dynamic x (covariant int y(x), type z), {covariant someType x()})')), '(dynamic x (covariant int y (x), type z), {covariant someType x ()})');
    runTest(FormalParameterList(lexer.parseStr('({dynamic x (covariant int y(x))})')), '({dynamic x (covariant int y (x))})');
    runTest(FormalParameterList(lexer.parseStr('(int x, var y, {final var x})')), '(int x, var y, {final var x})');
    runTest(FormalParameterList(lexer.parseStr('(var this.x (), const bool this.y, {someType this.x})')), '(var this.x (), const bool this.y, {someType this.x})');
    runTest(FormalParameterList(lexer.parseStr('({final var this.x (const var this.y ( final int z))})')), '({final var this.x (const var this.y (final int z))})');
    runTest(FormalParameterList(lexer.parseStr('(final var this.x (const var this.y ( final int z)), {dynamic x (covariant int y(x)) = 23 * 3,})')), '(final var this.x (const var this.y (final int z)), {dynamic x (covariant int y (x))=23 * 3,})');

  });

  test('Formal Parameter Part', () {
    //TODO: add typeParameters checks

    runTest(FormalParameterPart(lexer.parseStr('()')), '()');

    runTest(FormalParameterPart(lexer.parseStr('(int x, var y)')), '(int x, var y)');
    runTest(FormalParameterPart(lexer.parseStr('(int x, final var y)')), '(int x, final var y)');
    runTest(FormalParameterPart(lexer.parseStr('(covariant int x, final var y)')), '(covariant int x, final var y)');
    runTest(FormalParameterPart(lexer.parseStr('(int x(), var y())')), '(int x (), var y ())');
    runTest(FormalParameterPart(lexer.parseStr('(covariant someType x(), bool y)')), '(covariant someType x (), bool y)');
    runTest(FormalParameterPart(lexer.parseStr('(dynamic x (covariant int y(x), type z))')), '(dynamic x (covariant int y (x), type z))');
    runTest(FormalParameterPart(lexer.parseStr('(var this.x, bool this.y)')), '(var this.x, bool this.y)');
    runTest(FormalParameterPart(lexer.parseStr('(var this.x, const bool this.y)')), '(var this.x, const bool this.y)');
    runTest(FormalParameterPart(lexer.parseStr('(var this.x (), const bool this.y)')), '(var this.x (), const bool this.y)');
    runTest(FormalParameterPart(lexer.parseStr('(var this.x (type this.y), const bool this.y)')), '(var this.x (type this.y), const bool this.y)');
    runTest(FormalParameterPart(lexer.parseStr('(final var this.x (const var this.y ( final int z)))')), '(final var this.x (const var this.y (final int z)))');

    runTest(FormalParameterPart(lexer.parseStr('(int x, var y,)')), '(int x, var y,)');
    runTest(FormalParameterPart(lexer.parseStr('(int x, final var y,)')), '(int x, final var y,)');
    runTest(FormalParameterPart(lexer.parseStr('(covariant int x, final var y,)')), '(covariant int x, final var y,)');
    runTest(FormalParameterPart(lexer.parseStr('(int x(), var y(),)')), '(int x (), var y (),)');
    runTest(FormalParameterPart(lexer.parseStr('(covariant someType x(), bool y,)')), '(covariant someType x (), bool y,)');
    runTest(FormalParameterPart(lexer.parseStr('(dynamic x (covariant int y(x), type z),)')), '(dynamic x (covariant int y (x), type z),)');
    runTest(FormalParameterPart(lexer.parseStr('(var this.x, bool this.y,)')), '(var this.x, bool this.y,)');
    runTest(FormalParameterPart(lexer.parseStr('(var this.x, const bool this.y,)')), '(var this.x, const bool this.y,)');
    runTest(FormalParameterPart(lexer.parseStr('(var this.x (), const bool this.y,)')), '(var this.x (), const bool this.y,)');
    runTest(FormalParameterPart(lexer.parseStr('(var this.x (type this.y), const bool this.y,)')), '(var this.x (type this.y), const bool this.y,)');
    runTest(FormalParameterPart(lexer.parseStr('(final var this.x (const var this.y ( final int z)),)')), '(final var this.x (const var this.y (final int z)),)');

    // ---------------------------------

    runTest(FormalParameterPart(lexer.parseStr('([int x()])')), '([int x ()])');
    runTest(FormalParameterPart(lexer.parseStr('([covariant someType x()])')), '([covariant someType x ()])');
    runTest(FormalParameterPart(lexer.parseStr('([dynamic x (covariant int y(x))])')), '([dynamic x (covariant int y (x))])');
    runTest(FormalParameterPart(lexer.parseStr('([final var x])')), '([final var x])');
    runTest(FormalParameterPart(lexer.parseStr('([covariant x])')), '([covariant x])');
    runTest(FormalParameterPart(lexer.parseStr('([covariant const bool x])')), '([covariant const bool x])');
    runTest(FormalParameterPart(lexer.parseStr('([someType this.x])')), '([someType this.x])');
    runTest(FormalParameterPart(lexer.parseStr('([var this.x (var y)])')), '([var this.x (var y)])');
    runTest(FormalParameterPart(lexer.parseStr('([someType this.x (const var t)])')), '([someType this.x (const var t)])');
    runTest(FormalParameterPart(lexer.parseStr('([const bool this.x (covariant int y)])')), '([const bool this.x (covariant int y)])');
    runTest(FormalParameterPart(lexer.parseStr('([final var this.x (const var this.y ( final int z))])')), '([final var this.x (const var this.y (final int z))])');
    runTest(FormalParameterPart(lexer.parseStr('([var x = "testStr"])')), '([var x="testStr"])');
    runTest(FormalParameterPart(lexer.parseStr('([final var x = someVal])')), '([final var x=someVal])');
    runTest(FormalParameterPart(lexer.parseStr('([covariant int x = 0xfff])')), '([covariant int x=0xfff])');
    runTest(FormalParameterPart(lexer.parseStr('([const String this.x=msg])')), '([const String this.x=msg])');
    runTest(FormalParameterPart(lexer.parseStr('([var this.x (var y)=anotherVar])')), '([var this.x (var y)=anotherVar])');
    runTest(FormalParameterPart(lexer.parseStr('([int x() = 4,])')), '([int x ()=4,])');
    runTest(FormalParameterPart(lexer.parseStr('([dynamic x (covariant int y(x)) = 23 * 3,])')), '([dynamic x (covariant int y (x))=23 * 3,])');
    runTest(FormalParameterPart(lexer.parseStr('([String this.x=msg,])')), '([String this.x=msg,])');
    runTest(FormalParameterPart(lexer.parseStr('([int this.x () = 0,])')), '([int this.x ()=0,])');

    runTest(FormalParameterPart(lexer.parseStr('(int x, var y, [int x()])')), '(int x, var y, [int x ()])');
    runTest(FormalParameterPart(lexer.parseStr('(dynamic x (covariant int y(x), type z), [covariant someType x()])')), '(dynamic x (covariant int y (x), type z), [covariant someType x ()])');
    runTest(FormalParameterPart(lexer.parseStr('([dynamic x (covariant int y(x))])')), '([dynamic x (covariant int y (x))])');
    runTest(FormalParameterPart(lexer.parseStr('(int x, var y, [final var x])')), '(int x, var y, [final var x])');
    runTest(FormalParameterPart(lexer.parseStr('(var this.x (), const bool this.y, [someType this.x])')), '(var this.x (), const bool this.y, [someType this.x])');
    runTest(FormalParameterPart(lexer.parseStr('([final var this.x (const var this.y ( final int z))])')), '([final var this.x (const var this.y (final int z))])');
    runTest(FormalParameterPart(lexer.parseStr('(final var this.x (const var this.y ( final int z)), [dynamic x (covariant int y(x)) = 23 * 3,])')), '(final var this.x (const var this.y (final int z)), [dynamic x (covariant int y (x))=23 * 3,])');

    // ---------------------------------

    runTest(FormalParameterPart(lexer.parseStr('({int x()})')), '({int x ()})');
    runTest(FormalParameterPart(lexer.parseStr('({covariant someType x()})')), '({covariant someType x ()})');
    runTest(FormalParameterPart(lexer.parseStr('({dynamic x (covariant int y(x))})')), '({dynamic x (covariant int y (x))})');
    runTest(FormalParameterPart(lexer.parseStr('({final var x})')), '({final var x})');
    runTest(FormalParameterPart(lexer.parseStr('({covariant x})')), '({covariant x})');
    runTest(FormalParameterPart(lexer.parseStr('({covariant const bool x})')), '({covariant const bool x})');
    runTest(FormalParameterPart(lexer.parseStr('({someType this.x})')), '({someType this.x})');
    runTest(FormalParameterPart(lexer.parseStr('({var this.x (var y)})')), '({var this.x (var y)})');
    runTest(FormalParameterPart(lexer.parseStr('({someType this.x (const var t)})')), '({someType this.x (const var t)})');
    runTest(FormalParameterPart(lexer.parseStr('({const bool this.x (covariant int y)})')), '({const bool this.x (covariant int y)})');
    runTest(FormalParameterPart(lexer.parseStr('({final var this.x (const var this.y ( final int z))})')), '({final var this.x (const var this.y (final int z))})');
    runTest(FormalParameterPart(lexer.parseStr('({var x = "testStr"})')), '({var x="testStr"})');
    runTest(FormalParameterPart(lexer.parseStr('({final var x = someVal})')), '({final var x=someVal})');
    runTest(FormalParameterPart(lexer.parseStr('({covariant int x = 0xfff})')), '({covariant int x=0xfff})');
    runTest(FormalParameterPart(lexer.parseStr('({const String this.x=msg})')), '({const String this.x=msg})');
    runTest(FormalParameterPart(lexer.parseStr('({var this.x (var y)=anotherVar})')), '({var this.x (var y)=anotherVar})');
    runTest(FormalParameterPart(lexer.parseStr('({int x() = 4,})')), '({int x ()=4,})');
    runTest(FormalParameterPart(lexer.parseStr('({dynamic x (covariant int y(x)) = 23 * 3,})')), '({dynamic x (covariant int y (x))=23 * 3,})');
    runTest(FormalParameterPart(lexer.parseStr('({String this.x=msg,})')), '({String this.x=msg,})');
    runTest(FormalParameterPart(lexer.parseStr('({int this.x () = 0,})')), '({int this.x ()=0,})');

    runTest(FormalParameterPart(lexer.parseStr('(int x, var y, {int x()})')), '(int x, var y, {int x ()})');
    runTest(FormalParameterPart(lexer.parseStr('(dynamic x (covariant int y(x), type z), {covariant someType x()})')), '(dynamic x (covariant int y (x), type z), {covariant someType x ()})');
    runTest(FormalParameterPart(lexer.parseStr('({dynamic x (covariant int y(x))})')), '({dynamic x (covariant int y (x))})');
    runTest(FormalParameterPart(lexer.parseStr('(int x, var y, {final var x})')), '(int x, var y, {final var x})');
    runTest(FormalParameterPart(lexer.parseStr('(var this.x (), const bool this.y, {someType this.x})')), '(var this.x (), const bool this.y, {someType this.x})');
    runTest(FormalParameterPart(lexer.parseStr('({final var this.x (const var this.y ( final int z))})')), '({final var this.x (const var this.y (final int z))})');
    runTest(FormalParameterPart(lexer.parseStr('(final var this.x (const var this.y ( final int z)), {dynamic x (covariant int y(x)) = 23 * 3,})')), '(final var this.x (const var this.y (final int z)), {dynamic x (covariant int y (x))=23 * 3,})');

  });

  test('Function Signature', () {

    runTest(FunctionSignature(lexer.parseStr('testFunction()')), 'testFunction()');
    runTest(FunctionSignature(lexer.parseStr('void testFunction()')), 'void testFunction()');

    runTest(FunctionSignature(lexer.parseStr('void testFunction(int x, var y)')), 'void testFunction(int x, var y)');
    runTest(FunctionSignature(lexer.parseStr('testFunction(int x, final var y)')), 'testFunction(int x, final var y)');
    runTest(FunctionSignature(lexer.parseStr('String testFunction(covariant int x, final var y)')), 'String testFunction(covariant int x, final var y)');
    runTest(FunctionSignature(lexer.parseStr('void testFunction(int x(), var y())')), 'void testFunction(int x (), var y ())');
    runTest(FunctionSignature(lexer.parseStr('testFunction(covariant someType x(), bool y)')), 'testFunction(covariant someType x (), bool y)');
    runTest(FunctionSignature(lexer.parseStr('void testFunction(dynamic x (covariant int y(x), type z))')), 'void testFunction(dynamic x (covariant int y (x), type z))');
    runTest(FunctionSignature(lexer.parseStr('int testFunction(var this.x, bool this.y)')), 'int testFunction(var this.x, bool this.y)');
    runTest(FunctionSignature(lexer.parseStr('testFunction(var this.x, const bool this.y)')), 'testFunction(var this.x, const bool this.y)');
    runTest(FunctionSignature(lexer.parseStr('void testFunction(var this.x (), const bool this.y)')), 'void testFunction(var this.x (), const bool this.y)');
    runTest(FunctionSignature(lexer.parseStr('someType testFunction(var this.x (type this.y), const bool this.y)')), 'someType testFunction(var this.x (type this.y), const bool this.y)');
    runTest(FunctionSignature(lexer.parseStr('testFunction(final var this.x (const var this.y ( final int z)))')), 'testFunction(final var this.x (const var this.y (final int z)))');

    runTest(FunctionSignature(lexer.parseStr('testFunction(int x, var y,)')), 'testFunction(int x, var y,)');
    runTest(FunctionSignature(lexer.parseStr('void testFunction(int x, final var y,)')), 'void testFunction(int x, final var y,)');
    runTest(FunctionSignature(lexer.parseStr('someType testFunction(covariant int x, final var y,)')), 'someType testFunction(covariant int x, final var y,)');
    runTest(FunctionSignature(lexer.parseStr('testFunction(int x(), var y(),)')), 'testFunction(int x (), var y (),)');
    runTest(FunctionSignature(lexer.parseStr('void testFunction(covariant someType x(), bool y,)')), 'void testFunction(covariant someType x (), bool y,)');
    runTest(FunctionSignature(lexer.parseStr('testFunction(dynamic x (covariant int y(x), type z),)')), 'testFunction(dynamic x (covariant int y (x), type z),)');
    runTest(FunctionSignature(lexer.parseStr('testFunction(var this.x, bool this.y,)')), 'testFunction(var this.x, bool this.y,)');
    runTest(FunctionSignature(lexer.parseStr('int testFunction(var this.x, const bool this.y,)')), 'int testFunction(var this.x, const bool this.y,)');
    runTest(FunctionSignature(lexer.parseStr('void testFunction(var this.x (), const bool this.y,)')), 'void testFunction(var this.x (), const bool this.y,)');
    runTest(FunctionSignature(lexer.parseStr('testFunction(var this.x (type this.y), const bool this.y,)')), 'testFunction(var this.x (type this.y), const bool this.y,)');
    runTest(FunctionSignature(lexer.parseStr('void testFunction(final var this.x (const var this.y ( final int z)),)')), 'void testFunction(final var this.x (const var this.y (final int z)),)');

    // ---------------------------------

    runTest(FunctionSignature(lexer.parseStr('testFunction([int x()])')), 'testFunction([int x ()])');
    runTest(FunctionSignature(lexer.parseStr('void testFunction([covariant someType x()])')), 'void testFunction([covariant someType x ()])');
    runTest(FunctionSignature(lexer.parseStr('void testFunction([dynamic x (covariant int y(x))])')), 'void testFunction([dynamic x (covariant int y (x))])');
    runTest(FunctionSignature(lexer.parseStr('bool testFunction([final var x])')), 'bool testFunction([final var x])');
    runTest(FunctionSignature(lexer.parseStr('String testFunction([covariant x])')), 'String testFunction([covariant x])');
    runTest(FunctionSignature(lexer.parseStr('void testFunction([covariant const bool x])')), 'void testFunction([covariant const bool x])');
    runTest(FunctionSignature(lexer.parseStr('testFunction([someType this.x])')), 'testFunction([someType this.x])');
    runTest(FunctionSignature(lexer.parseStr('testFunction([var this.x (var y)])')), 'testFunction([var this.x (var y)])');
    runTest(FunctionSignature(lexer.parseStr('void testFunction([someType this.x (const var t)])')), 'void testFunction([someType this.x (const var t)])');
    runTest(FunctionSignature(lexer.parseStr('someType testFunction([const bool this.x (covariant int y)])')), 'someType testFunction([const bool this.x (covariant int y)])');
    runTest(FunctionSignature(lexer.parseStr('void testFunction([final var this.x (const var this.y ( final int z))])')), 'void testFunction([final var this.x (const var this.y (final int z))])');
    runTest(FunctionSignature(lexer.parseStr('testFunction([var x = "testStr"])')), 'testFunction([var x="testStr"])');
    runTest(FunctionSignature(lexer.parseStr('void testFunction([final var x = someVal])')), 'void testFunction([final var x=someVal])');
    runTest(FunctionSignature(lexer.parseStr('String testFunction([covariant int x = 0xfff])')), 'String testFunction([covariant int x=0xfff])');
    runTest(FunctionSignature(lexer.parseStr('void testFunction([const String this.x=msg])')), 'void testFunction([const String this.x=msg])');
    runTest(FunctionSignature(lexer.parseStr('int testFunction([var this.x (var y)=anotherVar])')), 'int testFunction([var this.x (var y)=anotherVar])');
    runTest(FunctionSignature(lexer.parseStr('void testFunction([int x() = 4,])')), 'void testFunction([int x ()=4,])');
    runTest(FunctionSignature(lexer.parseStr('testFunction([dynamic x (covariant int y(x)) = 23 * 3,])')), 'testFunction([dynamic x (covariant int y (x))=23 * 3,])');
    runTest(FunctionSignature(lexer.parseStr('someType_ testFunction([String this.x=msg,])')), 'someType_ testFunction([String this.x=msg,])');
    runTest(FunctionSignature(lexer.parseStr('void testFunction([int this.x () = 0,])')), 'void testFunction([int this.x ()=0,])');

    runTest(FunctionSignature(lexer.parseStr('void testFunction(int x, var y, [int x()])')), 'void testFunction(int x, var y, [int x ()])');
    runTest(FunctionSignature(lexer.parseStr('testFunction(dynamic x (covariant int y(x), type z), [covariant someType x()])')), 'testFunction(dynamic x (covariant int y (x), type z), [covariant someType x ()])');
    runTest(FunctionSignature(lexer.parseStr('String testFunction([dynamic x (covariant int y(x))])')), 'String testFunction([dynamic x (covariant int y (x))])');
    runTest(FunctionSignature(lexer.parseStr('bool testFunction(int x, var y, [final var x])')), 'bool testFunction(int x, var y, [final var x])');
    runTest(FunctionSignature(lexer.parseStr('int testFunction(var this.x (), const bool this.y, [someType this.x])')), 'int testFunction(var this.x (), const bool this.y, [someType this.x])');
    runTest(FunctionSignature(lexer.parseStr('testFunction([final var this.x (const var this.y ( final int z))])')), 'testFunction([final var this.x (const var this.y (final int z))])');
    runTest(FunctionSignature(lexer.parseStr('someType testFunction(final var this.x (const var this.y ( final int z)), [dynamic x (covariant int y(x)) = 23 * 3,])')), 'someType testFunction(final var this.x (const var this.y (final int z)), [dynamic x (covariant int y (x))=23 * 3,])');

    // ---------------------------------

    runTest(FunctionSignature(lexer.parseStr('testFunction({int x()})')), 'testFunction({int x ()})');
    runTest(FunctionSignature(lexer.parseStr('void testFunction({covariant someType x()})')), 'void testFunction({covariant someType x ()})');
    runTest(FunctionSignature(lexer.parseStr('void testFunction({dynamic x (covariant int y(x))})')), 'void testFunction({dynamic x (covariant int y (x))})');
    runTest(FunctionSignature(lexer.parseStr('bool testFunction({final var x})')), 'bool testFunction({final var x})');
    runTest(FunctionSignature(lexer.parseStr('String testFunction({covariant x})')), 'String testFunction({covariant x})');
    runTest(FunctionSignature(lexer.parseStr('void testFunction({covariant const bool x})')), 'void testFunction({covariant const bool x})');
    runTest(FunctionSignature(lexer.parseStr('testFunction({someType this.x})')), 'testFunction({someType this.x})');
    runTest(FunctionSignature(lexer.parseStr('testFunction({var this.x (var y)})')), 'testFunction({var this.x (var y)})');
    runTest(FunctionSignature(lexer.parseStr('void testFunction({someType this.x (const var t)})')), 'void testFunction({someType this.x (const var t)})');
    runTest(FunctionSignature(lexer.parseStr('someType testFunction({const bool this.x (covariant int y)})')), 'someType testFunction({const bool this.x (covariant int y)})');
    runTest(FunctionSignature(lexer.parseStr('void testFunction({final var this.x (const var this.y ( final int z))})')), 'void testFunction({final var this.x (const var this.y (final int z))})');
    runTest(FunctionSignature(lexer.parseStr('testFunction({var x = "testStr"})')), 'testFunction({var x="testStr"})');
    runTest(FunctionSignature(lexer.parseStr('void testFunction({final var x = someVal})')), 'void testFunction({final var x=someVal})');
    runTest(FunctionSignature(lexer.parseStr('String testFunction({covariant int x = 0xfff})')), 'String testFunction({covariant int x=0xfff})');
    runTest(FunctionSignature(lexer.parseStr('void testFunction({const String this.x=msg})')), 'void testFunction({const String this.x=msg})');
    runTest(FunctionSignature(lexer.parseStr('int testFunction({var this.x (var y)=anotherVar})')), 'int testFunction({var this.x (var y)=anotherVar})');
    runTest(FunctionSignature(lexer.parseStr('void testFunction({int x() = 4,})')), 'void testFunction({int x ()=4,})');
    runTest(FunctionSignature(lexer.parseStr('testFunction({dynamic x (covariant int y(x)) = 23 * 3,})')), 'testFunction({dynamic x (covariant int y (x))=23 * 3,})');
    runTest(FunctionSignature(lexer.parseStr('someType_ testFunction({String this.x=msg,})')), 'someType_ testFunction({String this.x=msg,})');
    runTest(FunctionSignature(lexer.parseStr('void testFunction({int this.x () = 0,})')), 'void testFunction({int this.x ()=0,})');

    runTest(FunctionSignature(lexer.parseStr('void testFunction(int x, var y, {int x()})')), 'void testFunction(int x, var y, {int x ()})');
    runTest(FunctionSignature(lexer.parseStr('testFunction(dynamic x (covariant int y(x), type z), {covariant someType x()})')), 'testFunction(dynamic x (covariant int y (x), type z), {covariant someType x ()})');
    runTest(FunctionSignature(lexer.parseStr('String testFunction({dynamic x (covariant int y(x))})')), 'String testFunction({dynamic x (covariant int y (x))})');
    runTest(FunctionSignature(lexer.parseStr('bool testFunction(int x, var y, {final var x})')), 'bool testFunction(int x, var y, {final var x})');
    runTest(FunctionSignature(lexer.parseStr('int testFunction(var this.x (), const bool this.y, {someType this.x})')), 'int testFunction(var this.x (), const bool this.y, {someType this.x})');
    runTest(FunctionSignature(lexer.parseStr('testFunction({final var this.x (const var this.y ( final int z))})')), 'testFunction({final var this.x (const var this.y (final int z))})');
    runTest(FunctionSignature(lexer.parseStr('someType testFunction(final var this.x (const var this.y ( final int z)), {dynamic x (covariant int y(x)) = 23 * 3,})')), 'someType testFunction(final var this.x (const var this.y (final int z)), {dynamic x (covariant int y (x))=23 * 3,})');

  });
  
}
