import 'package:test/test.dart';
import 'test_lib.dart';

import 'package:source_parser/src/lexer/Lexer.dart';

import 'package:source_parser/src/sub_types/Literals.dart';
import 'package:source_parser/src/core/Literal.dart';

void main() {

  Lexer lexer = Lexer();

  test('Null Literal', () {

    runTest(NullLiteral(lexer.parseStr('null')), 'null');

  });

  test('Bool Literal', () {

    runTest(BooleanLiteral(lexer.parseStr('true')), 'true');
    runTest(BooleanLiteral(lexer.parseStr('false')), 'false');

  });

  test('String Literal', () {
    // TODO: Add complete string (multi line and other escape characters) parsing (this is done in the lexer)

    runTest(StringLiteral(lexer.parseStr('"Hello World!"')), '"Hello World!"');
    runTest(StringLiteral(lexer.parseStr('"\'Testing escape character\'"')), '"\'Testing escape character\'"');

    runTest(StringLiteral(lexer.parseStr("'Hello World!'")), "'Hello World!'");
    runTest(StringLiteral(lexer.parseStr("'\"Testing escape character\"'")), "'\"Testing escape character\"'");

  });

  test('Numeric Literal', () {

    runTest(NumericLiteral(lexer.parseStr('0')), '0');
    runTest(NumericLiteral(lexer.parseStr('23')), '23');
    runTest(NumericLiteral(lexer.parseStr('239039203')), '239039203');

    runTest(NumericLiteral(lexer.parseStr('0.00000')), '0.00000');
    runTest(NumericLiteral(lexer.parseStr('23.06730')), '23.06730');
    runTest(NumericLiteral(lexer.parseStr('239039203.4781')), '239039203.4781');
    runTest(NumericLiteral(lexer.parseStr('.0562372')), '.0562372');

    runTest(NumericLiteral(lexer.parseStr('0e+100')), '0e+100');
    runTest(NumericLiteral(lexer.parseStr('23E+1')), '23E+1');
    runTest(NumericLiteral(lexer.parseStr('239039203e-343')), '239039203e-343');
    runTest(NumericLiteral(lexer.parseStr('0.00000E-09')), '0.00000E-09');
    runTest(NumericLiteral(lexer.parseStr('23.06730e+672')), '23.06730e+672');
    runTest(NumericLiteral(lexer.parseStr('.0562372E-0351')), '.0562372E-0351');

    runTest(NumericLiteral(lexer.parseStr('0xfff')), '0xfff');
    runTest(NumericLiteral(lexer.parseStr('0xFFF')), '0xFFF');
    runTest(NumericLiteral(lexer.parseStr('0x000')), '0x000');
    runTest(NumericLiteral(lexer.parseStr('0x000000')), '0x000000');
    runTest(NumericLiteral(lexer.parseStr('0x0f0f0f')), '0x0f0f0f');
    runTest(NumericLiteral(lexer.parseStr('0x0F0F0F')), '0x0F0F0F');
    runTest(NumericLiteral(lexer.parseStr('0xA0A0A0')), '0xA0A0A0');

  });

  test('Symbol Literal', () {
    //TODO: Check symbol literals

    // runTest(SymbolLiteral(lexer.parseStr('')), '');

  });

  test('List Literal', () {

    runTest(ListLiteral(lexer.parseStr('[]')), '[]');
    runTest(ListLiteral(lexer.parseStr('[234]')), '[234]');
    runTest(ListLiteral(lexer.parseStr('[234.432]')), '[234.432]');
    runTest(ListLiteral(lexer.parseStr('[1,2,3,4]')), '[1, 2, 3, 4]');
    runTest(ListLiteral(lexer.parseStr('["someString"]')), '["someString"]');
    runTest(ListLiteral(lexer.parseStr('["1","2","3","4"]')), '["1", "2", "3", "4"]');
    runTest(ListLiteral(lexer.parseStr('[someObject]')), '[someObject]');
    runTest(ListLiteral(lexer.parseStr('[someObject,someObject,someObject]')), '[someObject, someObject, someObject]');
    runTest(ListLiteral(lexer.parseStr('const []')), 'const []');
    runTest(ListLiteral(lexer.parseStr('const [234]')), 'const [234]');
    runTest(ListLiteral(lexer.parseStr('const [234.432]')), 'const [234.432]');
    runTest(ListLiteral(lexer.parseStr('const [1,2,3,4]')), 'const [1, 2, 3, 4]');
    runTest(ListLiteral(lexer.parseStr('const ["someString"]')), 'const ["someString"]');
    runTest(ListLiteral(lexer.parseStr('const ["1","2","3","4"]')), 'const ["1", "2", "3", "4"]');
    runTest(ListLiteral(lexer.parseStr('const [someObject]')), 'const [someObject]');
    runTest(ListLiteral(lexer.parseStr('const [someObject,someObject,someObject]')), 'const [someObject, someObject, someObject]');

    runTest(ListLiteral(lexer.parseStr('[234,]')), '[234,]');
    runTest(ListLiteral(lexer.parseStr('[234.432,]')), '[234.432,]');
    runTest(ListLiteral(lexer.parseStr('[1,2,3,4,]')), '[1, 2, 3, 4,]');
    runTest(ListLiteral(lexer.parseStr('["someString",]')), '["someString",]');
    runTest(ListLiteral(lexer.parseStr('["1","2","3","4",]')), '["1", "2", "3", "4",]');
    runTest(ListLiteral(lexer.parseStr('[someObject,]')), '[someObject,]');
    runTest(ListLiteral(lexer.parseStr('[someObject,someObject,someObject,]')), '[someObject, someObject, someObject,]');
    runTest(ListLiteral(lexer.parseStr('const [234,]')), 'const [234,]');
    runTest(ListLiteral(lexer.parseStr('const [234.432,]')), 'const [234.432,]');
    runTest(ListLiteral(lexer.parseStr('const [1,2,3,4,]')), 'const [1, 2, 3, 4,]');
    runTest(ListLiteral(lexer.parseStr('const ["someString",]')), 'const ["someString",]');
    runTest(ListLiteral(lexer.parseStr('const ["1","2","3","4",]')), 'const ["1", "2", "3", "4",]');
    runTest(ListLiteral(lexer.parseStr('const [someObject,]')), 'const [someObject,]');
    runTest(ListLiteral(lexer.parseStr('const [someObject,someObject,someObject,]')), 'const [someObject, someObject, someObject,]');

    runTest(ListLiteral(lexer.parseStr('<int, num>[]')), '<int, num>[]');
    runTest(ListLiteral(lexer.parseStr('<someType, num>[234]')), '<someType, num>[234]');
    runTest(ListLiteral(lexer.parseStr('<someType, int, num>[234.432]')), '<someType, int, num>[234.432]');
    runTest(ListLiteral(lexer.parseStr('<someType, int, num>[1,2,3,4]')), '<someType, int, num>[1, 2, 3, 4]');
    runTest(ListLiteral(lexer.parseStr('<someType, int, num>["someString",]')), '<someType, int, num>["someString",]');
    runTest(ListLiteral(lexer.parseStr('<someType, int, num>["1","2","3","4",]')), '<someType, int, num>["1", "2", "3", "4",]');
    runTest(ListLiteral(lexer.parseStr('<someType, int, num>[someObject,]')), '<someType, int, num>[someObject,]');
    runTest(ListLiteral(lexer.parseStr('<someType, int, num>[someObject,someObject,someObject,]')), '<someType, int, num>[someObject, someObject, someObject,]');
    runTest(ListLiteral(lexer.parseStr('const <someType, int, num>[234,]')), 'const <someType, int, num>[234,]');
    runTest(ListLiteral(lexer.parseStr('const <someType, int, num>[234.432,]')), 'const <someType, int, num>[234.432,]');
    runTest(ListLiteral(lexer.parseStr('const <someType, int, num>[1,2,3,4,]')), 'const <someType, int, num>[1, 2, 3, 4,]');
    runTest(ListLiteral(lexer.parseStr('const <someType, int, num>["someString"]')), 'const <someType, int, num>["someString"]');
    runTest(ListLiteral(lexer.parseStr('const <someType, int, num>["1","2","3","4"]')), 'const <someType, int, num>["1", "2", "3", "4"]');
    runTest(ListLiteral(lexer.parseStr('const <someType, int, num>[someObject]')), 'const <someType, int, num>[someObject]');
    runTest(ListLiteral(lexer.parseStr('const <someType, int, num>[someObject,someObject,someObject]')), 'const <someType, int, num>[someObject, someObject, someObject]');

  });

  test('Set Or Map Literal', () {

    runTest(SetOrMapLiteral(lexer.parseStr('{}')), '{}');
    runTest(SetOrMapLiteral(lexer.parseStr('const {}')), 'const {}');
    runTest(SetOrMapLiteral(lexer.parseStr('<int, num>{}')), '<int, num>{}');

  });

  test('Map Literal Entry', () {

    runTest(MapLiteralEntry(lexer.parseStr('int:5')), 'int:5');
    runTest(MapLiteralEntry(lexer.parseStr('bool:false')), 'bool:false');
    runTest(MapLiteralEntry(lexer.parseStr('String:"someString"')), 'String:"someString"');
    runTest(MapLiteralEntry(lexer.parseStr('someType: someObject()')), 'someType:someObject()');
    runTest(MapLiteralEntry(lexer.parseStr('bool:(test ? false:true)')), 'bool:(test ? false : true)');

  });

  test('Map Literal', () {

    runTest(MapLiteral(lexer.parseStr('{int:5}')), '{int:5}');
    runTest(MapLiteral(lexer.parseStr('{bool:false}')), '{bool:false}');
    runTest(MapLiteral(lexer.parseStr('{String:"someString"}')), '{String:"someString"}');
    runTest(MapLiteral(lexer.parseStr('{someType: someObject()}')), '{someType:someObject()}');
    runTest(MapLiteral(lexer.parseStr('{bool:(test ? false:true)}')), '{bool:(test ? false : true)}');
    runTest(MapLiteral(lexer.parseStr('{int:5,int:12,int:someInt}')), '{int:5, int:12, int:someInt}');
    runTest(MapLiteral(lexer.parseStr('{bool:false,bool:someBool,bool:true}')), '{bool:false, bool:someBool, bool:true}');
    runTest(MapLiteral(lexer.parseStr('{String:"someString", key:val, another:"str"}')), '{String:"someString", key:val, another:"str"}');
    runTest(MapLiteral(lexer.parseStr('{someType: someObject(), k:v, "strKey":(false ? true : false)}')), '{someType:someObject(), k:v, "strKey":(false ? true : false)}');
    runTest(MapLiteral(lexer.parseStr('const {int:5}')), 'const {int:5}');
    runTest(MapLiteral(lexer.parseStr('const {bool:false}')), 'const {bool:false}');
    runTest(MapLiteral(lexer.parseStr('const {String:"someString"}')), 'const {String:"someString"}');
    runTest(MapLiteral(lexer.parseStr('const {someType: someObject()}')), 'const {someType:someObject()}');
    runTest(MapLiteral(lexer.parseStr('const {bool:(test ? false:true)}')), 'const {bool:(test ? false : true)}');
    runTest(MapLiteral(lexer.parseStr('const {int:5,int:12,int:someInt}')), 'const {int:5, int:12, int:someInt}');
    runTest(MapLiteral(lexer.parseStr('const {bool:false,bool:someBool,bool:true}')), 'const {bool:false, bool:someBool, bool:true}');
    runTest(MapLiteral(lexer.parseStr('const {String:"someString", key:val, another:"str"}')), 'const {String:"someString", key:val, another:"str"}');
    runTest(MapLiteral(lexer.parseStr('const {someType: someObject(), k:v, "strKey":(false ? true : false)}')), 'const {someType:someObject(), k:v, "strKey":(false ? true : false)}');

    runTest(MapLiteral(lexer.parseStr('{int:5,}')), '{int:5,}');
    runTest(MapLiteral(lexer.parseStr('{bool:false,}')), '{bool:false,}');
    runTest(MapLiteral(lexer.parseStr('{String:"someString",}')), '{String:"someString",}');
    runTest(MapLiteral(lexer.parseStr('{someType: someObject(),}')), '{someType:someObject(),}');
    runTest(MapLiteral(lexer.parseStr('{bool:(test ? false:true),}')), '{bool:(test ? false : true),}');
    runTest(MapLiteral(lexer.parseStr('{int:5,int:12,int:someInt,}')), '{int:5, int:12, int:someInt,}');
    runTest(MapLiteral(lexer.parseStr('{bool:false,bool:someBool,bool:true,}')), '{bool:false, bool:someBool, bool:true,}');
    runTest(MapLiteral(lexer.parseStr('{String:"someString", key:val, another:"str",}')), '{String:"someString", key:val, another:"str",}');
    runTest(MapLiteral(lexer.parseStr('{someType: someObject(), k:v, "strKey":(false ? true : false),}')), '{someType:someObject(), k:v, "strKey":(false ? true : false),}');
    runTest(MapLiteral(lexer.parseStr('const {int:5,}')), 'const {int:5,}');
    runTest(MapLiteral(lexer.parseStr('const {bool:false,}')), 'const {bool:false,}');
    runTest(MapLiteral(lexer.parseStr('const {String:"someString",}')), 'const {String:"someString",}');
    runTest(MapLiteral(lexer.parseStr('const {someType: someObject(),}')), 'const {someType:someObject(),}');
    runTest(MapLiteral(lexer.parseStr('const {bool:(test ? false:true),}')), 'const {bool:(test ? false : true),}');
    runTest(MapLiteral(lexer.parseStr('const {int:5,int:12,int:someInt,}')), 'const {int:5, int:12, int:someInt,}');
    runTest(MapLiteral(lexer.parseStr('const {bool:false,bool:someBool,bool:true,}')), 'const {bool:false, bool:someBool, bool:true,}');
    runTest(MapLiteral(lexer.parseStr('const {String:"someString", key:val, another:"str",}')), 'const {String:"someString", key:val, another:"str",}');
    runTest(MapLiteral(lexer.parseStr('const {someType: someObject(), k:v, "strKey":(false ? true : false),}')), 'const {someType:someObject(), k:v, "strKey":(false ? true : false),}');

  });

  test('Set Literal', () {

    runTest(SetLiteral(lexer.parseStr('{234}')), '{234}');
    runTest(SetLiteral(lexer.parseStr('{234.432}')), '{234.432}');
    runTest(SetLiteral(lexer.parseStr('{1,2,3,4}')), '{1, 2, 3, 4}');
    runTest(SetLiteral(lexer.parseStr('{"someString"}')), '{"someString"}');
    runTest(SetLiteral(lexer.parseStr('{"1","2","3","4"}')), '{"1", "2", "3", "4"}');
    runTest(SetLiteral(lexer.parseStr('{someObject}')), '{someObject}');
    runTest(SetLiteral(lexer.parseStr('{someObject,someObject,someObject}')), '{someObject, someObject, someObject}');
    runTest(SetLiteral(lexer.parseStr('const {234}')), 'const {234}');
    runTest(SetLiteral(lexer.parseStr('const {234.432}')), 'const {234.432}');
    runTest(SetLiteral(lexer.parseStr('const {1,2,3,4}')), 'const {1, 2, 3, 4}');
    runTest(SetLiteral(lexer.parseStr('const {"someString"}')), 'const {"someString"}');
    runTest(SetLiteral(lexer.parseStr('const {"1","2","3","4"}')), 'const {"1", "2", "3", "4"}');
    runTest(SetLiteral(lexer.parseStr('const {someObject}')), 'const {someObject}');
    runTest(SetLiteral(lexer.parseStr('const {someObject,someObject,someObject}')), 'const {someObject, someObject, someObject}');

    runTest(SetLiteral(lexer.parseStr('{234,}')), '{234,}');
    runTest(SetLiteral(lexer.parseStr('{234.432,}')), '{234.432,}');
    runTest(SetLiteral(lexer.parseStr('{1,2,3,4,}')), '{1, 2, 3, 4,}');
    runTest(SetLiteral(lexer.parseStr('{"someString",}')), '{"someString",}');
    runTest(SetLiteral(lexer.parseStr('{"1","2","3","4",}')), '{"1", "2", "3", "4",}');
    runTest(SetLiteral(lexer.parseStr('{someObject,}')), '{someObject,}');
    runTest(SetLiteral(lexer.parseStr('{someObject,someObject,someObject,}')), '{someObject, someObject, someObject,}');
    runTest(SetLiteral(lexer.parseStr('const {234,}')), 'const {234,}');
    runTest(SetLiteral(lexer.parseStr('const {234.432,}')), 'const {234.432,}');
    runTest(SetLiteral(lexer.parseStr('const {1,2,3,4,}')), 'const {1, 2, 3, 4,}');
    runTest(SetLiteral(lexer.parseStr('const {"someString",}')), 'const {"someString",}');
    runTest(SetLiteral(lexer.parseStr('const {"1","2","3","4",}')), 'const {"1", "2", "3", "4",}');
    runTest(SetLiteral(lexer.parseStr('const {someObject,}')), 'const {someObject,}');
    runTest(SetLiteral(lexer.parseStr('const {someObject,someObject,someObject,}')), 'const {someObject, someObject, someObject,}');

    runTest(SetLiteral(lexer.parseStr('<someType, num>{234}')), '<someType, num>{234}');
    runTest(SetLiteral(lexer.parseStr('<someType, int, num>{234.432}')), '<someType, int, num>{234.432}');
    runTest(SetLiteral(lexer.parseStr('<someType, int, num>{1,2,3,4}')), '<someType, int, num>{1, 2, 3, 4}');
    runTest(SetLiteral(lexer.parseStr('<someType, int, num>{"someString",}')), '<someType, int, num>{"someString",}');
    runTest(SetLiteral(lexer.parseStr('<someType, int, num>{"1","2","3","4",}')), '<someType, int, num>{"1", "2", "3", "4",}');
    runTest(SetLiteral(lexer.parseStr('<someType, int, num>{someObject,}')), '<someType, int, num>{someObject,}');
    runTest(SetLiteral(lexer.parseStr('<someType, int, num>{someObject,someObject,someObject,}')), '<someType, int, num>{someObject, someObject, someObject,}');
    runTest(SetLiteral(lexer.parseStr('const <someType, int, num>{234,}')), 'const <someType, int, num>{234,}');
    runTest(SetLiteral(lexer.parseStr('const <someType, int, num>{234.432,}')), 'const <someType, int, num>{234.432,}');
    runTest(SetLiteral(lexer.parseStr('const <someType, int, num>{1,2,3,4,}')), 'const <someType, int, num>{1, 2, 3, 4,}');
    runTest(SetLiteral(lexer.parseStr('const <someType, int, num>{"someString"}')), 'const <someType, int, num>{"someString"}');
    runTest(SetLiteral(lexer.parseStr('const <someType, int, num>{"1","2","3","4"}')), 'const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(SetLiteral(lexer.parseStr('const <someType, int, num>{someObject}')), 'const <someType, int, num>{someObject}');
    runTest(SetLiteral(lexer.parseStr('const <someType, int, num>{someObject,someObject,someObject}')), 'const <someType, int, num>{someObject, someObject, someObject}');

  });

  test('Literal', () {

    runTest(Literal(lexer.parseStr('null')), 'null');

    runTest(Literal(lexer.parseStr('true')), 'true');
    runTest(Literal(lexer.parseStr('false')), 'false');

    // TODO: Add complete string (multi line and other escape characters) parsing (this is done in the lexer)
    runTest(Literal(lexer.parseStr('"Hello World!"')), '"Hello World!"');
    runTest(Literal(lexer.parseStr('"\'Testing escape character\'"')), '"\'Testing escape character\'"');
    runTest(Literal(lexer.parseStr("'Hello World!'")), "'Hello World!'");
    runTest(Literal(lexer.parseStr("'\"Testing escape character\"'")), "'\"Testing escape character\"'");

    runTest(Literal(lexer.parseStr('0')), '0');
    runTest(Literal(lexer.parseStr('23')), '23');
    runTest(Literal(lexer.parseStr('239039203')), '239039203');
    runTest(Literal(lexer.parseStr('0.00000')), '0.00000');
    runTest(Literal(lexer.parseStr('23.06730')), '23.06730');
    runTest(Literal(lexer.parseStr('239039203.4781')), '239039203.4781');
    runTest(Literal(lexer.parseStr('.0562372')), '.0562372');
    runTest(Literal(lexer.parseStr('0e+100')), '0e+100');
    runTest(Literal(lexer.parseStr('23E+1')), '23E+1');
    runTest(Literal(lexer.parseStr('239039203e-343')), '239039203e-343');
    runTest(Literal(lexer.parseStr('0.00000E-09')), '0.00000E-09');
    runTest(Literal(lexer.parseStr('23.06730e+672')), '23.06730e+672');
    runTest(Literal(lexer.parseStr('.0562372E-0351')), '.0562372E-0351');
    runTest(Literal(lexer.parseStr('0xfff')), '0xfff');
    runTest(Literal(lexer.parseStr('0xFFF')), '0xFFF');
    runTest(Literal(lexer.parseStr('0x000')), '0x000');
    runTest(Literal(lexer.parseStr('0x000000')), '0x000000');
    runTest(Literal(lexer.parseStr('0x0f0f0f')), '0x0f0f0f');
    runTest(Literal(lexer.parseStr('0x0F0F0F')), '0x0F0F0F');
    runTest(Literal(lexer.parseStr('0xA0A0A0')), '0xA0A0A0');

    //TODO: Check symbol literals

    runTest(Literal(lexer.parseStr('[]')), '[]');
    runTest(Literal(lexer.parseStr('[234]')), '[234]');
    runTest(Literal(lexer.parseStr('[234.432]')), '[234.432]');
    runTest(Literal(lexer.parseStr('[1,2,3,4]')), '[1, 2, 3, 4]');
    runTest(Literal(lexer.parseStr('["someString"]')), '["someString"]');
    runTest(Literal(lexer.parseStr('["1","2","3","4"]')), '["1", "2", "3", "4"]');
    runTest(Literal(lexer.parseStr('[someObject]')), '[someObject]');
    runTest(Literal(lexer.parseStr('[someObject,someObject,someObject]')), '[someObject, someObject, someObject]');
    runTest(Literal(lexer.parseStr('const []')), 'const []');
    runTest(Literal(lexer.parseStr('const [234]')), 'const [234]');
    runTest(Literal(lexer.parseStr('const [234.432]')), 'const [234.432]');
    runTest(Literal(lexer.parseStr('const [1,2,3,4]')), 'const [1, 2, 3, 4]');
    runTest(Literal(lexer.parseStr('const ["someString"]')), 'const ["someString"]');
    runTest(Literal(lexer.parseStr('const ["1","2","3","4"]')), 'const ["1", "2", "3", "4"]');
    runTest(Literal(lexer.parseStr('const [someObject]')), 'const [someObject]');
    runTest(Literal(lexer.parseStr('const [someObject,someObject,someObject]')), 'const [someObject, someObject, someObject]');
    runTest(Literal(lexer.parseStr('[234,]')), '[234,]');
    runTest(Literal(lexer.parseStr('[234.432,]')), '[234.432,]');
    runTest(Literal(lexer.parseStr('[1,2,3,4,]')), '[1, 2, 3, 4,]');
    runTest(Literal(lexer.parseStr('["someString",]')), '["someString",]');
    runTest(Literal(lexer.parseStr('["1","2","3","4",]')), '["1", "2", "3", "4",]');
    runTest(Literal(lexer.parseStr('[someObject,]')), '[someObject,]');
    runTest(Literal(lexer.parseStr('[someObject,someObject,someObject,]')), '[someObject, someObject, someObject,]');
    runTest(Literal(lexer.parseStr('const [234,]')), 'const [234,]');
    runTest(Literal(lexer.parseStr('const [234.432,]')), 'const [234.432,]');
    runTest(Literal(lexer.parseStr('const [1,2,3,4,]')), 'const [1, 2, 3, 4,]');
    runTest(Literal(lexer.parseStr('const ["someString",]')), 'const ["someString",]');
    runTest(Literal(lexer.parseStr('const ["1","2","3","4",]')), 'const ["1", "2", "3", "4",]');
    runTest(Literal(lexer.parseStr('const [someObject,]')), 'const [someObject,]');
    runTest(Literal(lexer.parseStr('const [someObject,someObject,someObject,]')), 'const [someObject, someObject, someObject,]');
    runTest(Literal(lexer.parseStr('<int, num>[]')), '<int, num>[]');
    runTest(Literal(lexer.parseStr('<someType, num>[234]')), '<someType, num>[234]');
    runTest(Literal(lexer.parseStr('<someType, int, num>[234.432]')), '<someType, int, num>[234.432]');
    runTest(Literal(lexer.parseStr('<someType, int, num>[1,2,3,4]')), '<someType, int, num>[1, 2, 3, 4]');
    runTest(Literal(lexer.parseStr('<someType, int, num>["someString",]')), '<someType, int, num>["someString",]');
    runTest(Literal(lexer.parseStr('<someType, int, num>["1","2","3","4",]')), '<someType, int, num>["1", "2", "3", "4",]');
    runTest(Literal(lexer.parseStr('<someType, int, num>[someObject,]')), '<someType, int, num>[someObject,]');
    runTest(Literal(lexer.parseStr('<someType, int, num>[someObject,someObject,someObject,]')), '<someType, int, num>[someObject, someObject, someObject,]');
    runTest(Literal(lexer.parseStr('const <someType, int, num>[234,]')), 'const <someType, int, num>[234,]');
    runTest(Literal(lexer.parseStr('const <someType, int, num>[234.432,]')), 'const <someType, int, num>[234.432,]');
    runTest(Literal(lexer.parseStr('const <someType, int, num>[1,2,3,4,]')), 'const <someType, int, num>[1, 2, 3, 4,]');
    runTest(Literal(lexer.parseStr('const <someType, int, num>["someString"]')), 'const <someType, int, num>["someString"]');
    runTest(Literal(lexer.parseStr('const <someType, int, num>["1","2","3","4"]')), 'const <someType, int, num>["1", "2", "3", "4"]');
    runTest(Literal(lexer.parseStr('const <someType, int, num>[someObject]')), 'const <someType, int, num>[someObject]');
    runTest(Literal(lexer.parseStr('const <someType, int, num>[someObject,someObject,someObject]')), 'const <someType, int, num>[someObject, someObject, someObject]');

    runTest(Literal(lexer.parseStr('{}')), '{}');
    runTest(Literal(lexer.parseStr('const {}')), 'const {}');
    runTest(Literal(lexer.parseStr('<int, num>{}')), '<int, num>{}');

    runTest(Literal(lexer.parseStr('{int:5}')), '{int:5}');
    runTest(Literal(lexer.parseStr('{bool:false}')), '{bool:false}');
    runTest(Literal(lexer.parseStr('{String:"someString"}')), '{String:"someString"}');
    runTest(Literal(lexer.parseStr('{someType: someObject()}')), '{someType:someObject()}');
    runTest(Literal(lexer.parseStr('{bool:(test ? false:true)}')), '{bool:(test ? false : true)}');
    runTest(Literal(lexer.parseStr('{int:5,int:12,int:someInt}')), '{int:5, int:12, int:someInt}');
    runTest(Literal(lexer.parseStr('{bool:false,bool:someBool,bool:true}')), '{bool:false, bool:someBool, bool:true}');
    runTest(Literal(lexer.parseStr('{String:"someString", key:val, another:"str"}')), '{String:"someString", key:val, another:"str"}');
    runTest(Literal(lexer.parseStr('{someType: someObject(), k:v, "strKey":(false ? true : false)}')), '{someType:someObject(), k:v, "strKey":(false ? true : false)}');
    runTest(Literal(lexer.parseStr('const {int:5}')), 'const {int:5}');
    runTest(Literal(lexer.parseStr('const {bool:false}')), 'const {bool:false}');
    runTest(Literal(lexer.parseStr('const {String:"someString"}')), 'const {String:"someString"}');
    runTest(Literal(lexer.parseStr('const {someType: someObject()}')), 'const {someType:someObject()}');
    runTest(Literal(lexer.parseStr('const {bool:(test ? false:true)}')), 'const {bool:(test ? false : true)}');
    runTest(Literal(lexer.parseStr('const {int:5,int:12,int:someInt}')), 'const {int:5, int:12, int:someInt}');
    runTest(Literal(lexer.parseStr('const {bool:false,bool:someBool,bool:true}')), 'const {bool:false, bool:someBool, bool:true}');
    runTest(Literal(lexer.parseStr('const {String:"someString", key:val, another:"str"}')), 'const {String:"someString", key:val, another:"str"}');
    runTest(Literal(lexer.parseStr('const {someType: someObject(), k:v, "strKey":(false ? true : false)}')), 'const {someType:someObject(), k:v, "strKey":(false ? true : false)}');
    runTest(Literal(lexer.parseStr('{int:5,}')), '{int:5,}');
    runTest(Literal(lexer.parseStr('{bool:false,}')), '{bool:false,}');
    runTest(Literal(lexer.parseStr('{String:"someString",}')), '{String:"someString",}');
    runTest(Literal(lexer.parseStr('{someType: someObject(),}')), '{someType:someObject(),}');
    runTest(Literal(lexer.parseStr('{bool:(test ? false:true),}')), '{bool:(test ? false : true),}');
    runTest(Literal(lexer.parseStr('{int:5,int:12,int:someInt,}')), '{int:5, int:12, int:someInt,}');
    runTest(Literal(lexer.parseStr('{bool:false,bool:someBool,bool:true,}')), '{bool:false, bool:someBool, bool:true,}');
    runTest(Literal(lexer.parseStr('{String:"someString", key:val, another:"str",}')), '{String:"someString", key:val, another:"str",}');
    runTest(Literal(lexer.parseStr('{someType: someObject(), k:v, "strKey":(false ? true : false),}')), '{someType:someObject(), k:v, "strKey":(false ? true : false),}');
    runTest(Literal(lexer.parseStr('const {int:5,}')), 'const {int:5,}');
    runTest(Literal(lexer.parseStr('const {bool:false,}')), 'const {bool:false,}');
    runTest(Literal(lexer.parseStr('const {String:"someString",}')), 'const {String:"someString",}');
    runTest(Literal(lexer.parseStr('const {someType: someObject(),}')), 'const {someType:someObject(),}');
    runTest(Literal(lexer.parseStr('const {bool:(test ? false:true),}')), 'const {bool:(test ? false : true),}');
    runTest(Literal(lexer.parseStr('const {int:5,int:12,int:someInt,}')), 'const {int:5, int:12, int:someInt,}');
    runTest(Literal(lexer.parseStr('const {bool:false,bool:someBool,bool:true,}')), 'const {bool:false, bool:someBool, bool:true,}');
    runTest(Literal(lexer.parseStr('const {String:"someString", key:val, another:"str",}')), 'const {String:"someString", key:val, another:"str",}');
    runTest(Literal(lexer.parseStr('const {someType: someObject(), k:v, "strKey":(false ? true : false),}')), 'const {someType:someObject(), k:v, "strKey":(false ? true : false),}');

    runTest(Literal(lexer.parseStr('{234}')), '{234}');
    runTest(Literal(lexer.parseStr('{234.432}')), '{234.432}');
    runTest(Literal(lexer.parseStr('{1,2,3,4}')), '{1, 2, 3, 4}');
    runTest(Literal(lexer.parseStr('{"someString"}')), '{"someString"}');
    runTest(Literal(lexer.parseStr('{"1","2","3","4"}')), '{"1", "2", "3", "4"}');
    runTest(Literal(lexer.parseStr('{someObject}')), '{someObject}');
    runTest(Literal(lexer.parseStr('{someObject,someObject,someObject}')), '{someObject, someObject, someObject}');
    runTest(Literal(lexer.parseStr('const {234}')), 'const {234}');
    runTest(Literal(lexer.parseStr('const {234.432}')), 'const {234.432}');
    runTest(Literal(lexer.parseStr('const {1,2,3,4}')), 'const {1, 2, 3, 4}');
    runTest(Literal(lexer.parseStr('const {"someString"}')), 'const {"someString"}');
    runTest(Literal(lexer.parseStr('const {"1","2","3","4"}')), 'const {"1", "2", "3", "4"}');
    runTest(Literal(lexer.parseStr('const {someObject}')), 'const {someObject}');
    runTest(Literal(lexer.parseStr('const {someObject,someObject,someObject}')), 'const {someObject, someObject, someObject}');
    runTest(Literal(lexer.parseStr('{234,}')), '{234,}');
    runTest(Literal(lexer.parseStr('{234.432,}')), '{234.432,}');
    runTest(Literal(lexer.parseStr('{1,2,3,4,}')), '{1, 2, 3, 4,}');
    runTest(Literal(lexer.parseStr('{"someString",}')), '{"someString",}');
    runTest(Literal(lexer.parseStr('{"1","2","3","4",}')), '{"1", "2", "3", "4",}');
    runTest(Literal(lexer.parseStr('{someObject,}')), '{someObject,}');
    runTest(Literal(lexer.parseStr('{someObject,someObject,someObject,}')), '{someObject, someObject, someObject,}');
    runTest(Literal(lexer.parseStr('const {234,}')), 'const {234,}');
    runTest(Literal(lexer.parseStr('const {234.432,}')), 'const {234.432,}');
    runTest(Literal(lexer.parseStr('const {1,2,3,4,}')), 'const {1, 2, 3, 4,}');
    runTest(Literal(lexer.parseStr('const {"someString",}')), 'const {"someString",}');
    runTest(Literal(lexer.parseStr('const {"1","2","3","4",}')), 'const {"1", "2", "3", "4",}');
    runTest(Literal(lexer.parseStr('const {someObject,}')), 'const {someObject,}');
    runTest(Literal(lexer.parseStr('const {someObject,someObject,someObject,}')), 'const {someObject, someObject, someObject,}');
    runTest(Literal(lexer.parseStr('<someType, num>{234}')), '<someType, num>{234}');
    runTest(Literal(lexer.parseStr('<someType, int, num>{234.432}')), '<someType, int, num>{234.432}');
    runTest(Literal(lexer.parseStr('<someType, int, num>{1,2,3,4}')), '<someType, int, num>{1, 2, 3, 4}');
    runTest(Literal(lexer.parseStr('<someType, int, num>{"someString",}')), '<someType, int, num>{"someString",}');
    runTest(Literal(lexer.parseStr('<someType, int, num>{"1","2","3","4",}')), '<someType, int, num>{"1", "2", "3", "4",}');
    runTest(Literal(lexer.parseStr('<someType, int, num>{someObject,}')), '<someType, int, num>{someObject,}');
    runTest(Literal(lexer.parseStr('<someType, int, num>{someObject,someObject,someObject,}')), '<someType, int, num>{someObject, someObject, someObject,}');
    runTest(Literal(lexer.parseStr('const <someType, int, num>{234,}')), 'const <someType, int, num>{234,}');
    runTest(Literal(lexer.parseStr('const <someType, int, num>{234.432,}')), 'const <someType, int, num>{234.432,}');
    runTest(Literal(lexer.parseStr('const <someType, int, num>{1,2,3,4,}')), 'const <someType, int, num>{1, 2, 3, 4,}');
    runTest(Literal(lexer.parseStr('const <someType, int, num>{"someString"}')), 'const <someType, int, num>{"someString"}');
    runTest(Literal(lexer.parseStr('const <someType, int, num>{"1","2","3","4"}')), 'const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(Literal(lexer.parseStr('const <someType, int, num>{someObject}')), 'const <someType, int, num>{someObject}');
    runTest(Literal(lexer.parseStr('const <someType, int, num>{someObject,someObject,someObject}')), 'const <someType, int, num>{someObject, someObject, someObject}');

  });

}
