import 'package:test/test.dart';
import 'test_lib.dart';

import 'package:source_parser/src/lexer/Lexer.dart';

import 'package:source_parser/src/core/Primary.dart';

void main() {

  Lexer lexer = Lexer();

  test('Primary', () {

    // This Expression
    runTest(Primary(lexer.parseStr('this')), 'this');

    // Unconditional Assignable Selector
    runTest(Primary(lexer.parseStr('super .someId')), 'super .someId');
    runTest(Primary(lexer.parseStr('super ._someId')), 'super ._someId');
    runTest(Primary(lexer.parseStr('super [someId]')), 'super [someId]');
    runTest(Primary(lexer.parseStr('super [(someId)]')), 'super [(someId)]');
    runTest(Primary(lexer.parseStr('super [3+2]')), 'super [3 + 2]');
    runTest(Primary(lexer.parseStr('super [4??6]')), 'super [4 ?? 6]');
    runTest(Primary(lexer.parseStr('super [1??7]')), 'super [1 ?? 7]');
    runTest(Primary(lexer.parseStr('super [9??someInt||8]')), 'super [9 ?? someInt || 8]');
    runTest(Primary(lexer.parseStr('super [3|6%someInt??2+0||8]')), 'super [3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(Primary(lexer.parseStr('super [1+1>=someInt??7&0*32]')), 'super [1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(Primary(lexer.parseStr('super [2~/9%5??8-someInt|2]')), 'super [2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(Primary(lexer.parseStr('super [true?false:true]')), 'super [true ? false : true]');
    runTest(Primary(lexer.parseStr('super [someVar == another?false ? true : false:true]')), 'super [someVar == another ? false ? true : false : true]');

    // TODO: Add Function Expression tests

    // Literals
    // TODO: Add complete string (multi line and other escape characters) parsing (this is done in the lexer)
    // TODO: Check symbol literals
    runTest(Primary(lexer.parseStr('null')), 'null');
    runTest(Primary(lexer.parseStr('true')), 'true');
    runTest(Primary(lexer.parseStr('false')), 'false');
    runTest(Primary(lexer.parseStr('"Hello World!"')), '"Hello World!"');
    runTest(Primary(lexer.parseStr('"\'Testing escape character\'"')), '"\'Testing escape character\'"');
    runTest(Primary(lexer.parseStr("'Hello World!'")), "'Hello World!'");
    runTest(Primary(lexer.parseStr("'\"Testing escape character\"'")), "'\"Testing escape character\"'");
    runTest(Primary(lexer.parseStr('0')), '0');
    runTest(Primary(lexer.parseStr('23')), '23');
    runTest(Primary(lexer.parseStr('239039203')), '239039203');
    runTest(Primary(lexer.parseStr('0.00000')), '0.00000');
    runTest(Primary(lexer.parseStr('23.06730')), '23.06730');
    runTest(Primary(lexer.parseStr('239039203.4781')), '239039203.4781');
    runTest(Primary(lexer.parseStr('.0562372')), '.0562372');
    runTest(Primary(lexer.parseStr('0e+100')), '0e+100');
    runTest(Primary(lexer.parseStr('23E+1')), '23E+1');
    runTest(Primary(lexer.parseStr('239039203e-343')), '239039203e-343');
    runTest(Primary(lexer.parseStr('0.00000E-09')), '0.00000E-09');
    runTest(Primary(lexer.parseStr('23.06730e+672')), '23.06730e+672');
    runTest(Primary(lexer.parseStr('.0562372E-0351')), '.0562372E-0351');
    runTest(Primary(lexer.parseStr('0xfff')), '0xfff');
    runTest(Primary(lexer.parseStr('0xFFF')), '0xFFF');
    runTest(Primary(lexer.parseStr('0x000')), '0x000');
    runTest(Primary(lexer.parseStr('0x000000')), '0x000000');
    runTest(Primary(lexer.parseStr('0x0f0f0f')), '0x0f0f0f');
    runTest(Primary(lexer.parseStr('0x0F0F0F')), '0x0F0F0F');
    runTest(Primary(lexer.parseStr('0xA0A0A0')), '0xA0A0A0');
    runTest(Primary(lexer.parseStr('[]')), '[]');
    runTest(Primary(lexer.parseStr('[234]')), '[234]');
    runTest(Primary(lexer.parseStr('[234.432]')), '[234.432]');
    runTest(Primary(lexer.parseStr('[1,2,3,4]')), '[1, 2, 3, 4]');
    runTest(Primary(lexer.parseStr('["someString"]')), '["someString"]');
    runTest(Primary(lexer.parseStr('["1","2","3","4"]')), '["1", "2", "3", "4"]');
    runTest(Primary(lexer.parseStr('[someObject]')), '[someObject]');
    runTest(Primary(lexer.parseStr('[someObject,someObject,someObject]')), '[someObject, someObject, someObject]');
    runTest(Primary(lexer.parseStr('const []')), 'const []');
    runTest(Primary(lexer.parseStr('const [234]')), 'const [234]');
    runTest(Primary(lexer.parseStr('const [234.432]')), 'const [234.432]');
    runTest(Primary(lexer.parseStr('const [1,2,3,4]')), 'const [1, 2, 3, 4]');
    runTest(Primary(lexer.parseStr('const ["someString"]')), 'const ["someString"]');
    runTest(Primary(lexer.parseStr('const ["1","2","3","4"]')), 'const ["1", "2", "3", "4"]');
    runTest(Primary(lexer.parseStr('const [someObject]')), 'const [someObject]');
    runTest(Primary(lexer.parseStr('const [someObject,someObject,someObject]')), 'const [someObject, someObject, someObject]');
    runTest(Primary(lexer.parseStr('[234,]')), '[234,]');
    runTest(Primary(lexer.parseStr('[234.432,]')), '[234.432,]');
    runTest(Primary(lexer.parseStr('[1,2,3,4,]')), '[1, 2, 3, 4,]');
    runTest(Primary(lexer.parseStr('["someString",]')), '["someString",]');
    runTest(Primary(lexer.parseStr('["1","2","3","4",]')), '["1", "2", "3", "4",]');
    runTest(Primary(lexer.parseStr('[someObject,]')), '[someObject,]');
    runTest(Primary(lexer.parseStr('[someObject,someObject,someObject,]')), '[someObject, someObject, someObject,]');
    runTest(Primary(lexer.parseStr('const [234,]')), 'const [234,]');
    runTest(Primary(lexer.parseStr('const [234.432,]')), 'const [234.432,]');
    runTest(Primary(lexer.parseStr('const [1,2,3,4,]')), 'const [1, 2, 3, 4,]');
    runTest(Primary(lexer.parseStr('const ["someString",]')), 'const ["someString",]');
    runTest(Primary(lexer.parseStr('const ["1","2","3","4",]')), 'const ["1", "2", "3", "4",]');
    runTest(Primary(lexer.parseStr('const [someObject,]')), 'const [someObject,]');
    runTest(Primary(lexer.parseStr('const [someObject,someObject,someObject,]')), 'const [someObject, someObject, someObject,]');
    runTest(Primary(lexer.parseStr('<int, num>[]')), '<int, num>[]');
    runTest(Primary(lexer.parseStr('<someType, num>[234]')), '<someType, num>[234]');
    runTest(Primary(lexer.parseStr('<someType, int, num>[234.432]')), '<someType, int, num>[234.432]');
    runTest(Primary(lexer.parseStr('<someType, int, num>[1,2,3,4]')), '<someType, int, num>[1, 2, 3, 4]');
    runTest(Primary(lexer.parseStr('<someType, int, num>["someString",]')), '<someType, int, num>["someString",]');
    runTest(Primary(lexer.parseStr('<someType, int, num>["1","2","3","4",]')), '<someType, int, num>["1", "2", "3", "4",]');
    runTest(Primary(lexer.parseStr('<someType, int, num>[someObject,]')), '<someType, int, num>[someObject,]');
    runTest(Primary(lexer.parseStr('<someType, int, num>[someObject,someObject,someObject,]')), '<someType, int, num>[someObject, someObject, someObject,]');
    runTest(Primary(lexer.parseStr('const <someType, int, num>[234,]')), 'const <someType, int, num>[234,]');
    runTest(Primary(lexer.parseStr('const <someType, int, num>[234.432,]')), 'const <someType, int, num>[234.432,]');
    runTest(Primary(lexer.parseStr('const <someType, int, num>[1,2,3,4,]')), 'const <someType, int, num>[1, 2, 3, 4,]');
    runTest(Primary(lexer.parseStr('const <someType, int, num>["someString"]')), 'const <someType, int, num>["someString"]');
    runTest(Primary(lexer.parseStr('const <someType, int, num>["1","2","3","4"]')), 'const <someType, int, num>["1", "2", "3", "4"]');
    runTest(Primary(lexer.parseStr('const <someType, int, num>[someObject]')), 'const <someType, int, num>[someObject]');
    runTest(Primary(lexer.parseStr('const <someType, int, num>[someObject,someObject,someObject]')), 'const <someType, int, num>[someObject, someObject, someObject]');
    runTest(Primary(lexer.parseStr('{}')), '{}');
    runTest(Primary(lexer.parseStr('const {}')), 'const {}');
    runTest(Primary(lexer.parseStr('<int, num>{}')), '<int, num>{}');
    runTest(Primary(lexer.parseStr('{int:5}')), '{int:5}');
    runTest(Primary(lexer.parseStr('{bool:false}')), '{bool:false}');
    runTest(Primary(lexer.parseStr('{String:"someString"}')), '{String:"someString"}');
    runTest(Primary(lexer.parseStr('{someType: someObject()}')), '{someType:someObject()}');
    runTest(Primary(lexer.parseStr('{bool:(test ? false:true)}')), '{bool:(test ? false : true)}');
    runTest(Primary(lexer.parseStr('{int:5,int:12,int:someInt}')), '{int:5, int:12, int:someInt}');
    runTest(Primary(lexer.parseStr('{bool:false,bool:someBool,bool:true}')), '{bool:false, bool:someBool, bool:true}');
    runTest(Primary(lexer.parseStr('{String:"someString", key:val, another:"str"}')), '{String:"someString", key:val, another:"str"}');
    runTest(Primary(lexer.parseStr('{someType: someObject(), k:v, "strKey":(false ? true : false)}')), '{someType:someObject(), k:v, "strKey":(false ? true : false)}');
    runTest(Primary(lexer.parseStr('const {int:5}')), 'const {int:5}');
    runTest(Primary(lexer.parseStr('const {bool:false}')), 'const {bool:false}');
    runTest(Primary(lexer.parseStr('const {String:"someString"}')), 'const {String:"someString"}');
    runTest(Primary(lexer.parseStr('const {someType: someObject()}')), 'const {someType:someObject()}');
    runTest(Primary(lexer.parseStr('const {bool:(test ? false:true)}')), 'const {bool:(test ? false : true)}');
    runTest(Primary(lexer.parseStr('const {int:5,int:12,int:someInt}')), 'const {int:5, int:12, int:someInt}');
    runTest(Primary(lexer.parseStr('const {bool:false,bool:someBool,bool:true}')), 'const {bool:false, bool:someBool, bool:true}');
    runTest(Primary(lexer.parseStr('const {String:"someString", key:val, another:"str"}')), 'const {String:"someString", key:val, another:"str"}');
    runTest(Primary(lexer.parseStr('const {someType: someObject(), k:v, "strKey":(false ? true : false)}')), 'const {someType:someObject(), k:v, "strKey":(false ? true : false)}');
    runTest(Primary(lexer.parseStr('{int:5,}')), '{int:5,}');
    runTest(Primary(lexer.parseStr('{bool:false,}')), '{bool:false,}');
    runTest(Primary(lexer.parseStr('{String:"someString",}')), '{String:"someString",}');
    runTest(Primary(lexer.parseStr('{someType: someObject(),}')), '{someType:someObject(),}');
    runTest(Primary(lexer.parseStr('{bool:(test ? false:true),}')), '{bool:(test ? false : true),}');
    runTest(Primary(lexer.parseStr('{int:5,int:12,int:someInt,}')), '{int:5, int:12, int:someInt,}');
    runTest(Primary(lexer.parseStr('{bool:false,bool:someBool,bool:true,}')), '{bool:false, bool:someBool, bool:true,}');
    runTest(Primary(lexer.parseStr('{String:"someString", key:val, another:"str",}')), '{String:"someString", key:val, another:"str",}');
    runTest(Primary(lexer.parseStr('{someType: someObject(), k:v, "strKey":(false ? true : false),}')), '{someType:someObject(), k:v, "strKey":(false ? true : false),}');
    runTest(Primary(lexer.parseStr('const {int:5,}')), 'const {int:5,}');
    runTest(Primary(lexer.parseStr('const {bool:false,}')), 'const {bool:false,}');
    runTest(Primary(lexer.parseStr('const {String:"someString",}')), 'const {String:"someString",}');
    runTest(Primary(lexer.parseStr('const {someType: someObject(),}')), 'const {someType:someObject(),}');
    runTest(Primary(lexer.parseStr('const {bool:(test ? false:true),}')), 'const {bool:(test ? false : true),}');
    runTest(Primary(lexer.parseStr('const {int:5,int:12,int:someInt,}')), 'const {int:5, int:12, int:someInt,}');
    runTest(Primary(lexer.parseStr('const {bool:false,bool:someBool,bool:true,}')), 'const {bool:false, bool:someBool, bool:true,}');
    runTest(Primary(lexer.parseStr('const {String:"someString", key:val, another:"str",}')), 'const {String:"someString", key:val, another:"str",}');
    runTest(Primary(lexer.parseStr('const {someType: someObject(), k:v, "strKey":(false ? true : false),}')), 'const {someType:someObject(), k:v, "strKey":(false ? true : false),}');
    runTest(Primary(lexer.parseStr('{234}')), '{234}');
    runTest(Primary(lexer.parseStr('{234.432}')), '{234.432}');
    runTest(Primary(lexer.parseStr('{1,2,3,4}')), '{1, 2, 3, 4}');
    runTest(Primary(lexer.parseStr('{"someString"}')), '{"someString"}');
    runTest(Primary(lexer.parseStr('{"1","2","3","4"}')), '{"1", "2", "3", "4"}');
    runTest(Primary(lexer.parseStr('{someObject}')), '{someObject}');
    runTest(Primary(lexer.parseStr('{someObject,someObject,someObject}')), '{someObject, someObject, someObject}');
    runTest(Primary(lexer.parseStr('const {234}')), 'const {234}');
    runTest(Primary(lexer.parseStr('const {234.432}')), 'const {234.432}');
    runTest(Primary(lexer.parseStr('const {1,2,3,4}')), 'const {1, 2, 3, 4}');
    runTest(Primary(lexer.parseStr('const {"someString"}')), 'const {"someString"}');
    runTest(Primary(lexer.parseStr('const {"1","2","3","4"}')), 'const {"1", "2", "3", "4"}');
    runTest(Primary(lexer.parseStr('const {someObject}')), 'const {someObject}');
    runTest(Primary(lexer.parseStr('const {someObject,someObject,someObject}')), 'const {someObject, someObject, someObject}');
    runTest(Primary(lexer.parseStr('{234,}')), '{234,}');
    runTest(Primary(lexer.parseStr('{234.432,}')), '{234.432,}');
    runTest(Primary(lexer.parseStr('{1,2,3,4,}')), '{1, 2, 3, 4,}');
    runTest(Primary(lexer.parseStr('{"someString",}')), '{"someString",}');
    runTest(Primary(lexer.parseStr('{"1","2","3","4",}')), '{"1", "2", "3", "4",}');
    runTest(Primary(lexer.parseStr('{someObject,}')), '{someObject,}');
    runTest(Primary(lexer.parseStr('{someObject,someObject,someObject,}')), '{someObject, someObject, someObject,}');
    runTest(Primary(lexer.parseStr('const {234,}')), 'const {234,}');
    runTest(Primary(lexer.parseStr('const {234.432,}')), 'const {234.432,}');
    runTest(Primary(lexer.parseStr('const {1,2,3,4,}')), 'const {1, 2, 3, 4,}');
    runTest(Primary(lexer.parseStr('const {"someString",}')), 'const {"someString",}');
    runTest(Primary(lexer.parseStr('const {"1","2","3","4",}')), 'const {"1", "2", "3", "4",}');
    runTest(Primary(lexer.parseStr('const {someObject,}')), 'const {someObject,}');
    runTest(Primary(lexer.parseStr('const {someObject,someObject,someObject,}')), 'const {someObject, someObject, someObject,}');
    runTest(Primary(lexer.parseStr('<someType, num>{234}')), '<someType, num>{234}');
    runTest(Primary(lexer.parseStr('<someType, int, num>{234.432}')), '<someType, int, num>{234.432}');
    runTest(Primary(lexer.parseStr('<someType, int, num>{1,2,3,4}')), '<someType, int, num>{1, 2, 3, 4}');
    runTest(Primary(lexer.parseStr('<someType, int, num>{"someString",}')), '<someType, int, num>{"someString",}');
    runTest(Primary(lexer.parseStr('<someType, int, num>{"1","2","3","4",}')), '<someType, int, num>{"1", "2", "3", "4",}');
    runTest(Primary(lexer.parseStr('<someType, int, num>{someObject,}')), '<someType, int, num>{someObject,}');
    runTest(Primary(lexer.parseStr('<someType, int, num>{someObject,someObject,someObject,}')), '<someType, int, num>{someObject, someObject, someObject,}');
    runTest(Primary(lexer.parseStr('const <someType, int, num>{234,}')), 'const <someType, int, num>{234,}');
    runTest(Primary(lexer.parseStr('const <someType, int, num>{234.432,}')), 'const <someType, int, num>{234.432,}');
    runTest(Primary(lexer.parseStr('const <someType, int, num>{1,2,3,4,}')), 'const <someType, int, num>{1, 2, 3, 4,}');
    runTest(Primary(lexer.parseStr('const <someType, int, num>{"someString"}')), 'const <someType, int, num>{"someString"}');
    runTest(Primary(lexer.parseStr('const <someType, int, num>{"1","2","3","4"}')), 'const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(Primary(lexer.parseStr('const <someType, int, num>{someObject}')), 'const <someType, int, num>{someObject}');
    runTest(Primary(lexer.parseStr('const <someType, int, num>{someObject,someObject,someObject}')), 'const <someType, int, num>{someObject, someObject, someObject}');

    // TODO: Add Identifier tests


    // New Expression
    runTest(Primary(lexer.parseStr('new Test()')), 'new Test()');
    runTest(Primary(lexer.parseStr('new Test(234)')), 'new Test(234)');
    runTest(Primary(lexer.parseStr('new Test(234.432)')), 'new Test(234.432)');
    runTest(Primary(lexer.parseStr('new Test(1,2,3,4)')), 'new Test(1, 2, 3, 4)');
    runTest(Primary(lexer.parseStr('new Test("someString")')), 'new Test("someString")');
    runTest(Primary(lexer.parseStr('new Test("1","2","3","4")')), 'new Test("1", "2", "3", "4")');
    runTest(Primary(lexer.parseStr('new Test(someObject)')), 'new Test(someObject)');
    runTest(Primary(lexer.parseStr('new Test(someObject,someObject,someObject)')), 'new Test(someObject, someObject, someObject)');
    runTest(Primary(lexer.parseStr('new Test(someId:4)')), 'new Test(someId: 4)');
    runTest(Primary(lexer.parseStr('new Test(someId:true)')), 'new Test(someId: true)');
    runTest(Primary(lexer.parseStr('new Test(someId:"HI!")')), 'new Test(someId: "HI!")');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject())')), 'new Test(someId: someObject())');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject.param)')), 'new Test(someId: someObject.param)');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject.param ? another.param : meh.someVal)')), 'new Test(someId: someObject.param ? another.param : meh.someVal)');
    runTest(Primary(lexer.parseStr('new Test(someId:4,id2:.23)')), 'new Test(someId: 4, id2: .23)');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject(),id2:object2(),id3:object3())')), 'new Test(someId: someObject(), id2: object2(), id3: object3())');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject.param, _id2:otherParam.param.subparam)')), 'new Test(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'new Test(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Primary(lexer.parseStr('new Test(234.432,someId:4)')), 'new Test(234.432, someId: 4)');
    runTest(Primary(lexer.parseStr('new Test(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), 'new Test(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(Primary(lexer.parseStr('new Test("someString", someId:someObject(),id2:object2(),id3:object3())')), 'new Test("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(Primary(lexer.parseStr('new Test(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'new Test(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Primary(lexer.parseStr('new Test(234,)')), 'new Test(234,)');
    runTest(Primary(lexer.parseStr('new Test(234.432,)')), 'new Test(234.432,)');
    runTest(Primary(lexer.parseStr('new Test(1,2,3,4,)')), 'new Test(1, 2, 3, 4,)');
    runTest(Primary(lexer.parseStr('new Test("someString",)')), 'new Test("someString",)');
    runTest(Primary(lexer.parseStr('new Test("1","2","3","4",)')), 'new Test("1", "2", "3", "4",)');
    runTest(Primary(lexer.parseStr('new Test(someObject,)')), 'new Test(someObject,)');
    runTest(Primary(lexer.parseStr('new Test(someObject,someObject,someObject,)')), 'new Test(someObject, someObject, someObject,)');
    runTest(Primary(lexer.parseStr('new Test(someId:4,)')), 'new Test(someId: 4,)');
    runTest(Primary(lexer.parseStr('new Test(someId:true,)')), 'new Test(someId: true,)');
    runTest(Primary(lexer.parseStr('new Test(someId:"HI!",)')), 'new Test(someId: "HI!",)');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject(),)')), 'new Test(someId: someObject(),)');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject.param,)')), 'new Test(someId: someObject.param,)');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject.param ? another.param : meh.someVal,)')), 'new Test(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Primary(lexer.parseStr('new Test(someId:4,id2:.23,)')), 'new Test(someId: 4, id2: .23,)');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject(),id2:object2(),id3:object3(),)')), 'new Test(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'new Test(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(Primary(lexer.parseStr('new Test(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(Primary(lexer.parseStr('new Test(234.432,someId:4,)')), 'new Test(234.432, someId: 4,)');
    runTest(Primary(lexer.parseStr('new Test(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'new Test(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Primary(lexer.parseStr('new Test("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'new Test("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Primary(lexer.parseStr('new Test(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(Primary(lexer.parseStr('new Test.subType()')), 'new Test.subType()');
    runTest(Primary(lexer.parseStr('new Test.subType(234)')), 'new Test.subType(234)');
    runTest(Primary(lexer.parseStr('new Test.subType(234.432)')), 'new Test.subType(234.432)');
    runTest(Primary(lexer.parseStr('new Test.subType(1,2,3,4)')), 'new Test.subType(1, 2, 3, 4)');
    runTest(Primary(lexer.parseStr('new Test.subType("someString")')), 'new Test.subType("someString")');
    runTest(Primary(lexer.parseStr('new Test.subType("1","2","3","4")')), 'new Test.subType("1", "2", "3", "4")');
    runTest(Primary(lexer.parseStr('new Test.subType(someObject)')), 'new Test.subType(someObject)');
    runTest(Primary(lexer.parseStr('new Test.subType(someObject,someObject,someObject)')), 'new Test.subType(someObject, someObject, someObject)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:4)')), 'new Test.subType(someId: 4)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:true)')), 'new Test.subType(someId: true)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:"HI!")')), 'new Test.subType(someId: "HI!")');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject())')), 'new Test.subType(someId: someObject())');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject.param)')), 'new Test.subType(someId: someObject.param)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject.param ? another.param : meh.someVal)')), 'new Test.subType(someId: someObject.param ? another.param : meh.someVal)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:4,id2:.23)')), 'new Test.subType(someId: 4, id2: .23)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject(),id2:object2(),id3:object3())')), 'new Test.subType(someId: someObject(), id2: object2(), id3: object3())');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject.param, _id2:otherParam.param.subparam)')), 'new Test.subType(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'new Test.subType(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Primary(lexer.parseStr('new Test.subType(234.432,someId:4)')), 'new Test.subType(234.432, someId: 4)');
    runTest(Primary(lexer.parseStr('new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), 'new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(Primary(lexer.parseStr('new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3())')), 'new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(Primary(lexer.parseStr('new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Primary(lexer.parseStr('new Test.subType(234,)')), 'new Test.subType(234,)');
    runTest(Primary(lexer.parseStr('new Test.subType(234.432,)')), 'new Test.subType(234.432,)');
    runTest(Primary(lexer.parseStr('new Test.subType(1,2,3,4,)')), 'new Test.subType(1, 2, 3, 4,)');
    runTest(Primary(lexer.parseStr('new Test.subType("someString",)')), 'new Test.subType("someString",)');
    runTest(Primary(lexer.parseStr('new Test.subType("1","2","3","4",)')), 'new Test.subType("1", "2", "3", "4",)');
    runTest(Primary(lexer.parseStr('new Test.subType(someObject,)')), 'new Test.subType(someObject,)');
    runTest(Primary(lexer.parseStr('new Test.subType(someObject,someObject,someObject,)')), 'new Test.subType(someObject, someObject, someObject,)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:4,)')), 'new Test.subType(someId: 4,)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:true,)')), 'new Test.subType(someId: true,)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:"HI!",)')), 'new Test.subType(someId: "HI!",)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject(),)')), 'new Test.subType(someId: someObject(),)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject.param,)')), 'new Test.subType(someId: someObject.param,)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), 'new Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:4,id2:.23,)')), 'new Test.subType(someId: 4, id2: .23,)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), 'new Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'new Test.subType(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(Primary(lexer.parseStr('new Test.subType(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test.subType(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(Primary(lexer.parseStr('new Test.subType(234.432,someId:4,)')), 'new Test.subType(234.432, someId: 4,)');
    runTest(Primary(lexer.parseStr('new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Primary(lexer.parseStr('new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Primary(lexer.parseStr('new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

    // Const Object Expression
    runTest(Primary(lexer.parseStr('const Test()')), 'const Test()');
    runTest(Primary(lexer.parseStr('const Test(234)')), 'const Test(234)');
    runTest(Primary(lexer.parseStr('const Test(234.432)')), 'const Test(234.432)');
    runTest(Primary(lexer.parseStr('const Test(1,2,3,4)')), 'const Test(1, 2, 3, 4)');
    runTest(Primary(lexer.parseStr('const Test("someString")')), 'const Test("someString")');
    runTest(Primary(lexer.parseStr('const Test("1","2","3","4")')), 'const Test("1", "2", "3", "4")');
    runTest(Primary(lexer.parseStr('const Test(someObject)')), 'const Test(someObject)');
    runTest(Primary(lexer.parseStr('const Test(someObject,someObject,someObject)')), 'const Test(someObject, someObject, someObject)');
    runTest(Primary(lexer.parseStr('const Test(someId:4)')), 'const Test(someId: 4)');
    runTest(Primary(lexer.parseStr('const Test(someId:true)')), 'const Test(someId: true)');
    runTest(Primary(lexer.parseStr('const Test(someId:"HI!")')), 'const Test(someId: "HI!")');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject())')), 'const Test(someId: someObject())');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject.param)')), 'const Test(someId: someObject.param)');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject.param ? another.param : meh.someVal)')), 'const Test(someId: someObject.param ? another.param : meh.someVal)');
    runTest(Primary(lexer.parseStr('const Test(someId:4,id2:.23)')), 'const Test(someId: 4, id2: .23)');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject(),id2:object2(),id3:object3())')), 'const Test(someId: someObject(), id2: object2(), id3: object3())');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject.param, _id2:otherParam.param.subparam)')), 'const Test(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'const Test(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Primary(lexer.parseStr('const Test(234.432,someId:4)')), 'const Test(234.432, someId: 4)');
    runTest(Primary(lexer.parseStr('const Test(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), 'const Test(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(Primary(lexer.parseStr('const Test("someString", someId:someObject(),id2:object2(),id3:object3())')), 'const Test("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(Primary(lexer.parseStr('const Test(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'const Test(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Primary(lexer.parseStr('const Test(234,)')), 'const Test(234,)');
    runTest(Primary(lexer.parseStr('const Test(234.432,)')), 'const Test(234.432,)');
    runTest(Primary(lexer.parseStr('const Test(1,2,3,4,)')), 'const Test(1, 2, 3, 4,)');
    runTest(Primary(lexer.parseStr('const Test("someString",)')), 'const Test("someString",)');
    runTest(Primary(lexer.parseStr('const Test("1","2","3","4",)')), 'const Test("1", "2", "3", "4",)');
    runTest(Primary(lexer.parseStr('const Test(someObject,)')), 'const Test(someObject,)');
    runTest(Primary(lexer.parseStr('const Test(someObject,someObject,someObject,)')), 'const Test(someObject, someObject, someObject,)');
    runTest(Primary(lexer.parseStr('const Test(someId:4,)')), 'const Test(someId: 4,)');
    runTest(Primary(lexer.parseStr('const Test(someId:true,)')), 'const Test(someId: true,)');
    runTest(Primary(lexer.parseStr('const Test(someId:"HI!",)')), 'const Test(someId: "HI!",)');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject(),)')), 'const Test(someId: someObject(),)');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject.param,)')), 'const Test(someId: someObject.param,)');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject.param ? another.param : meh.someVal,)')), 'const Test(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Primary(lexer.parseStr('const Test(someId:4,id2:.23,)')), 'const Test(someId: 4, id2: .23,)');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject(),id2:object2(),id3:object3(),)')), 'const Test(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'const Test(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(Primary(lexer.parseStr('const Test(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const Test(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(Primary(lexer.parseStr('const Test(234.432,someId:4,)')), 'const Test(234.432, someId: 4,)');
    runTest(Primary(lexer.parseStr('const Test(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'const Test(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Primary(lexer.parseStr('const Test("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'const Test("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Primary(lexer.parseStr('const Test(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const Test(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(Primary(lexer.parseStr('const Test.subType()')), 'const Test.subType()');
    runTest(Primary(lexer.parseStr('const Test.subType(234)')), 'const Test.subType(234)');
    runTest(Primary(lexer.parseStr('const Test.subType(234.432)')), 'const Test.subType(234.432)');
    runTest(Primary(lexer.parseStr('const Test.subType(1,2,3,4)')), 'const Test.subType(1, 2, 3, 4)');
    runTest(Primary(lexer.parseStr('const Test.subType("someString")')), 'const Test.subType("someString")');
    runTest(Primary(lexer.parseStr('const Test.subType("1","2","3","4")')), 'const Test.subType("1", "2", "3", "4")');
    runTest(Primary(lexer.parseStr('const Test.subType(someObject)')), 'const Test.subType(someObject)');
    runTest(Primary(lexer.parseStr('const Test.subType(someObject,someObject,someObject)')), 'const Test.subType(someObject, someObject, someObject)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:4)')), 'const Test.subType(someId: 4)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:true)')), 'const Test.subType(someId: true)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:"HI!")')), 'const Test.subType(someId: "HI!")');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject())')), 'const Test.subType(someId: someObject())');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject.param)')), 'const Test.subType(someId: someObject.param)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal)')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:4,id2:.23)')), 'const Test.subType(someId: 4, id2: .23)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject(),id2:object2(),id3:object3())')), 'const Test.subType(someId: someObject(), id2: object2(), id3: object3())');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject.param, _id2:otherParam.param.subparam)')), 'const Test.subType(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Primary(lexer.parseStr('const Test.subType(234.432,someId:4)')), 'const Test.subType(234.432, someId: 4)');
    runTest(Primary(lexer.parseStr('const Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), 'const Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(Primary(lexer.parseStr('const Test.subType("someString", someId:someObject(),id2:object2(),id3:object3())')), 'const Test.subType("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(Primary(lexer.parseStr('const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Primary(lexer.parseStr('const Test.subType(234,)')), 'const Test.subType(234,)');
    runTest(Primary(lexer.parseStr('const Test.subType(234.432,)')), 'const Test.subType(234.432,)');
    runTest(Primary(lexer.parseStr('const Test.subType(1,2,3,4,)')), 'const Test.subType(1, 2, 3, 4,)');
    runTest(Primary(lexer.parseStr('const Test.subType("someString",)')), 'const Test.subType("someString",)');
    runTest(Primary(lexer.parseStr('const Test.subType("1","2","3","4",)')), 'const Test.subType("1", "2", "3", "4",)');
    runTest(Primary(lexer.parseStr('const Test.subType(someObject,)')), 'const Test.subType(someObject,)');
    runTest(Primary(lexer.parseStr('const Test.subType(someObject,someObject,someObject,)')), 'const Test.subType(someObject, someObject, someObject,)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:4,)')), 'const Test.subType(someId: 4,)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:true,)')), 'const Test.subType(someId: true,)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:"HI!",)')), 'const Test.subType(someId: "HI!",)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject(),)')), 'const Test.subType(someId: someObject(),)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject.param,)')), 'const Test.subType(someId: someObject.param,)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:4,id2:.23,)')), 'const Test.subType(someId: 4, id2: .23,)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), 'const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'const Test.subType(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(Primary(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(Primary(lexer.parseStr('const Test.subType(234.432,someId:4,)')), 'const Test.subType(234.432, someId: 4,)');
    runTest(Primary(lexer.parseStr('const Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'const Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Primary(lexer.parseStr('const Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'const Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Primary(lexer.parseStr('const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

    // TODO: Add (Expression)

  });

}
