import 'package:test/test.dart';
import 'test_lib.dart';

import 'package:source_parser/src/lexer/Lexer.dart';

import 'package:source_parser/src/sub_types/UnconditionalAssignableSelector.dart';
import 'package:source_parser/src/sub_types/AssignableSelectorPart.dart';
import 'package:source_parser/src/sub_types/ConstructorInvocation.dart';
import 'package:source_parser/src/sub_types/AssignableSelector.dart';
import 'package:source_parser/src/main/Selector.dart';

void main() {

  Lexer lexer = Lexer();

  test('Unconditional Assignable Selector', () { 

    runTest(UnconditionalAssignableSelector(lexer.parseStr('.someId')), '.someId');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('._someId')), '._someId');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[someId]')), '[someId]');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[(someId)]')), '[(someId)]');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[3+2]')), '[3 + 2]');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[4??6]')), '[4 ?? 6]');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[1??7]')), '[1 ?? 7]');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[9??someInt||8]')), '[9 ?? someInt || 8]');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[3|6%someInt??2+0||8]')), '[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[1+1>=someInt??7&0*32]')), '[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[2~/9%5??8-someInt|2]')), '[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[true?false:true]')), '[true ? false : true]');
    runTest(UnconditionalAssignableSelector(lexer.parseStr('[someVar == another?false ? true : false:true]')), '[someVar == another ? false ? true : false : true]');

  });

  test('Assignable Selector', () { 

    runTest(AssignableSelector(lexer.parseStr('.someId')), '.someId');
    runTest(AssignableSelector(lexer.parseStr('._someId')), '._someId');
    runTest(AssignableSelector(lexer.parseStr('?.someId')), '?.someId');
    runTest(AssignableSelector(lexer.parseStr('?._someId')), '?._someId');
    runTest(AssignableSelector(lexer.parseStr('[someId]')), '[someId]');
    runTest(AssignableSelector(lexer.parseStr('[(someId)]')), '[(someId)]');
    runTest(AssignableSelector(lexer.parseStr('[3+2]')), '[3 + 2]');
    runTest(AssignableSelector(lexer.parseStr('[4??6]')), '[4 ?? 6]');
    runTest(AssignableSelector(lexer.parseStr('[1??7]')), '[1 ?? 7]');
    runTest(AssignableSelector(lexer.parseStr('[9??someInt||8]')), '[9 ?? someInt || 8]');
    runTest(AssignableSelector(lexer.parseStr('[3|6%someInt??2+0||8]')), '[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(AssignableSelector(lexer.parseStr('[1+1>=someInt??7&0*32]')), '[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(AssignableSelector(lexer.parseStr('[2~/9%5??8-someInt|2]')), '[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(AssignableSelector(lexer.parseStr('[true?false:true]')), '[true ? false : true]');
    runTest(AssignableSelector(lexer.parseStr('[someVar == another?false ? true : false:true]')), '[someVar == another ? false ? true : false : true]');

  });

  test('Assignable Selector Part', () { 

    runTest(AssignableSelectorPart(lexer.parseStr('.someId')), '.someId');
    runTest(AssignableSelectorPart(lexer.parseStr('._someId')), '._someId');
    runTest(AssignableSelectorPart(lexer.parseStr('?.someId')), '?.someId');
    runTest(AssignableSelectorPart(lexer.parseStr('?._someId')), '?._someId');
    runTest(AssignableSelectorPart(lexer.parseStr('[someId]')), '[someId]');
    runTest(AssignableSelectorPart(lexer.parseStr('[(someId)]')), '[(someId)]');
    runTest(AssignableSelectorPart(lexer.parseStr('[3+2]')), '[3 + 2]');
    runTest(AssignableSelectorPart(lexer.parseStr('[4??6]')), '[4 ?? 6]');
    runTest(AssignableSelectorPart(lexer.parseStr('[1??7]')), '[1 ?? 7]');
    runTest(AssignableSelectorPart(lexer.parseStr('[9??someInt||8]')), '[9 ?? someInt || 8]');
    runTest(AssignableSelectorPart(lexer.parseStr('[3|6%someInt??2+0||8]')), '[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(AssignableSelectorPart(lexer.parseStr('[1+1>=someInt??7&0*32]')), '[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(AssignableSelectorPart(lexer.parseStr('[2~/9%5??8-someInt|2]')), '[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(AssignableSelectorPart(lexer.parseStr('[true?false:true]')), '[true ? false : true]');
    runTest(AssignableSelectorPart(lexer.parseStr('[someVar == another?false ? true : false:true]')), '[someVar == another ? false ? true : false : true]');

    runTest(AssignableSelectorPart(lexer.parseStr('(x, y, {x()}).someId')), '(x, y, {x()}).someId');
    runTest(AssignableSelectorPart(lexer.parseStr('(x (y(x), z), {x()})[9??someInt||8]')), '(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(AssignableSelectorPart(lexer.parseStr('({x (y(x))})[3|6%someInt??2+0||8]')), '({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(AssignableSelectorPart(lexer.parseStr('(x, y, {x})[1+1>=someInt??7&0*32]')), '(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(AssignableSelectorPart(lexer.parseStr('(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(AssignableSelectorPart(lexer.parseStr('({this.x (this.y ( z))})[true?false:true]')), '({this.x(this.y(z))})[true ? false : true]');
    runTest(AssignableSelectorPart(lexer.parseStr('(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');

    runTest(AssignableSelectorPart(lexer.parseStr('<int>(x, y, {x()}).someId')), '<int>(x, y, {x()}).someId');
    runTest(AssignableSelectorPart(lexer.parseStr('<int,num>(x (y(x), z), {x()})[9??someInt||8]')), '<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(AssignableSelectorPart(lexer.parseStr('<int,num>({x (y(x))})[3|6%someInt??2+0||8]')), '<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(AssignableSelectorPart(lexer.parseStr('<int,num>(x, y, {x})[1+1>=someInt??7&0*32]')), '<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(AssignableSelectorPart(lexer.parseStr('<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(AssignableSelectorPart(lexer.parseStr('<int,num>({this.x (this.y ( z))})[true?false:true]')), '<int, num>({this.x(this.y(z))})[true ? false : true]');
    runTest(AssignableSelectorPart(lexer.parseStr('<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');

  });

  test('Constructor Invocation', () {

    runTest(ConstructorInvocation(lexer.parseStr('someType<int>.someId()')), 'someType<int>.someId()');
    runTest(ConstructorInvocation(lexer.parseStr('someType<int>.someId(x, y, {x()})')), 'someType<int>.someId(x, y, {x()})');
    runTest(ConstructorInvocation(lexer.parseStr('someType<int,String,anotherType>.someId(x, y, {x(),})')), 'someType<int, String, anotherType>.someId(x, y, {x(),})');

  });

  test('Selector', () {

    runTest(Selector(lexer.parseStr('.someId')), '.someId');
    runTest(Selector(lexer.parseStr('._someId')), '._someId');
    runTest(Selector(lexer.parseStr('?.someId')), '?.someId');
    runTest(Selector(lexer.parseStr('?._someId')), '?._someId');
    runTest(Selector(lexer.parseStr('[someId]')), '[someId]');
    runTest(Selector(lexer.parseStr('[(someId)]')), '[(someId)]');
    runTest(Selector(lexer.parseStr('[3+2]')), '[3 + 2]');
    runTest(Selector(lexer.parseStr('[4??6]')), '[4 ?? 6]');
    runTest(Selector(lexer.parseStr('[1??7]')), '[1 ?? 7]');
    runTest(Selector(lexer.parseStr('[9??someInt||8]')), '[9 ?? someInt || 8]');
    runTest(Selector(lexer.parseStr('[3|6%someInt??2+0||8]')), '[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(Selector(lexer.parseStr('[1+1>=someInt??7&0*32]')), '[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(Selector(lexer.parseStr('[2~/9%5??8-someInt|2]')), '[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(Selector(lexer.parseStr('[true?false:true]')), '[true ? false : true]');
    runTest(Selector(lexer.parseStr('[someVar == another?false ? true : false:true]')), '[someVar == another ? false ? true : false : true]');

    runTest(Selector(lexer.parseStr('()')), '()');
    runTest(Selector(lexer.parseStr('(234)')), '(234)');
    runTest(Selector(lexer.parseStr('(234.432)')), '(234.432)');
    runTest(Selector(lexer.parseStr('(1,2,3,4)')), '(1, 2, 3, 4)');
    runTest(Selector(lexer.parseStr('("someString")')), '("someString")');
    runTest(Selector(lexer.parseStr('("1","2","3","4")')), '("1", "2", "3", "4")');
    runTest(Selector(lexer.parseStr('(someObject)')), '(someObject)');
    runTest(Selector(lexer.parseStr('(someObject,someObject,someObject)')), '(someObject, someObject, someObject)');
    runTest(Selector(lexer.parseStr('(someId:4)')), '(someId: 4)');
    runTest(Selector(lexer.parseStr('(someId:true)')), '(someId: true)');
    runTest(Selector(lexer.parseStr('(someId:"HI!")')), '(someId: "HI!")');
    runTest(Selector(lexer.parseStr('(someId:someObject())')), '(someId: someObject())');
    runTest(Selector(lexer.parseStr('(someId:someObject.param)')), '(someId: someObject.param)');
    runTest(Selector(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal)')), '(someId: someObject.param ? another.param : meh.someVal)');
    runTest(Selector(lexer.parseStr('(someId:4,id2:.23)')), '(someId: 4, id2: .23)');
    runTest(Selector(lexer.parseStr('(someId:someObject(),id2:object2(),id3:object3())')), '(someId: someObject(), id2: object2(), id3: object3())');
    runTest(Selector(lexer.parseStr('(someId:someObject.param, _id2:otherParam.param.subparam)')), '(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(Selector(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), '(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Selector(lexer.parseStr('(234.432,someId:4)')), '(234.432, someId: 4)');
    runTest(Selector(lexer.parseStr('(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), '(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(Selector(lexer.parseStr('("someString", someId:someObject(),id2:object2(),id3:object3())')), '("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(Selector(lexer.parseStr('(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), '(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Selector(lexer.parseStr('(234,)')), '(234,)');
    runTest(Selector(lexer.parseStr('(234.432,)')), '(234.432,)');
    runTest(Selector(lexer.parseStr('(1,2,3,4,)')), '(1, 2, 3, 4,)');
    runTest(Selector(lexer.parseStr('("someString",)')), '("someString",)');
    runTest(Selector(lexer.parseStr('("1","2","3","4",)')), '("1", "2", "3", "4",)');
    runTest(Selector(lexer.parseStr('(someObject,)')), '(someObject,)');
    runTest(Selector(lexer.parseStr('(someObject,someObject,someObject,)')), '(someObject, someObject, someObject,)');
    runTest(Selector(lexer.parseStr('(someId:4,)')), '(someId: 4,)');
    runTest(Selector(lexer.parseStr('(someId:true,)')), '(someId: true,)');
    runTest(Selector(lexer.parseStr('(someId:"HI!",)')), '(someId: "HI!",)');
    runTest(Selector(lexer.parseStr('(someId:someObject(),)')), '(someId: someObject(),)');
    runTest(Selector(lexer.parseStr('(someId:someObject.param,)')), '(someId: someObject.param,)');
    runTest(Selector(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal,)')), '(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Selector(lexer.parseStr('(someId:4,id2:.23,)')), '(someId: 4, id2: .23,)');
    runTest(Selector(lexer.parseStr('(someId:someObject(),id2:object2(),id3:object3(),)')), '(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Selector(lexer.parseStr('(someId:someObject.param, _id2:otherParam.param.subparam,)')), '(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(Selector(lexer.parseStr('(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(Selector(lexer.parseStr('(234.432,someId:4,)')), '(234.432, someId: 4,)');
    runTest(Selector(lexer.parseStr('(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Selector(lexer.parseStr('("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Selector(lexer.parseStr('(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(Selector(lexer.parseStr('<int>()')), '<int>()');
    runTest(Selector(lexer.parseStr('<int,String>(234)')), '<int, String>(234)');
    runTest(Selector(lexer.parseStr('<int,String>(234.432)')), '<int, String>(234.432)');
    runTest(Selector(lexer.parseStr('<int,String>(1,2,3,4)')), '<int, String>(1, 2, 3, 4)');
    runTest(Selector(lexer.parseStr('<int,String>("someString")')), '<int, String>("someString")');
    runTest(Selector(lexer.parseStr('<int,String>("1","2","3","4")')), '<int, String>("1", "2", "3", "4")');
    runTest(Selector(lexer.parseStr('<int,String>(someObject)')), '<int, String>(someObject)');
    runTest(Selector(lexer.parseStr('<int,String>(someObject,someObject,someObject)')), '<int, String>(someObject, someObject, someObject)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:4)')), '<int, String>(someId: 4)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:true)')), '<int, String>(someId: true)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:"HI!")')), '<int, String>(someId: "HI!")');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject())')), '<int, String>(someId: someObject())');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject.param)')), '<int, String>(someId: someObject.param)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject.param ? another.param : meh.someVal)')), '<int, String>(someId: someObject.param ? another.param : meh.someVal)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:4,id2:.23)')), '<int, String>(someId: 4, id2: .23)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject(),id2:object2(),id3:object3())')), '<int, String>(someId: someObject(), id2: object2(), id3: object3())');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject.param, _id2:otherParam.param.subparam)')), '<int, String>(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), '<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Selector(lexer.parseStr('<int,String>(234.432,someId:4)')), '<int, String>(234.432, someId: 4)');
    runTest(Selector(lexer.parseStr('<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), '<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(Selector(lexer.parseStr('<int,String>("someString", someId:someObject(),id2:object2(),id3:object3())')), '<int, String>("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(Selector(lexer.parseStr('<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), '<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(Selector(lexer.parseStr('<int,String>(234,)')), '<int, String>(234,)');
    runTest(Selector(lexer.parseStr('<int,String>(234.432,)')), '<int, String>(234.432,)');
    runTest(Selector(lexer.parseStr('<int,String>(1,2,3,4,)')), '<int, String>(1, 2, 3, 4,)');
    runTest(Selector(lexer.parseStr('<int,String>("someString",)')), '<int, String>("someString",)');
    runTest(Selector(lexer.parseStr('<int,String>("1","2","3","4",)')), '<int, String>("1", "2", "3", "4",)');
    runTest(Selector(lexer.parseStr('<int,String>(someObject,)')), '<int, String>(someObject,)');
    runTest(Selector(lexer.parseStr('<int,String>(someObject,someObject,someObject,)')), '<int, String>(someObject, someObject, someObject,)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:4,)')), '<int, String>(someId: 4,)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:true,)')), '<int, String>(someId: true,)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:"HI!",)')), '<int, String>(someId: "HI!",)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject(),)')), '<int, String>(someId: someObject(),)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject.param,)')), '<int, String>(someId: someObject.param,)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject.param ? another.param : meh.someVal,)')), '<int, String>(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:4,id2:.23,)')), '<int, String>(someId: 4, id2: .23,)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject(),id2:object2(),id3:object3(),)')), '<int, String>(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(Selector(lexer.parseStr('<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(Selector(lexer.parseStr('<int,String>(234.432,someId:4,)')), '<int, String>(234.432, someId: 4,)');
    runTest(Selector(lexer.parseStr('<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(Selector(lexer.parseStr('<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(Selector(lexer.parseStr('<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

  });

}
