import 'package:test/test.dart';
import 'test_lib.dart';

import 'package:source_parser/src/lexer/Lexer.dart';

import 'package:source_parser/src/sub_types/Operators.dart';

void main() {

  Lexer lexer = Lexer();

  test('Compound Assignment Operator', () {

    runTest(CompoundAssignmentOperator(lexer.parseStr('*=')), '*=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('/=')), '/=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('~/=')), '~/=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('%=')), '%=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('+=')), '+=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('-=')), '-=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('<<=')), '<<=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('>>=')), '>>=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('>>>=')), '>>>=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('&=')), '&=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('^=')), '^=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('|=')), '|=');
    runTest(CompoundAssignmentOperator(lexer.parseStr('??=')), '??=');

  });

  test('Assignment Operator', () {

    runTest(AssignmentOperator(lexer.parseStr('=')), '=');
    runTest(AssignmentOperator(lexer.parseStr('*=')), '*=');
    runTest(AssignmentOperator(lexer.parseStr('/=')), '/=');
    runTest(AssignmentOperator(lexer.parseStr('~/=')), '~/=');
    runTest(AssignmentOperator(lexer.parseStr('%=')), '%=');
    runTest(AssignmentOperator(lexer.parseStr('+=')), '+=');
    runTest(AssignmentOperator(lexer.parseStr('-=')), '-=');
    runTest(AssignmentOperator(lexer.parseStr('<<=')), '<<=');
    runTest(AssignmentOperator(lexer.parseStr('>>=')), '>>=');
    runTest(AssignmentOperator(lexer.parseStr('>>>=')), '>>>=');
    runTest(AssignmentOperator(lexer.parseStr('&=')), '&=');
    runTest(AssignmentOperator(lexer.parseStr('^=')), '^=');
    runTest(AssignmentOperator(lexer.parseStr('|=')), '|=');
    runTest(AssignmentOperator(lexer.parseStr('??=')), '??=');

  });

  test('Equality Operator', () {

    runTest(EqualityOperator(lexer.parseStr('==')), '==');
    runTest(EqualityOperator(lexer.parseStr('!=')), '!=');

  });

  test('Relational Operator', () {

    runTest(RelationalOperator(lexer.parseStr('<=')), '<=');
    runTest(RelationalOperator(lexer.parseStr('>=')), '>=');
    runTest(RelationalOperator(lexer.parseStr('<')), '<');
    runTest(RelationalOperator(lexer.parseStr('>')), '>');

  });

  test('Bitwise Operator', () {

    runTest(BitwiseOperator(lexer.parseStr('|')), '|');
    runTest(BitwiseOperator(lexer.parseStr('&')), '&');
    runTest(BitwiseOperator(lexer.parseStr('^')), '^');

  });

  test('Shift Operator', () {

    runTest(ShiftOperator(lexer.parseStr('<<')), '<<');
    runTest(ShiftOperator(lexer.parseStr('>>')), '>>');
    runTest(ShiftOperator(lexer.parseStr('>>>')), '>>>');

  });

  test('Additive Operator', () {

    runTest(AdditiveOperator(lexer.parseStr('+')), '+');
    runTest(AdditiveOperator(lexer.parseStr('-')), '-');

  });

  test('Multiplicative Operator', () {

    runTest(MultiplicativeOperator(lexer.parseStr('*')), '*');
    runTest(MultiplicativeOperator(lexer.parseStr('/')), '/');
    runTest(MultiplicativeOperator(lexer.parseStr('%')), '%');
    runTest(MultiplicativeOperator(lexer.parseStr('~/')), '~/');

  });

  test('Prefix Operator', () {

    runTest(MinusOperator(lexer.parseStr('-')), '-');
    runTest(NegationOperator(lexer.parseStr('!')), '!');
    runTest(TildeOperator(lexer.parseStr('~')), '~');

    runTest(PrefixOperator(lexer.parseStr('-')), '-');
    runTest(PrefixOperator(lexer.parseStr('!')), '!');
    runTest(PrefixOperator(lexer.parseStr('~')), '~');
  });

  test('Postfix Operator (Increment Operators)', () {

    runTest(IncrementOperator(lexer.parseStr('++')), '++');
    runTest(IncrementOperator(lexer.parseStr('--')), '--');

    runTest(PostfixOperator(lexer.parseStr('++')), '++');
    runTest(PostfixOperator(lexer.parseStr('--')), '--');

  });

  test('Is Operator', () {

    runTest(IsOperator(lexer.parseStr('is')), 'is');
    runTest(IsOperator(lexer.parseStr('is!')), 'is!');

  });

  test('As Operator', () {

    runTest(AsOperator(lexer.parseStr('as')), 'as');

  });

}
