import 'package:test/test.dart';
import 'test_lib.dart';

import 'package:source_parser/src/lexer/Lexer.dart';

import 'package:source_parser/src/main/InitializedVariableDeclaration.dart';
import 'package:source_parser/src/lists/InitializedIdentifierList.dart';
import 'package:source_parser/src/main/VariableDeclaration.dart';
import 'package:source_parser/src/sub_types/Identifiers.dart';
import 'package:source_parser/src/sub_types/Types.dart';

void main() {

  Lexer lexer = Lexer();

  test('Var Or Type', () {

    runTest(VarOrType(lexer.parseStr('int')), 'int');
    runTest(VarOrType(lexer.parseStr('var')), 'var');
    runTest(VarOrType(lexer.parseStr('Future')), 'Future');

  });
  
  test('Final Const Var Or Type', () {

    runTest(FinalConstVarOrType(lexer.parseStr('int')), 'int');
    runTest(FinalConstVarOrType(lexer.parseStr('final')), 'final');
    runTest(FinalConstVarOrType(lexer.parseStr('const')), 'const');
    runTest(FinalConstVarOrType(lexer.parseStr('final _someType')), 'final _someType');
    runTest(FinalConstVarOrType(lexer.parseStr('const _someType')), 'const _someType');

  });
  
  test('Declared Identifier', () {

    runTest(DeclaredIdentifier(lexer.parseStr('int x')), 'int x');
    runTest(DeclaredIdentifier(lexer.parseStr('var x')), 'var x');
    runTest(DeclaredIdentifier(lexer.parseStr('covariant someType x')), 'covariant someType x');
    runTest(DeclaredIdentifier(lexer.parseStr('const bool x')), 'const bool x');
    runTest(DeclaredIdentifier(lexer.parseStr('final var x')), 'final var x');
    runTest(DeclaredIdentifier(lexer.parseStr('dynamic x')), 'dynamic x');

  });

  test('Variable Declarations', () {

    runTest(VariableDeclaration(lexer.parseStr('int x')), 'int x');
    runTest(VariableDeclaration(lexer.parseStr('var x')), 'var x');
    runTest(VariableDeclaration(lexer.parseStr('final var x')), 'final var x');
    runTest(VariableDeclaration(lexer.parseStr('const bool x')), 'const bool x');
    runTest(VariableDeclaration(lexer.parseStr('var x,y,z')), 'var x, y, z');
    runTest(VariableDeclaration(lexer.parseStr('final String someVal, anotherVal,third')), 'final String someVal, anotherVal, third');

  });

  test('Initialized Identifier', () {

    runTest(InitializedIdentifier(lexer.parseStr('x')), 'x');
    runTest(InitializedIdentifier(lexer.parseStr('x="test"')), 'x = "test"');
    runTest(InitializedIdentifier(lexer.parseStr('someVar=someOtherVar')), 'someVar = someOtherVar');
    runTest(InitializedIdentifier(lexer.parseStr('someVar=someOtherVar.subParam')), 'someVar = someOtherVar.subParam');
    runTest(InitializedIdentifier(lexer.parseStr('x=someVar?true:false')), 'x = someVar ? true : false');
    runTest(InitializedIdentifier(lexer.parseStr('x=( 403-2+0xfff)')), 'x = (403 - 2 + 0xfff)');
    runTest(InitializedIdentifier(lexer.parseStr('x=funcCall()')), 'x = funcCall()');
    runTest(InitializedIdentifier(lexer.parseStr('x=funcCall("someArg")')), 'x = funcCall("someArg")');

  });

  test('Initialized Variable Declaration', () {

    runTest(InitializedVariableDeclaration(lexer.parseStr('int x')), 'int x');
    runTest(InitializedVariableDeclaration(lexer.parseStr('var x')), 'var x');
    runTest(InitializedVariableDeclaration(lexer.parseStr('var x=someVar')), 'var x = someVar');
    runTest(InitializedVariableDeclaration(lexer.parseStr('var x=someVar.subParam')), 'var x = someVar.subParam');
    runTest(InitializedVariableDeclaration(lexer.parseStr('var x=someVar, y="someStr",z=false')), 'var x = someVar, y = "someStr", z = false');
    runTest(InitializedVariableDeclaration(lexer.parseStr('const bool x=someVar?true:false')), 'const bool x = someVar ? true : false');

  });

  test('Initialized Identifier List', () {

    runTest(InitializedIdentifierList(lexer.parseStr('x = 0')), 'x = 0');
    runTest(InitializedIdentifierList(lexer.parseStr('x = 0, _y="false",another=true?3:4')), 'x = 0, _y = "false", another = true ? 3 : 4');

  });

}
