import 'package:test/test.dart';
import 'test_lib.dart';

import 'package:source_parser/src/lexer/Lexer.dart';

import 'package:source_parser/src/sub_types/Expressions.dart';

// TODO: Make expressions check further types beyond basic integers
void main() {

  Lexer lexer = Lexer();

  test('This Expression', () {

    runTest(ThisExpression(lexer.parseStr('this')), 'this');

  });

  test('Instance Creation (New Expression)', () {

    runTest(NewExpression(lexer.parseStr('new Test()')), 'new Test()');
    runTest(NewExpression(lexer.parseStr('new Test(234)')), 'new Test(234)');
    runTest(NewExpression(lexer.parseStr('new Test(234.432)')), 'new Test(234.432)');
    runTest(NewExpression(lexer.parseStr('new Test(1,2,3,4)')), 'new Test(1, 2, 3, 4)');
    runTest(NewExpression(lexer.parseStr('new Test("someString")')), 'new Test("someString")');
    runTest(NewExpression(lexer.parseStr('new Test("1","2","3","4")')), 'new Test("1", "2", "3", "4")');
    runTest(NewExpression(lexer.parseStr('new Test(someObject)')), 'new Test(someObject)');
    runTest(NewExpression(lexer.parseStr('new Test(someObject,someObject,someObject)')), 'new Test(someObject, someObject, someObject)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:4)')), 'new Test(someId: 4)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:true)')), 'new Test(someId: true)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:"HI!")')), 'new Test(someId: "HI!")');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject())')), 'new Test(someId: someObject())');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject.param)')), 'new Test(someId: someObject.param)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject.param ? another.param : meh.someVal)')), 'new Test(someId: someObject.param ? another.param : meh.someVal)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:4,id2:.23)')), 'new Test(someId: 4, id2: .23)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject(),id2:object2(),id3:object3())')), 'new Test(someId: someObject(), id2: object2(), id3: object3())');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject.param, _id2:otherParam.param.subparam)')), 'new Test(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'new Test(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(NewExpression(lexer.parseStr('new Test(234.432,someId:4)')), 'new Test(234.432, someId: 4)');
    runTest(NewExpression(lexer.parseStr('new Test(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), 'new Test(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(NewExpression(lexer.parseStr('new Test("someString", someId:someObject(),id2:object2(),id3:object3())')), 'new Test("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(NewExpression(lexer.parseStr('new Test(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'new Test(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(NewExpression(lexer.parseStr('new Test(234,)')), 'new Test(234,)');
    runTest(NewExpression(lexer.parseStr('new Test(234.432,)')), 'new Test(234.432,)');
    runTest(NewExpression(lexer.parseStr('new Test(1,2,3,4,)')), 'new Test(1, 2, 3, 4,)');
    runTest(NewExpression(lexer.parseStr('new Test("someString",)')), 'new Test("someString",)');
    runTest(NewExpression(lexer.parseStr('new Test("1","2","3","4",)')), 'new Test("1", "2", "3", "4",)');
    runTest(NewExpression(lexer.parseStr('new Test(someObject,)')), 'new Test(someObject,)');
    runTest(NewExpression(lexer.parseStr('new Test(someObject,someObject,someObject,)')), 'new Test(someObject, someObject, someObject,)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:4,)')), 'new Test(someId: 4,)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:true,)')), 'new Test(someId: true,)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:"HI!",)')), 'new Test(someId: "HI!",)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject(),)')), 'new Test(someId: someObject(),)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject.param,)')), 'new Test(someId: someObject.param,)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject.param ? another.param : meh.someVal,)')), 'new Test(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:4,id2:.23,)')), 'new Test(someId: 4, id2: .23,)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject(),id2:object2(),id3:object3(),)')), 'new Test(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'new Test(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(NewExpression(lexer.parseStr('new Test(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(NewExpression(lexer.parseStr('new Test(234.432,someId:4,)')), 'new Test(234.432, someId: 4,)');
    runTest(NewExpression(lexer.parseStr('new Test(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'new Test(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(NewExpression(lexer.parseStr('new Test("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'new Test("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(NewExpression(lexer.parseStr('new Test(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

    runTest(NewExpression(lexer.parseStr('new Test.subType()')), 'new Test.subType()');
    runTest(NewExpression(lexer.parseStr('new Test.subType(234)')), 'new Test.subType(234)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(234.432)')), 'new Test.subType(234.432)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(1,2,3,4)')), 'new Test.subType(1, 2, 3, 4)');
    runTest(NewExpression(lexer.parseStr('new Test.subType("someString")')), 'new Test.subType("someString")');
    runTest(NewExpression(lexer.parseStr('new Test.subType("1","2","3","4")')), 'new Test.subType("1", "2", "3", "4")');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someObject)')), 'new Test.subType(someObject)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someObject,someObject,someObject)')), 'new Test.subType(someObject, someObject, someObject)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:4)')), 'new Test.subType(someId: 4)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:true)')), 'new Test.subType(someId: true)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:"HI!")')), 'new Test.subType(someId: "HI!")');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject())')), 'new Test.subType(someId: someObject())');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject.param)')), 'new Test.subType(someId: someObject.param)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject.param ? another.param : meh.someVal)')), 'new Test.subType(someId: someObject.param ? another.param : meh.someVal)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:4,id2:.23)')), 'new Test.subType(someId: 4, id2: .23)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject(),id2:object2(),id3:object3())')), 'new Test.subType(someId: someObject(), id2: object2(), id3: object3())');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject.param, _id2:otherParam.param.subparam)')), 'new Test.subType(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'new Test.subType(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(234.432,someId:4)')), 'new Test.subType(234.432, someId: 4)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), 'new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(NewExpression(lexer.parseStr('new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3())')), 'new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(234,)')), 'new Test.subType(234,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(234.432,)')), 'new Test.subType(234.432,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(1,2,3,4,)')), 'new Test.subType(1, 2, 3, 4,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType("someString",)')), 'new Test.subType("someString",)');
    runTest(NewExpression(lexer.parseStr('new Test.subType("1","2","3","4",)')), 'new Test.subType("1", "2", "3", "4",)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someObject,)')), 'new Test.subType(someObject,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someObject,someObject,someObject,)')), 'new Test.subType(someObject, someObject, someObject,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:4,)')), 'new Test.subType(someId: 4,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:true,)')), 'new Test.subType(someId: true,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:"HI!",)')), 'new Test.subType(someId: "HI!",)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject(),)')), 'new Test.subType(someId: someObject(),)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject.param,)')), 'new Test.subType(someId: someObject.param,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), 'new Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:4,id2:.23,)')), 'new Test.subType(someId: 4, id2: .23,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), 'new Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'new Test.subType(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test.subType(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(234.432,someId:4,)')), 'new Test.subType(234.432, someId: 4,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(NewExpression(lexer.parseStr('new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(NewExpression(lexer.parseStr('new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

  });

  test('Const Object Expression', () {

    runTest(ConstObjectExpression(lexer.parseStr('const Test()')), 'const Test()');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(234)')), 'const Test(234)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(234.432)')), 'const Test(234.432)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(1,2,3,4)')), 'const Test(1, 2, 3, 4)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test("someString")')), 'const Test("someString")');
    runTest(ConstObjectExpression(lexer.parseStr('const Test("1","2","3","4")')), 'const Test("1", "2", "3", "4")');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someObject)')), 'const Test(someObject)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someObject,someObject,someObject)')), 'const Test(someObject, someObject, someObject)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:4)')), 'const Test(someId: 4)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:true)')), 'const Test(someId: true)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:"HI!")')), 'const Test(someId: "HI!")');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject())')), 'const Test(someId: someObject())');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject.param)')), 'const Test(someId: someObject.param)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject.param ? another.param : meh.someVal)')), 'const Test(someId: someObject.param ? another.param : meh.someVal)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:4,id2:.23)')), 'const Test(someId: 4, id2: .23)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject(),id2:object2(),id3:object3())')), 'const Test(someId: someObject(), id2: object2(), id3: object3())');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject.param, _id2:otherParam.param.subparam)')), 'const Test(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'const Test(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(234.432,someId:4)')), 'const Test(234.432, someId: 4)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), 'const Test(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test("someString", someId:someObject(),id2:object2(),id3:object3())')), 'const Test("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'const Test(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(234,)')), 'const Test(234,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(234.432,)')), 'const Test(234.432,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(1,2,3,4,)')), 'const Test(1, 2, 3, 4,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test("someString",)')), 'const Test("someString",)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test("1","2","3","4",)')), 'const Test("1", "2", "3", "4",)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someObject,)')), 'const Test(someObject,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someObject,someObject,someObject,)')), 'const Test(someObject, someObject, someObject,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:4,)')), 'const Test(someId: 4,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:true,)')), 'const Test(someId: true,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:"HI!",)')), 'const Test(someId: "HI!",)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject(),)')), 'const Test(someId: someObject(),)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject.param,)')), 'const Test(someId: someObject.param,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject.param ? another.param : meh.someVal,)')), 'const Test(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:4,id2:.23,)')), 'const Test(someId: 4, id2: .23,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject(),id2:object2(),id3:object3(),)')), 'const Test(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'const Test(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const Test(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(234.432,someId:4,)')), 'const Test(234.432, someId: 4,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'const Test(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'const Test("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const Test(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType()')), 'const Test.subType()');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(234)')), 'const Test.subType(234)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(234.432)')), 'const Test.subType(234.432)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(1,2,3,4)')), 'const Test.subType(1, 2, 3, 4)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType("someString")')), 'const Test.subType("someString")');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType("1","2","3","4")')), 'const Test.subType("1", "2", "3", "4")');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someObject)')), 'const Test.subType(someObject)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someObject,someObject,someObject)')), 'const Test.subType(someObject, someObject, someObject)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:4)')), 'const Test.subType(someId: 4)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:true)')), 'const Test.subType(someId: true)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:"HI!")')), 'const Test.subType(someId: "HI!")');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject())')), 'const Test.subType(someId: someObject())');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject.param)')), 'const Test.subType(someId: someObject.param)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal)')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:4,id2:.23)')), 'const Test.subType(someId: 4, id2: .23)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject(),id2:object2(),id3:object3())')), 'const Test.subType(someId: someObject(), id2: object2(), id3: object3())');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject.param, _id2:otherParam.param.subparam)')), 'const Test.subType(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(234.432,someId:4)')), 'const Test.subType(234.432, someId: 4)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), 'const Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType("someString", someId:someObject(),id2:object2(),id3:object3())')), 'const Test.subType("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(234,)')), 'const Test.subType(234,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(234.432,)')), 'const Test.subType(234.432,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(1,2,3,4,)')), 'const Test.subType(1, 2, 3, 4,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType("someString",)')), 'const Test.subType("someString",)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType("1","2","3","4",)')), 'const Test.subType("1", "2", "3", "4",)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someObject,)')), 'const Test.subType(someObject,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someObject,someObject,someObject,)')), 'const Test.subType(someObject, someObject, someObject,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:4,)')), 'const Test.subType(someId: 4,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:true,)')), 'const Test.subType(someId: true,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:"HI!",)')), 'const Test.subType(someId: "HI!",)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject(),)')), 'const Test.subType(someId: someObject(),)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject.param,)')), 'const Test.subType(someId: someObject.param,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:4,id2:.23,)')), 'const Test.subType(someId: 4, id2: .23,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), 'const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'const Test.subType(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(234.432,someId:4,)')), 'const Test.subType(234.432, someId: 4,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'const Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'const Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ConstObjectExpression(lexer.parseStr('const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

  });

  test('Throw Expression', () {
    // TODO: Add more expression tests (conditionals, etc)

    runTest(ThrowExpression(lexer.parseStr('throw 3')), 'throw 3');
    runTest(ThrowExpression(lexer.parseStr('throw "testThrow"')), 'throw "testThrow"');

    runTest(ThrowExpression(lexer.parseStr('throw Test()')), 'throw Test()');
    runTest(ThrowExpression(lexer.parseStr('throw Test(234)')), 'throw Test(234)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(234.432)')), 'throw Test(234.432)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(1,2,3,4)')), 'throw Test(1, 2, 3, 4)');
    runTest(ThrowExpression(lexer.parseStr('throw Test("someString")')), 'throw Test("someString")');
    runTest(ThrowExpression(lexer.parseStr('throw Test("1","2","3","4")')), 'throw Test("1", "2", "3", "4")');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someObject)')), 'throw Test(someObject)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someObject,someObject,someObject)')), 'throw Test(someObject, someObject, someObject)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:4)')), 'throw Test(someId: 4)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:true)')), 'throw Test(someId: true)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:"HI!")')), 'throw Test(someId: "HI!")');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject())')), 'throw Test(someId: someObject())');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject.param)')), 'throw Test(someId: someObject.param)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject.param ? another.param : meh.someVal)')), 'throw Test(someId: someObject.param ? another.param : meh.someVal)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:4,id2:.23)')), 'throw Test(someId: 4, id2: .23)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject(),id2:object2(),id3:object3())')), 'throw Test(someId: someObject(), id2: object2(), id3: object3())');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject.param, _id2:otherParam.param.subparam)')), 'throw Test(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'throw Test(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(234.432,someId:4)')), 'throw Test(234.432, someId: 4)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), 'throw Test(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(ThrowExpression(lexer.parseStr('throw Test("someString", someId:someObject(),id2:object2(),id3:object3())')), 'throw Test("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'throw Test(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(234,)')), 'throw Test(234,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(234.432,)')), 'throw Test(234.432,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(1,2,3,4,)')), 'throw Test(1, 2, 3, 4,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test("someString",)')), 'throw Test("someString",)');
    runTest(ThrowExpression(lexer.parseStr('throw Test("1","2","3","4",)')), 'throw Test("1", "2", "3", "4",)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someObject,)')), 'throw Test(someObject,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someObject,someObject,someObject,)')), 'throw Test(someObject, someObject, someObject,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:4,)')), 'throw Test(someId: 4,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:true,)')), 'throw Test(someId: true,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:"HI!",)')), 'throw Test(someId: "HI!",)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject(),)')), 'throw Test(someId: someObject(),)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject.param,)')), 'throw Test(someId: someObject.param,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject.param ? another.param : meh.someVal,)')), 'throw Test(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:4,id2:.23,)')), 'throw Test(someId: 4, id2: .23,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject(),id2:object2(),id3:object3(),)')), 'throw Test(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'throw Test(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'throw Test(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(234.432,someId:4,)')), 'throw Test(234.432, someId: 4,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'throw Test(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'throw Test("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ThrowExpression(lexer.parseStr('throw Test(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'throw Test(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType()')), 'throw Test.subType()');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(234)')), 'throw Test.subType(234)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(234.432)')), 'throw Test.subType(234.432)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(1,2,3,4)')), 'throw Test.subType(1, 2, 3, 4)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType("someString")')), 'throw Test.subType("someString")');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType("1","2","3","4")')), 'throw Test.subType("1", "2", "3", "4")');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someObject)')), 'throw Test.subType(someObject)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someObject,someObject,someObject)')), 'throw Test.subType(someObject, someObject, someObject)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:4)')), 'throw Test.subType(someId: 4)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:true)')), 'throw Test.subType(someId: true)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:"HI!")')), 'throw Test.subType(someId: "HI!")');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject())')), 'throw Test.subType(someId: someObject())');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject.param)')), 'throw Test.subType(someId: someObject.param)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject.param ? another.param : meh.someVal)')), 'throw Test.subType(someId: someObject.param ? another.param : meh.someVal)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:4,id2:.23)')), 'throw Test.subType(someId: 4, id2: .23)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject(),id2:object2(),id3:object3())')), 'throw Test.subType(someId: someObject(), id2: object2(), id3: object3())');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject.param, _id2:otherParam.param.subparam)')), 'throw Test.subType(someId: someObject.param, _id2: otherParam.param.subparam)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'throw Test.subType(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(234.432,someId:4)')), 'throw Test.subType(234.432, someId: 4)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal)')), 'throw Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType("someString", someId:someObject(),id2:object2(),id3:object3())')), 'throw Test.subType("someString", someId: someObject(), id2: object2(), id3: object3())');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10)')), 'throw Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(234,)')), 'throw Test.subType(234,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(234.432,)')), 'throw Test.subType(234.432,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(1,2,3,4,)')), 'throw Test.subType(1, 2, 3, 4,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType("someString",)')), 'throw Test.subType("someString",)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType("1","2","3","4",)')), 'throw Test.subType("1", "2", "3", "4",)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someObject,)')), 'throw Test.subType(someObject,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someObject,someObject,someObject,)')), 'throw Test.subType(someObject, someObject, someObject,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:4,)')), 'throw Test.subType(someId: 4,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:true,)')), 'throw Test.subType(someId: true,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:"HI!",)')), 'throw Test.subType(someId: "HI!",)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject(),)')), 'throw Test.subType(someId: someObject(),)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject.param,)')), 'throw Test.subType(someId: someObject.param,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), 'throw Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:4,id2:.23,)')), 'throw Test.subType(someId: 4, id2: .23,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), 'throw Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'throw Test.subType(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'throw Test.subType(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(234.432,someId:4,)')), 'throw Test.subType(234.432, someId: 4,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'throw Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'throw Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(ThrowExpression(lexer.parseStr('throw Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'throw Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

  });

  test('Multiplicative Expression', () {

    runTest(MultiplicativeExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(MultiplicativeExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(MultiplicativeExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(MultiplicativeExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(MultiplicativeExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(MultiplicativeExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(MultiplicativeExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(MultiplicativeExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(MultiplicativeExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(MultiplicativeExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(MultiplicativeExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(MultiplicativeExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');

    runTest(MultiplicativeExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(MultiplicativeExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(MultiplicativeExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(MultiplicativeExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(MultiplicativeExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(MultiplicativeExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(MultiplicativeExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(MultiplicativeExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(MultiplicativeExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(MultiplicativeExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(MultiplicativeExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(MultiplicativeExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');

  });

  test('Additive Expression', () {

    runTest(AdditiveExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(AdditiveExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(AdditiveExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(AdditiveExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(AdditiveExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(AdditiveExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(AdditiveExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(AdditiveExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(AdditiveExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(AdditiveExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(AdditiveExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(AdditiveExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(AdditiveExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(AdditiveExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(AdditiveExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(AdditiveExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(AdditiveExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(AdditiveExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(AdditiveExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(AdditiveExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(AdditiveExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(AdditiveExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(AdditiveExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(AdditiveExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');

    runTest(AdditiveExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(AdditiveExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(AdditiveExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(AdditiveExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(AdditiveExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(AdditiveExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(AdditiveExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(AdditiveExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2');

    runTest(AdditiveExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(AdditiveExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(AdditiveExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(AdditiveExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(AdditiveExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(AdditiveExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(AdditiveExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(AdditiveExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');

  });

  test('Shift Expression', () {

    runTest(ShiftExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(ShiftExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(ShiftExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(ShiftExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(ShiftExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(ShiftExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(ShiftExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(ShiftExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(ShiftExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(ShiftExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(ShiftExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(ShiftExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(ShiftExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(ShiftExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(ShiftExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(ShiftExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(ShiftExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(ShiftExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(ShiftExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(ShiftExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(ShiftExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(ShiftExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(ShiftExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(ShiftExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');
    runTest(ShiftExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(ShiftExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(ShiftExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(ShiftExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(ShiftExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(ShiftExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(ShiftExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(ShiftExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2');
    runTest(ShiftExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(ShiftExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(ShiftExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(ShiftExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(ShiftExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(ShiftExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(ShiftExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(ShiftExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');

    runTest(ShiftExpression(lexer.parseStr('9<<3')), '9 << 3');
    runTest(ShiftExpression(lexer.parseStr('3>>7')), '3 >> 7');
    runTest(ShiftExpression(lexer.parseStr('4>>6')), '4 >> 6');
    runTest(ShiftExpression(lexer.parseStr('1<<7>>0')), '1 << 7 >> 0');
    runTest(ShiftExpression(lexer.parseStr('2<<5<<2')), '2 << 5 << 2');
    runTest(ShiftExpression(lexer.parseStr('9>>>someInt>>8<<3')), '9 >>> someInt >> 8 << 3');
    runTest(ShiftExpression(lexer.parseStr('3+6%someInt>>>2')), '3 + 6 % someInt >>> 2');
    runTest(ShiftExpression(lexer.parseStr('1+someInt>>7~/0')), '1 + someInt >> 7 ~/ 0');
    runTest(ShiftExpression(lexer.parseStr('2~/5-someInt>>2')), '2 ~/ 5 - someInt >> 2');

    runTest(ShiftExpression(lexer.parseStr('super <<3')), 'super << 3');
    runTest(ShiftExpression(lexer.parseStr('super >>7')), 'super >> 7');
    runTest(ShiftExpression(lexer.parseStr('super >>6')), 'super >> 6');
    runTest(ShiftExpression(lexer.parseStr('super <<7>>0')), 'super << 7 >> 0');
    runTest(ShiftExpression(lexer.parseStr('super <<5<<2')), 'super << 5 << 2');
    runTest(ShiftExpression(lexer.parseStr('super >>>someInt>>8<<3')), 'super >>> someInt >> 8 << 3');
    runTest(ShiftExpression(lexer.parseStr('super +6%someInt>>>2')), 'super + 6 % someInt >>> 2');
    runTest(ShiftExpression(lexer.parseStr('super -someInt>>7~/0')), 'super - someInt >> 7 ~/ 0');
    runTest(ShiftExpression(lexer.parseStr('super ~/5-someInt>>2')), 'super ~/ 5 - someInt >> 2');

  });

  test('Bitwise And Expression', () {

    runTest(BitwiseAndExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(BitwiseAndExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(BitwiseAndExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(BitwiseAndExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(BitwiseAndExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(BitwiseAndExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(BitwiseAndExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(BitwiseAndExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(BitwiseAndExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(BitwiseAndExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(BitwiseAndExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(BitwiseAndExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(BitwiseAndExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(BitwiseAndExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(BitwiseAndExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(BitwiseAndExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(BitwiseAndExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(BitwiseAndExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(BitwiseAndExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(BitwiseAndExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(BitwiseAndExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(BitwiseAndExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(BitwiseAndExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(BitwiseAndExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');
    runTest(BitwiseAndExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(BitwiseAndExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(BitwiseAndExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(BitwiseAndExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(BitwiseAndExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(BitwiseAndExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(BitwiseAndExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(BitwiseAndExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2'); runTest(BitwiseAndExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(BitwiseAndExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(BitwiseAndExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(BitwiseAndExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(BitwiseAndExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(BitwiseAndExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(BitwiseAndExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(BitwiseAndExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');
    runTest(BitwiseAndExpression(lexer.parseStr('9<<3')), '9 << 3');
    runTest(BitwiseAndExpression(lexer.parseStr('3>>7')), '3 >> 7');
    runTest(BitwiseAndExpression(lexer.parseStr('4>>6')), '4 >> 6');
    runTest(BitwiseAndExpression(lexer.parseStr('1<<7>>0')), '1 << 7 >> 0');
    runTest(BitwiseAndExpression(lexer.parseStr('2<<5<<2')), '2 << 5 << 2');
    runTest(BitwiseAndExpression(lexer.parseStr('9>>>someInt>>8<<3')), '9 >>> someInt >> 8 << 3');
    runTest(BitwiseAndExpression(lexer.parseStr('3+6%someInt>>>2')), '3 + 6 % someInt >>> 2');
    runTest(BitwiseAndExpression(lexer.parseStr('1+someInt>>7~/0')), '1 + someInt >> 7 ~/ 0');
    runTest(BitwiseAndExpression(lexer.parseStr('2~/5-someInt>>2')), '2 ~/ 5 - someInt >> 2');
    runTest(BitwiseAndExpression(lexer.parseStr('super <<3')), 'super << 3');
    runTest(BitwiseAndExpression(lexer.parseStr('super >>7')), 'super >> 7');
    runTest(BitwiseAndExpression(lexer.parseStr('super >>6')), 'super >> 6');
    runTest(BitwiseAndExpression(lexer.parseStr('super <<7>>0')), 'super << 7 >> 0');
    runTest(BitwiseAndExpression(lexer.parseStr('super <<5<<2')), 'super << 5 << 2');
    runTest(BitwiseAndExpression(lexer.parseStr('super >>>someInt>>8<<3')), 'super >>> someInt >> 8 << 3');
    runTest(BitwiseAndExpression(lexer.parseStr('super +6%someInt>>>2')), 'super + 6 % someInt >>> 2');
    runTest(BitwiseAndExpression(lexer.parseStr('super -someInt>>7~/0')), 'super - someInt >> 7 ~/ 0');
    runTest(BitwiseAndExpression(lexer.parseStr('super ~/5-someInt>>2')), 'super ~/ 5 - someInt >> 2');

    runTest(BitwiseAndExpression(lexer.parseStr('4&6')), '4 & 6');
    runTest(BitwiseAndExpression(lexer.parseStr('1&7&0')), '1 & 7 & 0');
    runTest(BitwiseAndExpression(lexer.parseStr('9&someInt&8&3')), '9 & someInt & 8 & 3');
    runTest(BitwiseAndExpression(lexer.parseStr('3&6%someInt>>>2+0')), '3 & 6 % someInt >>> 2 + 0');
    runTest(BitwiseAndExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(BitwiseAndExpression(lexer.parseStr('2~/9%5-someInt&2')), '2 ~/ 9 % 5 - someInt & 2');

    runTest(BitwiseAndExpression(lexer.parseStr('super &6')), 'super & 6');
    runTest(BitwiseAndExpression(lexer.parseStr('super &7&0')), 'super & 7 & 0');
    runTest(BitwiseAndExpression(lexer.parseStr('super &someInt&8&3')), 'super & someInt & 8 & 3');
    runTest(BitwiseAndExpression(lexer.parseStr('super &6%someInt>>>2+0')), 'super & 6 % someInt >>> 2 + 0');
    runTest(BitwiseAndExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(BitwiseAndExpression(lexer.parseStr('super ~/9%5-someInt&2')), 'super ~/ 9 % 5 - someInt & 2');

  });

  test('Bitwise Xor Expression', () {

    runTest(BitwiseXorExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(BitwiseXorExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(BitwiseXorExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(BitwiseXorExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(BitwiseXorExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(BitwiseXorExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(BitwiseXorExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(BitwiseXorExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(BitwiseXorExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(BitwiseXorExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(BitwiseXorExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(BitwiseXorExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(BitwiseXorExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(BitwiseXorExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(BitwiseXorExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(BitwiseXorExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');
    runTest(BitwiseXorExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(BitwiseXorExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(BitwiseXorExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(BitwiseXorExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(BitwiseXorExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(BitwiseXorExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(BitwiseXorExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(BitwiseXorExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(BitwiseXorExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(BitwiseXorExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');
    runTest(BitwiseXorExpression(lexer.parseStr('9<<3')), '9 << 3');
    runTest(BitwiseXorExpression(lexer.parseStr('3>>7')), '3 >> 7');
    runTest(BitwiseXorExpression(lexer.parseStr('4>>6')), '4 >> 6');
    runTest(BitwiseXorExpression(lexer.parseStr('1<<7>>0')), '1 << 7 >> 0');
    runTest(BitwiseXorExpression(lexer.parseStr('2<<5<<2')), '2 << 5 << 2');
    runTest(BitwiseXorExpression(lexer.parseStr('9>>>someInt>>8<<3')), '9 >>> someInt >> 8 << 3');
    runTest(BitwiseXorExpression(lexer.parseStr('3+6%someInt>>>2')), '3 + 6 % someInt >>> 2');
    runTest(BitwiseXorExpression(lexer.parseStr('1+someInt>>7~/0')), '1 + someInt >> 7 ~/ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('2~/5-someInt>>2')), '2 ~/ 5 - someInt >> 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super <<3')), 'super << 3');
    runTest(BitwiseXorExpression(lexer.parseStr('super >>7')), 'super >> 7');
    runTest(BitwiseXorExpression(lexer.parseStr('super >>6')), 'super >> 6');
    runTest(BitwiseXorExpression(lexer.parseStr('super <<7>>0')), 'super << 7 >> 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super <<5<<2')), 'super << 5 << 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super >>>someInt>>8<<3')), 'super >>> someInt >> 8 << 3');
    runTest(BitwiseXorExpression(lexer.parseStr('super +6%someInt>>>2')), 'super + 6 % someInt >>> 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super -someInt>>7~/0')), 'super - someInt >> 7 ~/ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super ~/5-someInt>>2')), 'super ~/ 5 - someInt >> 2');
    runTest(BitwiseXorExpression(lexer.parseStr('4&6')), '4 & 6');
    runTest(BitwiseXorExpression(lexer.parseStr('1&7&0')), '1 & 7 & 0');
    runTest(BitwiseXorExpression(lexer.parseStr('9&someInt&8&3')), '9 & someInt & 8 & 3');
    runTest(BitwiseXorExpression(lexer.parseStr('3&6%someInt>>>2+0')), '3 & 6 % someInt >>> 2 + 0');
    runTest(BitwiseXorExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(BitwiseXorExpression(lexer.parseStr('2~/9%5-someInt&2')), '2 ~/ 9 % 5 - someInt & 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super &6')), 'super & 6');
    runTest(BitwiseXorExpression(lexer.parseStr('super &7&0')), 'super & 7 & 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super &someInt&8&3')), 'super & someInt & 8 & 3');
    runTest(BitwiseXorExpression(lexer.parseStr('super &6%someInt>>>2+0')), 'super & 6 % someInt >>> 2 + 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(BitwiseXorExpression(lexer.parseStr('super ~/9%5-someInt&2')), 'super ~/ 9 % 5 - someInt & 2');

    runTest(BitwiseXorExpression(lexer.parseStr('4^6')), '4 ^ 6');
    runTest(BitwiseXorExpression(lexer.parseStr('1^7^0')), '1 ^ 7 ^ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('9^someInt^8^3')), '9 ^ someInt ^ 8 ^ 3');
    runTest(BitwiseXorExpression(lexer.parseStr('3^6%someInt>>>2+0')), '3 ^ 6 % someInt >>> 2 + 0');
    runTest(BitwiseXorExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(BitwiseXorExpression(lexer.parseStr('2^9%5-someInt&2')), '2 ^ 9 % 5 - someInt & 2');

    runTest(BitwiseXorExpression(lexer.parseStr('super ^6')), 'super ^ 6');
    runTest(BitwiseXorExpression(lexer.parseStr('super ^7^0')), 'super ^ 7 ^ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super ^someInt^8^3')), 'super ^ someInt ^ 8 ^ 3');
    runTest(BitwiseXorExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(BitwiseXorExpression(lexer.parseStr('super ^9%5-someInt&2')), 'super ^ 9 % 5 - someInt & 2');

  });

  test('Bitwise And Expression', () {

    runTest(BitwiseOrExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(BitwiseOrExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(BitwiseOrExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(BitwiseOrExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(BitwiseOrExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(BitwiseOrExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(BitwiseOrExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(BitwiseOrExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(BitwiseOrExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(BitwiseOrExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(BitwiseOrExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(BitwiseOrExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(BitwiseOrExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(BitwiseOrExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(BitwiseOrExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(BitwiseOrExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(BitwiseOrExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(BitwiseOrExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(BitwiseOrExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(BitwiseOrExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');
    runTest(BitwiseOrExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(BitwiseOrExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(BitwiseOrExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(BitwiseOrExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(BitwiseOrExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(BitwiseOrExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(BitwiseOrExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(BitwiseOrExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(BitwiseOrExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(BitwiseOrExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(BitwiseOrExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(BitwiseOrExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(BitwiseOrExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');
    runTest(BitwiseOrExpression(lexer.parseStr('9<<3')), '9 << 3');
    runTest(BitwiseOrExpression(lexer.parseStr('3>>7')), '3 >> 7');
    runTest(BitwiseOrExpression(lexer.parseStr('4>>6')), '4 >> 6');
    runTest(BitwiseOrExpression(lexer.parseStr('1<<7>>0')), '1 << 7 >> 0');
    runTest(BitwiseOrExpression(lexer.parseStr('2<<5<<2')), '2 << 5 << 2');
    runTest(BitwiseOrExpression(lexer.parseStr('9>>>someInt>>8<<3')), '9 >>> someInt >> 8 << 3');
    runTest(BitwiseOrExpression(lexer.parseStr('3+6%someInt>>>2')), '3 + 6 % someInt >>> 2');
    runTest(BitwiseOrExpression(lexer.parseStr('1+someInt>>7~/0')), '1 + someInt >> 7 ~/ 0');
    runTest(BitwiseOrExpression(lexer.parseStr('2~/5-someInt>>2')), '2 ~/ 5 - someInt >> 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super <<3')), 'super << 3');
    runTest(BitwiseOrExpression(lexer.parseStr('super >>7')), 'super >> 7');
    runTest(BitwiseOrExpression(lexer.parseStr('super >>6')), 'super >> 6');
    runTest(BitwiseOrExpression(lexer.parseStr('super <<7>>0')), 'super << 7 >> 0');
    runTest(BitwiseOrExpression(lexer.parseStr('super <<5<<2')), 'super << 5 << 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super >>>someInt>>8<<3')), 'super >>> someInt >> 8 << 3');
    runTest(BitwiseOrExpression(lexer.parseStr('super +6%someInt>>>2')), 'super + 6 % someInt >>> 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super -someInt>>7~/0')), 'super - someInt >> 7 ~/ 0');
    runTest(BitwiseOrExpression(lexer.parseStr('super ~/5-someInt>>2')), 'super ~/ 5 - someInt >> 2');
    runTest(BitwiseOrExpression(lexer.parseStr('4&6')), '4 & 6');
    runTest(BitwiseOrExpression(lexer.parseStr('1&7&0')), '1 & 7 & 0');
    runTest(BitwiseOrExpression(lexer.parseStr('9&someInt&8&3')), '9 & someInt & 8 & 3');
    runTest(BitwiseOrExpression(lexer.parseStr('3&6%someInt>>>2+0')), '3 & 6 % someInt >>> 2 + 0');
    runTest(BitwiseOrExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(BitwiseOrExpression(lexer.parseStr('2~/9%5-someInt&2')), '2 ~/ 9 % 5 - someInt & 2');
    runTest(BitwiseOrExpression(lexer.parseStr('super &6')), 'super & 6');
    runTest(BitwiseOrExpression(lexer.parseStr('super &7&0')), 'super & 7 & 0');
    runTest(BitwiseOrExpression(lexer.parseStr('super &someInt&8&3')), 'super & someInt & 8 & 3');
    runTest(BitwiseOrExpression(lexer.parseStr('super &6%someInt>>>2+0')), 'super & 6 % someInt >>> 2 + 0');
    runTest(BitwiseOrExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(BitwiseOrExpression(lexer.parseStr('super ~/9%5-someInt&2')), 'super ~/ 9 % 5 - someInt & 2');
    runTest(BitwiseXorExpression(lexer.parseStr('4^6')), '4 ^ 6');
    runTest(BitwiseXorExpression(lexer.parseStr('1^7^0')), '1 ^ 7 ^ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('9^someInt^8^3')), '9 ^ someInt ^ 8 ^ 3');
    runTest(BitwiseXorExpression(lexer.parseStr('3^6%someInt>>>2+0')), '3 ^ 6 % someInt >>> 2 + 0');
    runTest(BitwiseXorExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(BitwiseXorExpression(lexer.parseStr('2^9%5-someInt&2')), '2 ^ 9 % 5 - someInt & 2');
    runTest(BitwiseXorExpression(lexer.parseStr('super ^6')), 'super ^ 6');
    runTest(BitwiseXorExpression(lexer.parseStr('super ^7^0')), 'super ^ 7 ^ 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super ^someInt^8^3')), 'super ^ someInt ^ 8 ^ 3');
    runTest(BitwiseXorExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(BitwiseXorExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(BitwiseXorExpression(lexer.parseStr('super ^9%5-someInt&2')), 'super ^ 9 % 5 - someInt & 2');


    runTest(BitwiseOrExpression(lexer.parseStr('4|6')), '4 | 6');
    runTest(BitwiseOrExpression(lexer.parseStr('1|7|0')), '1 | 7 | 0');
    runTest(BitwiseOrExpression(lexer.parseStr('9|someInt|8|3')), '9 | someInt | 8 | 3');
    runTest(BitwiseOrExpression(lexer.parseStr('3|6%someInt>>>2+0')), '3 | 6 % someInt >>> 2 + 0');
    runTest(BitwiseOrExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(BitwiseOrExpression(lexer.parseStr('2~/9%5-someInt|2')), '2 ~/ 9 % 5 - someInt | 2');

    runTest(BitwiseOrExpression(lexer.parseStr('super |6')), 'super | 6');
    runTest(BitwiseOrExpression(lexer.parseStr('super |7|0')), 'super | 7 | 0');
    runTest(BitwiseOrExpression(lexer.parseStr('super |someInt|8|3')), 'super | someInt | 8 | 3');
    runTest(BitwiseOrExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(BitwiseOrExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(BitwiseOrExpression(lexer.parseStr('super ~/9%5-someInt|2')), 'super ~/ 9 % 5 - someInt | 2');

  });

  test('Relational Expression', () {

    runTest(RelationalExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(RelationalExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(RelationalExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(RelationalExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(RelationalExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(RelationalExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(RelationalExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(RelationalExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(RelationalExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(RelationalExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(RelationalExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(RelationalExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(RelationalExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(RelationalExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(RelationalExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(RelationalExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(RelationalExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(RelationalExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(RelationalExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(RelationalExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(RelationalExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(RelationalExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(RelationalExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(RelationalExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');
    runTest(RelationalExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(RelationalExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(RelationalExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(RelationalExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(RelationalExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(RelationalExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(RelationalExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(RelationalExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2');
    runTest(RelationalExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(RelationalExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(RelationalExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(RelationalExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(RelationalExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(RelationalExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(RelationalExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(RelationalExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');
    runTest(RelationalExpression(lexer.parseStr('9<<3')), '9 << 3');
    runTest(RelationalExpression(lexer.parseStr('3>>7')), '3 >> 7');
    runTest(RelationalExpression(lexer.parseStr('4>>6')), '4 >> 6');
    runTest(RelationalExpression(lexer.parseStr('1<<7>>0')), '1 << 7 >> 0');
    runTest(RelationalExpression(lexer.parseStr('2<<5<<2')), '2 << 5 << 2');
    runTest(RelationalExpression(lexer.parseStr('9>>>someInt>>8<<3')), '9 >>> someInt >> 8 << 3');
    runTest(RelationalExpression(lexer.parseStr('3+6%someInt>>>2')), '3 + 6 % someInt >>> 2');
    runTest(RelationalExpression(lexer.parseStr('1+someInt>>7~/0')), '1 + someInt >> 7 ~/ 0');
    runTest(RelationalExpression(lexer.parseStr('2~/5-someInt>>2')), '2 ~/ 5 - someInt >> 2');
    runTest(RelationalExpression(lexer.parseStr('super <<3')), 'super << 3');
    runTest(RelationalExpression(lexer.parseStr('super >>7')), 'super >> 7');
    runTest(RelationalExpression(lexer.parseStr('super >>6')), 'super >> 6');
    runTest(RelationalExpression(lexer.parseStr('super <<7>>0')), 'super << 7 >> 0');
    runTest(RelationalExpression(lexer.parseStr('super <<5<<2')), 'super << 5 << 2');
    runTest(RelationalExpression(lexer.parseStr('super >>>someInt>>8<<3')), 'super >>> someInt >> 8 << 3');
    runTest(RelationalExpression(lexer.parseStr('super +6%someInt>>>2')), 'super + 6 % someInt >>> 2');
    runTest(RelationalExpression(lexer.parseStr('super -someInt>>7~/0')), 'super - someInt >> 7 ~/ 0');
    runTest(RelationalExpression(lexer.parseStr('super ~/5-someInt>>2')), 'super ~/ 5 - someInt >> 2');
    runTest(RelationalExpression(lexer.parseStr('4&6')), '4 & 6');
    runTest(RelationalExpression(lexer.parseStr('1&7&0')), '1 & 7 & 0');
    runTest(RelationalExpression(lexer.parseStr('9&someInt&8&3')), '9 & someInt & 8 & 3');
    runTest(RelationalExpression(lexer.parseStr('3&6%someInt>>>2+0')), '3 & 6 % someInt >>> 2 + 0');
    runTest(RelationalExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(RelationalExpression(lexer.parseStr('2~/9%5-someInt&2')), '2 ~/ 9 % 5 - someInt & 2');
    runTest(RelationalExpression(lexer.parseStr('super &6')), 'super & 6');
    runTest(RelationalExpression(lexer.parseStr('super &7&0')), 'super & 7 & 0');
    runTest(RelationalExpression(lexer.parseStr('super &someInt&8&3')), 'super & someInt & 8 & 3');
    runTest(RelationalExpression(lexer.parseStr('super &6%someInt>>>2+0')), 'super & 6 % someInt >>> 2 + 0');
    runTest(RelationalExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(RelationalExpression(lexer.parseStr('super ~/9%5-someInt&2')), 'super ~/ 9 % 5 - someInt & 2');
    runTest(RelationalExpression(lexer.parseStr('4^6')), '4 ^ 6');
    runTest(RelationalExpression(lexer.parseStr('1^7^0')), '1 ^ 7 ^ 0');
    runTest(RelationalExpression(lexer.parseStr('9^someInt^8^3')), '9 ^ someInt ^ 8 ^ 3');
    runTest(RelationalExpression(lexer.parseStr('3^6%someInt>>>2+0')), '3 ^ 6 % someInt >>> 2 + 0');
    runTest(RelationalExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(RelationalExpression(lexer.parseStr('2^9%5-someInt&2')), '2 ^ 9 % 5 - someInt & 2');
    runTest(RelationalExpression(lexer.parseStr('super ^6')), 'super ^ 6');
    runTest(RelationalExpression(lexer.parseStr('super ^7^0')), 'super ^ 7 ^ 0');
    runTest(RelationalExpression(lexer.parseStr('super ^someInt^8^3')), 'super ^ someInt ^ 8 ^ 3');
    runTest(RelationalExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(RelationalExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(RelationalExpression(lexer.parseStr('super ^9%5-someInt&2')), 'super ^ 9 % 5 - someInt & 2');
    runTest(RelationalExpression(lexer.parseStr('4|6')), '4 | 6');
    runTest(RelationalExpression(lexer.parseStr('1|7|0')), '1 | 7 | 0');
    runTest(RelationalExpression(lexer.parseStr('9|someInt|8|3')), '9 | someInt | 8 | 3');
    runTest(RelationalExpression(lexer.parseStr('3|6%someInt>>>2+0')), '3 | 6 % someInt >>> 2 + 0');
    runTest(RelationalExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(RelationalExpression(lexer.parseStr('2~/9%5-someInt|2')), '2 ~/ 9 % 5 - someInt | 2');
    runTest(RelationalExpression(lexer.parseStr('super |6')), 'super | 6');
    runTest(RelationalExpression(lexer.parseStr('super |7|0')), 'super | 7 | 0');
    runTest(RelationalExpression(lexer.parseStr('super |someInt|8|3')), 'super | someInt | 8 | 3');
    runTest(RelationalExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(RelationalExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(RelationalExpression(lexer.parseStr('super ~/9%5-someInt|2')), 'super ~/ 9 % 5 - someInt | 2');

    runTest(RelationalExpression(lexer.parseStr('4<=6')), '4 <= 6');
    runTest(RelationalExpression(lexer.parseStr('1<7')), '1 < 7');
    runTest(RelationalExpression(lexer.parseStr('9>=someInt<<8')), '9 >= someInt << 8');
    runTest(RelationalExpression(lexer.parseStr('3|6%someInt>>>2+0<=8')), '3 | 6 % someInt >>> 2 + 0 <= 8');
    runTest(RelationalExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(RelationalExpression(lexer.parseStr('2~/9%5>8-someInt|2')), '2 ~/ 9 % 5 > 8 - someInt | 2');

    runTest(RelationalExpression(lexer.parseStr('super <=6')), 'super <= 6');
    runTest(RelationalExpression(lexer.parseStr('super <7')), 'super < 7');
    runTest(RelationalExpression(lexer.parseStr('super >=someInt<<8')), 'super >= someInt << 8');
    runTest(RelationalExpression(lexer.parseStr('super |6%someInt>>>2+0<=8')), 'super | 6 % someInt >>> 2 + 0 <= 8');
    runTest(RelationalExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(RelationalExpression(lexer.parseStr('super ~/9%5>8-someInt|2')), 'super ~/ 9 % 5 > 8 - someInt | 2');

    runTest(RelationalExpression(lexer.parseStr('4is someType')), '4 is someType');
    runTest(RelationalExpression(lexer.parseStr('1 is! someType')), '1 is! someType');
    runTest(RelationalExpression(lexer.parseStr('5as someType')), '5 as someType');

    //TODO: add more tests for different test types
  });

  test('Equality Expression', () {

    runTest(EqualityExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(EqualityExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(EqualityExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(EqualityExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(EqualityExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(EqualityExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(EqualityExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(EqualityExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(EqualityExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(EqualityExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(EqualityExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(EqualityExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(EqualityExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(EqualityExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(EqualityExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(EqualityExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(EqualityExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(EqualityExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(EqualityExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(EqualityExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(EqualityExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(EqualityExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(EqualityExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(EqualityExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');
    runTest(EqualityExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(EqualityExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(EqualityExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(EqualityExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(EqualityExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(EqualityExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(EqualityExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(EqualityExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2');
    runTest(EqualityExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(EqualityExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(EqualityExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(EqualityExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(EqualityExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(EqualityExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(EqualityExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(EqualityExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');
    runTest(EqualityExpression(lexer.parseStr('9<<3')), '9 << 3');
    runTest(EqualityExpression(lexer.parseStr('3>>7')), '3 >> 7');
    runTest(EqualityExpression(lexer.parseStr('4>>6')), '4 >> 6');
    runTest(EqualityExpression(lexer.parseStr('1<<7>>0')), '1 << 7 >> 0');
    runTest(EqualityExpression(lexer.parseStr('2<<5<<2')), '2 << 5 << 2');
    runTest(EqualityExpression(lexer.parseStr('9>>>someInt>>8<<3')), '9 >>> someInt >> 8 << 3');
    runTest(EqualityExpression(lexer.parseStr('3+6%someInt>>>2')), '3 + 6 % someInt >>> 2');
    runTest(EqualityExpression(lexer.parseStr('1+someInt>>7~/0')), '1 + someInt >> 7 ~/ 0');
    runTest(EqualityExpression(lexer.parseStr('2~/5-someInt>>2')), '2 ~/ 5 - someInt >> 2');
    runTest(EqualityExpression(lexer.parseStr('super <<3')), 'super << 3');
    runTest(EqualityExpression(lexer.parseStr('super >>7')), 'super >> 7');
    runTest(EqualityExpression(lexer.parseStr('super >>6')), 'super >> 6');
    runTest(EqualityExpression(lexer.parseStr('super <<7>>0')), 'super << 7 >> 0');
    runTest(EqualityExpression(lexer.parseStr('super <<5<<2')), 'super << 5 << 2');
    runTest(EqualityExpression(lexer.parseStr('super >>>someInt>>8<<3')), 'super >>> someInt >> 8 << 3');
    runTest(EqualityExpression(lexer.parseStr('super +6%someInt>>>2')), 'super + 6 % someInt >>> 2');
    runTest(EqualityExpression(lexer.parseStr('super -someInt>>7~/0')), 'super - someInt >> 7 ~/ 0');
    runTest(EqualityExpression(lexer.parseStr('super ~/5-someInt>>2')), 'super ~/ 5 - someInt >> 2');
    runTest(EqualityExpression(lexer.parseStr('4&6')), '4 & 6');
    runTest(EqualityExpression(lexer.parseStr('1&7&0')), '1 & 7 & 0');
    runTest(EqualityExpression(lexer.parseStr('9&someInt&8&3')), '9 & someInt & 8 & 3');
    runTest(EqualityExpression(lexer.parseStr('3&6%someInt>>>2+0')), '3 & 6 % someInt >>> 2 + 0');
    runTest(EqualityExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(EqualityExpression(lexer.parseStr('2~/9%5-someInt&2')), '2 ~/ 9 % 5 - someInt & 2');
    runTest(EqualityExpression(lexer.parseStr('super &6')), 'super & 6');
    runTest(EqualityExpression(lexer.parseStr('super &7&0')), 'super & 7 & 0');
    runTest(EqualityExpression(lexer.parseStr('super &someInt&8&3')), 'super & someInt & 8 & 3');
    runTest(EqualityExpression(lexer.parseStr('super &6%someInt>>>2+0')), 'super & 6 % someInt >>> 2 + 0');
    runTest(EqualityExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(EqualityExpression(lexer.parseStr('super ~/9%5-someInt&2')), 'super ~/ 9 % 5 - someInt & 2');
    runTest(EqualityExpression(lexer.parseStr('4^6')), '4 ^ 6');
    runTest(EqualityExpression(lexer.parseStr('1^7^0')), '1 ^ 7 ^ 0');
    runTest(EqualityExpression(lexer.parseStr('9^someInt^8^3')), '9 ^ someInt ^ 8 ^ 3');
    runTest(EqualityExpression(lexer.parseStr('3^6%someInt>>>2+0')), '3 ^ 6 % someInt >>> 2 + 0');
    runTest(EqualityExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(EqualityExpression(lexer.parseStr('2^9%5-someInt&2')), '2 ^ 9 % 5 - someInt & 2');
    runTest(EqualityExpression(lexer.parseStr('super ^6')), 'super ^ 6');
    runTest(EqualityExpression(lexer.parseStr('super ^7^0')), 'super ^ 7 ^ 0');
    runTest(EqualityExpression(lexer.parseStr('super ^someInt^8^3')), 'super ^ someInt ^ 8 ^ 3');
    runTest(EqualityExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(EqualityExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(EqualityExpression(lexer.parseStr('super ^9%5-someInt&2')), 'super ^ 9 % 5 - someInt & 2');
    runTest(EqualityExpression(lexer.parseStr('4|6')), '4 | 6');
    runTest(EqualityExpression(lexer.parseStr('1|7|0')), '1 | 7 | 0');
    runTest(EqualityExpression(lexer.parseStr('9|someInt|8|3')), '9 | someInt | 8 | 3');
    runTest(EqualityExpression(lexer.parseStr('3|6%someInt>>>2+0')), '3 | 6 % someInt >>> 2 + 0');
    runTest(EqualityExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(EqualityExpression(lexer.parseStr('2~/9%5-someInt|2')), '2 ~/ 9 % 5 - someInt | 2');
    runTest(EqualityExpression(lexer.parseStr('super |6')), 'super | 6');
    runTest(EqualityExpression(lexer.parseStr('super |7|0')), 'super | 7 | 0');
    runTest(EqualityExpression(lexer.parseStr('super |someInt|8|3')), 'super | someInt | 8 | 3');
    runTest(EqualityExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(EqualityExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(EqualityExpression(lexer.parseStr('super ~/9%5-someInt|2')), 'super ~/ 9 % 5 - someInt | 2');
    runTest(EqualityExpression(lexer.parseStr('4<=6')), '4 <= 6');
    runTest(EqualityExpression(lexer.parseStr('1<7')), '1 < 7');
    runTest(EqualityExpression(lexer.parseStr('9>=someInt<<8')), '9 >= someInt << 8');
    runTest(EqualityExpression(lexer.parseStr('3|6%someInt>>>2+0<=8')), '3 | 6 % someInt >>> 2 + 0 <= 8');
    runTest(EqualityExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(EqualityExpression(lexer.parseStr('2~/9%5>8-someInt|2')), '2 ~/ 9 % 5 > 8 - someInt | 2');
    runTest(EqualityExpression(lexer.parseStr('super <=6')), 'super <= 6');
    runTest(EqualityExpression(lexer.parseStr('super <7')), 'super < 7');
    runTest(EqualityExpression(lexer.parseStr('super >=someInt<<8')), 'super >= someInt << 8');
    runTest(EqualityExpression(lexer.parseStr('super |6%someInt>>>2+0<=8')), 'super | 6 % someInt >>> 2 + 0 <= 8');
    runTest(EqualityExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(EqualityExpression(lexer.parseStr('super ~/9%5>8-someInt|2')), 'super ~/ 9 % 5 > 8 - someInt | 2');
    runTest(EqualityExpression(lexer.parseStr('4is someType')), '4 is someType');
    runTest(EqualityExpression(lexer.parseStr('1 is! someType')), '1 is! someType');
    runTest(EqualityExpression(lexer.parseStr('5as someType')), '5 as someType');

    runTest(EqualityExpression(lexer.parseStr('4==6')), '4 == 6');
    runTest(EqualityExpression(lexer.parseStr('1!=7')), '1 != 7');
    runTest(EqualityExpression(lexer.parseStr('9==someInt<<8')), '9 == someInt << 8');
    runTest(EqualityExpression(lexer.parseStr('3|6%someInt>>>2+0!=8')), '3 | 6 % someInt >>> 2 + 0 != 8');
    runTest(EqualityExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(EqualityExpression(lexer.parseStr('2~/9%5==8-someInt|2')), '2 ~/ 9 % 5 == 8 - someInt | 2');

    runTest(EqualityExpression(lexer.parseStr('super ==6')), 'super == 6');
    runTest(EqualityExpression(lexer.parseStr('super !=7')), 'super != 7');
    runTest(EqualityExpression(lexer.parseStr('super ==someInt<<8')), 'super == someInt << 8');
    runTest(EqualityExpression(lexer.parseStr('super |6%someInt>>>2+0!=8')), 'super | 6 % someInt >>> 2 + 0 != 8');
    runTest(EqualityExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(EqualityExpression(lexer.parseStr('super ~/9%5==8-someInt|2')), 'super ~/ 9 % 5 == 8 - someInt | 2');

  });

  test('Logical And Expression', () {

    runTest(LogicalAndExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(LogicalAndExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(LogicalAndExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(LogicalAndExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(LogicalAndExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(LogicalAndExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(LogicalAndExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(LogicalAndExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(LogicalAndExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(LogicalAndExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(LogicalAndExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(LogicalAndExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(LogicalAndExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(LogicalAndExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(LogicalAndExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(LogicalAndExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(LogicalAndExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(LogicalAndExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(LogicalAndExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(LogicalAndExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(LogicalAndExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(LogicalAndExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(LogicalAndExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(LogicalAndExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');
    runTest(LogicalAndExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(LogicalAndExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(LogicalAndExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(LogicalAndExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(LogicalAndExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(LogicalAndExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(LogicalAndExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(LogicalAndExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2');
    runTest(LogicalAndExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(LogicalAndExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(LogicalAndExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(LogicalAndExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(LogicalAndExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(LogicalAndExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(LogicalAndExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(LogicalAndExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');
    runTest(LogicalAndExpression(lexer.parseStr('9<<3')), '9 << 3');
    runTest(LogicalAndExpression(lexer.parseStr('3>>7')), '3 >> 7');
    runTest(LogicalAndExpression(lexer.parseStr('4>>6')), '4 >> 6');
    runTest(LogicalAndExpression(lexer.parseStr('1<<7>>0')), '1 << 7 >> 0');
    runTest(LogicalAndExpression(lexer.parseStr('2<<5<<2')), '2 << 5 << 2');
    runTest(LogicalAndExpression(lexer.parseStr('9>>>someInt>>8<<3')), '9 >>> someInt >> 8 << 3');
    runTest(LogicalAndExpression(lexer.parseStr('3+6%someInt>>>2')), '3 + 6 % someInt >>> 2');
    runTest(LogicalAndExpression(lexer.parseStr('1+someInt>>7~/0')), '1 + someInt >> 7 ~/ 0');
    runTest(LogicalAndExpression(lexer.parseStr('2~/5-someInt>>2')), '2 ~/ 5 - someInt >> 2');
    runTest(LogicalAndExpression(lexer.parseStr('super <<3')), 'super << 3');
    runTest(LogicalAndExpression(lexer.parseStr('super >>7')), 'super >> 7');
    runTest(LogicalAndExpression(lexer.parseStr('super >>6')), 'super >> 6');
    runTest(LogicalAndExpression(lexer.parseStr('super <<7>>0')), 'super << 7 >> 0');
    runTest(LogicalAndExpression(lexer.parseStr('super <<5<<2')), 'super << 5 << 2');
    runTest(LogicalAndExpression(lexer.parseStr('super >>>someInt>>8<<3')), 'super >>> someInt >> 8 << 3');
    runTest(LogicalAndExpression(lexer.parseStr('super +6%someInt>>>2')), 'super + 6 % someInt >>> 2');
    runTest(LogicalAndExpression(lexer.parseStr('super -someInt>>7~/0')), 'super - someInt >> 7 ~/ 0');
    runTest(LogicalAndExpression(lexer.parseStr('super ~/5-someInt>>2')), 'super ~/ 5 - someInt >> 2');
    runTest(LogicalAndExpression(lexer.parseStr('4&6')), '4 & 6');
    runTest(LogicalAndExpression(lexer.parseStr('1&7&0')), '1 & 7 & 0');
    runTest(LogicalAndExpression(lexer.parseStr('9&someInt&8&3')), '9 & someInt & 8 & 3');
    runTest(LogicalAndExpression(lexer.parseStr('3&6%someInt>>>2+0')), '3 & 6 % someInt >>> 2 + 0');
    runTest(LogicalAndExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('2~/9%5-someInt&2')), '2 ~/ 9 % 5 - someInt & 2');
    runTest(LogicalAndExpression(lexer.parseStr('super &6')), 'super & 6');
    runTest(LogicalAndExpression(lexer.parseStr('super &7&0')), 'super & 7 & 0');
    runTest(LogicalAndExpression(lexer.parseStr('super &someInt&8&3')), 'super & someInt & 8 & 3');
    runTest(LogicalAndExpression(lexer.parseStr('super &6%someInt>>>2+0')), 'super & 6 % someInt >>> 2 + 0');
    runTest(LogicalAndExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('super ~/9%5-someInt&2')), 'super ~/ 9 % 5 - someInt & 2');
    runTest(LogicalAndExpression(lexer.parseStr('4^6')), '4 ^ 6');
    runTest(LogicalAndExpression(lexer.parseStr('1^7^0')), '1 ^ 7 ^ 0');
    runTest(LogicalAndExpression(lexer.parseStr('9^someInt^8^3')), '9 ^ someInt ^ 8 ^ 3');
    runTest(LogicalAndExpression(lexer.parseStr('3^6%someInt>>>2+0')), '3 ^ 6 % someInt >>> 2 + 0');
    runTest(LogicalAndExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('2^9%5-someInt&2')), '2 ^ 9 % 5 - someInt & 2');
    runTest(LogicalAndExpression(lexer.parseStr('super ^6')), 'super ^ 6');
    runTest(LogicalAndExpression(lexer.parseStr('super ^7^0')), 'super ^ 7 ^ 0');
    runTest(LogicalAndExpression(lexer.parseStr('super ^someInt^8^3')), 'super ^ someInt ^ 8 ^ 3');
    runTest(LogicalAndExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(LogicalAndExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('super ^9%5-someInt&2')), 'super ^ 9 % 5 - someInt & 2');
    runTest(LogicalAndExpression(lexer.parseStr('4|6')), '4 | 6');
    runTest(LogicalAndExpression(lexer.parseStr('1|7|0')), '1 | 7 | 0');
    runTest(LogicalAndExpression(lexer.parseStr('9|someInt|8|3')), '9 | someInt | 8 | 3');
    runTest(LogicalAndExpression(lexer.parseStr('3|6%someInt>>>2+0')), '3 | 6 % someInt >>> 2 + 0');
    runTest(LogicalAndExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('2~/9%5-someInt|2')), '2 ~/ 9 % 5 - someInt | 2');
    runTest(LogicalAndExpression(lexer.parseStr('super |6')), 'super | 6');
    runTest(LogicalAndExpression(lexer.parseStr('super |7|0')), 'super | 7 | 0');
    runTest(LogicalAndExpression(lexer.parseStr('super |someInt|8|3')), 'super | someInt | 8 | 3');
    runTest(LogicalAndExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(LogicalAndExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('super ~/9%5-someInt|2')), 'super ~/ 9 % 5 - someInt | 2');
    runTest(LogicalAndExpression(lexer.parseStr('4<=6')), '4 <= 6');
    runTest(LogicalAndExpression(lexer.parseStr('1<7')), '1 < 7');
    runTest(LogicalAndExpression(lexer.parseStr('9>=someInt<<8')), '9 >= someInt << 8');
    runTest(LogicalAndExpression(lexer.parseStr('3|6%someInt>>>2+0<=8')), '3 | 6 % someInt >>> 2 + 0 <= 8');
    runTest(LogicalAndExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('2~/9%5>8-someInt|2')), '2 ~/ 9 % 5 > 8 - someInt | 2');
    runTest(LogicalAndExpression(lexer.parseStr('super <=6')), 'super <= 6');
    runTest(LogicalAndExpression(lexer.parseStr('super <7')), 'super < 7');
    runTest(LogicalAndExpression(lexer.parseStr('super >=someInt<<8')), 'super >= someInt << 8');
    runTest(LogicalAndExpression(lexer.parseStr('super |6%someInt>>>2+0<=8')), 'super | 6 % someInt >>> 2 + 0 <= 8');
    runTest(LogicalAndExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('super ~/9%5>8-someInt|2')), 'super ~/ 9 % 5 > 8 - someInt | 2');
    runTest(LogicalAndExpression(lexer.parseStr('4is someType')), '4 is someType');
    runTest(LogicalAndExpression(lexer.parseStr('1 is! someType')), '1 is! someType');
    runTest(LogicalAndExpression(lexer.parseStr('5as someType')), '5 as someType');
    runTest(LogicalAndExpression(lexer.parseStr('4==6')), '4 == 6');
    runTest(LogicalAndExpression(lexer.parseStr('1!=7')), '1 != 7');
    runTest(LogicalAndExpression(lexer.parseStr('9==someInt<<8')), '9 == someInt << 8');
    runTest(LogicalAndExpression(lexer.parseStr('3|6%someInt>>>2+0!=8')), '3 | 6 % someInt >>> 2 + 0 != 8');
    runTest(LogicalAndExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('2~/9%5==8-someInt|2')), '2 ~/ 9 % 5 == 8 - someInt | 2');
    runTest(LogicalAndExpression(lexer.parseStr('super ==6')), 'super == 6');
    runTest(LogicalAndExpression(lexer.parseStr('super !=7')), 'super != 7');
    runTest(LogicalAndExpression(lexer.parseStr('super ==someInt<<8')), 'super == someInt << 8');
    runTest(LogicalAndExpression(lexer.parseStr('super |6%someInt>>>2+0!=8')), 'super | 6 % someInt >>> 2 + 0 != 8');
    runTest(LogicalAndExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('super ~/9%5==8-someInt|2')), 'super ~/ 9 % 5 == 8 - someInt | 2');

    runTest(LogicalAndExpression(lexer.parseStr('4&&6')), '4 && 6');
    runTest(LogicalAndExpression(lexer.parseStr('1&&7')), '1 && 7');
    runTest(LogicalAndExpression(lexer.parseStr('9&&someInt<<8')), '9 && someInt << 8');
    runTest(LogicalAndExpression(lexer.parseStr('3|6%someInt>>>2+0&&8')), '3 | 6 % someInt >>> 2 + 0 && 8');
    runTest(LogicalAndExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('2~/9%5&&8-someInt|2')), '2 ~/ 9 % 5 && 8 - someInt | 2');

    runTest(LogicalAndExpression(lexer.parseStr('super &&6')), 'super && 6');
    runTest(LogicalAndExpression(lexer.parseStr('super &&7')), 'super && 7');
    runTest(LogicalAndExpression(lexer.parseStr('super &&someInt<<8')), 'super && someInt << 8');
    runTest(LogicalAndExpression(lexer.parseStr('super |6%someInt>>>2+0&&8')), 'super | 6 % someInt >>> 2 + 0 && 8');
    runTest(LogicalAndExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalAndExpression(lexer.parseStr('super ~/9%5&&8-someInt|2')), 'super ~/ 9 % 5 && 8 - someInt | 2');

  });

  test('Logical Or Expression', () {

    runTest(LogicalOrExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(LogicalOrExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(LogicalOrExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(LogicalOrExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(LogicalOrExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(LogicalOrExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(LogicalOrExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(LogicalOrExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(LogicalOrExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(LogicalOrExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(LogicalOrExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(LogicalOrExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(LogicalOrExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(LogicalOrExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(LogicalOrExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(LogicalOrExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(LogicalOrExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(LogicalOrExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(LogicalOrExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(LogicalOrExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(LogicalOrExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');
    runTest(LogicalOrExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(LogicalOrExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(LogicalOrExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(LogicalOrExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(LogicalOrExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(LogicalOrExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(LogicalOrExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(LogicalOrExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2');
    runTest(LogicalOrExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(LogicalOrExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(LogicalOrExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(LogicalOrExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(LogicalOrExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(LogicalOrExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(LogicalOrExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');
    runTest(LogicalOrExpression(lexer.parseStr('9<<3')), '9 << 3');
    runTest(LogicalOrExpression(lexer.parseStr('3>>7')), '3 >> 7');
    runTest(LogicalOrExpression(lexer.parseStr('4>>6')), '4 >> 6');
    runTest(LogicalOrExpression(lexer.parseStr('1<<7>>0')), '1 << 7 >> 0');
    runTest(LogicalOrExpression(lexer.parseStr('2<<5<<2')), '2 << 5 << 2');
    runTest(LogicalOrExpression(lexer.parseStr('9>>>someInt>>8<<3')), '9 >>> someInt >> 8 << 3');
    runTest(LogicalOrExpression(lexer.parseStr('3+6%someInt>>>2')), '3 + 6 % someInt >>> 2');
    runTest(LogicalOrExpression(lexer.parseStr('1+someInt>>7~/0')), '1 + someInt >> 7 ~/ 0');
    runTest(LogicalOrExpression(lexer.parseStr('2~/5-someInt>>2')), '2 ~/ 5 - someInt >> 2');
    runTest(LogicalOrExpression(lexer.parseStr('super <<3')), 'super << 3');
    runTest(LogicalOrExpression(lexer.parseStr('super >>7')), 'super >> 7');
    runTest(LogicalOrExpression(lexer.parseStr('super >>6')), 'super >> 6');
    runTest(LogicalOrExpression(lexer.parseStr('super <<7>>0')), 'super << 7 >> 0');
    runTest(LogicalOrExpression(lexer.parseStr('super <<5<<2')), 'super << 5 << 2');
    runTest(LogicalOrExpression(lexer.parseStr('super >>>someInt>>8<<3')), 'super >>> someInt >> 8 << 3');
    runTest(LogicalOrExpression(lexer.parseStr('super +6%someInt>>>2')), 'super + 6 % someInt >>> 2');
    runTest(LogicalOrExpression(lexer.parseStr('super -someInt>>7~/0')), 'super - someInt >> 7 ~/ 0');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/5-someInt>>2')), 'super ~/ 5 - someInt >> 2');
    runTest(LogicalOrExpression(lexer.parseStr('4&6')), '4 & 6');
    runTest(LogicalOrExpression(lexer.parseStr('1&7&0')), '1 & 7 & 0');
    runTest(LogicalOrExpression(lexer.parseStr('9&someInt&8&3')), '9 & someInt & 8 & 3');
    runTest(LogicalOrExpression(lexer.parseStr('3&6%someInt>>>2+0')), '3 & 6 % someInt >>> 2 + 0');
    runTest(LogicalOrExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('2~/9%5-someInt&2')), '2 ~/ 9 % 5 - someInt & 2');
    runTest(LogicalOrExpression(lexer.parseStr('super &6')), 'super & 6');
    runTest(LogicalOrExpression(lexer.parseStr('super &7&0')), 'super & 7 & 0');
    runTest(LogicalOrExpression(lexer.parseStr('super &someInt&8&3')), 'super & someInt & 8 & 3');
    runTest(LogicalOrExpression(lexer.parseStr('super &6%someInt>>>2+0')), 'super & 6 % someInt >>> 2 + 0');
    runTest(LogicalOrExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/9%5-someInt&2')), 'super ~/ 9 % 5 - someInt & 2');
    runTest(LogicalOrExpression(lexer.parseStr('4^6')), '4 ^ 6');
    runTest(LogicalOrExpression(lexer.parseStr('1^7^0')), '1 ^ 7 ^ 0');
    runTest(LogicalOrExpression(lexer.parseStr('9^someInt^8^3')), '9 ^ someInt ^ 8 ^ 3');
    runTest(LogicalOrExpression(lexer.parseStr('3^6%someInt>>>2+0')), '3 ^ 6 % someInt >>> 2 + 0');
    runTest(LogicalOrExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('2^9%5-someInt&2')), '2 ^ 9 % 5 - someInt & 2');
    runTest(LogicalOrExpression(lexer.parseStr('super ^6')), 'super ^ 6');
    runTest(LogicalOrExpression(lexer.parseStr('super ^7^0')), 'super ^ 7 ^ 0');
    runTest(LogicalOrExpression(lexer.parseStr('super ^someInt^8^3')), 'super ^ someInt ^ 8 ^ 3');
    runTest(LogicalOrExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(LogicalOrExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('super ^9%5-someInt&2')), 'super ^ 9 % 5 - someInt & 2');
    runTest(LogicalOrExpression(lexer.parseStr('4|6')), '4 | 6');
    runTest(LogicalOrExpression(lexer.parseStr('1|7|0')), '1 | 7 | 0');
    runTest(LogicalOrExpression(lexer.parseStr('9|someInt|8|3')), '9 | someInt | 8 | 3');
    runTest(LogicalOrExpression(lexer.parseStr('3|6%someInt>>>2+0')), '3 | 6 % someInt >>> 2 + 0');
    runTest(LogicalOrExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('2~/9%5-someInt|2')), '2 ~/ 9 % 5 - someInt | 2');
    runTest(LogicalOrExpression(lexer.parseStr('super |6')), 'super | 6');
    runTest(LogicalOrExpression(lexer.parseStr('super |7|0')), 'super | 7 | 0');
    runTest(LogicalOrExpression(lexer.parseStr('super |someInt|8|3')), 'super | someInt | 8 | 3');
    runTest(LogicalOrExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(LogicalOrExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/9%5-someInt|2')), 'super ~/ 9 % 5 - someInt | 2');
    runTest(LogicalOrExpression(lexer.parseStr('4<=6')), '4 <= 6');
    runTest(LogicalOrExpression(lexer.parseStr('1<7')), '1 < 7');
    runTest(LogicalOrExpression(lexer.parseStr('9>=someInt<<8')), '9 >= someInt << 8');
    runTest(LogicalOrExpression(lexer.parseStr('3|6%someInt>>>2+0<=8')), '3 | 6 % someInt >>> 2 + 0 <= 8');
    runTest(LogicalOrExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('2~/9%5>8-someInt|2')), '2 ~/ 9 % 5 > 8 - someInt | 2');
    runTest(LogicalOrExpression(lexer.parseStr('super <=6')), 'super <= 6');
    runTest(LogicalOrExpression(lexer.parseStr('super <7')), 'super < 7');
    runTest(LogicalOrExpression(lexer.parseStr('super >=someInt<<8')), 'super >= someInt << 8');
    runTest(LogicalOrExpression(lexer.parseStr('super |6%someInt>>>2+0<=8')), 'super | 6 % someInt >>> 2 + 0 <= 8');
    runTest(LogicalOrExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/9%5>8-someInt|2')), 'super ~/ 9 % 5 > 8 - someInt | 2');
    runTest(LogicalOrExpression(lexer.parseStr('4is someType')), '4 is someType');
    runTest(LogicalOrExpression(lexer.parseStr('1 is! someType')), '1 is! someType');
    runTest(LogicalOrExpression(lexer.parseStr('5as someType')), '5 as someType');
    runTest(LogicalOrExpression(lexer.parseStr('4==6')), '4 == 6');
    runTest(LogicalOrExpression(lexer.parseStr('1!=7')), '1 != 7');
    runTest(LogicalOrExpression(lexer.parseStr('9==someInt<<8')), '9 == someInt << 8');
    runTest(LogicalOrExpression(lexer.parseStr('3|6%someInt>>>2+0!=8')), '3 | 6 % someInt >>> 2 + 0 != 8');
    runTest(LogicalOrExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('2~/9%5==8-someInt|2')), '2 ~/ 9 % 5 == 8 - someInt | 2');
    runTest(LogicalOrExpression(lexer.parseStr('super ==6')), 'super == 6');
    runTest(LogicalOrExpression(lexer.parseStr('super !=7')), 'super != 7');
    runTest(LogicalOrExpression(lexer.parseStr('super ==someInt<<8')), 'super == someInt << 8');
    runTest(LogicalOrExpression(lexer.parseStr('super |6%someInt>>>2+0!=8')), 'super | 6 % someInt >>> 2 + 0 != 8');
    runTest(LogicalOrExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/9%5==8-someInt|2')), 'super ~/ 9 % 5 == 8 - someInt | 2');
    runTest(LogicalOrExpression(lexer.parseStr('4&&6')), '4 && 6');
    runTest(LogicalOrExpression(lexer.parseStr('1&&7')), '1 && 7');
    runTest(LogicalOrExpression(lexer.parseStr('9&&someInt<<8')), '9 && someInt << 8');
    runTest(LogicalOrExpression(lexer.parseStr('3|6%someInt>>>2+0&&8')), '3 | 6 % someInt >>> 2 + 0 && 8');
    runTest(LogicalOrExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('2~/9%5&&8-someInt|2')), '2 ~/ 9 % 5 && 8 - someInt | 2');
    runTest(LogicalOrExpression(lexer.parseStr('super &&6')), 'super && 6');
    runTest(LogicalOrExpression(lexer.parseStr('super &&7')), 'super && 7');
    runTest(LogicalOrExpression(lexer.parseStr('super &&someInt<<8')), 'super && someInt << 8');
    runTest(LogicalOrExpression(lexer.parseStr('super |6%someInt>>>2+0&&8')), 'super | 6 % someInt >>> 2 + 0 && 8');
    runTest(LogicalOrExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/9%5&&8-someInt|2')), 'super ~/ 9 % 5 && 8 - someInt | 2');

    runTest(LogicalOrExpression(lexer.parseStr('4||6')), '4 || 6');
    runTest(LogicalOrExpression(lexer.parseStr('1||7')), '1 || 7');
    runTest(LogicalOrExpression(lexer.parseStr('9&&someInt||8')), '9 && someInt || 8');
    runTest(LogicalOrExpression(lexer.parseStr('3|6%someInt>>>2+0||8')), '3 | 6 % someInt >>> 2 + 0 || 8');
    runTest(LogicalOrExpression(lexer.parseStr('1+1>=someInt||7&0*32')), '1 + 1 >= someInt || 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('2~/9%5&&8-someInt|2')), '2 ~/ 9 % 5 && 8 - someInt | 2');

    runTest(LogicalOrExpression(lexer.parseStr('super ||6')), 'super || 6');
    runTest(LogicalOrExpression(lexer.parseStr('super ||7')), 'super || 7');
    runTest(LogicalOrExpression(lexer.parseStr('super &&someInt||8')), 'super && someInt || 8');
    runTest(LogicalOrExpression(lexer.parseStr('super |6%someInt>>>2+0||8')), 'super | 6 % someInt >>> 2 + 0 || 8');
    runTest(LogicalOrExpression(lexer.parseStr('super +1>=someInt||7&0*32')), 'super + 1 >= someInt || 7 & 0 * 32');
    runTest(LogicalOrExpression(lexer.parseStr('super ~/9%5&&8-someInt|2')), 'super ~/ 9 % 5 && 8 - someInt | 2');

  });

  test('If Null Expression', () {

    runTest(IfNullExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(IfNullExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(IfNullExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(IfNullExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(IfNullExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(IfNullExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(IfNullExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(IfNullExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(IfNullExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(IfNullExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(IfNullExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(IfNullExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(IfNullExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(IfNullExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(IfNullExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(IfNullExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(IfNullExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(IfNullExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(IfNullExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(IfNullExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(IfNullExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(IfNullExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(IfNullExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(IfNullExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');
    runTest(IfNullExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(IfNullExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(IfNullExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(IfNullExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(IfNullExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(IfNullExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(IfNullExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(IfNullExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2');
    runTest(IfNullExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(IfNullExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(IfNullExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(IfNullExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(IfNullExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(IfNullExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(IfNullExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(IfNullExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');
    runTest(IfNullExpression(lexer.parseStr('9<<3')), '9 << 3');
    runTest(IfNullExpression(lexer.parseStr('3>>7')), '3 >> 7');
    runTest(IfNullExpression(lexer.parseStr('4>>6')), '4 >> 6');
    runTest(IfNullExpression(lexer.parseStr('1<<7>>0')), '1 << 7 >> 0');
    runTest(IfNullExpression(lexer.parseStr('2<<5<<2')), '2 << 5 << 2');
    runTest(IfNullExpression(lexer.parseStr('9>>>someInt>>8<<3')), '9 >>> someInt >> 8 << 3');
    runTest(IfNullExpression(lexer.parseStr('3+6%someInt>>>2')), '3 + 6 % someInt >>> 2');
    runTest(IfNullExpression(lexer.parseStr('1+someInt>>7~/0')), '1 + someInt >> 7 ~/ 0');
    runTest(IfNullExpression(lexer.parseStr('2~/5-someInt>>2')), '2 ~/ 5 - someInt >> 2');
    runTest(IfNullExpression(lexer.parseStr('super <<3')), 'super << 3');
    runTest(IfNullExpression(lexer.parseStr('super >>7')), 'super >> 7');
    runTest(IfNullExpression(lexer.parseStr('super >>6')), 'super >> 6');
    runTest(IfNullExpression(lexer.parseStr('super <<7>>0')), 'super << 7 >> 0');
    runTest(IfNullExpression(lexer.parseStr('super <<5<<2')), 'super << 5 << 2');
    runTest(IfNullExpression(lexer.parseStr('super >>>someInt>>8<<3')), 'super >>> someInt >> 8 << 3');
    runTest(IfNullExpression(lexer.parseStr('super +6%someInt>>>2')), 'super + 6 % someInt >>> 2');
    runTest(IfNullExpression(lexer.parseStr('super -someInt>>7~/0')), 'super - someInt >> 7 ~/ 0');
    runTest(IfNullExpression(lexer.parseStr('super ~/5-someInt>>2')), 'super ~/ 5 - someInt >> 2');
    runTest(IfNullExpression(lexer.parseStr('4&6')), '4 & 6');
    runTest(IfNullExpression(lexer.parseStr('1&7&0')), '1 & 7 & 0');
    runTest(IfNullExpression(lexer.parseStr('9&someInt&8&3')), '9 & someInt & 8 & 3');
    runTest(IfNullExpression(lexer.parseStr('3&6%someInt>>>2+0')), '3 & 6 % someInt >>> 2 + 0');
    runTest(IfNullExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('2~/9%5-someInt&2')), '2 ~/ 9 % 5 - someInt & 2');
    runTest(IfNullExpression(lexer.parseStr('super &6')), 'super & 6');
    runTest(IfNullExpression(lexer.parseStr('super &7&0')), 'super & 7 & 0');
    runTest(IfNullExpression(lexer.parseStr('super &someInt&8&3')), 'super & someInt & 8 & 3');
    runTest(IfNullExpression(lexer.parseStr('super &6%someInt>>>2+0')), 'super & 6 % someInt >>> 2 + 0');
    runTest(IfNullExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('super ~/9%5-someInt&2')), 'super ~/ 9 % 5 - someInt & 2');
    runTest(IfNullExpression(lexer.parseStr('4^6')), '4 ^ 6');
    runTest(IfNullExpression(lexer.parseStr('1^7^0')), '1 ^ 7 ^ 0');
    runTest(IfNullExpression(lexer.parseStr('9^someInt^8^3')), '9 ^ someInt ^ 8 ^ 3');
    runTest(IfNullExpression(lexer.parseStr('3^6%someInt>>>2+0')), '3 ^ 6 % someInt >>> 2 + 0');
    runTest(IfNullExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('2^9%5-someInt&2')), '2 ^ 9 % 5 - someInt & 2');
    runTest(IfNullExpression(lexer.parseStr('super ^6')), 'super ^ 6');
    runTest(IfNullExpression(lexer.parseStr('super ^7^0')), 'super ^ 7 ^ 0');
    runTest(IfNullExpression(lexer.parseStr('super ^someInt^8^3')), 'super ^ someInt ^ 8 ^ 3');
    runTest(IfNullExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(IfNullExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('super ^9%5-someInt&2')), 'super ^ 9 % 5 - someInt & 2');
    runTest(IfNullExpression(lexer.parseStr('4|6')), '4 | 6');
    runTest(IfNullExpression(lexer.parseStr('1|7|0')), '1 | 7 | 0');
    runTest(IfNullExpression(lexer.parseStr('9|someInt|8|3')), '9 | someInt | 8 | 3');
    runTest(IfNullExpression(lexer.parseStr('3|6%someInt>>>2+0')), '3 | 6 % someInt >>> 2 + 0');
    runTest(IfNullExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('2~/9%5-someInt|2')), '2 ~/ 9 % 5 - someInt | 2');
    runTest(IfNullExpression(lexer.parseStr('super |6')), 'super | 6');
    runTest(IfNullExpression(lexer.parseStr('super |7|0')), 'super | 7 | 0');
    runTest(IfNullExpression(lexer.parseStr('super |someInt|8|3')), 'super | someInt | 8 | 3');
    runTest(IfNullExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(IfNullExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('super ~/9%5-someInt|2')), 'super ~/ 9 % 5 - someInt | 2');
    runTest(IfNullExpression(lexer.parseStr('4<=6')), '4 <= 6');
    runTest(IfNullExpression(lexer.parseStr('1<7')), '1 < 7');
    runTest(IfNullExpression(lexer.parseStr('9>=someInt<<8')), '9 >= someInt << 8');
    runTest(IfNullExpression(lexer.parseStr('3|6%someInt>>>2+0<=8')), '3 | 6 % someInt >>> 2 + 0 <= 8');
    runTest(IfNullExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('2~/9%5>8-someInt|2')), '2 ~/ 9 % 5 > 8 - someInt | 2');
    runTest(IfNullExpression(lexer.parseStr('super <=6')), 'super <= 6');
    runTest(IfNullExpression(lexer.parseStr('super <7')), 'super < 7');
    runTest(IfNullExpression(lexer.parseStr('super >=someInt<<8')), 'super >= someInt << 8');
    runTest(IfNullExpression(lexer.parseStr('super |6%someInt>>>2+0<=8')), 'super | 6 % someInt >>> 2 + 0 <= 8');
    runTest(IfNullExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('super ~/9%5>8-someInt|2')), 'super ~/ 9 % 5 > 8 - someInt | 2');
    runTest(IfNullExpression(lexer.parseStr('4is someType')), '4 is someType');
    runTest(IfNullExpression(lexer.parseStr('1 is! someType')), '1 is! someType');
    runTest(IfNullExpression(lexer.parseStr('5as someType')), '5 as someType');
    runTest(IfNullExpression(lexer.parseStr('4==6')), '4 == 6');
    runTest(IfNullExpression(lexer.parseStr('1!=7')), '1 != 7');
    runTest(IfNullExpression(lexer.parseStr('9==someInt<<8')), '9 == someInt << 8');
    runTest(IfNullExpression(lexer.parseStr('3|6%someInt>>>2+0!=8')), '3 | 6 % someInt >>> 2 + 0 != 8');
    runTest(IfNullExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('2~/9%5==8-someInt|2')), '2 ~/ 9 % 5 == 8 - someInt | 2');
    runTest(IfNullExpression(lexer.parseStr('super ==6')), 'super == 6');
    runTest(IfNullExpression(lexer.parseStr('super !=7')), 'super != 7');
    runTest(IfNullExpression(lexer.parseStr('super ==someInt<<8')), 'super == someInt << 8');
    runTest(IfNullExpression(lexer.parseStr('super |6%someInt>>>2+0!=8')), 'super | 6 % someInt >>> 2 + 0 != 8');
    runTest(IfNullExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('super ~/9%5==8-someInt|2')), 'super ~/ 9 % 5 == 8 - someInt | 2');
    runTest(IfNullExpression(lexer.parseStr('4&&6')), '4 && 6');
    runTest(IfNullExpression(lexer.parseStr('1&&7')), '1 && 7');
    runTest(IfNullExpression(lexer.parseStr('9&&someInt<<8')), '9 && someInt << 8');
    runTest(IfNullExpression(lexer.parseStr('3|6%someInt>>>2+0&&8')), '3 | 6 % someInt >>> 2 + 0 && 8');
    runTest(IfNullExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('2~/9%5&&8-someInt|2')), '2 ~/ 9 % 5 && 8 - someInt | 2');
    runTest(IfNullExpression(lexer.parseStr('super &&6')), 'super && 6');
    runTest(IfNullExpression(lexer.parseStr('super &&7')), 'super && 7');
    runTest(IfNullExpression(lexer.parseStr('super &&someInt<<8')), 'super && someInt << 8');
    runTest(IfNullExpression(lexer.parseStr('super |6%someInt>>>2+0&&8')), 'super | 6 % someInt >>> 2 + 0 && 8');
    runTest(IfNullExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('super ~/9%5&&8-someInt|2')), 'super ~/ 9 % 5 && 8 - someInt | 2');
    runTest(IfNullExpression(lexer.parseStr('4||6')), '4 || 6');
    runTest(IfNullExpression(lexer.parseStr('1||7')), '1 || 7');
    runTest(IfNullExpression(lexer.parseStr('9&&someInt||8')), '9 && someInt || 8');
    runTest(IfNullExpression(lexer.parseStr('3|6%someInt>>>2+0||8')), '3 | 6 % someInt >>> 2 + 0 || 8');
    runTest(IfNullExpression(lexer.parseStr('1+1>=someInt||7&0*32')), '1 + 1 >= someInt || 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('2~/9%5&&8-someInt|2')), '2 ~/ 9 % 5 && 8 - someInt | 2');
    runTest(IfNullExpression(lexer.parseStr('super ||6')), 'super || 6');
    runTest(IfNullExpression(lexer.parseStr('super ||7')), 'super || 7');
    runTest(IfNullExpression(lexer.parseStr('super &&someInt||8')), 'super && someInt || 8');
    runTest(IfNullExpression(lexer.parseStr('super |6%someInt>>>2+0||8')), 'super | 6 % someInt >>> 2 + 0 || 8');
    runTest(IfNullExpression(lexer.parseStr('super +1>=someInt||7&0*32')), 'super + 1 >= someInt || 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('super ~/9%5&&8-someInt|2')), 'super ~/ 9 % 5 && 8 - someInt | 2');

    runTest(IfNullExpression(lexer.parseStr('4??6')), '4 ?? 6');
    runTest(IfNullExpression(lexer.parseStr('1??7')), '1 ?? 7');
    runTest(IfNullExpression(lexer.parseStr('9??someInt||8')), '9 ?? someInt || 8');
    runTest(IfNullExpression(lexer.parseStr('3|6%someInt??2+0||8')), '3 | 6 % someInt ?? 2 + 0 || 8');
    runTest(IfNullExpression(lexer.parseStr('1+1>=someInt??7&0*32')), '1 + 1 >= someInt ?? 7 & 0 * 32');
    runTest(IfNullExpression(lexer.parseStr('2~/9%5??8-someInt|2')), '2 ~/ 9 % 5 ?? 8 - someInt | 2');

  });

  test('Conditional Expression', () {

    runTest(ConditionalExpression(lexer.parseStr('9*8')), '9 * 8');
    runTest(ConditionalExpression(lexer.parseStr('3/6')), '3 / 6');
    runTest(ConditionalExpression(lexer.parseStr('1%7')), '1 % 7');
    runTest(ConditionalExpression(lexer.parseStr('2~/5')), '2 ~/ 5');
    runTest(ConditionalExpression(lexer.parseStr('9*8/3')), '9 * 8 / 3');
    runTest(ConditionalExpression(lexer.parseStr('3/6%2')), '3 / 6 % 2');
    runTest(ConditionalExpression(lexer.parseStr('1%7~/0')), '1 % 7 ~/ 0');
    runTest(ConditionalExpression(lexer.parseStr('2~/5/2')), '2 ~/ 5 / 2');
    runTest(ConditionalExpression(lexer.parseStr('9*someInt%8/3')), '9 * someInt % 8 / 3');
    runTest(ConditionalExpression(lexer.parseStr('3/6%someInt/2')), '3 / 6 % someInt / 2');
    runTest(ConditionalExpression(lexer.parseStr('1%someInt*7~/0')), '1 % someInt * 7 ~/ 0');
    runTest(ConditionalExpression(lexer.parseStr('2~/5%someInt/2')), '2 ~/ 5 % someInt / 2');
    runTest(ConditionalExpression(lexer.parseStr('super *8')), 'super * 8');
    runTest(ConditionalExpression(lexer.parseStr('super /6')), 'super / 6');
    runTest(ConditionalExpression(lexer.parseStr('super %7')), 'super % 7');
    runTest(ConditionalExpression(lexer.parseStr('super ~/5')), 'super ~/ 5');
    runTest(ConditionalExpression(lexer.parseStr('super *8/3')), 'super * 8 / 3');
    runTest(ConditionalExpression(lexer.parseStr('super /6%2')), 'super / 6 % 2');
    runTest(ConditionalExpression(lexer.parseStr('super %7~/0')), 'super % 7 ~/ 0');
    runTest(ConditionalExpression(lexer.parseStr('super ~/5/2')), 'super ~/ 5 / 2');
    runTest(ConditionalExpression(lexer.parseStr('super *someInt%8/3')), 'super * someInt % 8 / 3');
    runTest(ConditionalExpression(lexer.parseStr('super /6%someInt/2')), 'super / 6 % someInt / 2');
    runTest(ConditionalExpression(lexer.parseStr('super %someInt*7~/0')), 'super % someInt * 7 ~/ 0');
    runTest(ConditionalExpression(lexer.parseStr('super ~/5%someInt/2')), 'super ~/ 5 % someInt / 2');
    runTest(ConditionalExpression(lexer.parseStr('9+3')), '9 + 3');
    runTest(ConditionalExpression(lexer.parseStr('3-7')), '3 - 7');
    runTest(ConditionalExpression(lexer.parseStr('1+7-0')), '1 + 7 - 0');
    runTest(ConditionalExpression(lexer.parseStr('2+5+2')), '2 + 5 + 2');
    runTest(ConditionalExpression(lexer.parseStr('9-someInt-8-3')), '9 - someInt - 8 - 3');
    runTest(ConditionalExpression(lexer.parseStr('3+6%someInt/2')), '3 + 6 % someInt / 2');
    runTest(ConditionalExpression(lexer.parseStr('1+someInt-7~/0')), '1 + someInt - 7 ~/ 0');
    runTest(ConditionalExpression(lexer.parseStr('2~/5-someInt+2')), '2 ~/ 5 - someInt + 2');
    runTest(ConditionalExpression(lexer.parseStr('super +3')), 'super + 3');
    runTest(ConditionalExpression(lexer.parseStr('super -7')), 'super - 7');
    runTest(ConditionalExpression(lexer.parseStr('super +7-0')), 'super + 7 - 0');
    runTest(ConditionalExpression(lexer.parseStr('super +5+2')), 'super + 5 + 2');
    runTest(ConditionalExpression(lexer.parseStr('super -someInt-8-3')), 'super - someInt - 8 - 3');
    runTest(ConditionalExpression(lexer.parseStr('super +6%someInt/2')), 'super + 6 % someInt / 2');
    runTest(ConditionalExpression(lexer.parseStr('super +someInt-7~/0')), 'super + someInt - 7 ~/ 0');
    runTest(ConditionalExpression(lexer.parseStr('super ~/5-someInt+2')), 'super ~/ 5 - someInt + 2');
    runTest(ConditionalExpression(lexer.parseStr('9<<3')), '9 << 3');
    runTest(ConditionalExpression(lexer.parseStr('3>>7')), '3 >> 7');
    runTest(ConditionalExpression(lexer.parseStr('4>>6')), '4 >> 6');
    runTest(ConditionalExpression(lexer.parseStr('1<<7>>0')), '1 << 7 >> 0');
    runTest(ConditionalExpression(lexer.parseStr('2<<5<<2')), '2 << 5 << 2');
    runTest(ConditionalExpression(lexer.parseStr('9>>>someInt>>8<<3')), '9 >>> someInt >> 8 << 3');
    runTest(ConditionalExpression(lexer.parseStr('3+6%someInt>>>2')), '3 + 6 % someInt >>> 2');
    runTest(ConditionalExpression(lexer.parseStr('1+someInt>>7~/0')), '1 + someInt >> 7 ~/ 0');
    runTest(ConditionalExpression(lexer.parseStr('2~/5-someInt>>2')), '2 ~/ 5 - someInt >> 2');
    runTest(ConditionalExpression(lexer.parseStr('super <<3')), 'super << 3');
    runTest(ConditionalExpression(lexer.parseStr('super >>7')), 'super >> 7');
    runTest(ConditionalExpression(lexer.parseStr('super >>6')), 'super >> 6');
    runTest(ConditionalExpression(lexer.parseStr('super <<7>>0')), 'super << 7 >> 0');
    runTest(ConditionalExpression(lexer.parseStr('super <<5<<2')), 'super << 5 << 2');
    runTest(ConditionalExpression(lexer.parseStr('super >>>someInt>>8<<3')), 'super >>> someInt >> 8 << 3');
    runTest(ConditionalExpression(lexer.parseStr('super +6%someInt>>>2')), 'super + 6 % someInt >>> 2');
    runTest(ConditionalExpression(lexer.parseStr('super -someInt>>7~/0')), 'super - someInt >> 7 ~/ 0');
    runTest(ConditionalExpression(lexer.parseStr('super ~/5-someInt>>2')), 'super ~/ 5 - someInt >> 2');
    runTest(ConditionalExpression(lexer.parseStr('4&6')), '4 & 6');
    runTest(ConditionalExpression(lexer.parseStr('1&7&0')), '1 & 7 & 0');
    runTest(ConditionalExpression(lexer.parseStr('9&someInt&8&3')), '9 & someInt & 8 & 3');
    runTest(ConditionalExpression(lexer.parseStr('3&6%someInt>>>2+0')), '3 & 6 % someInt >>> 2 + 0');
    runTest(ConditionalExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('2~/9%5-someInt&2')), '2 ~/ 9 % 5 - someInt & 2');
    runTest(ConditionalExpression(lexer.parseStr('super &6')), 'super & 6');
    runTest(ConditionalExpression(lexer.parseStr('super &7&0')), 'super & 7 & 0');
    runTest(ConditionalExpression(lexer.parseStr('super &someInt&8&3')), 'super & someInt & 8 & 3');
    runTest(ConditionalExpression(lexer.parseStr('super &6%someInt>>>2+0')), 'super & 6 % someInt >>> 2 + 0');
    runTest(ConditionalExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('super ~/9%5-someInt&2')), 'super ~/ 9 % 5 - someInt & 2');
    runTest(ConditionalExpression(lexer.parseStr('4^6')), '4 ^ 6');
    runTest(ConditionalExpression(lexer.parseStr('1^7^0')), '1 ^ 7 ^ 0');
    runTest(ConditionalExpression(lexer.parseStr('9^someInt^8^3')), '9 ^ someInt ^ 8 ^ 3');
    runTest(ConditionalExpression(lexer.parseStr('3^6%someInt>>>2+0')), '3 ^ 6 % someInt >>> 2 + 0');
    runTest(ConditionalExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('2^9%5-someInt&2')), '2 ^ 9 % 5 - someInt & 2');
    runTest(ConditionalExpression(lexer.parseStr('super ^6')), 'super ^ 6');
    runTest(ConditionalExpression(lexer.parseStr('super ^7^0')), 'super ^ 7 ^ 0');
    runTest(ConditionalExpression(lexer.parseStr('super ^someInt^8^3')), 'super ^ someInt ^ 8 ^ 3');
    runTest(ConditionalExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(ConditionalExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('super ^9%5-someInt&2')), 'super ^ 9 % 5 - someInt & 2');
    runTest(ConditionalExpression(lexer.parseStr('4|6')), '4 | 6');
    runTest(ConditionalExpression(lexer.parseStr('1|7|0')), '1 | 7 | 0');
    runTest(ConditionalExpression(lexer.parseStr('9|someInt|8|3')), '9 | someInt | 8 | 3');
    runTest(ConditionalExpression(lexer.parseStr('3|6%someInt>>>2+0')), '3 | 6 % someInt >>> 2 + 0');
    runTest(ConditionalExpression(lexer.parseStr('1+someInt>>7&0*32')), '1 + someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('2~/9%5-someInt|2')), '2 ~/ 9 % 5 - someInt | 2');
    runTest(ConditionalExpression(lexer.parseStr('super |6')), 'super | 6');
    runTest(ConditionalExpression(lexer.parseStr('super |7|0')), 'super | 7 | 0');
    runTest(ConditionalExpression(lexer.parseStr('super |someInt|8|3')), 'super | someInt | 8 | 3');
    runTest(ConditionalExpression(lexer.parseStr('super ^6%someInt>>>2+0')), 'super ^ 6 % someInt >>> 2 + 0');
    runTest(ConditionalExpression(lexer.parseStr('super +someInt>>7&0*32')), 'super + someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('super ~/9%5-someInt|2')), 'super ~/ 9 % 5 - someInt | 2');
    runTest(ConditionalExpression(lexer.parseStr('4<=6')), '4 <= 6');
    runTest(ConditionalExpression(lexer.parseStr('1<7')), '1 < 7');
    runTest(ConditionalExpression(lexer.parseStr('9>=someInt<<8')), '9 >= someInt << 8');
    runTest(ConditionalExpression(lexer.parseStr('3|6%someInt>>>2+0<=8')), '3 | 6 % someInt >>> 2 + 0 <= 8');
    runTest(ConditionalExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('2~/9%5>8-someInt|2')), '2 ~/ 9 % 5 > 8 - someInt | 2');
    runTest(ConditionalExpression(lexer.parseStr('super <=6')), 'super <= 6');
    runTest(ConditionalExpression(lexer.parseStr('super <7')), 'super < 7');
    runTest(ConditionalExpression(lexer.parseStr('super >=someInt<<8')), 'super >= someInt << 8');
    runTest(ConditionalExpression(lexer.parseStr('super |6%someInt>>>2+0<=8')), 'super | 6 % someInt >>> 2 + 0 <= 8');
    runTest(ConditionalExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('super ~/9%5>8-someInt|2')), 'super ~/ 9 % 5 > 8 - someInt | 2');
    runTest(ConditionalExpression(lexer.parseStr('4is someType')), '4 is someType');
    runTest(ConditionalExpression(lexer.parseStr('1 is! someType')), '1 is! someType');
    runTest(ConditionalExpression(lexer.parseStr('5as someType')), '5 as someType');
    runTest(ConditionalExpression(lexer.parseStr('4==6')), '4 == 6');
    runTest(ConditionalExpression(lexer.parseStr('1!=7')), '1 != 7');
    runTest(ConditionalExpression(lexer.parseStr('9==someInt<<8')), '9 == someInt << 8');
    runTest(ConditionalExpression(lexer.parseStr('3|6%someInt>>>2+0!=8')), '3 | 6 % someInt >>> 2 + 0 != 8');
    runTest(ConditionalExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('2~/9%5==8-someInt|2')), '2 ~/ 9 % 5 == 8 - someInt | 2');
    runTest(ConditionalExpression(lexer.parseStr('super ==6')), 'super == 6');
    runTest(ConditionalExpression(lexer.parseStr('super !=7')), 'super != 7');
    runTest(ConditionalExpression(lexer.parseStr('super ==someInt<<8')), 'super == someInt << 8');
    runTest(ConditionalExpression(lexer.parseStr('super |6%someInt>>>2+0!=8')), 'super | 6 % someInt >>> 2 + 0 != 8');
    runTest(ConditionalExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('super ~/9%5==8-someInt|2')), 'super ~/ 9 % 5 == 8 - someInt | 2');
    runTest(ConditionalExpression(lexer.parseStr('4&&6')), '4 && 6');
    runTest(ConditionalExpression(lexer.parseStr('1&&7')), '1 && 7');
    runTest(ConditionalExpression(lexer.parseStr('9&&someInt<<8')), '9 && someInt << 8');
    runTest(ConditionalExpression(lexer.parseStr('3|6%someInt>>>2+0&&8')), '3 | 6 % someInt >>> 2 + 0 && 8');
    runTest(ConditionalExpression(lexer.parseStr('1+1>=someInt>>7&0*32')), '1 + 1 >= someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('2~/9%5&&8-someInt|2')), '2 ~/ 9 % 5 && 8 - someInt | 2');
    runTest(ConditionalExpression(lexer.parseStr('super &&6')), 'super && 6');
    runTest(ConditionalExpression(lexer.parseStr('super &&7')), 'super && 7');
    runTest(ConditionalExpression(lexer.parseStr('super &&someInt<<8')), 'super && someInt << 8');
    runTest(ConditionalExpression(lexer.parseStr('super |6%someInt>>>2+0&&8')), 'super | 6 % someInt >>> 2 + 0 && 8');
    runTest(ConditionalExpression(lexer.parseStr('super +1>=someInt>>7&0*32')), 'super + 1 >= someInt >> 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('super ~/9%5&&8-someInt|2')), 'super ~/ 9 % 5 && 8 - someInt | 2');
    runTest(ConditionalExpression(lexer.parseStr('4||6')), '4 || 6');
    runTest(ConditionalExpression(lexer.parseStr('1||7')), '1 || 7');
    runTest(ConditionalExpression(lexer.parseStr('9&&someInt||8')), '9 && someInt || 8');
    runTest(ConditionalExpression(lexer.parseStr('3|6%someInt>>>2+0||8')), '3 | 6 % someInt >>> 2 + 0 || 8');
    runTest(ConditionalExpression(lexer.parseStr('1+1>=someInt||7&0*32')), '1 + 1 >= someInt || 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('2~/9%5&&8-someInt|2')), '2 ~/ 9 % 5 && 8 - someInt | 2');
    runTest(ConditionalExpression(lexer.parseStr('super ||6')), 'super || 6');
    runTest(ConditionalExpression(lexer.parseStr('super ||7')), 'super || 7');
    runTest(ConditionalExpression(lexer.parseStr('super &&someInt||8')), 'super && someInt || 8');
    runTest(ConditionalExpression(lexer.parseStr('super |6%someInt>>>2+0||8')), 'super | 6 % someInt >>> 2 + 0 || 8');
    runTest(ConditionalExpression(lexer.parseStr('super +1>=someInt||7&0*32')), 'super + 1 >= someInt || 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('super ~/9%5&&8-someInt|2')), 'super ~/ 9 % 5 && 8 - someInt | 2');
    runTest(ConditionalExpression(lexer.parseStr('4??6')), '4 ?? 6');
    runTest(ConditionalExpression(lexer.parseStr('1??7')), '1 ?? 7');
    runTest(ConditionalExpression(lexer.parseStr('9??someInt||8')), '9 ?? someInt || 8');
    runTest(ConditionalExpression(lexer.parseStr('3|6%someInt??2+0||8')), '3 | 6 % someInt ?? 2 + 0 || 8');
    runTest(ConditionalExpression(lexer.parseStr('1+1>=someInt??7&0*32')), '1 + 1 >= someInt ?? 7 & 0 * 32');
    runTest(ConditionalExpression(lexer.parseStr('2~/9%5??8-someInt|2')), '2 ~/ 9 % 5 ?? 8 - someInt | 2');

    runTest(ConditionalExpression(lexer.parseStr('true?false:true')), 'true ? false : true');
    runTest(ConditionalExpression(lexer.parseStr('someVar == another?false ? true : false:true')), 'someVar == another ? false ? true : false : true');

  });

  test('Assignable Expression', () {

    runTest(AssignableExpression(lexer.parseStr('somePrimary.someId')), 'somePrimary.someId');
    runTest(AssignableExpression(lexer.parseStr('somePrimary._someId')), 'somePrimary._someId');
    runTest(AssignableExpression(lexer.parseStr('somePrimary?.someId')), 'somePrimary?.someId');
    runTest(AssignableExpression(lexer.parseStr('somePrimary?._someId')), 'somePrimary?._someId');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[someId]')), 'somePrimary[someId]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[(someId)]')), 'somePrimary[(someId)]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[3+2]')), 'somePrimary[3 + 2]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[4??6]')), 'somePrimary[4 ?? 6]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[1??7]')), 'somePrimary[1 ?? 7]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[9??someInt||8]')), 'somePrimary[9 ?? someInt || 8]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[3|6%someInt??2+0||8]')), 'somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[1+1>=someInt??7&0*32]')), 'somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[2~/9%5??8-someInt|2]')), 'somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[true?false:true]')), 'somePrimary[true ? false : true]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[someVar == another?false ? true : false:true]')), 'somePrimary[someVar == another ? false ? true : false : true]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary(x, y, {x()}).someId')), 'somePrimary(x, y, {x()}).someId');
    runTest(AssignableExpression(lexer.parseStr('somePrimary(x (y(x), z), {x()})[9??someInt||8]')), 'somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary({x (y(x))})[3|6%someInt??2+0||8]')), 'somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary(x, y, {x})[1+1>=someInt??7&0*32]')), 'somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), 'somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary({this.x (this.y ( z))})[true?false:true]')), 'somePrimary({this.x(this.y(z))})[true ? false : true]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), 'somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary<int>(x, y, {x()}).someId')), 'somePrimary<int>(x, y, {x()}).someId');
    runTest(AssignableExpression(lexer.parseStr('somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]')), 'somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]')), 'somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]')), 'somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), 'somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]')), 'somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), 'somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');

    runTest(AssignableExpression(lexer.parseStr('somePrimary.someId._another')), 'somePrimary.someId._another');
    runTest(AssignableExpression(lexer.parseStr('somePrimary?.someId._another')), 'somePrimary?.someId._another');
    runTest(AssignableExpression(lexer.parseStr('somePrimary?.someId?._another')), 'somePrimary?.someId?._another');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[someId][another]')), 'somePrimary[someId][another]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[someId][1??7].third')), 'somePrimary[someId][1 ?? 7].third');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]')), 'somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]')), 'somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]');
    runTest(AssignableExpression(lexer.parseStr('somePrimary(x, y, {x()}).someId.another')), 'somePrimary(x, y, {x()}).someId.another');
    runTest(AssignableExpression(lexer.parseStr('somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]')), 'somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]');

    // ---------------------------------

    runTest(AssignableExpression(lexer.parseStr('super .someId')), 'super .someId');
    runTest(AssignableExpression(lexer.parseStr('super ._someId')), 'super ._someId');
    runTest(AssignableExpression(lexer.parseStr('super [someId]')), 'super [someId]');
    runTest(AssignableExpression(lexer.parseStr('super [(someId)]')), 'super [(someId)]');
    runTest(AssignableExpression(lexer.parseStr('super [3+2]')), 'super [3 + 2]');
    runTest(AssignableExpression(lexer.parseStr('super [4??6]')), 'super [4 ?? 6]');
    runTest(AssignableExpression(lexer.parseStr('super [1??7]')), 'super [1 ?? 7]');
    runTest(AssignableExpression(lexer.parseStr('super [9??someInt||8]')), 'super [9 ?? someInt || 8]');
    runTest(AssignableExpression(lexer.parseStr('super [3|6%someInt??2+0||8]')), 'super [3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(AssignableExpression(lexer.parseStr('super [1+1>=someInt??7&0*32]')), 'super [1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(AssignableExpression(lexer.parseStr('super [2~/9%5??8-someInt|2]')), 'super [2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(AssignableExpression(lexer.parseStr('super [true?false:true]')), 'super [true ? false : true]');
    runTest(AssignableExpression(lexer.parseStr('super [someVar == another?false ? true : false:true]')), 'super [someVar == another ? false ? true : false : true]');

    // ---------------------------------

    // TODO: Add constructorInvocation case

  });

  test('Postfix Expression', () {
    // TODO: Add constructorInvocation case

    runTest(PostfixExpression(lexer.parseStr('somePrimary.someId')), 'somePrimary.someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary._someId')), 'somePrimary._someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary?.someId')), 'somePrimary?.someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary?._someId')), 'somePrimary?._someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[someId]++')), 'somePrimary[someId]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[(someId)]++')), 'somePrimary[(someId)]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[3+2]++')), 'somePrimary[3 + 2]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[4??6]++')), 'somePrimary[4 ?? 6]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[1??7]++')), 'somePrimary[1 ?? 7]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[9??someInt||8]++')), 'somePrimary[9 ?? someInt || 8]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[3|6%someInt??2+0||8]++')), 'somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[1+1>=someInt??7&0*32]++')), 'somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[2~/9%5??8-someInt|2]++')), 'somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[true?false:true]++')), 'somePrimary[true ? false : true]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[someVar == another?false ? true : false:true]++')), 'somePrimary[someVar == another ? false ? true : false : true]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(x, y, {x()}).someId')), 'somePrimary(x, y, {x()}).someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(x (y(x), z), {x()})[9??someInt||8]++')), 'somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary({x (y(x))})[3|6%someInt??2+0||8]++')), 'somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(x, y, {x})[1+1>=someInt??7&0*32]++')), 'somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), 'somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary({this.x (this.y ( z))})[true?false:true]++')), 'somePrimary({this.x(this.y(z))})[true ? false : true]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), 'somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int>(x, y, {x()}).someId')), 'somePrimary<int>(x, y, {x()}).someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]++')), 'somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]++')), 'somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]++')), 'somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), 'somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]++')), 'somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), 'somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary.someId._another')), 'somePrimary.someId._another');
    runTest(PostfixExpression(lexer.parseStr('somePrimary?.someId._another')), 'somePrimary?.someId._another');
    runTest(PostfixExpression(lexer.parseStr('somePrimary?.someId?._another')), 'somePrimary?.someId?._another');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[someId][another]++')), 'somePrimary[someId][another]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[someId][1??7].third')), 'somePrimary[someId][1 ?? 7].third');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]++')), 'somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]++')), 'somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]++');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(x, y, {x()}).someId.another')), 'somePrimary(x, y, {x()}).someId.another');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]++')), 'somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]++');
    runTest(PostfixExpression(lexer.parseStr('super .someId')), 'super .someId');
    runTest(PostfixExpression(lexer.parseStr('super ._someId')), 'super ._someId');
    runTest(PostfixExpression(lexer.parseStr('super [someId]++')), 'super [someId]++');
    runTest(PostfixExpression(lexer.parseStr('super [(someId)]++')), 'super [(someId)]++');
    runTest(PostfixExpression(lexer.parseStr('super [3+2]++')), 'super [3 + 2]++');
    runTest(PostfixExpression(lexer.parseStr('super [4??6]++')), 'super [4 ?? 6]++');
    runTest(PostfixExpression(lexer.parseStr('super [1??7]++')), 'super [1 ?? 7]++');
    runTest(PostfixExpression(lexer.parseStr('super [9??someInt||8]++')), 'super [9 ?? someInt || 8]++');
    runTest(PostfixExpression(lexer.parseStr('super [3|6%someInt??2+0||8]++')), 'super [3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(PostfixExpression(lexer.parseStr('super [1+1>=someInt??7&0*32]++')), 'super [1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(PostfixExpression(lexer.parseStr('super [2~/9%5??8-someInt|2]++')), 'super [2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(PostfixExpression(lexer.parseStr('super [true?false:true]++')), 'super [true ? false : true]++');
    runTest(PostfixExpression(lexer.parseStr('super [someVar == another?false ? true : false:true]++')), 'super [someVar == another ? false ? true : false : true]++');

    runTest(PostfixExpression(lexer.parseStr('somePrimary.someId')), 'somePrimary.someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary._someId')), 'somePrimary._someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary?.someId')), 'somePrimary?.someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary?._someId')), 'somePrimary?._someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[someId]--')), 'somePrimary[someId]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[(someId)]--')), 'somePrimary[(someId)]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[3+2]--')), 'somePrimary[3 + 2]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[4??6]--')), 'somePrimary[4 ?? 6]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[1??7]--')), 'somePrimary[1 ?? 7]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[9??someInt||8]--')), 'somePrimary[9 ?? someInt || 8]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[3|6%someInt??2+0||8]--')), 'somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[1+1>=someInt??7&0*32]--')), 'somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[2~/9%5??8-someInt|2]--')), 'somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[true?false:true]--')), 'somePrimary[true ? false : true]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[someVar == another?false ? true : false:true]--')), 'somePrimary[someVar == another ? false ? true : false : true]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(x, y, {x()}).someId')), 'somePrimary(x, y, {x()}).someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(x (y(x), z), {x()})[9??someInt||8]--')), 'somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary({x (y(x))})[3|6%someInt??2+0||8]--')), 'somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(x, y, {x})[1+1>=someInt??7&0*32]--')), 'somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), 'somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary({this.x (this.y ( z))})[true?false:true]--')), 'somePrimary({this.x(this.y(z))})[true ? false : true]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), 'somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int>(x, y, {x()}).someId')), 'somePrimary<int>(x, y, {x()}).someId');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]--')), 'somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]--')), 'somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]--')), 'somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), 'somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]--')), 'somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), 'somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary.someId._another')), 'somePrimary.someId._another');
    runTest(PostfixExpression(lexer.parseStr('somePrimary?.someId._another')), 'somePrimary?.someId._another');
    runTest(PostfixExpression(lexer.parseStr('somePrimary?.someId?._another')), 'somePrimary?.someId?._another');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[someId][another]--')), 'somePrimary[someId][another]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[someId][1??7].third')), 'somePrimary[someId][1 ?? 7].third');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]--')), 'somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]--')), 'somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]--');
    runTest(PostfixExpression(lexer.parseStr('somePrimary(x, y, {x()}).someId.another')), 'somePrimary(x, y, {x()}).someId.another');
    runTest(PostfixExpression(lexer.parseStr('somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]--')), 'somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]--');
    runTest(PostfixExpression(lexer.parseStr('super .someId')), 'super .someId');
    runTest(PostfixExpression(lexer.parseStr('super ._someId')), 'super ._someId');
    runTest(PostfixExpression(lexer.parseStr('super [someId]--')), 'super [someId]--');
    runTest(PostfixExpression(lexer.parseStr('super [(someId)]--')), 'super [(someId)]--');
    runTest(PostfixExpression(lexer.parseStr('super [3+2]--')), 'super [3 + 2]--');
    runTest(PostfixExpression(lexer.parseStr('super [4??6]--')), 'super [4 ?? 6]--');
    runTest(PostfixExpression(lexer.parseStr('super [1??7]--')), 'super [1 ?? 7]--');
    runTest(PostfixExpression(lexer.parseStr('super [9??someInt||8]--')), 'super [9 ?? someInt || 8]--');
    runTest(PostfixExpression(lexer.parseStr('super [3|6%someInt??2+0||8]--')), 'super [3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(PostfixExpression(lexer.parseStr('super [1+1>=someInt??7&0*32]--')), 'super [1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(PostfixExpression(lexer.parseStr('super [2~/9%5??8-someInt|2]--')), 'super [2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(PostfixExpression(lexer.parseStr('super [true?false:true]--')), 'super [true ? false : true]--');
    runTest(PostfixExpression(lexer.parseStr('super [someVar == another?false ? true : false:true]--')), 'super [someVar == another ? false ? true : false : true]--');

    // ---------------------------------

    runTest(PostfixExpression(lexer.parseStr('new Test()')), 'new Test()');
    runTest(PostfixExpression(lexer.parseStr('const Test()')), 'const Test()');
    runTest(PostfixExpression(lexer.parseStr('const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(PostfixExpression(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(PostfixExpression(lexer.parseStr('const Test.subType(someId:4,id2:.23,)')), 'const Test.subType(someId: 4, id2: .23,)');
    runTest(PostfixExpression(lexer.parseStr('const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), 'const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(PostfixExpression(lexer.parseStr('new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(PostfixExpression(lexer.parseStr('new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(PostfixExpression(lexer.parseStr('new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(PostfixExpression(lexer.parseStr('const <someType, int, num>{"1","2","3","4"}')), 'const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(PostfixExpression(lexer.parseStr('const <someType, int, num>{someObject}')), 'const <someType, int, num>{someObject}');
    runTest(PostfixExpression(lexer.parseStr('const <someType, int, num>{someObject,someObject,someObject}')), 'const <someType, int, num>{someObject, someObject, someObject}');

    runTest(PostfixExpression(lexer.parseStr('new Test()()')), 'new Test()()');
    runTest(PostfixExpression(lexer.parseStr('const Test()(4900.34)')), 'const Test()(4900.34)');
    runTest(PostfixExpression(lexer.parseStr('const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,).someId')), 'const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,).someId');
    runTest(PostfixExpression(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal,)?.someId')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal,)?.someId');
    runTest(PostfixExpression(lexer.parseStr('const Test.subType(someId:4,id2:.23,)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'const Test.subType(someId: 4, id2: .23,)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(PostfixExpression(lexer.parseStr('const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(PostfixExpression(lexer.parseStr('new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(PostfixExpression(lexer.parseStr('new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)<int,String>(234.432,someId:4,)')), 'new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)<int, String>(234.432, someId: 4,)');
    runTest(PostfixExpression(lexer.parseStr('new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(PostfixExpression(lexer.parseStr('const <someType, int, num>{"1","2","3","4"}<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'const <someType, int, num>{"1", "2", "3", "4"}<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(PostfixExpression(lexer.parseStr('const <someType, int, num>{someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const <someType, int, num>{someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(PostfixExpression(lexer.parseStr('const <someType, int, num>{someObject,someObject,someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const <someType, int, num>{someObject, someObject, someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

  });

  test('Await Expression', () {

    runTest(AwaitExpression(lexer.parseStr('await somePrimary.someId')), 'await somePrimary.someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary._someId')), 'await somePrimary._someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary?.someId')), 'await somePrimary?.someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary?._someId')), 'await somePrimary?._someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[someId]++')), 'await somePrimary[someId]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[(someId)]++')), 'await somePrimary[(someId)]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[3+2]++')), 'await somePrimary[3 + 2]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[4??6]++')), 'await somePrimary[4 ?? 6]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[1??7]++')), 'await somePrimary[1 ?? 7]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[9??someInt||8]++')), 'await somePrimary[9 ?? someInt || 8]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[3|6%someInt??2+0||8]++')), 'await somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[1+1>=someInt??7&0*32]++')), 'await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[2~/9%5??8-someInt|2]++')), 'await somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[true?false:true]++')), 'await somePrimary[true ? false : true]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[someVar == another?false ? true : false:true]++')), 'await somePrimary[someVar == another ? false ? true : false : true]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(x, y, {x()}).someId')), 'await somePrimary(x, y, {x()}).someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(x (y(x), z), {x()})[9??someInt||8]++')), 'await somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary({x (y(x))})[3|6%someInt??2+0||8]++')), 'await somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(x, y, {x})[1+1>=someInt??7&0*32]++')), 'await somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), 'await somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary({this.x (this.y ( z))})[true?false:true]++')), 'await somePrimary({this.x(this.y(z))})[true ? false : true]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), 'await somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int>(x, y, {x()}).someId')), 'await somePrimary<int>(x, y, {x()}).someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]++')), 'await somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]++')), 'await somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]++')), 'await somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), 'await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]++')), 'await somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), 'await somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary.someId._another')), 'await somePrimary.someId._another');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary?.someId._another')), 'await somePrimary?.someId._another');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary?.someId?._another')), 'await somePrimary?.someId?._another');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[someId][another]++')), 'await somePrimary[someId][another]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[someId][1??7].third')), 'await somePrimary[someId][1 ?? 7].third');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]++')), 'await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]++')), 'await somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(x, y, {x()}).someId.another')), 'await somePrimary(x, y, {x()}).someId.another');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]++')), 'await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]++');
    runTest(AwaitExpression(lexer.parseStr('await super .someId')), 'await super .someId');
    runTest(AwaitExpression(lexer.parseStr('await super ._someId')), 'await super ._someId');
    runTest(AwaitExpression(lexer.parseStr('await super [someId]++')), 'await super [someId]++');
    runTest(AwaitExpression(lexer.parseStr('await super [(someId)]++')), 'await super [(someId)]++');
    runTest(AwaitExpression(lexer.parseStr('await super [3+2]++')), 'await super [3 + 2]++');
    runTest(AwaitExpression(lexer.parseStr('await super [4??6]++')), 'await super [4 ?? 6]++');
    runTest(AwaitExpression(lexer.parseStr('await super [1??7]++')), 'await super [1 ?? 7]++');
    runTest(AwaitExpression(lexer.parseStr('await super [9??someInt||8]++')), 'await super [9 ?? someInt || 8]++');
    runTest(AwaitExpression(lexer.parseStr('await super [3|6%someInt??2+0||8]++')), 'await super [3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(AwaitExpression(lexer.parseStr('await super [1+1>=someInt??7&0*32]++')), 'await super [1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(AwaitExpression(lexer.parseStr('await super [2~/9%5??8-someInt|2]++')), 'await super [2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(AwaitExpression(lexer.parseStr('await super [true?false:true]++')), 'await super [true ? false : true]++');
    runTest(AwaitExpression(lexer.parseStr('await super [someVar == another?false ? true : false:true]++')), 'await super [someVar == another ? false ? true : false : true]++');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary.someId')), 'await somePrimary.someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary._someId')), 'await somePrimary._someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary?.someId')), 'await somePrimary?.someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary?._someId')), 'await somePrimary?._someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[someId]--')), 'await somePrimary[someId]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[(someId)]--')), 'await somePrimary[(someId)]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[3+2]--')), 'await somePrimary[3 + 2]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[4??6]--')), 'await somePrimary[4 ?? 6]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[1??7]--')), 'await somePrimary[1 ?? 7]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[9??someInt||8]--')), 'await somePrimary[9 ?? someInt || 8]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[3|6%someInt??2+0||8]--')), 'await somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[1+1>=someInt??7&0*32]--')), 'await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[2~/9%5??8-someInt|2]--')), 'await somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[true?false:true]--')), 'await somePrimary[true ? false : true]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[someVar == another?false ? true : false:true]--')), 'await somePrimary[someVar == another ? false ? true : false : true]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(x, y, {x()}).someId')), 'await somePrimary(x, y, {x()}).someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(x (y(x), z), {x()})[9??someInt||8]--')), 'await somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary({x (y(x))})[3|6%someInt??2+0||8]--')), 'await somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(x, y, {x})[1+1>=someInt??7&0*32]--')), 'await somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), 'await somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary({this.x (this.y ( z))})[true?false:true]--')), 'await somePrimary({this.x(this.y(z))})[true ? false : true]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), 'await somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int>(x, y, {x()}).someId')), 'await somePrimary<int>(x, y, {x()}).someId');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]--')), 'await somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]--')), 'await somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]--')), 'await somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), 'await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]--')), 'await somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), 'await somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary.someId._another')), 'await somePrimary.someId._another');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary?.someId._another')), 'await somePrimary?.someId._another');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary?.someId?._another')), 'await somePrimary?.someId?._another');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[someId][another]--')), 'await somePrimary[someId][another]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[someId][1??7].third')), 'await somePrimary[someId][1 ?? 7].third');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]--')), 'await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]--')), 'await somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]--');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary(x, y, {x()}).someId.another')), 'await somePrimary(x, y, {x()}).someId.another');
    runTest(AwaitExpression(lexer.parseStr('await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]--')), 'await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]--');
    runTest(AwaitExpression(lexer.parseStr('await super .someId')), 'await super .someId');
    runTest(AwaitExpression(lexer.parseStr('await super ._someId')), 'await super ._someId');
    runTest(AwaitExpression(lexer.parseStr('await super [someId]--')), 'await super [someId]--');
    runTest(AwaitExpression(lexer.parseStr('await super [(someId)]--')), 'await super [(someId)]--');
    runTest(AwaitExpression(lexer.parseStr('await super [3+2]--')), 'await super [3 + 2]--');
    runTest(AwaitExpression(lexer.parseStr('await super [4??6]--')), 'await super [4 ?? 6]--');
    runTest(AwaitExpression(lexer.parseStr('await super [1??7]--')), 'await super [1 ?? 7]--');
    runTest(AwaitExpression(lexer.parseStr('await super [9??someInt||8]--')), 'await super [9 ?? someInt || 8]--');
    runTest(AwaitExpression(lexer.parseStr('await super [3|6%someInt??2+0||8]--')), 'await super [3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(AwaitExpression(lexer.parseStr('await super [1+1>=someInt??7&0*32]--')), 'await super [1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(AwaitExpression(lexer.parseStr('await super [2~/9%5??8-someInt|2]--')), 'await super [2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(AwaitExpression(lexer.parseStr('await super [true?false:true]--')), 'await super [true ? false : true]--');
    runTest(AwaitExpression(lexer.parseStr('await super [someVar == another?false ? true : false:true]--')), 'await super [someVar == another ? false ? true : false : true]--');
    runTest(AwaitExpression(lexer.parseStr('await new Test()')), 'await new Test()');
    runTest(AwaitExpression(lexer.parseStr('await const Test()')), 'await const Test()');
    runTest(AwaitExpression(lexer.parseStr('await const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'await const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(AwaitExpression(lexer.parseStr('await const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), 'await const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(AwaitExpression(lexer.parseStr('await const Test.subType(someId:4,id2:.23,)')), 'await const Test.subType(someId: 4, id2: .23,)');
    runTest(AwaitExpression(lexer.parseStr('await const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), 'await const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(AwaitExpression(lexer.parseStr('await new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'await new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(AwaitExpression(lexer.parseStr('await new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'await new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(AwaitExpression(lexer.parseStr('await new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'await new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(AwaitExpression(lexer.parseStr('await const <someType, int, num>{"1","2","3","4"}')), 'await const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(AwaitExpression(lexer.parseStr('await const <someType, int, num>{someObject}')), 'await const <someType, int, num>{someObject}');
    runTest(AwaitExpression(lexer.parseStr('await const <someType, int, num>{someObject,someObject,someObject}')), 'await const <someType, int, num>{someObject, someObject, someObject}');
    runTest(AwaitExpression(lexer.parseStr('await new Test()()')), 'await new Test()()');
    runTest(AwaitExpression(lexer.parseStr('await const Test()(4900.34)')), 'await const Test()(4900.34)');
    runTest(AwaitExpression(lexer.parseStr('await const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,).someId')), 'await const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,).someId');
    runTest(AwaitExpression(lexer.parseStr('await const Test.subType(someId:someObject.param ? another.param : meh.someVal,)?.someId')), 'await const Test.subType(someId: someObject.param ? another.param : meh.someVal,)?.someId');
    runTest(AwaitExpression(lexer.parseStr('await const Test.subType(someId:4,id2:.23,)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'await const Test.subType(someId: 4, id2: .23,)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(AwaitExpression(lexer.parseStr('await const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'await const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(AwaitExpression(lexer.parseStr('await new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'await new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(AwaitExpression(lexer.parseStr('await new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)<int,String>(234.432,someId:4,)')), 'await new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)<int, String>(234.432, someId: 4,)');
    runTest(AwaitExpression(lexer.parseStr('await new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'await new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(AwaitExpression(lexer.parseStr('await const <someType, int, num>{"1","2","3","4"}<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'await const <someType, int, num>{"1", "2", "3", "4"}<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(AwaitExpression(lexer.parseStr('await const <someType, int, num>{someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'await const <someType, int, num>{someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(AwaitExpression(lexer.parseStr('await const <someType, int, num>{someObject,someObject,someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'await const <someType, int, num>{someObject, someObject, someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

  });

  test('Unary Expression', () {

    runTest(UnaryExpression(lexer.parseStr('-super')), '-super');
    runTest(UnaryExpression(lexer.parseStr('~super')), '~super');

    runTest(UnaryExpression(lexer.parseStr('++somePrimary.someId')), '++somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary._someId')), '++somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary?.someId')), '++somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary?._someId')), '++somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[someId]')), '++somePrimary[someId]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[(someId)]')), '++somePrimary[(someId)]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[3+2]')), '++somePrimary[3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[4??6]')), '++somePrimary[4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[1??7]')), '++somePrimary[1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[9??someInt||8]')), '++somePrimary[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[3|6%someInt??2+0||8]')), '++somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[1+1>=someInt??7&0*32]')), '++somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[2~/9%5??8-someInt|2]')), '++somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[true?false:true]')), '++somePrimary[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[someVar == another?false ? true : false:true]')), '++somePrimary[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary(x, y, {x()}).someId')), '++somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary(x (y(x), z), {x()})[9??someInt||8]')), '++somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary({x (y(x))})[3|6%someInt??2+0||8]')), '++somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary(x, y, {x})[1+1>=someInt??7&0*32]')), '++somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '++somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary({this.x (this.y ( z))})[true?false:true]')), '++somePrimary({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '++somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary<int>(x, y, {x()}).someId')), '++somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]')), '++somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]')), '++somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]')), '++somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '++somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]')), '++somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '++somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary.someId._another')), '++somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary?.someId._another')), '++somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary?.someId?._another')), '++somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[someId][another]')), '++somePrimary[someId][another]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[someId][1??7].third')), '++somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]')), '++somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]')), '++somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary(x, y, {x()}).someId.another')), '++somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('++somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]')), '++somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('++super .someId')), '++super .someId');
    runTest(UnaryExpression(lexer.parseStr('++super ._someId')), '++super ._someId');
    runTest(UnaryExpression(lexer.parseStr('++super [someId]')), '++super [someId]');
    runTest(UnaryExpression(lexer.parseStr('++super [(someId)]')), '++super [(someId)]');
    runTest(UnaryExpression(lexer.parseStr('++super [3+2]')), '++super [3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('++super [4??6]')), '++super [4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('++super [1??7]')), '++super [1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('++super [9??someInt||8]')), '++super [9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('++super [3|6%someInt??2+0||8]')), '++super [3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('++super [1+1>=someInt??7&0*32]')), '++super [1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('++super [2~/9%5??8-someInt|2]')), '++super [2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('++super [true?false:true]')), '++super [true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('++super [someVar == another?false ? true : false:true]')), '++super [someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary.someId')), '--somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary._someId')), '--somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary?.someId')), '--somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary?._someId')), '--somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[someId]')), '--somePrimary[someId]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[(someId)]')), '--somePrimary[(someId)]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[3+2]')), '--somePrimary[3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[4??6]')), '--somePrimary[4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[1??7]')), '--somePrimary[1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[9??someInt||8]')), '--somePrimary[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[3|6%someInt??2+0||8]')), '--somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[1+1>=someInt??7&0*32]')), '--somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[2~/9%5??8-someInt|2]')), '--somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[true?false:true]')), '--somePrimary[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[someVar == another?false ? true : false:true]')), '--somePrimary[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary(x, y, {x()}).someId')), '--somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary(x (y(x), z), {x()})[9??someInt||8]')), '--somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary({x (y(x))})[3|6%someInt??2+0||8]')), '--somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary(x, y, {x})[1+1>=someInt??7&0*32]')), '--somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '--somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary({this.x (this.y ( z))})[true?false:true]')), '--somePrimary({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '--somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary<int>(x, y, {x()}).someId')), '--somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]')), '--somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]')), '--somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]')), '--somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '--somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]')), '--somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '--somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary.someId._another')), '--somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary?.someId._another')), '--somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary?.someId?._another')), '--somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[someId][another]')), '--somePrimary[someId][another]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[someId][1??7].third')), '--somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]')), '--somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]')), '--somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary(x, y, {x()}).someId.another')), '--somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('--somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]')), '--somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('--super .someId')), '--super .someId');
    runTest(UnaryExpression(lexer.parseStr('--super ._someId')), '--super ._someId');
    runTest(UnaryExpression(lexer.parseStr('--super [someId]')), '--super [someId]');
    runTest(UnaryExpression(lexer.parseStr('--super [(someId)]')), '--super [(someId)]');
    runTest(UnaryExpression(lexer.parseStr('--super [3+2]')), '--super [3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('--super [4??6]')), '--super [4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('--super [1??7]')), '--super [1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('--super [9??someInt||8]')), '--super [9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('--super [3|6%someInt??2+0||8]')), '--super [3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('--super [1+1>=someInt??7&0*32]')), '--super [1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('--super [2~/9%5??8-someInt|2]')), '--super [2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('--super [true?false:true]')), '--super [true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('--super [someVar == another?false ? true : false:true]')), '--super [someVar == another ? false ? true : false : true]');

    runTest(UnaryExpression(lexer.parseStr('somePrimary.someId')), 'somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary._someId')), 'somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary?.someId')), 'somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary?._someId')), 'somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[someId]++')), 'somePrimary[someId]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[(someId)]++')), 'somePrimary[(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[3+2]++')), 'somePrimary[3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[4??6]++')), 'somePrimary[4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[1??7]++')), 'somePrimary[1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[9??someInt||8]++')), 'somePrimary[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[3|6%someInt??2+0||8]++')), 'somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[1+1>=someInt??7&0*32]++')), 'somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[2~/9%5??8-someInt|2]++')), 'somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[true?false:true]++')), 'somePrimary[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[someVar == another?false ? true : false:true]++')), 'somePrimary[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(x, y, {x()}).someId')), 'somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(x (y(x), z), {x()})[9??someInt||8]++')), 'somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary({x (y(x))})[3|6%someInt??2+0||8]++')), 'somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(x, y, {x})[1+1>=someInt??7&0*32]++')), 'somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), 'somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary({this.x (this.y ( z))})[true?false:true]++')), 'somePrimary({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), 'somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int>(x, y, {x()}).someId')), 'somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]++')), 'somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]++')), 'somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]++')), 'somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), 'somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]++')), 'somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), 'somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary.someId._another')), 'somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('somePrimary?.someId._another')), 'somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('somePrimary?.someId?._another')), 'somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[someId][another]++')), 'somePrimary[someId][another]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[someId][1??7].third')), 'somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]++')), 'somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]++')), 'somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(x, y, {x()}).someId.another')), 'somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]++')), 'somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('super .someId')), 'super .someId');
    runTest(UnaryExpression(lexer.parseStr('super ._someId')), 'super ._someId');
    runTest(UnaryExpression(lexer.parseStr('super [someId]++')), 'super [someId]++');
    runTest(UnaryExpression(lexer.parseStr('super [(someId)]++')), 'super [(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('super [3+2]++')), 'super [3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('super [4??6]++')), 'super [4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('super [1??7]++')), 'super [1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('super [9??someInt||8]++')), 'super [9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('super [3|6%someInt??2+0||8]++')), 'super [3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('super [1+1>=someInt??7&0*32]++')), 'super [1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('super [2~/9%5??8-someInt|2]++')), 'super [2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('super [true?false:true]++')), 'super [true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('super [someVar == another?false ? true : false:true]++')), 'super [someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('somePrimary.someId')), 'somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary._someId')), 'somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary?.someId')), 'somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary?._someId')), 'somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[someId]--')), 'somePrimary[someId]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[(someId)]--')), 'somePrimary[(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[3+2]--')), 'somePrimary[3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[4??6]--')), 'somePrimary[4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[1??7]--')), 'somePrimary[1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[9??someInt||8]--')), 'somePrimary[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[3|6%someInt??2+0||8]--')), 'somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[1+1>=someInt??7&0*32]--')), 'somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[2~/9%5??8-someInt|2]--')), 'somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[true?false:true]--')), 'somePrimary[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[someVar == another?false ? true : false:true]--')), 'somePrimary[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(x, y, {x()}).someId')), 'somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(x (y(x), z), {x()})[9??someInt||8]--')), 'somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary({x (y(x))})[3|6%someInt??2+0||8]--')), 'somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(x, y, {x})[1+1>=someInt??7&0*32]--')), 'somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), 'somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary({this.x (this.y ( z))})[true?false:true]--')), 'somePrimary({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), 'somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int>(x, y, {x()}).someId')), 'somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]--')), 'somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]--')), 'somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]--')), 'somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), 'somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]--')), 'somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), 'somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary.someId._another')), 'somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('somePrimary?.someId._another')), 'somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('somePrimary?.someId?._another')), 'somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[someId][another]--')), 'somePrimary[someId][another]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[someId][1??7].third')), 'somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]--')), 'somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]--')), 'somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('somePrimary(x, y, {x()}).someId.another')), 'somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]--')), 'somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('super .someId')), 'super .someId');
    runTest(UnaryExpression(lexer.parseStr('super ._someId')), 'super ._someId');
    runTest(UnaryExpression(lexer.parseStr('super [someId]--')), 'super [someId]--');
    runTest(UnaryExpression(lexer.parseStr('super [(someId)]--')), 'super [(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('super [3+2]--')), 'super [3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('super [4??6]--')), 'super [4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('super [1??7]--')), 'super [1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('super [9??someInt||8]--')), 'super [9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('super [3|6%someInt??2+0||8]--')), 'super [3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('super [1+1>=someInt??7&0*32]--')), 'super [1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('super [2~/9%5??8-someInt|2]--')), 'super [2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('super [true?false:true]--')), 'super [true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('super [someVar == another?false ? true : false:true]--')), 'super [someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('new Test()')), 'new Test()');
    runTest(UnaryExpression(lexer.parseStr('const Test()')), 'const Test()');
    runTest(UnaryExpression(lexer.parseStr('const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('const Test.subType(someId:4,id2:.23,)')), 'const Test.subType(someId: 4, id2: .23,)');
    runTest(UnaryExpression(lexer.parseStr('const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), 'const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('const <someType, int, num>{"1","2","3","4"}')), 'const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(UnaryExpression(lexer.parseStr('const <someType, int, num>{someObject}')), 'const <someType, int, num>{someObject}');
    runTest(UnaryExpression(lexer.parseStr('const <someType, int, num>{someObject,someObject,someObject}')), 'const <someType, int, num>{someObject, someObject, someObject}');
    runTest(UnaryExpression(lexer.parseStr('new Test()()')), 'new Test()()');
    runTest(UnaryExpression(lexer.parseStr('const Test()(4900.34)')), 'const Test()(4900.34)');
    runTest(UnaryExpression(lexer.parseStr('const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,).someId')), 'const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,).someId');
    runTest(UnaryExpression(lexer.parseStr('const Test.subType(someId:someObject.param ? another.param : meh.someVal,)?.someId')), 'const Test.subType(someId: someObject.param ? another.param : meh.someVal,)?.someId');
    runTest(UnaryExpression(lexer.parseStr('const Test.subType(someId:4,id2:.23,)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'const Test.subType(someId: 4, id2: .23,)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)<int,String>(234.432,someId:4,)')), 'new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)<int, String>(234.432, someId: 4,)');
    runTest(UnaryExpression(lexer.parseStr('new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('const <someType, int, num>{"1","2","3","4"}<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'const <someType, int, num>{"1", "2", "3", "4"}<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('const <someType, int, num>{someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const <someType, int, num>{someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('const <someType, int, num>{someObject,someObject,someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'const <someType, int, num>{someObject, someObject, someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

    runTest(UnaryExpression(lexer.parseStr('await somePrimary.someId')), 'await somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary._someId')), 'await somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary?.someId')), 'await somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary?._someId')), 'await somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[someId]++')), 'await somePrimary[someId]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[(someId)]++')), 'await somePrimary[(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[3+2]++')), 'await somePrimary[3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[4??6]++')), 'await somePrimary[4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[1??7]++')), 'await somePrimary[1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[9??someInt||8]++')), 'await somePrimary[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[3|6%someInt??2+0||8]++')), 'await somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[1+1>=someInt??7&0*32]++')), 'await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[2~/9%5??8-someInt|2]++')), 'await somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[true?false:true]++')), 'await somePrimary[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[someVar == another?false ? true : false:true]++')), 'await somePrimary[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(x, y, {x()}).someId')), 'await somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(x (y(x), z), {x()})[9??someInt||8]++')), 'await somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary({x (y(x))})[3|6%someInt??2+0||8]++')), 'await somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(x, y, {x})[1+1>=someInt??7&0*32]++')), 'await somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), 'await somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary({this.x (this.y ( z))})[true?false:true]++')), 'await somePrimary({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), 'await somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int>(x, y, {x()}).someId')), 'await somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]++')), 'await somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]++')), 'await somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]++')), 'await somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), 'await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]++')), 'await somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), 'await somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary.someId._another')), 'await somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary?.someId._another')), 'await somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary?.someId?._another')), 'await somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[someId][another]++')), 'await somePrimary[someId][another]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[someId][1??7].third')), 'await somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]++')), 'await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]++')), 'await somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(x, y, {x()}).someId.another')), 'await somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]++')), 'await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('await super .someId')), 'await super .someId');
    runTest(UnaryExpression(lexer.parseStr('await super ._someId')), 'await super ._someId');
    runTest(UnaryExpression(lexer.parseStr('await super [someId]++')), 'await super [someId]++');
    runTest(UnaryExpression(lexer.parseStr('await super [(someId)]++')), 'await super [(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('await super [3+2]++')), 'await super [3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('await super [4??6]++')), 'await super [4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('await super [1??7]++')), 'await super [1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('await super [9??someInt||8]++')), 'await super [9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('await super [3|6%someInt??2+0||8]++')), 'await super [3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('await super [1+1>=someInt??7&0*32]++')), 'await super [1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('await super [2~/9%5??8-someInt|2]++')), 'await super [2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('await super [true?false:true]++')), 'await super [true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('await super [someVar == another?false ? true : false:true]++')), 'await super [someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary.someId')), 'await somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary._someId')), 'await somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary?.someId')), 'await somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary?._someId')), 'await somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[someId]--')), 'await somePrimary[someId]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[(someId)]--')), 'await somePrimary[(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[3+2]--')), 'await somePrimary[3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[4??6]--')), 'await somePrimary[4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[1??7]--')), 'await somePrimary[1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[9??someInt||8]--')), 'await somePrimary[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[3|6%someInt??2+0||8]--')), 'await somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[1+1>=someInt??7&0*32]--')), 'await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[2~/9%5??8-someInt|2]--')), 'await somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[true?false:true]--')), 'await somePrimary[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[someVar == another?false ? true : false:true]--')), 'await somePrimary[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(x, y, {x()}).someId')), 'await somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(x (y(x), z), {x()})[9??someInt||8]--')), 'await somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary({x (y(x))})[3|6%someInt??2+0||8]--')), 'await somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(x, y, {x})[1+1>=someInt??7&0*32]--')), 'await somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), 'await somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary({this.x (this.y ( z))})[true?false:true]--')), 'await somePrimary({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), 'await somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int>(x, y, {x()}).someId')), 'await somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]--')), 'await somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]--')), 'await somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]--')), 'await somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), 'await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]--')), 'await somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), 'await somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary.someId._another')), 'await somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary?.someId._another')), 'await somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary?.someId?._another')), 'await somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[someId][another]--')), 'await somePrimary[someId][another]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[someId][1??7].third')), 'await somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]--')), 'await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]--')), 'await somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary(x, y, {x()}).someId.another')), 'await somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]--')), 'await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('await super .someId')), 'await super .someId');
    runTest(UnaryExpression(lexer.parseStr('await super ._someId')), 'await super ._someId');
    runTest(UnaryExpression(lexer.parseStr('await super [someId]--')), 'await super [someId]--');
    runTest(UnaryExpression(lexer.parseStr('await super [(someId)]--')), 'await super [(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('await super [3+2]--')), 'await super [3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('await super [4??6]--')), 'await super [4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('await super [1??7]--')), 'await super [1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('await super [9??someInt||8]--')), 'await super [9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('await super [3|6%someInt??2+0||8]--')), 'await super [3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('await super [1+1>=someInt??7&0*32]--')), 'await super [1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('await super [2~/9%5??8-someInt|2]--')), 'await super [2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('await super [true?false:true]--')), 'await super [true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('await super [someVar == another?false ? true : false:true]--')), 'await super [someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('await new Test()')), 'await new Test()');
    runTest(UnaryExpression(lexer.parseStr('await const Test()')), 'await const Test()');
    runTest(UnaryExpression(lexer.parseStr('await const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'await const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('await const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), 'await const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('await const Test.subType(someId:4,id2:.23,)')), 'await const Test.subType(someId: 4, id2: .23,)');
    runTest(UnaryExpression(lexer.parseStr('await const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), 'await const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('await new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'await new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('await new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'await new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('await new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'await new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('await const <someType, int, num>{"1","2","3","4"}')), 'await const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(UnaryExpression(lexer.parseStr('await const <someType, int, num>{someObject}')), 'await const <someType, int, num>{someObject}');
    runTest(UnaryExpression(lexer.parseStr('await const <someType, int, num>{someObject,someObject,someObject}')), 'await const <someType, int, num>{someObject, someObject, someObject}');
    runTest(UnaryExpression(lexer.parseStr('await new Test()()')), 'await new Test()()');
    runTest(UnaryExpression(lexer.parseStr('await const Test()(4900.34)')), 'await const Test()(4900.34)');
    runTest(UnaryExpression(lexer.parseStr('await const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,).someId')), 'await const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,).someId');
    runTest(UnaryExpression(lexer.parseStr('await const Test.subType(someId:someObject.param ? another.param : meh.someVal,)?.someId')), 'await const Test.subType(someId: someObject.param ? another.param : meh.someVal,)?.someId');
    runTest(UnaryExpression(lexer.parseStr('await const Test.subType(someId:4,id2:.23,)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'await const Test.subType(someId: 4, id2: .23,)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('await const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), 'await const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('await new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'await new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('await new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)<int,String>(234.432,someId:4,)')), 'await new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)<int, String>(234.432, someId: 4,)');
    runTest(UnaryExpression(lexer.parseStr('await new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), 'await new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('await const <someType, int, num>{"1","2","3","4"}<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), 'await const <someType, int, num>{"1", "2", "3", "4"}<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('await const <someType, int, num>{someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'await const <someType, int, num>{someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('await const <someType, int, num>{someObject,someObject,someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), 'await const <someType, int, num>{someObject, someObject, someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

    runTest(UnaryExpression(lexer.parseStr('!-super')), '!-super');
    runTest(UnaryExpression(lexer.parseStr('!~super')), '!~super');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary.someId')), '!++somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary._someId')), '!++somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary?.someId')), '!++somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary?._someId')), '!++somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[someId]')), '!++somePrimary[someId]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[(someId)]')), '!++somePrimary[(someId)]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[3+2]')), '!++somePrimary[3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[4??6]')), '!++somePrimary[4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[1??7]')), '!++somePrimary[1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[9??someInt||8]')), '!++somePrimary[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[3|6%someInt??2+0||8]')), '!++somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[1+1>=someInt??7&0*32]')), '!++somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[2~/9%5??8-someInt|2]')), '!++somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[true?false:true]')), '!++somePrimary[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[someVar == another?false ? true : false:true]')), '!++somePrimary[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary(x, y, {x()}).someId')), '!++somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary(x (y(x), z), {x()})[9??someInt||8]')), '!++somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary({x (y(x))})[3|6%someInt??2+0||8]')), '!++somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary(x, y, {x})[1+1>=someInt??7&0*32]')), '!++somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '!++somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary({this.x (this.y ( z))})[true?false:true]')), '!++somePrimary({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '!++somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary<int>(x, y, {x()}).someId')), '!++somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]')), '!++somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]')), '!++somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]')), '!++somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '!++somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]')), '!++somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '!++somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary.someId._another')), '!++somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary?.someId._another')), '!++somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary?.someId?._another')), '!++somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[someId][another]')), '!++somePrimary[someId][another]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[someId][1??7].third')), '!++somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]')), '!++somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]')), '!++somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary(x, y, {x()}).someId.another')), '!++somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('!++somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]')), '!++somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!++super .someId')), '!++super .someId');
    runTest(UnaryExpression(lexer.parseStr('!++super ._someId')), '!++super ._someId');
    runTest(UnaryExpression(lexer.parseStr('!++super [someId]')), '!++super [someId]');
    runTest(UnaryExpression(lexer.parseStr('!++super [(someId)]')), '!++super [(someId)]');
    runTest(UnaryExpression(lexer.parseStr('!++super [3+2]')), '!++super [3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('!++super [4??6]')), '!++super [4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('!++super [1??7]')), '!++super [1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('!++super [9??someInt||8]')), '!++super [9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('!++super [3|6%someInt??2+0||8]')), '!++super [3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('!++super [1+1>=someInt??7&0*32]')), '!++super [1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('!++super [2~/9%5??8-someInt|2]')), '!++super [2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('!++super [true?false:true]')), '!++super [true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('!++super [someVar == another?false ? true : false:true]')), '!++super [someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary.someId')), '!--somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary._someId')), '!--somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary?.someId')), '!--somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary?._someId')), '!--somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[someId]')), '!--somePrimary[someId]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[(someId)]')), '!--somePrimary[(someId)]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[3+2]')), '!--somePrimary[3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[4??6]')), '!--somePrimary[4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[1??7]')), '!--somePrimary[1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[9??someInt||8]')), '!--somePrimary[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[3|6%someInt??2+0||8]')), '!--somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[1+1>=someInt??7&0*32]')), '!--somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[2~/9%5??8-someInt|2]')), '!--somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[true?false:true]')), '!--somePrimary[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[someVar == another?false ? true : false:true]')), '!--somePrimary[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary(x, y, {x()}).someId')), '!--somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary(x (y(x), z), {x()})[9??someInt||8]')), '!--somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary({x (y(x))})[3|6%someInt??2+0||8]')), '!--somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary(x, y, {x})[1+1>=someInt??7&0*32]')), '!--somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '!--somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary({this.x (this.y ( z))})[true?false:true]')), '!--somePrimary({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '!--somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary<int>(x, y, {x()}).someId')), '!--somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]')), '!--somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]')), '!--somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]')), '!--somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '!--somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]')), '!--somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '!--somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary.someId._another')), '!--somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary?.someId._another')), '!--somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary?.someId?._another')), '!--somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[someId][another]')), '!--somePrimary[someId][another]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[someId][1??7].third')), '!--somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]')), '!--somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]')), '!--somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary(x, y, {x()}).someId.another')), '!--somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('!--somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]')), '!--somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!--super .someId')), '!--super .someId');
    runTest(UnaryExpression(lexer.parseStr('!--super ._someId')), '!--super ._someId');
    runTest(UnaryExpression(lexer.parseStr('!--super [someId]')), '!--super [someId]');
    runTest(UnaryExpression(lexer.parseStr('!--super [(someId)]')), '!--super [(someId)]');
    runTest(UnaryExpression(lexer.parseStr('!--super [3+2]')), '!--super [3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('!--super [4??6]')), '!--super [4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('!--super [1??7]')), '!--super [1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('!--super [9??someInt||8]')), '!--super [9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('!--super [3|6%someInt??2+0||8]')), '!--super [3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('!--super [1+1>=someInt??7&0*32]')), '!--super [1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('!--super [2~/9%5??8-someInt|2]')), '!--super [2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('!--super [true?false:true]')), '!--super [true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('!--super [someVar == another?false ? true : false:true]')), '!--super [someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary.someId')), '!somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary._someId')), '!somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary?.someId')), '!somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary?._someId')), '!somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[someId]++')), '!somePrimary[someId]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[(someId)]++')), '!somePrimary[(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[3+2]++')), '!somePrimary[3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[4??6]++')), '!somePrimary[4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[1??7]++')), '!somePrimary[1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[9??someInt||8]++')), '!somePrimary[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[3|6%someInt??2+0||8]++')), '!somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[1+1>=someInt??7&0*32]++')), '!somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[2~/9%5??8-someInt|2]++')), '!somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[true?false:true]++')), '!somePrimary[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[someVar == another?false ? true : false:true]++')), '!somePrimary[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(x, y, {x()}).someId')), '!somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(x (y(x), z), {x()})[9??someInt||8]++')), '!somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary({x (y(x))})[3|6%someInt??2+0||8]++')), '!somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(x, y, {x})[1+1>=someInt??7&0*32]++')), '!somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '!somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary({this.x (this.y ( z))})[true?false:true]++')), '!somePrimary({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '!somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int>(x, y, {x()}).someId')), '!somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]++')), '!somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]++')), '!somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]++')), '!somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '!somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]++')), '!somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '!somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary.someId._another')), '!somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary?.someId._another')), '!somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary?.someId?._another')), '!somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[someId][another]++')), '!somePrimary[someId][another]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[someId][1??7].third')), '!somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]++')), '!somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]++')), '!somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(x, y, {x()}).someId.another')), '!somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]++')), '!somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!super .someId')), '!super .someId');
    runTest(UnaryExpression(lexer.parseStr('!super ._someId')), '!super ._someId');
    runTest(UnaryExpression(lexer.parseStr('!super [someId]++')), '!super [someId]++');
    runTest(UnaryExpression(lexer.parseStr('!super [(someId)]++')), '!super [(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('!super [3+2]++')), '!super [3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('!super [4??6]++')), '!super [4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('!super [1??7]++')), '!super [1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('!super [9??someInt||8]++')), '!super [9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!super [3|6%someInt??2+0||8]++')), '!super [3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!super [1+1>=someInt??7&0*32]++')), '!super [1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('!super [2~/9%5??8-someInt|2]++')), '!super [2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('!super [true?false:true]++')), '!super [true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!super [someVar == another?false ? true : false:true]++')), '!super [someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary.someId')), '!somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary._someId')), '!somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary?.someId')), '!somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary?._someId')), '!somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[someId]--')), '!somePrimary[someId]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[(someId)]--')), '!somePrimary[(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[3+2]--')), '!somePrimary[3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[4??6]--')), '!somePrimary[4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[1??7]--')), '!somePrimary[1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[9??someInt||8]--')), '!somePrimary[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[3|6%someInt??2+0||8]--')), '!somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[1+1>=someInt??7&0*32]--')), '!somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[2~/9%5??8-someInt|2]--')), '!somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[true?false:true]--')), '!somePrimary[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[someVar == another?false ? true : false:true]--')), '!somePrimary[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(x, y, {x()}).someId')), '!somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(x (y(x), z), {x()})[9??someInt||8]--')), '!somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary({x (y(x))})[3|6%someInt??2+0||8]--')), '!somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(x, y, {x})[1+1>=someInt??7&0*32]--')), '!somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '!somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary({this.x (this.y ( z))})[true?false:true]--')), '!somePrimary({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '!somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int>(x, y, {x()}).someId')), '!somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]--')), '!somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]--')), '!somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]--')), '!somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '!somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]--')), '!somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '!somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary.someId._another')), '!somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary?.someId._another')), '!somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary?.someId?._another')), '!somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[someId][another]--')), '!somePrimary[someId][another]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[someId][1??7].third')), '!somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]--')), '!somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]--')), '!somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary(x, y, {x()}).someId.another')), '!somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('!somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]--')), '!somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!super .someId')), '!super .someId');
    runTest(UnaryExpression(lexer.parseStr('!super ._someId')), '!super ._someId');
    runTest(UnaryExpression(lexer.parseStr('!super [someId]--')), '!super [someId]--');
    runTest(UnaryExpression(lexer.parseStr('!super [(someId)]--')), '!super [(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('!super [3+2]--')), '!super [3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('!super [4??6]--')), '!super [4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('!super [1??7]--')), '!super [1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('!super [9??someInt||8]--')), '!super [9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!super [3|6%someInt??2+0||8]--')), '!super [3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!super [1+1>=someInt??7&0*32]--')), '!super [1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('!super [2~/9%5??8-someInt|2]--')), '!super [2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('!super [true?false:true]--')), '!super [true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!super [someVar == another?false ? true : false:true]--')), '!super [someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!new Test()')), '!new Test()');
    runTest(UnaryExpression(lexer.parseStr('!const Test()')), '!const Test()');
    runTest(UnaryExpression(lexer.parseStr('!const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '!const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('!const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), '!const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('!const Test.subType(someId:4,id2:.23,)')), '!const Test.subType(someId: 4, id2: .23,)');
    runTest(UnaryExpression(lexer.parseStr('!const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), '!const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('!new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '!new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('!new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '!new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('!new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '!new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('!const <someType, int, num>{"1","2","3","4"}')), '!const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(UnaryExpression(lexer.parseStr('!const <someType, int, num>{someObject}')), '!const <someType, int, num>{someObject}');
    runTest(UnaryExpression(lexer.parseStr('!const <someType, int, num>{someObject,someObject,someObject}')), '!const <someType, int, num>{someObject, someObject, someObject}');
    runTest(UnaryExpression(lexer.parseStr('!new Test()()')), '!new Test()()');
    runTest(UnaryExpression(lexer.parseStr('!const Test()(4900.34)')), '!const Test()(4900.34)');
    runTest(UnaryExpression(lexer.parseStr('!const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,).someId')), '!const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,).someId');
    runTest(UnaryExpression(lexer.parseStr('!const Test.subType(someId:someObject.param ? another.param : meh.someVal,)?.someId')), '!const Test.subType(someId: someObject.param ? another.param : meh.someVal,)?.someId');
    runTest(UnaryExpression(lexer.parseStr('!const Test.subType(someId:4,id2:.23,)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '!const Test.subType(someId: 4, id2: .23,)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('!const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '!const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('!new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '!new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('!new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)<int,String>(234.432,someId:4,)')), '!new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)<int, String>(234.432, someId: 4,)');
    runTest(UnaryExpression(lexer.parseStr('!new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '!new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('!const <someType, int, num>{"1","2","3","4"}<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '!const <someType, int, num>{"1", "2", "3", "4"}<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('!const <someType, int, num>{someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '!const <someType, int, num>{someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('!const <someType, int, num>{someObject,someObject,someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '!const <someType, int, num>{someObject, someObject, someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary.someId')), '!await somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary._someId')), '!await somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary?.someId')), '!await somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary?._someId')), '!await somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[someId]++')), '!await somePrimary[someId]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[(someId)]++')), '!await somePrimary[(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[3+2]++')), '!await somePrimary[3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[4??6]++')), '!await somePrimary[4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[1??7]++')), '!await somePrimary[1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[9??someInt||8]++')), '!await somePrimary[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[3|6%someInt??2+0||8]++')), '!await somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[1+1>=someInt??7&0*32]++')), '!await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[2~/9%5??8-someInt|2]++')), '!await somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[true?false:true]++')), '!await somePrimary[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[someVar == another?false ? true : false:true]++')), '!await somePrimary[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(x, y, {x()}).someId')), '!await somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(x (y(x), z), {x()})[9??someInt||8]++')), '!await somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary({x (y(x))})[3|6%someInt??2+0||8]++')), '!await somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(x, y, {x})[1+1>=someInt??7&0*32]++')), '!await somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '!await somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary({this.x (this.y ( z))})[true?false:true]++')), '!await somePrimary({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '!await somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int>(x, y, {x()}).someId')), '!await somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]++')), '!await somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]++')), '!await somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]++')), '!await somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '!await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]++')), '!await somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '!await somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary.someId._another')), '!await somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary?.someId._another')), '!await somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary?.someId?._another')), '!await somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[someId][another]++')), '!await somePrimary[someId][another]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[someId][1??7].third')), '!await somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]++')), '!await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]++')), '!await somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(x, y, {x()}).someId.another')), '!await somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]++')), '!await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!await super .someId')), '!await super .someId');
    runTest(UnaryExpression(lexer.parseStr('!await super ._someId')), '!await super ._someId');
    runTest(UnaryExpression(lexer.parseStr('!await super [someId]++')), '!await super [someId]++');
    runTest(UnaryExpression(lexer.parseStr('!await super [(someId)]++')), '!await super [(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('!await super [3+2]++')), '!await super [3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('!await super [4??6]++')), '!await super [4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('!await super [1??7]++')), '!await super [1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('!await super [9??someInt||8]++')), '!await super [9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!await super [3|6%someInt??2+0||8]++')), '!await super [3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('!await super [1+1>=someInt??7&0*32]++')), '!await super [1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('!await super [2~/9%5??8-someInt|2]++')), '!await super [2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('!await super [true?false:true]++')), '!await super [true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!await super [someVar == another?false ? true : false:true]++')), '!await super [someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary.someId')), '!await somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary._someId')), '!await somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary?.someId')), '!await somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary?._someId')), '!await somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[someId]--')), '!await somePrimary[someId]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[(someId)]--')), '!await somePrimary[(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[3+2]--')), '!await somePrimary[3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[4??6]--')), '!await somePrimary[4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[1??7]--')), '!await somePrimary[1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[9??someInt||8]--')), '!await somePrimary[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[3|6%someInt??2+0||8]--')), '!await somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[1+1>=someInt??7&0*32]--')), '!await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[2~/9%5??8-someInt|2]--')), '!await somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[true?false:true]--')), '!await somePrimary[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[someVar == another?false ? true : false:true]--')), '!await somePrimary[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(x, y, {x()}).someId')), '!await somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(x (y(x), z), {x()})[9??someInt||8]--')), '!await somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary({x (y(x))})[3|6%someInt??2+0||8]--')), '!await somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(x, y, {x})[1+1>=someInt??7&0*32]--')), '!await somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '!await somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary({this.x (this.y ( z))})[true?false:true]--')), '!await somePrimary({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '!await somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int>(x, y, {x()}).someId')), '!await somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]--')), '!await somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]--')), '!await somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]--')), '!await somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '!await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]--')), '!await somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '!await somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary.someId._another')), '!await somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary?.someId._another')), '!await somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary?.someId?._another')), '!await somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[someId][another]--')), '!await somePrimary[someId][another]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[someId][1??7].third')), '!await somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]--')), '!await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]--')), '!await somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary(x, y, {x()}).someId.another')), '!await somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('!await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]--')), '!await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!await super .someId')), '!await super .someId');
    runTest(UnaryExpression(lexer.parseStr('!await super ._someId')), '!await super ._someId');
    runTest(UnaryExpression(lexer.parseStr('!await super [someId]--')), '!await super [someId]--');
    runTest(UnaryExpression(lexer.parseStr('!await super [(someId)]--')), '!await super [(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('!await super [3+2]--')), '!await super [3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('!await super [4??6]--')), '!await super [4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('!await super [1??7]--')), '!await super [1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('!await super [9??someInt||8]--')), '!await super [9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!await super [3|6%someInt??2+0||8]--')), '!await super [3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('!await super [1+1>=someInt??7&0*32]--')), '!await super [1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('!await super [2~/9%5??8-someInt|2]--')), '!await super [2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('!await super [true?false:true]--')), '!await super [true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!await super [someVar == another?false ? true : false:true]--')), '!await super [someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('!await new Test()')), '!await new Test()');
    runTest(UnaryExpression(lexer.parseStr('!await const Test()')), '!await const Test()');
    runTest(UnaryExpression(lexer.parseStr('!await const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '!await const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('!await const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), '!await const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('!await const Test.subType(someId:4,id2:.23,)')), '!await const Test.subType(someId: 4, id2: .23,)');
    runTest(UnaryExpression(lexer.parseStr('!await const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), '!await const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('!await new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '!await new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('!await new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '!await new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('!await new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '!await new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('!await const <someType, int, num>{"1","2","3","4"}')), '!await const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(UnaryExpression(lexer.parseStr('!await const <someType, int, num>{someObject}')), '!await const <someType, int, num>{someObject}');
    runTest(UnaryExpression(lexer.parseStr('!await const <someType, int, num>{someObject,someObject,someObject}')), '!await const <someType, int, num>{someObject, someObject, someObject}');
    runTest(UnaryExpression(lexer.parseStr('!await new Test()()')), '!await new Test()()');
    runTest(UnaryExpression(lexer.parseStr('!await const Test()(4900.34)')), '!await const Test()(4900.34)');
    runTest(UnaryExpression(lexer.parseStr('!await const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,).someId')), '!await const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,).someId');
    runTest(UnaryExpression(lexer.parseStr('!await const Test.subType(someId:someObject.param ? another.param : meh.someVal,)?.someId')), '!await const Test.subType(someId: someObject.param ? another.param : meh.someVal,)?.someId');
    runTest(UnaryExpression(lexer.parseStr('!await const Test.subType(someId:4,id2:.23,)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '!await const Test.subType(someId: 4, id2: .23,)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('!await const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '!await const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('!await new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '!await new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('!await new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)<int,String>(234.432,someId:4,)')), '!await new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)<int, String>(234.432, someId: 4,)');
    runTest(UnaryExpression(lexer.parseStr('!await new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '!await new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('!await const <someType, int, num>{"1","2","3","4"}<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '!await const <someType, int, num>{"1", "2", "3", "4"}<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('!await const <someType, int, num>{someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '!await const <someType, int, num>{someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('!await const <someType, int, num>{someObject,someObject,someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '!await const <someType, int, num>{someObject, someObject, someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

    runTest(UnaryExpression(lexer.parseStr('~++somePrimary.someId')), '~++somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary._someId')), '~++somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary?.someId')), '~++somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary?._someId')), '~++somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[someId]')), '~++somePrimary[someId]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[(someId)]')), '~++somePrimary[(someId)]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[3+2]')), '~++somePrimary[3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[4??6]')), '~++somePrimary[4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[1??7]')), '~++somePrimary[1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[9??someInt||8]')), '~++somePrimary[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[3|6%someInt??2+0||8]')), '~++somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[1+1>=someInt??7&0*32]')), '~++somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[2~/9%5??8-someInt|2]')), '~++somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[true?false:true]')), '~++somePrimary[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[someVar == another?false ? true : false:true]')), '~++somePrimary[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary(x, y, {x()}).someId')), '~++somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary(x (y(x), z), {x()})[9??someInt||8]')), '~++somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary({x (y(x))})[3|6%someInt??2+0||8]')), '~++somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary(x, y, {x})[1+1>=someInt??7&0*32]')), '~++somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '~++somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary({this.x (this.y ( z))})[true?false:true]')), '~++somePrimary({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '~++somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary<int>(x, y, {x()}).someId')), '~++somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]')), '~++somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]')), '~++somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]')), '~++somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '~++somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]')), '~++somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '~++somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary.someId._another')), '~++somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary?.someId._another')), '~++somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary?.someId?._another')), '~++somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[someId][another]')), '~++somePrimary[someId][another]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[someId][1??7].third')), '~++somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]')), '~++somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]')), '~++somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary(x, y, {x()}).someId.another')), '~++somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('~++somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]')), '~++somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~++super .someId')), '~++super .someId');
    runTest(UnaryExpression(lexer.parseStr('~++super ._someId')), '~++super ._someId');
    runTest(UnaryExpression(lexer.parseStr('~++super [someId]')), '~++super [someId]');
    runTest(UnaryExpression(lexer.parseStr('~++super [(someId)]')), '~++super [(someId)]');
    runTest(UnaryExpression(lexer.parseStr('~++super [3+2]')), '~++super [3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('~++super [4??6]')), '~++super [4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('~++super [1??7]')), '~++super [1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('~++super [9??someInt||8]')), '~++super [9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('~++super [3|6%someInt??2+0||8]')), '~++super [3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('~++super [1+1>=someInt??7&0*32]')), '~++super [1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('~++super [2~/9%5??8-someInt|2]')), '~++super [2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('~++super [true?false:true]')), '~++super [true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('~++super [someVar == another?false ? true : false:true]')), '~++super [someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary.someId')), '~--somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary._someId')), '~--somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary?.someId')), '~--somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary?._someId')), '~--somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[someId]')), '~--somePrimary[someId]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[(someId)]')), '~--somePrimary[(someId)]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[3+2]')), '~--somePrimary[3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[4??6]')), '~--somePrimary[4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[1??7]')), '~--somePrimary[1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[9??someInt||8]')), '~--somePrimary[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[3|6%someInt??2+0||8]')), '~--somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[1+1>=someInt??7&0*32]')), '~--somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[2~/9%5??8-someInt|2]')), '~--somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[true?false:true]')), '~--somePrimary[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[someVar == another?false ? true : false:true]')), '~--somePrimary[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary(x, y, {x()}).someId')), '~--somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary(x (y(x), z), {x()})[9??someInt||8]')), '~--somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary({x (y(x))})[3|6%someInt??2+0||8]')), '~--somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary(x, y, {x})[1+1>=someInt??7&0*32]')), '~--somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '~--somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary({this.x (this.y ( z))})[true?false:true]')), '~--somePrimary({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '~--somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary<int>(x, y, {x()}).someId')), '~--somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]')), '~--somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]')), '~--somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]')), '~--somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '~--somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]')), '~--somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '~--somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary.someId._another')), '~--somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary?.someId._another')), '~--somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary?.someId?._another')), '~--somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[someId][another]')), '~--somePrimary[someId][another]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[someId][1??7].third')), '~--somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]')), '~--somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]')), '~--somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary(x, y, {x()}).someId.another')), '~--somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('~--somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]')), '~--somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~--super .someId')), '~--super .someId');
    runTest(UnaryExpression(lexer.parseStr('~--super ._someId')), '~--super ._someId');
    runTest(UnaryExpression(lexer.parseStr('~--super [someId]')), '~--super [someId]');
    runTest(UnaryExpression(lexer.parseStr('~--super [(someId)]')), '~--super [(someId)]');
    runTest(UnaryExpression(lexer.parseStr('~--super [3+2]')), '~--super [3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('~--super [4??6]')), '~--super [4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('~--super [1??7]')), '~--super [1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('~--super [9??someInt||8]')), '~--super [9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('~--super [3|6%someInt??2+0||8]')), '~--super [3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('~--super [1+1>=someInt??7&0*32]')), '~--super [1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('~--super [2~/9%5??8-someInt|2]')), '~--super [2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('~--super [true?false:true]')), '~--super [true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('~--super [someVar == another?false ? true : false:true]')), '~--super [someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary.someId')), '~somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary._someId')), '~somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary?.someId')), '~somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary?._someId')), '~somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[someId]++')), '~somePrimary[someId]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[(someId)]++')), '~somePrimary[(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[3+2]++')), '~somePrimary[3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[4??6]++')), '~somePrimary[4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[1??7]++')), '~somePrimary[1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[9??someInt||8]++')), '~somePrimary[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[3|6%someInt??2+0||8]++')), '~somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[1+1>=someInt??7&0*32]++')), '~somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[2~/9%5??8-someInt|2]++')), '~somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[true?false:true]++')), '~somePrimary[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[someVar == another?false ? true : false:true]++')), '~somePrimary[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(x, y, {x()}).someId')), '~somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(x (y(x), z), {x()})[9??someInt||8]++')), '~somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary({x (y(x))})[3|6%someInt??2+0||8]++')), '~somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(x, y, {x})[1+1>=someInt??7&0*32]++')), '~somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '~somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary({this.x (this.y ( z))})[true?false:true]++')), '~somePrimary({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '~somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int>(x, y, {x()}).someId')), '~somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]++')), '~somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]++')), '~somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]++')), '~somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '~somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]++')), '~somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '~somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary.someId._another')), '~somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary?.someId._another')), '~somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary?.someId?._another')), '~somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[someId][another]++')), '~somePrimary[someId][another]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[someId][1??7].third')), '~somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]++')), '~somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]++')), '~somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(x, y, {x()}).someId.another')), '~somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]++')), '~somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~super .someId')), '~super .someId');
    runTest(UnaryExpression(lexer.parseStr('~super ._someId')), '~super ._someId');
    runTest(UnaryExpression(lexer.parseStr('~super [someId]++')), '~super [someId]++');
    runTest(UnaryExpression(lexer.parseStr('~super [(someId)]++')), '~super [(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('~super [3+2]++')), '~super [3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('~super [4??6]++')), '~super [4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('~super [1??7]++')), '~super [1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('~super [9??someInt||8]++')), '~super [9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~super [3|6%someInt??2+0||8]++')), '~super [3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~super [1+1>=someInt??7&0*32]++')), '~super [1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('~super [2~/9%5??8-someInt|2]++')), '~super [2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('~super [true?false:true]++')), '~super [true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~super [someVar == another?false ? true : false:true]++')), '~super [someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary.someId')), '~somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary._someId')), '~somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary?.someId')), '~somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary?._someId')), '~somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[someId]--')), '~somePrimary[someId]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[(someId)]--')), '~somePrimary[(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[3+2]--')), '~somePrimary[3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[4??6]--')), '~somePrimary[4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[1??7]--')), '~somePrimary[1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[9??someInt||8]--')), '~somePrimary[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[3|6%someInt??2+0||8]--')), '~somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[1+1>=someInt??7&0*32]--')), '~somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[2~/9%5??8-someInt|2]--')), '~somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[true?false:true]--')), '~somePrimary[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[someVar == another?false ? true : false:true]--')), '~somePrimary[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(x, y, {x()}).someId')), '~somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(x (y(x), z), {x()})[9??someInt||8]--')), '~somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary({x (y(x))})[3|6%someInt??2+0||8]--')), '~somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(x, y, {x})[1+1>=someInt??7&0*32]--')), '~somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '~somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary({this.x (this.y ( z))})[true?false:true]--')), '~somePrimary({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '~somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int>(x, y, {x()}).someId')), '~somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]--')), '~somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]--')), '~somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]--')), '~somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '~somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]--')), '~somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '~somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary.someId._another')), '~somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary?.someId._another')), '~somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary?.someId?._another')), '~somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[someId][another]--')), '~somePrimary[someId][another]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[someId][1??7].third')), '~somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]--')), '~somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]--')), '~somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary(x, y, {x()}).someId.another')), '~somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('~somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]--')), '~somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~super .someId')), '~super .someId');
    runTest(UnaryExpression(lexer.parseStr('~super ._someId')), '~super ._someId');
    runTest(UnaryExpression(lexer.parseStr('~super [someId]--')), '~super [someId]--');
    runTest(UnaryExpression(lexer.parseStr('~super [(someId)]--')), '~super [(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('~super [3+2]--')), '~super [3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('~super [4??6]--')), '~super [4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('~super [1??7]--')), '~super [1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('~super [9??someInt||8]--')), '~super [9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~super [3|6%someInt??2+0||8]--')), '~super [3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~super [1+1>=someInt??7&0*32]--')), '~super [1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('~super [2~/9%5??8-someInt|2]--')), '~super [2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('~super [true?false:true]--')), '~super [true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~super [someVar == another?false ? true : false:true]--')), '~super [someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~new Test()')), '~new Test()');
    runTest(UnaryExpression(lexer.parseStr('~const Test()')), '~const Test()');
    runTest(UnaryExpression(lexer.parseStr('~const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '~const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('~const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), '~const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('~const Test.subType(someId:4,id2:.23,)')), '~const Test.subType(someId: 4, id2: .23,)');
    runTest(UnaryExpression(lexer.parseStr('~const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), '~const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('~new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '~new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('~new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '~new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('~new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '~new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('~const <someType, int, num>{"1","2","3","4"}')), '~const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(UnaryExpression(lexer.parseStr('~const <someType, int, num>{someObject}')), '~const <someType, int, num>{someObject}');
    runTest(UnaryExpression(lexer.parseStr('~const <someType, int, num>{someObject,someObject,someObject}')), '~const <someType, int, num>{someObject, someObject, someObject}');
    runTest(UnaryExpression(lexer.parseStr('~new Test()()')), '~new Test()()');
    runTest(UnaryExpression(lexer.parseStr('~const Test()(4900.34)')), '~const Test()(4900.34)');
    runTest(UnaryExpression(lexer.parseStr('~const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,).someId')), '~const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,).someId');
    runTest(UnaryExpression(lexer.parseStr('~const Test.subType(someId:someObject.param ? another.param : meh.someVal,)?.someId')), '~const Test.subType(someId: someObject.param ? another.param : meh.someVal,)?.someId');
    runTest(UnaryExpression(lexer.parseStr('~const Test.subType(someId:4,id2:.23,)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '~const Test.subType(someId: 4, id2: .23,)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('~const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '~const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('~new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '~new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('~new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)<int,String>(234.432,someId:4,)')), '~new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)<int, String>(234.432, someId: 4,)');
    runTest(UnaryExpression(lexer.parseStr('~new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '~new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('~const <someType, int, num>{"1","2","3","4"}<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '~const <someType, int, num>{"1", "2", "3", "4"}<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('~const <someType, int, num>{someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '~const <someType, int, num>{someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('~const <someType, int, num>{someObject,someObject,someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '~const <someType, int, num>{someObject, someObject, someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary.someId')), '~await somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary._someId')), '~await somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary?.someId')), '~await somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary?._someId')), '~await somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[someId]++')), '~await somePrimary[someId]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[(someId)]++')), '~await somePrimary[(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[3+2]++')), '~await somePrimary[3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[4??6]++')), '~await somePrimary[4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[1??7]++')), '~await somePrimary[1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[9??someInt||8]++')), '~await somePrimary[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[3|6%someInt??2+0||8]++')), '~await somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[1+1>=someInt??7&0*32]++')), '~await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[2~/9%5??8-someInt|2]++')), '~await somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[true?false:true]++')), '~await somePrimary[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[someVar == another?false ? true : false:true]++')), '~await somePrimary[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(x, y, {x()}).someId')), '~await somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(x (y(x), z), {x()})[9??someInt||8]++')), '~await somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary({x (y(x))})[3|6%someInt??2+0||8]++')), '~await somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(x, y, {x})[1+1>=someInt??7&0*32]++')), '~await somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '~await somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary({this.x (this.y ( z))})[true?false:true]++')), '~await somePrimary({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '~await somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int>(x, y, {x()}).someId')), '~await somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]++')), '~await somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]++')), '~await somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]++')), '~await somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '~await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]++')), '~await somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '~await somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary.someId._another')), '~await somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary?.someId._another')), '~await somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary?.someId?._another')), '~await somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[someId][another]++')), '~await somePrimary[someId][another]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[someId][1??7].third')), '~await somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]++')), '~await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]++')), '~await somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(x, y, {x()}).someId.another')), '~await somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]++')), '~await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~await super .someId')), '~await super .someId');
    runTest(UnaryExpression(lexer.parseStr('~await super ._someId')), '~await super ._someId');
    runTest(UnaryExpression(lexer.parseStr('~await super [someId]++')), '~await super [someId]++');
    runTest(UnaryExpression(lexer.parseStr('~await super [(someId)]++')), '~await super [(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('~await super [3+2]++')), '~await super [3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('~await super [4??6]++')), '~await super [4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('~await super [1??7]++')), '~await super [1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('~await super [9??someInt||8]++')), '~await super [9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~await super [3|6%someInt??2+0||8]++')), '~await super [3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('~await super [1+1>=someInt??7&0*32]++')), '~await super [1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('~await super [2~/9%5??8-someInt|2]++')), '~await super [2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('~await super [true?false:true]++')), '~await super [true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~await super [someVar == another?false ? true : false:true]++')), '~await super [someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary.someId')), '~await somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary._someId')), '~await somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary?.someId')), '~await somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary?._someId')), '~await somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[someId]--')), '~await somePrimary[someId]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[(someId)]--')), '~await somePrimary[(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[3+2]--')), '~await somePrimary[3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[4??6]--')), '~await somePrimary[4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[1??7]--')), '~await somePrimary[1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[9??someInt||8]--')), '~await somePrimary[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[3|6%someInt??2+0||8]--')), '~await somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[1+1>=someInt??7&0*32]--')), '~await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[2~/9%5??8-someInt|2]--')), '~await somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[true?false:true]--')), '~await somePrimary[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[someVar == another?false ? true : false:true]--')), '~await somePrimary[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(x, y, {x()}).someId')), '~await somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(x (y(x), z), {x()})[9??someInt||8]--')), '~await somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary({x (y(x))})[3|6%someInt??2+0||8]--')), '~await somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(x, y, {x})[1+1>=someInt??7&0*32]--')), '~await somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '~await somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary({this.x (this.y ( z))})[true?false:true]--')), '~await somePrimary({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '~await somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int>(x, y, {x()}).someId')), '~await somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]--')), '~await somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]--')), '~await somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]--')), '~await somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '~await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]--')), '~await somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '~await somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary.someId._another')), '~await somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary?.someId._another')), '~await somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary?.someId?._another')), '~await somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[someId][another]--')), '~await somePrimary[someId][another]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[someId][1??7].third')), '~await somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]--')), '~await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]--')), '~await somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary(x, y, {x()}).someId.another')), '~await somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('~await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]--')), '~await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~await super .someId')), '~await super .someId');
    runTest(UnaryExpression(lexer.parseStr('~await super ._someId')), '~await super ._someId');
    runTest(UnaryExpression(lexer.parseStr('~await super [someId]--')), '~await super [someId]--');
    runTest(UnaryExpression(lexer.parseStr('~await super [(someId)]--')), '~await super [(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('~await super [3+2]--')), '~await super [3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('~await super [4??6]--')), '~await super [4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('~await super [1??7]--')), '~await super [1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('~await super [9??someInt||8]--')), '~await super [9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~await super [3|6%someInt??2+0||8]--')), '~await super [3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('~await super [1+1>=someInt??7&0*32]--')), '~await super [1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('~await super [2~/9%5??8-someInt|2]--')), '~await super [2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('~await super [true?false:true]--')), '~await super [true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~await super [someVar == another?false ? true : false:true]--')), '~await super [someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('~await new Test()')), '~await new Test()');
    runTest(UnaryExpression(lexer.parseStr('~await const Test()')), '~await const Test()');
    runTest(UnaryExpression(lexer.parseStr('~await const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '~await const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('~await const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), '~await const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('~await const Test.subType(someId:4,id2:.23,)')), '~await const Test.subType(someId: 4, id2: .23,)');
    runTest(UnaryExpression(lexer.parseStr('~await const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), '~await const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('~await new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '~await new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('~await new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '~await new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('~await new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '~await new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('~await const <someType, int, num>{"1","2","3","4"}')), '~await const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(UnaryExpression(lexer.parseStr('~await const <someType, int, num>{someObject}')), '~await const <someType, int, num>{someObject}');
    runTest(UnaryExpression(lexer.parseStr('~await const <someType, int, num>{someObject,someObject,someObject}')), '~await const <someType, int, num>{someObject, someObject, someObject}');
    runTest(UnaryExpression(lexer.parseStr('~await new Test()()')), '~await new Test()()');
    runTest(UnaryExpression(lexer.parseStr('~await const Test()(4900.34)')), '~await const Test()(4900.34)');
    runTest(UnaryExpression(lexer.parseStr('~await const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,).someId')), '~await const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,).someId');
    runTest(UnaryExpression(lexer.parseStr('~await const Test.subType(someId:someObject.param ? another.param : meh.someVal,)?.someId')), '~await const Test.subType(someId: someObject.param ? another.param : meh.someVal,)?.someId');
    runTest(UnaryExpression(lexer.parseStr('~await const Test.subType(someId:4,id2:.23,)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '~await const Test.subType(someId: 4, id2: .23,)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('~await const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '~await const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('~await new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '~await new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('~await new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)<int,String>(234.432,someId:4,)')), '~await new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)<int, String>(234.432, someId: 4,)');
    runTest(UnaryExpression(lexer.parseStr('~await new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '~await new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('~await const <someType, int, num>{"1","2","3","4"}<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '~await const <someType, int, num>{"1", "2", "3", "4"}<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('~await const <someType, int, num>{someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '~await const <someType, int, num>{someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('~await const <someType, int, num>{someObject,someObject,someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '~await const <someType, int, num>{someObject, someObject, someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

    runTest(UnaryExpression(lexer.parseStr('-++somePrimary.someId')), '-++somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary._someId')), '-++somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary?.someId')), '-++somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary?._someId')), '-++somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[someId]')), '-++somePrimary[someId]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[(someId)]')), '-++somePrimary[(someId)]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[3+2]')), '-++somePrimary[3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[4??6]')), '-++somePrimary[4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[1??7]')), '-++somePrimary[1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[9??someInt||8]')), '-++somePrimary[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[3|6%someInt??2+0||8]')), '-++somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[1+1>=someInt??7&0*32]')), '-++somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[2~/9%5??8-someInt|2]')), '-++somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[true?false:true]')), '-++somePrimary[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[someVar == another?false ? true : false:true]')), '-++somePrimary[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary(x, y, {x()}).someId')), '-++somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary(x (y(x), z), {x()})[9??someInt||8]')), '-++somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary({x (y(x))})[3|6%someInt??2+0||8]')), '-++somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary(x, y, {x})[1+1>=someInt??7&0*32]')), '-++somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '-++somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary({this.x (this.y ( z))})[true?false:true]')), '-++somePrimary({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '-++somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary<int>(x, y, {x()}).someId')), '-++somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]')), '-++somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]')), '-++somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]')), '-++somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]')), '-++somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]')), '-++somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]')), '-++somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary.someId._another')), '-++somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary?.someId._another')), '-++somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary?.someId?._another')), '-++somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[someId][another]')), '-++somePrimary[someId][another]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[someId][1??7].third')), '-++somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]')), '-++somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]')), '-++somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary(x, y, {x()}).someId.another')), '-++somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('-++somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]')), '-++somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('-++super .someId')), '-++super .someId');
    runTest(UnaryExpression(lexer.parseStr('-++super ._someId')), '-++super ._someId');
    runTest(UnaryExpression(lexer.parseStr('-++super [someId]')), '-++super [someId]');
    runTest(UnaryExpression(lexer.parseStr('-++super [(someId)]')), '-++super [(someId)]');
    runTest(UnaryExpression(lexer.parseStr('-++super [3+2]')), '-++super [3 + 2]');
    runTest(UnaryExpression(lexer.parseStr('-++super [4??6]')), '-++super [4 ?? 6]');
    runTest(UnaryExpression(lexer.parseStr('-++super [1??7]')), '-++super [1 ?? 7]');
    runTest(UnaryExpression(lexer.parseStr('-++super [9??someInt||8]')), '-++super [9 ?? someInt || 8]');
    runTest(UnaryExpression(lexer.parseStr('-++super [3|6%someInt??2+0||8]')), '-++super [3 | 6 % someInt ?? 2 + 0 || 8]');
    runTest(UnaryExpression(lexer.parseStr('-++super [1+1>=someInt??7&0*32]')), '-++super [1 + 1 >= someInt ?? 7 & 0 * 32]');
    runTest(UnaryExpression(lexer.parseStr('-++super [2~/9%5??8-someInt|2]')), '-++super [2 ~/ 9 % 5 ?? 8 - someInt | 2]');
    runTest(UnaryExpression(lexer.parseStr('-++super [true?false:true]')), '-++super [true ? false : true]');
    runTest(UnaryExpression(lexer.parseStr('-++super [someVar == another?false ? true : false:true]')), '-++super [someVar == another ? false ? true : false : true]');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary.someId')), '-somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary._someId')), '-somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary?.someId')), '-somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary?._someId')), '-somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[someId]++')), '-somePrimary[someId]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[(someId)]++')), '-somePrimary[(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[3+2]++')), '-somePrimary[3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[4??6]++')), '-somePrimary[4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[1??7]++')), '-somePrimary[1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[9??someInt||8]++')), '-somePrimary[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[3|6%someInt??2+0||8]++')), '-somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[1+1>=someInt??7&0*32]++')), '-somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[2~/9%5??8-someInt|2]++')), '-somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[true?false:true]++')), '-somePrimary[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[someVar == another?false ? true : false:true]++')), '-somePrimary[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(x, y, {x()}).someId')), '-somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(x (y(x), z), {x()})[9??someInt||8]++')), '-somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary({x (y(x))})[3|6%someInt??2+0||8]++')), '-somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(x, y, {x})[1+1>=someInt??7&0*32]++')), '-somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '-somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary({this.x (this.y ( z))})[true?false:true]++')), '-somePrimary({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '-somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int>(x, y, {x()}).someId')), '-somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]++')), '-somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]++')), '-somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]++')), '-somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '-somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]++')), '-somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '-somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary.someId._another')), '-somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary?.someId._another')), '-somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary?.someId?._another')), '-somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[someId][another]++')), '-somePrimary[someId][another]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[someId][1??7].third')), '-somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]++')), '-somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]++')), '-somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(x, y, {x()}).someId.another')), '-somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]++')), '-somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-super .someId')), '-super .someId');
    runTest(UnaryExpression(lexer.parseStr('-super ._someId')), '-super ._someId');
    runTest(UnaryExpression(lexer.parseStr('-super [someId]++')), '-super [someId]++');
    runTest(UnaryExpression(lexer.parseStr('-super [(someId)]++')), '-super [(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('-super [3+2]++')), '-super [3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('-super [4??6]++')), '-super [4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('-super [1??7]++')), '-super [1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('-super [9??someInt||8]++')), '-super [9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-super [3|6%someInt??2+0||8]++')), '-super [3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-super [1+1>=someInt??7&0*32]++')), '-super [1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('-super [2~/9%5??8-someInt|2]++')), '-super [2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('-super [true?false:true]++')), '-super [true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-super [someVar == another?false ? true : false:true]++')), '-super [someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary.someId')), '-somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary._someId')), '-somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary?.someId')), '-somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary?._someId')), '-somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[someId]--')), '-somePrimary[someId]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[(someId)]--')), '-somePrimary[(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[3+2]--')), '-somePrimary[3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[4??6]--')), '-somePrimary[4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[1??7]--')), '-somePrimary[1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[9??someInt||8]--')), '-somePrimary[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[3|6%someInt??2+0||8]--')), '-somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[1+1>=someInt??7&0*32]--')), '-somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[2~/9%5??8-someInt|2]--')), '-somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[true?false:true]--')), '-somePrimary[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[someVar == another?false ? true : false:true]--')), '-somePrimary[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(x, y, {x()}).someId')), '-somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(x (y(x), z), {x()})[9??someInt||8]--')), '-somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary({x (y(x))})[3|6%someInt??2+0||8]--')), '-somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(x, y, {x})[1+1>=someInt??7&0*32]--')), '-somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '-somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary({this.x (this.y ( z))})[true?false:true]--')), '-somePrimary({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '-somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int>(x, y, {x()}).someId')), '-somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]--')), '-somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]--')), '-somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]--')), '-somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '-somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]--')), '-somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '-somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary.someId._another')), '-somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary?.someId._another')), '-somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary?.someId?._another')), '-somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[someId][another]--')), '-somePrimary[someId][another]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[someId][1??7].third')), '-somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]--')), '-somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]--')), '-somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary(x, y, {x()}).someId.another')), '-somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('-somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]--')), '-somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-super .someId')), '-super .someId');
    runTest(UnaryExpression(lexer.parseStr('-super ._someId')), '-super ._someId');
    runTest(UnaryExpression(lexer.parseStr('-super [someId]--')), '-super [someId]--');
    runTest(UnaryExpression(lexer.parseStr('-super [(someId)]--')), '-super [(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('-super [3+2]--')), '-super [3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('-super [4??6]--')), '-super [4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('-super [1??7]--')), '-super [1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('-super [9??someInt||8]--')), '-super [9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-super [3|6%someInt??2+0||8]--')), '-super [3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-super [1+1>=someInt??7&0*32]--')), '-super [1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('-super [2~/9%5??8-someInt|2]--')), '-super [2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('-super [true?false:true]--')), '-super [true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-super [someVar == another?false ? true : false:true]--')), '-super [someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-new Test()')), '-new Test()');
    runTest(UnaryExpression(lexer.parseStr('-const Test()')), '-const Test()');
    runTest(UnaryExpression(lexer.parseStr('-const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '-const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('-const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), '-const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('-const Test.subType(someId:4,id2:.23,)')), '-const Test.subType(someId: 4, id2: .23,)');
    runTest(UnaryExpression(lexer.parseStr('-const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), '-const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('-new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '-new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('-new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '-new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('-new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '-new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('-const <someType, int, num>{"1","2","3","4"}')), '-const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(UnaryExpression(lexer.parseStr('-const <someType, int, num>{someObject}')), '-const <someType, int, num>{someObject}');
    runTest(UnaryExpression(lexer.parseStr('-const <someType, int, num>{someObject,someObject,someObject}')), '-const <someType, int, num>{someObject, someObject, someObject}');
    runTest(UnaryExpression(lexer.parseStr('-new Test()()')), '-new Test()()');
    runTest(UnaryExpression(lexer.parseStr('-const Test()(4900.34)')), '-const Test()(4900.34)');
    runTest(UnaryExpression(lexer.parseStr('-const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,).someId')), '-const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,).someId');
    runTest(UnaryExpression(lexer.parseStr('-const Test.subType(someId:someObject.param ? another.param : meh.someVal,)?.someId')), '-const Test.subType(someId: someObject.param ? another.param : meh.someVal,)?.someId');
    runTest(UnaryExpression(lexer.parseStr('-const Test.subType(someId:4,id2:.23,)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '-const Test.subType(someId: 4, id2: .23,)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('-const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '-const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('-new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '-new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('-new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)<int,String>(234.432,someId:4,)')), '-new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)<int, String>(234.432, someId: 4,)');
    runTest(UnaryExpression(lexer.parseStr('-new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '-new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('-const <someType, int, num>{"1","2","3","4"}<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '-const <someType, int, num>{"1", "2", "3", "4"}<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('-const <someType, int, num>{someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '-const <someType, int, num>{someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('-const <someType, int, num>{someObject,someObject,someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '-const <someType, int, num>{someObject, someObject, someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary.someId')), '-await somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary._someId')), '-await somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary?.someId')), '-await somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary?._someId')), '-await somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[someId]++')), '-await somePrimary[someId]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[(someId)]++')), '-await somePrimary[(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[3+2]++')), '-await somePrimary[3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[4??6]++')), '-await somePrimary[4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[1??7]++')), '-await somePrimary[1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[9??someInt||8]++')), '-await somePrimary[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[3|6%someInt??2+0||8]++')), '-await somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[1+1>=someInt??7&0*32]++')), '-await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[2~/9%5??8-someInt|2]++')), '-await somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[true?false:true]++')), '-await somePrimary[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[someVar == another?false ? true : false:true]++')), '-await somePrimary[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(x, y, {x()}).someId')), '-await somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(x (y(x), z), {x()})[9??someInt||8]++')), '-await somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary({x (y(x))})[3|6%someInt??2+0||8]++')), '-await somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(x, y, {x})[1+1>=someInt??7&0*32]++')), '-await somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '-await somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary({this.x (this.y ( z))})[true?false:true]++')), '-await somePrimary({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '-await somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int>(x, y, {x()}).someId')), '-await somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]++')), '-await somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]++')), '-await somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]++')), '-await somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]++')), '-await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]++')), '-await somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]++')), '-await somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary.someId._another')), '-await somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary?.someId._another')), '-await somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary?.someId?._another')), '-await somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[someId][another]++')), '-await somePrimary[someId][another]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[someId][1??7].third')), '-await somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]++')), '-await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]++')), '-await somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(x, y, {x()}).someId.another')), '-await somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]++')), '-await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-await super .someId')), '-await super .someId');
    runTest(UnaryExpression(lexer.parseStr('-await super ._someId')), '-await super ._someId');
    runTest(UnaryExpression(lexer.parseStr('-await super [someId]++')), '-await super [someId]++');
    runTest(UnaryExpression(lexer.parseStr('-await super [(someId)]++')), '-await super [(someId)]++');
    runTest(UnaryExpression(lexer.parseStr('-await super [3+2]++')), '-await super [3 + 2]++');
    runTest(UnaryExpression(lexer.parseStr('-await super [4??6]++')), '-await super [4 ?? 6]++');
    runTest(UnaryExpression(lexer.parseStr('-await super [1??7]++')), '-await super [1 ?? 7]++');
    runTest(UnaryExpression(lexer.parseStr('-await super [9??someInt||8]++')), '-await super [9 ?? someInt || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-await super [3|6%someInt??2+0||8]++')), '-await super [3 | 6 % someInt ?? 2 + 0 || 8]++');
    runTest(UnaryExpression(lexer.parseStr('-await super [1+1>=someInt??7&0*32]++')), '-await super [1 + 1 >= someInt ?? 7 & 0 * 32]++');
    runTest(UnaryExpression(lexer.parseStr('-await super [2~/9%5??8-someInt|2]++')), '-await super [2 ~/ 9 % 5 ?? 8 - someInt | 2]++');
    runTest(UnaryExpression(lexer.parseStr('-await super [true?false:true]++')), '-await super [true ? false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-await super [someVar == another?false ? true : false:true]++')), '-await super [someVar == another ? false ? true : false : true]++');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary.someId')), '-await somePrimary.someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary._someId')), '-await somePrimary._someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary?.someId')), '-await somePrimary?.someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary?._someId')), '-await somePrimary?._someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[someId]--')), '-await somePrimary[someId]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[(someId)]--')), '-await somePrimary[(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[3+2]--')), '-await somePrimary[3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[4??6]--')), '-await somePrimary[4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[1??7]--')), '-await somePrimary[1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[9??someInt||8]--')), '-await somePrimary[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[3|6%someInt??2+0||8]--')), '-await somePrimary[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[1+1>=someInt??7&0*32]--')), '-await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[2~/9%5??8-someInt|2]--')), '-await somePrimary[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[true?false:true]--')), '-await somePrimary[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[someVar == another?false ? true : false:true]--')), '-await somePrimary[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(x, y, {x()}).someId')), '-await somePrimary(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(x (y(x), z), {x()})[9??someInt||8]--')), '-await somePrimary(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary({x (y(x))})[3|6%someInt??2+0||8]--')), '-await somePrimary({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(x, y, {x})[1+1>=someInt??7&0*32]--')), '-await somePrimary(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '-await somePrimary(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary({this.x (this.y ( z))})[true?false:true]--')), '-await somePrimary({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '-await somePrimary(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int>(x, y, {x()}).someId')), '-await somePrimary<int>(x, y, {x()}).someId');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>(x (y(x), z), {x()})[9??someInt||8]--')), '-await somePrimary<int, num>(x(y(x), z), {x()})[9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>({x (y(x))})[3|6%someInt??2+0||8]--')), '-await somePrimary<int, num>({x(y(x))})[3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>(x, y, {x})[1+1>=someInt??7&0*32]--')), '-await somePrimary<int, num>(x, y, {x})[1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2]--')), '-await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>({this.x (this.y ( z))})[true?false:true]--')), '-await somePrimary<int, num>({this.x(this.y(z))})[true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>(this.x (this.y ( z)), {x (y(x)),})[someVar == another?false ? true : false:true]--')), '-await somePrimary<int, num>(this.x(this.y(z)), {x(y(x)),})[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary.someId._another')), '-await somePrimary.someId._another');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary?.someId._another')), '-await somePrimary?.someId._another');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary?.someId?._another')), '-await somePrimary?.someId?._another');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[someId][another]--')), '-await somePrimary[someId][another]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[someId][1??7].third')), '-await somePrimary[someId][1 ?? 7].third');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[1+1>=someInt??7&0*32][2~/9%5??8-someInt|2]--')), '-await somePrimary[1 + 1 >= someInt ?? 7 & 0 * 32][2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary[true?false:true][someVar == another?false ? true : false:true][someVar == another?false ? true : false:true]--')), '-await somePrimary[true ? false : true][someVar == another ? false ? true : false : true][someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary(x, y, {x()}).someId.another')), '-await somePrimary(x, y, {x()}).someId.another');
    runTest(UnaryExpression(lexer.parseStr('-await somePrimary<int,num>(this.x (), this.y, {this.x})[2~/9%5??8-someInt|2][true?false:true].third[someVar == another?false ? true : false:true]--')), '-await somePrimary<int, num>(this.x(), this.y, {this.x})[2 ~/ 9 % 5 ?? 8 - someInt | 2][true ? false : true].third[someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-await super .someId')), '-await super .someId');
    runTest(UnaryExpression(lexer.parseStr('-await super ._someId')), '-await super ._someId');
    runTest(UnaryExpression(lexer.parseStr('-await super [someId]--')), '-await super [someId]--');
    runTest(UnaryExpression(lexer.parseStr('-await super [(someId)]--')), '-await super [(someId)]--');
    runTest(UnaryExpression(lexer.parseStr('-await super [3+2]--')), '-await super [3 + 2]--');
    runTest(UnaryExpression(lexer.parseStr('-await super [4??6]--')), '-await super [4 ?? 6]--');
    runTest(UnaryExpression(lexer.parseStr('-await super [1??7]--')), '-await super [1 ?? 7]--');
    runTest(UnaryExpression(lexer.parseStr('-await super [9??someInt||8]--')), '-await super [9 ?? someInt || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-await super [3|6%someInt??2+0||8]--')), '-await super [3 | 6 % someInt ?? 2 + 0 || 8]--');
    runTest(UnaryExpression(lexer.parseStr('-await super [1+1>=someInt??7&0*32]--')), '-await super [1 + 1 >= someInt ?? 7 & 0 * 32]--');
    runTest(UnaryExpression(lexer.parseStr('-await super [2~/9%5??8-someInt|2]--')), '-await super [2 ~/ 9 % 5 ?? 8 - someInt | 2]--');
    runTest(UnaryExpression(lexer.parseStr('-await super [true?false:true]--')), '-await super [true ? false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-await super [someVar == another?false ? true : false:true]--')), '-await super [someVar == another ? false ? true : false : true]--');
    runTest(UnaryExpression(lexer.parseStr('-await new Test()')), '-await new Test()');
    runTest(UnaryExpression(lexer.parseStr('-await const Test()')), '-await const Test()');
    runTest(UnaryExpression(lexer.parseStr('-await const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '-await const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('-await const Test.subType(someId:someObject.param ? another.param : meh.someVal,)')), '-await const Test.subType(someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('-await const Test.subType(someId:4,id2:.23,)')), '-await const Test.subType(someId: 4, id2: .23,)');
    runTest(UnaryExpression(lexer.parseStr('-await const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)')), '-await const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('-await new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '-await new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('-await new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '-await new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('-await new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '-await new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('-await const <someType, int, num>{"1","2","3","4"}')), '-await const <someType, int, num>{"1", "2", "3", "4"}');
    runTest(UnaryExpression(lexer.parseStr('-await const <someType, int, num>{someObject}')), '-await const <someType, int, num>{someObject}');
    runTest(UnaryExpression(lexer.parseStr('-await const <someType, int, num>{someObject,someObject,someObject}')), '-await const <someType, int, num>{someObject, someObject, someObject}');
    runTest(UnaryExpression(lexer.parseStr('-await new Test()()')), '-await new Test()()');
    runTest(UnaryExpression(lexer.parseStr('-await const Test()(4900.34)')), '-await const Test()(4900.34)');
    runTest(UnaryExpression(lexer.parseStr('-await const Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,).someId')), '-await const Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,).someId');
    runTest(UnaryExpression(lexer.parseStr('-await const Test.subType(someId:someObject.param ? another.param : meh.someVal,)?.someId')), '-await const Test.subType(someId: someObject.param ? another.param : meh.someVal,)?.someId');
    runTest(UnaryExpression(lexer.parseStr('-await const Test.subType(someId:4,id2:.23,)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '-await const Test.subType(someId: 4, id2: .23,)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('-await const Test.subType(someId:someObject(),id2:object2(),id3:object3(),)<int,String>(someId:someObject.param, _id2:otherParam.param.subparam,)')), '-await const Test.subType(someId: someObject(), id2: object2(), id3: object3(),)<int, String>(someId: someObject.param, _id2: otherParam.param.subparam,)');
    runTest(UnaryExpression(lexer.parseStr('-await new Test.subType(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)<int,String>(someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '-await new Test.subType(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)<int, String>(someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('-await new Test.subType("someString", someId:someObject(),id2:object2(),id3:object3(),)<int,String>(234.432,someId:4,)')), '-await new Test.subType("someString", someId: someObject(), id2: object2(), id3: object3(),)<int, String>(234.432, someId: 4,)');
    runTest(UnaryExpression(lexer.parseStr('-await new Test.subType(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)<int,String>(1,2,3,4,someId:someObject.param ? another.param : meh.someVal,)')), '-await new Test.subType(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)<int, String>(1, 2, 3, 4, someId: someObject.param ? another.param : meh.someVal,)');
    runTest(UnaryExpression(lexer.parseStr('-await const <someType, int, num>{"1","2","3","4"}<int,String>("someString", someId:someObject(),id2:object2(),id3:object3(),)')), '-await const <someType, int, num>{"1", "2", "3", "4"}<int, String>("someString", someId: someObject(), id2: object2(), id3: object3(),)');
    runTest(UnaryExpression(lexer.parseStr('-await const <someType, int, num>{someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '-await const <someType, int, num>{someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');
    runTest(UnaryExpression(lexer.parseStr('-await const <someType, int, num>{someObject,someObject,someObject}<int,String>(someObject,someObject,someObject,someId:someObject.param ? another.param : meh.someVal, withExpression: 3+3/2%10,)')), '-await const <someType, int, num>{someObject, someObject, someObject}<int, String>(someObject, someObject, someObject, someId: someObject.param ? another.param : meh.someVal, withExpression: 3 + 3 / 2 % 10,)');

  });

  test('Expression', () {
    // TODO
  });

}
